/*
 * (C) Copyright 2010 WyPlay SAS.
 * Frederic Mazuel <fmazuel@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/device.h>
#include <linux/ctype.h>
#include <linux/sysfs.h>
#include <linux/stm/pio.h>

#define POWER_BUTTON_PORT	4
#define POWER_BUTTON_PIN 	3

static dev_t nangmaa_button_dev_number;
static struct class *nangmaa_button_class;
static struct class_device *nangmaa_button_class_dev;
static struct stpio_pin *power_button_pin;
static unsigned int power_button_state;

ssize_t show_power_button_state(struct class_device *cdev, char *buf)
{
	char *pos = buf;

	power_button_pin = stpio_request_pin(POWER_BUTTON_PORT,
					     POWER_BUTTON_PIN,
					     "nangmaa power_button",
					     STPIO_IN);
	power_button_state = stpio_get_pin(power_button_pin);
	stpio_free_pin(power_button_pin);

	/* Update the status buffer */
	pos += sprintf(pos, "%u\n", power_button_state);

	return pos - buf;
}

static CLASS_DEVICE_ATTR(state, S_IRUGO, show_power_button_state, NULL);

static int __init power_button_state_init(void)
{
	int ret;

	printk("%s\n",__FUNCTION__);

	ret = alloc_chrdev_region (&nangmaa_button_dev_number, 0, 1, "button");
	if (ret < 0) {
		printk (KERN_ERR
			"%s: nangmaa button can't register device err: 0x%x\n",
			__FUNCTION__, ret);
		goto exit;
	}

	nangmaa_button_class = class_create (THIS_MODULE, "nangmaa_button");
	if (IS_ERR(nangmaa_button_class)) {
		printk( KERN_ERR
			"%s: ERR creating nangmaa_button class: %ld)\n",
			__FUNCTION__, PTR_ERR(nangmaa_button_class));
		ret = PTR_ERR(nangmaa_button_class);
		goto err1;
	}

	nangmaa_button_class_dev = class_device_create (nangmaa_button_class, NULL,
						 nangmaa_button_dev_number, NULL,
						 "power");
	if (IS_ERR(nangmaa_button_class_dev)) {
		printk( KERN_ERR
			"%s: ERR creating rcuserv0 class device: %ld)\n",
			__FUNCTION__, PTR_ERR(nangmaa_button_class_dev));
		ret = PTR_ERR(nangmaa_button_class_dev);
		goto exit;
	}

	ret = class_device_create_file(nangmaa_button_class_dev,
				       &class_device_attr_state);
	if (ret < 0) {
		printk (KERN_ERR
			"%s: ERR creating power button state file (%d)\n",
			__FUNCTION__, ret);
		goto err2;
	}

	goto exit;
err2:
	class_destroy(nangmaa_button_class);
err1:
	unregister_chrdev_region(nangmaa_button_dev_number, 1);
exit:
	return ret;
}

static void __exit power_button_state_exit(void)
{
	class_destroy(nangmaa_button_class);
	unregister_chrdev_region(nangmaa_button_dev_number, 1);
}

module_init(power_button_state_init);
module_exit(power_button_state_exit);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("nagmaa Power button state");
MODULE_AUTHOR("Frederic Mazuel <fmazuel@wyplay.com>");
