/*
 * board_mb618.h
 *
 * Copyright (C) STMicroelectronics Limited 2003. All rights reserved.
 *
 * Code to initialize EMBX2 on an MB618.
 */

#define MAILBOX0	0xfe211000	/* Video LX */
#define MAILBOX1	0xfe212000	/* Audio LX */

#if defined(__SH4__)

#if defined(__OS21__)
#include <os21/st40/sti7105.h>

extern interrupt_name_t OS21_INTERRUPT_MBOX_DMU_SH4;
extern interrupt_name_t OS21_INTERRUPT_MBOX_AUD_SH4;

static EMBX_FACTORY embxPlatformInit(void)
{
	EMBXSHM_MailboxConfig_t config = {
		"shm",			/* name */
		0,			/* cpuID */
		{ 1, 1, 0, 0, 0, 0, 0, 0 },		/* participants */
                0x60000000,		/* pointerWarp */
		0,			/* maxPorts */
		16,			/* maxObjects */
		16,			/* freeListSize */
		0,			/* sharedAddr */
		2 * 1024 * 1024,	/* sharedSize */
		NULL, 0,		/* Primary Warp range */
		NULL, 0			/* Secondary Warp range */
	};

	EMBXSHM_MailboxConfig_t config2 = {
		"shm2",			/* name */
		0,			/* cpuID */
		{ 1, 0, 1, 0, 0, 0, 0, 0 },		/* participants */
                0x60000000,		/* pointerWarp */
		0,			/* maxPorts */
		16,			/* maxObjects */
		16,			/* freeListSize */
		0,			/* sharedAddr */
		2 * 1024 * 1024,	/* sharedSize */
		NULL, 0,		/* Primary Warp range */
		NULL, 0			/* Secondary Warp range */
	};

        EMBX_ERROR err;
        EMBX_FACTORY hFactory, hFactory2;

        err = EMBX_Mailbox_Init();
	assert (EMBX_SUCCESS == err || EMBX_ALREADY_INITIALIZED == err); 

	if (err != EMBX_ALREADY_INITIALIZED) {
		err = EMBX_Mailbox_Register((void *) MAILBOX0, OS21_INTERRUPT_MBOX_DMU_SH4, 0, 
					    EMBX_MAILBOX_FLAGS_SET2);
		assert (EMBX_SUCCESS == err); 

		err = EMBX_Mailbox_Register((void *) MAILBOX1, OS21_INTERRUPT_MBOX_AUD_SH4, 0, 
					    EMBX_MAILBOX_FLAGS_SET2);
		assert (EMBX_SUCCESS == err); 
	}

	err = EMBX_Init();
	assert (EMBX_SUCCESS == err || EMBX_ALREADY_INITIALIZED == err); 

	/* register the transport */
	err = EMBX_RegisterTransport( EMBXSHM_mailbox_factory,
				      &config, sizeof(config), &hFactory);
	assert (EMBX_SUCCESS == err); 

	/* Bugzilla #4661: register and unregister a dummy second transport */
	err = EMBX_RegisterTransport( EMBXSHM_mailbox_factory,
				      &config2, sizeof(config2), &hFactory2);
	assert (EMBX_SUCCESS == err); 
	err = EMBX_UnregisterTransport( hFactory2 );
	
	return hFactory;
}
#elif defined(__LINUX__)

#if defined __KERNEL__

int main(char *argv[], int argc);

int __init harness_init(void)
{
	int res;
	
	printk("******** Loading test module ********\n");

	/* run the test */
	res = main(NULL, 0);

	printk("******** Test returned %d ********\n", res);

	return res;
}

void __exit harness_deinit(void)
{
	printk("******** Unloaded test module ********\n");
}

module_init(harness_init);
module_exit(harness_deinit);
#endif

static EMBX_FACTORY embxPlatformInit(void)
{
	EMBXSHM_MailboxConfig_t config = {
		"shm",			/* name */
		0,			/* cpuID */
		{ 1, 1, 0, 0, 0, 0, 0, 0 },		/* participants */
                0x60000000,		/* pointerWarp */
		0,			/* maxPorts */
		16,			/* maxObjects */
		16,			/* freeListSize */
		0,			/* sharedAddr */
		2 * 1024 * 1024,	/* sharedSize */
		NULL, 0,		/* Primary Warp range */
		NULL, 0			/* Secondary Warp range */
	};

        EMBX_ERROR err;
        EMBX_FACTORY hFactory;

	/* on Linux EMBX and the RPC uServer are setup using module
	 * parameters
	 */

	err = EMBX_Init();
	printf("EMBX_INIT returned %d\n", err);

	/* register the transport */
	err = EMBX_RegisterTransport( EMBXSHM_mailbox_factory,
				      &config, sizeof(config), &hFactory);

	return hFactory;
}

#elif defined(__WINCE__)
#include <nkintr.h>
#include <SH4202T_intc2_irq.h>

static void embxPlatformInit(void)
{
	EMBXSHM_MailboxConfig_t config = { 
		"shm",		/* name */
		0,		/* cpuID */
		{ 1, 1, 0, 0, 0, 0, 0, 0 },	/* participants */
                0x60000000,     /* pointerWarp */
		0,		/* maxPorts */
		16,		/* maxObjects */
		16,		/* freeListSize */
		(void *) 0xa6000000, /* sharedAddr */
		2 * 1024 * 1024,/* sharedSize */
		NULL, 0,	/* Primary Warp range */
		NULL, 0		/* Secondary Warp range */
	};

        EMBX_ERROR err;
        EMBX_FACTORY hFactory;

	UINT32 irq;
	DWORD sysintr, out;

        err = EMBX_Mailbox_Init();
	assert (EMBX_SUCCESS == err); 

	/* Obtain an logical interrupt number from the hardware derived one.
	 * When this interrupt is no longer required it should be released using
	 * IOCTL_HAL_RELEASE_SYSINTR but this is not implemented because the
	 * Multicom example framework does not support de-initialization.
	 */
	irq = IRQ_ST231_DELPHI;
	sysintr = SYSINTR_NOP;
	err = KernelIoControl(IOCTL_HAL_REQUEST_SYSINTR, 
	                      &irq, sizeof(irq),
			      &sysintr, sizeof(sysintr), &out);
	assert(err && out == sizeof(sysintr));

        err = EMBX_Mailbox_Register((void *) MAILBOX0, sysintr, 0, 
				    EMBX_MAILBOX_FLAGS_SET2 | EMBX_MAILBOX_FLAGS_LOCKS);
	assert (EMBX_SUCCESS == err); 

	/* strictly speaking we don't need the audio interrupt so we'll register
	 * a mailbox as though it is not capable of interrupt us.
	 */
	err = EMBX_Mailbox_Register((void *) MAILBOX1, -1, -1, EMBX_MAILBOX_FLAGS_LOCKS);
	assert (EMBX_SUCCESS == err); 

        /* register the transport */
        err = EMBX_RegisterTransport( EMBXSHM_mailbox_factory,
                        &config, sizeof(config), &hFactory);
	assert (EMBX_SUCCESS == err); 

	err = EMBX_Init();
	assert (EMBX_SUCCESS == err); 
}

#else
#error Unsupported operating system
#endif /* __OS21__ */

#endif /* __SH4__ */


#if defined(__ST200__)

#if defined(__OS21__)
/* #include <os21/st200/sti7105_audio.h> */
extern interrupt_name_t OS21_INTERRUPT_MBOX_SH4;
#endif

static EMBX_FACTORY embxPlatformInit(void)
{
	EMBXSHM_MailboxConfig_t config = {
		{ 's', 'h', 'm' },	/* name */
		1,			/* cpuID */
		{ 1, 1, 0, 0 },		/* participants */
		0,			/* pointerWarp */
	};

        EMBX_ERROR err;
        EMBX_FACTORY hFactory;

        err = EMBX_Mailbox_Init();
	assert (EMBX_SUCCESS == err); 


	err = EMBX_Mailbox_Register((void *) MAILBOX0, -1, -1, 0);
	assert (EMBX_SUCCESS == err);
        err = EMBX_Mailbox_Register((void *) MAILBOX1, OS21_INTERRUPT_MBOX_SH4, 0, 
			            EMBX_MAILBOX_FLAGS_SET1);
	assert (EMBX_SUCCESS == err); 


        /* register the transport */
        err = EMBX_RegisterTransport( EMBXSHM_mailbox_factory,
                        &config, sizeof(config), &hFactory);
	assert (EMBX_SUCCESS == err); 


	err = EMBX_Init();
	assert (EMBX_SUCCESS == err);

	return hFactory;
}
#endif /* __ST200__ */

/*
 * Local Variables:
 *  tab-width: 8
 *  c-indent-level: 8
 *  c-basic-offset: 8
 * End:
 */
