/*
 * os_linux.h
 *
 * Copyright (C) STMicroelectronics Limited 2003. All rights reserved.
 *
 * Code to get Linux up and running.
 */

#if defined __KERNEL__

#include <linux/errno.h>
#include <linux/kernel.h>
#define  __NO_VERSION__
#include <linux/module.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/version.h>
#include <linux/sched.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
#include <linux/bigphysarea.h>
#else
#include <linux/bpa2.h>
#endif
#include <linux/reboot.h>

#include <linux/moduleparam.h>

/* mangle some of the normal C functions that do not exist in kernel space */
#define exit(code) harness_exit(code, NULL)
#define free(block) kfree(block)
#define fprintf(stream, fmt, args...) printk(fmt, ##args)
#define fflush(stream)
#define printf(fmt, args...) printk(fmt, ##args)
#define malloc(size) kmalloc(size, GFP_KERNEL)

static void osRuntimeInit(void)
{
}

#else /* __KERNEL__ */

#define USERMODE
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

static void osRuntimeInit(void)
{
	/* force line buffering even isatty() is zero (gdbserver) */
	setvbuf(stdout, NULL, _IOLBF, 0);
}

#endif /* __KERNEL__ */

