/*
 * hanger.c
 *
 * Copyright (C) STMicroelectronics Limited 2004. All rights reserved.
 *
 * Very simple transformer that hangs forever on execution (H.264 simulator)
 */

#include <assert.h>
#include <stdlib.h>

#include <os21.h>

#include <mme.h>

#include "hanger.h"

struct mixerCtx {
	float volume[2]; 
};

static MME_ERROR abortCommand(void *ctx, MME_CommandId_t cmdId)
{
	/* This transformer supports neither aborting the currently executing 
	 * transform nor does it utilize deferred transforms. For this reason
	 * there is nothing useful it can do here.
	 */

	return MME_INVALID_ARGUMENT;
}

static MME_ERROR getTransformerCapability(MME_TransformerCapability_t *capability)
{
	capability->Version = 0x010040;
	return MME_SUCCESS;
}

static MME_ERROR initTransformer(MME_UINT paramsSize, MME_GenericParams_t params, void **ctx)
{
	struct mixerCtx *mixerCtx;

	/* if we have a parameter structure ensure it is the correct size */
	if (paramsSize != 0 && paramsSize != MME_LENGTH_BYTES(MixerParamsInit)) {
		return MME_INVALID_ARGUMENT;
	}

	/* allocate the context structure */
	mixerCtx = malloc(sizeof(struct mixerCtx));
	if (!mixerCtx) {
		return MME_NOMEM;
	}

	/* extract the initialization parameters (if they are present) */
	mixerCtx->volume[0] = params ? MME_INDEXED_PARAM(params, MixerParamsInit_volume, 0) : 1.0f;
	mixerCtx->volume[1] = params ? MME_INDEXED_PARAM(params, MixerParamsInit_volume, 1) : 1.0f;

	/* supply the context pointer and return */
	*ctx = mixerCtx;
	return MME_SUCCESS;
}

static MME_ERROR transform(void *ctx, MME_Command_t *cmd)
{
	int restore = interrupt_mask_all();
	exit(42);
	while (1) {
		/* do nothing (just in case exit fails for some reason) */
	}
	interrupt_unmask(restore);

	return MME_SUCCESS;
}

static MME_ERROR processCommand(void *ctx, MME_Command_t *cmd)
{
	switch (cmd->CmdCode) {
	case MME_TRANSFORM:
		return transform(ctx, cmd);

	case MME_SET_GLOBAL_TRANSFORM_PARAMS: /* not supported by this transformer */
	case MME_SEND_BUFFERS: /* not supported by this transformer */
	default:
		break;
	}

	return MME_INVALID_ARGUMENT;
}

static MME_ERROR termTransformer(void *ctx)
{
	free(ctx);
	return MME_SUCCESS;
}


MME_ERROR Mixer_RegisterTransformer(const char *name)
{
	return MME_RegisterTransformer(name, abortCommand, getTransformerCapability,
	                               initTransformer, processCommand, termTransformer);
}
