#
# mkfiles/sparc_solaris.mak
#
# Copyright (C) STMicroelectronics Ltd. 2001
#
# Setup macros for SH4 Linux userspace
#

ifndef DISABLE_SPARC_SOLARIS
  ifneq (,$(call FINDEXEC,gcc))
    # gcc is found on most Unix boxen so we'd better check the host type as well
    ifeq ($(HOSTTYPE),Solaris)
      ENABLE_SPARC_SOLARIS = 1
    endif
  endif
endif

OBJDIR_SPARC_SOLARIS   = obj/solaris/sparc
ifdef ENABLE_PLATFORM_SPECIFIC_OBJECTS
OBJDIR_SPARC_SOLARIS  := $(OBJDIR_SPARC_SOLARIS)/$(PLATFORM)
endif

ifdef ENABLE_SPARC_SOLARIS

CLEAN_DIRS             += $(OBJDIR_SPARC_SOLARIS)/*
MAKE_DIRS              += $(OBJDIR_SPARC_SOLARIS)
LIBDIR_SPARC_SOLARIS       = $(RPC_ROOT)/lib/solaris/sparc

CC_SPARC_SOLARIS        = gcc
CPP_SPARC_SOLARIS       = gcc -E
CFLAGS_SPARC_SOLARIS    = -pedantic -Wall \
                          $(DEFS_SPARC_SOLARIS) $(OPT_SPARC_SOLARIS) $(DEBUG_SPARC_SOLARIS) \
		          $(LOCAL_CFLAGS) $(LOCAL_CFLAGS_SPARC_SOLARIS) $(DEBUG_CFLAGS) $(INCS_SPARC_SOLARIS)
DEFS_SPARC_SOLARIS      = -D__SOLARIS__ -D__SPARC__
INCS_SPARC_SOLARIS      = -I$(RPC_ROOT)/include

ifdef ENABLE_RPCCC
CC_SPARC_SOLARIS       := rpccc $(CC_SPARC_SOLARIS)
endif
ifdef ENABLE_DEBUG
OPT_SPARC_SOLARIS       = 
DEBUG_SPARC_SOLARIS     = -g
else
OPT_SPARC_SOLARIS       = -O3
DEBUG_SPARC_SOLARIS     =
endif

OBJ_SPARC_SOLARIS       = o

$(OBJDIR_SPARC_SOLARIS) :
	-$(MKDIR) $(call DOSCMD,$@)

LIB_SPARC_SOLARIS       = ar
LIBFLAGS_SPARC_SOLARIS  = -rcs $(LOCAL_LIBFLAGS_SPARC_SOLARIS)

LD_SPARC_SOLARIS        = gcc
LDFLAGS_SPARC_SOLARIS   = -L$(LIBDIR_SPARC_SOLARIS) $(LOCAL_LDFLAGS_SPARC_SOLARIS) \
                          $(LIBS_SPARC_SOLARIS)
ifndef DISABLE_EMBXSHELL
LIBS_SPARC_SOLARIS     += -lembxshell
endif
LIBS_SPARC_SOLARIS     += -lposix4 -lpthread

RTLD_SPARC_SOLARIS      = /usr/bin/env

.SUFFIXES: .$(OBJ_SPARC_SOLARIS) .c

$(OBJDIR_SPARC_SOLARIS)/%.$(OBJ_SPARC_SOLARIS) : %.c
ifdef ENABLE_COMENTARY
	@$(ECHO) +++ Compile SPARC/Solaris source [$<]. +++
endif
	$(CC_SPARC_SOLARIS) -c $(CFLAGS_SPARC_SOLARIS) $< -o $@ 

ifdef OBJS_SPARC_SOLARIS

ifdef LIBRARY
TARGET_FOR_SPARC_SOLARIS = $(OBJDIR_SPARC_SOLARIS)/lib$(LIBRARY).a
DIST_SPARC_SOLARIS_LIB  += $(TARGET_FOR_SPARC_SOLARIS)

$(TARGET_FOR_SPARC_SOLARIS) : $(OBJS_SPARC_SOLARIS)
	$(LIB_SPARC_SOLARIS) $(LIBFLAGS_SPARC_SOLARIS) $@ $(OBJS_SPARC_SOLARIS)
endif

ifdef APPLICATION
TARGET_FOR_SPARC_SOLARIS = $(OBJDIR_SPARC_SOLARIS)/$(APPLICATION)
DIST_SPARC_SOLARIS_BIN  += $(TARGET_FOR_SPARC_SOLARIS)

$(TARGET_FOR_SPARC_SOLARIS) : $(OBJS_SPARC_SOLARIS)
	$(LD_SPARC_SOLARIS) -o $@ $(OBJS_SPARC_SOLARIS) $(LDFLAGS_SPARC_SOLARIS)
endif 

endif # OBJS_SPARC_SOLARIS

MAKE_TARGETS         += $(TARGET_FOR_SPARC_SOLARIS)

endif # ENABLE_SPARC_SOLARIS

