/*
 * ArgHD X1
 * PCB revision ARGHDA_MBRD_V0040
 *
 * Derived from STMicroelectronics STb7100 Reference board support.
 *
 * (C) Copyright 2005 STMicroelectronics Limited.
 * Stuart Menefy <stuart.menefy@st.com>
 *
 * (C) Copyright 2006-2009 WyPlay SAS.
 * Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>
 * Aubin Constans <aconstans@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * TODO:
 *      - sort out the USBPWR stuff from stb7100ref/setup.c
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/stm/pio.h>
#include <linux/stm/soc.h>
#include <linux/stm/emi.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/partitions.h>
#include <linux/i2c.h>
#include <linux/i2c-pio.h>
#include <linux/wymcu/mcu.h>
#include <linux/phy.h>
#include <linux/leds.h>
#include <asm/io.h>
#include <asm/irl.h>

#ifdef CONFIG_BPA2

#include <linux/bpa2.h>

static const char *LMI_VID_partalias[] = {
	"BPA2_Region1",
	"coredisplay-video",
	"gfx-memory",
	"v4l2-video-buffers",
	NULL
};

static const char *LMI_SYS_partalias[] = {
	"BPA2_Region0",
	"bigphysarea",
	"v4l2-coded-video-buffers",
	NULL
};

static struct bpa2_partition_desc bpa2_parts_table[] = {
	{
		.name	= "LMI_VID",
		.start	= 0x10800000,
		.size	= 0x03800000,	/* 64-8=56MB */
		.flags	= 0,
		.aka	= LMI_VID_partalias,
	}, {
		.name	= "LMI_SYS",
		.start	= 0,
		.size	= 0x03800000,	/* 56MB */
		.flags	= 0,
		.aka	= LMI_SYS_partalias,
	},
};

static void arghda_bpa2_init(void)
{
	u32 size;

	/* Find out the amount of DDR-SDRAM on system then video LMI */
	size = 0;
	stx7100_get_ram_size(0, &size);
	printk("%luMiB system DDR-SDRAM, ", size / (1024ul * 1024ul));
	size = 0;
	stx7100_get_ram_size(1, &size);
	printk("%luMiB video DDR-SDRAM\n", size / (1024ul * 1024ul));

	bpa2_init(bpa2_parts_table, ARRAY_SIZE(bpa2_parts_table));
}

#else

static inline void arghda_bpa2_init(void) { return; }

#endif  /* CONFIG_BPA2 */


void __init arghda_setup(char **cmdline_p)
{
	extern void arghda_pio_init(void);
	int ascs[1] = { 2 | (STASC_FLAG_NORTSCTS << 8) };

	stx7100_early_device_init();
	stb7100_configure_asc(ascs, ARRAY_SIZE(ascs), 1);

	arghda_pio_init();
	arghda_bpa2_init();
}

/*
 * NOR flash
 */
#ifdef CONFIG_MTD_PHYSMAP
/* default partition table, overridden by mtdparts command line option */
static struct mtd_partition mtd_parts_table[] = {
	/* preloader area */
	{
		.name           = "Boot Firmware",
		.size           = 0x003f0000,
		.offset         = 0x00000000,
	},

	/* WM-DRM */
	{
		.name           = "DRM",
		.size           = 0x00004000,
		.offset         = 0x003f8000,
	},

	/* read-write production/installer data */
	{
		.name           = "rwdata",
		.size           = 0x00002000,
		.offset         = 0x003fc000,
	},

	/* read-only production data */
	{
		.name           = "rodata",
		.size           = 0x00002000,
		.offset         = 0x003fe000,
	},
};

static struct physmap_flash_data physmap_flash_data = {
	.width		= 2,
	.set_vpp	= NULL,
	.nr_parts	= ARRAY_SIZE(mtd_parts_table),
	.parts		= mtd_parts_table,
};
#endif

static struct resource physmap_flash_resource = {
	/* completed at runtime */
	.flags	= IORESOURCE_MEM,
};

static struct platform_device physmap_flash = {
	.name	= "physmap-flash",
	.id	= -1,
	.dev	= {
#ifdef CONFIG_MTD_PHYSMAP
		.platform_data = &physmap_flash_data,
#endif
	},
	.num_resources	= 1,
	.resource	= &physmap_flash_resource,
};

static struct usb_init_data usb_init __initdata = {
	.oc_en = 0,
	.pwr_en = 0,
};

static int phy_reset(void* priv)
{
	struct stpio_pin* pio_ptr = NULL;

	/* PIO5_3 = LAN8700:nRST */
	pio_ptr = stpio_request_set_pin(5, 3, "ETH_PHY_RESET", STPIO_OUT, 1);
	if (pio_ptr == NULL)
		return 0;
	/* Reset the Ethernet PHY SMSC LAN8700
	 * assert nRST (low) for at least 100usec,
	 * with MII clock fed in at the same time */
	udelay(1);
	stpio_set_pin(pio_ptr, 0);
	udelay(110);
	stpio_set_pin(pio_ptr, 1);
	udelay(1);
	stpio_free_pin(pio_ptr);
	return 0;
}

static struct plat_stmmacphy_data phy_private_data = {
	.bus_id = 0,
	.phy_addr = 0x1f,
	.phy_mask = ~(1UL << 0x1f),
	.interface = PHY_INTERFACE_MODE_MII,
	.phy_reset = phy_reset,
};

static struct platform_device stmmac_phy_device = {
	.name		= "stmmacphy",
	.id		= 0,
	.num_resources	= 1,
	.resource	= (struct resource[]) {
		{
			.name	= "phyirq",
			.start	= IRL3_IRQ,
			.end	= IRL3_IRQ,
			.flags	= IORESOURCE_IRQ,
		},
	},
	.dev = {
		.platform_data = &phy_private_data,
	 }
};

/*
 * Infrared interfaces
 */
static unsigned lirc_enabled_pios[] = {
	[0] = 3 * 8 + 3
};

/*
 * SSC
 */
static struct plat_ssc_data ssc_private_info = {
	.capability  = ssc0_has(SSC_I2C_CAPABILITY) |
			ssc1_has(SSC_I2C_CAPABILITY) |
			ssc2_has(SSC_UNCONFIGURED)
};

/*
 * I2C
 */
static struct i2c_adapter_info i2c_adapt_data = {
	.pio = {
		{0, 4},	/* SCL port, pin */
		{0, 3},	/* SDA port, pin */
	},
	.bus_freq = 1000,	/* HZ */
};

static struct platform_device i2c_adapter_device = {
	.name = "i2c_st_2",
	.id = 3,
	.dev = {
		.platform_data = &i2c_adapt_data,
	}
};

/*
 * MCU
 */
static struct mcu_dev mcu_bus_devices_info[] = {
	{
		.bus_type = I2C,
		.bus_data = (void *)0x41,
		.bus_id	  = 3,
		.name = "mcu_x1_1",
	},
};

static struct mcu_devs mcu_bus_devices = {
	.nb = ARRAY_SIZE(mcu_bus_devices_info),
	.devs = mcu_bus_devices_info,
};

static struct platform_device mcu_bus = {
	.name = "mcu",
	.dev = {
		.platform_data = &mcu_bus_devices,
	}
};


#if defined(CONFIG_LEDS_GPIO)
static struct platform_device leds_gpio = {
	.name = "leds-gpio",
	.id = 0,
	.dev.platform_data = &(struct gpio_led_platform_data) {
		.num_leds = 1,
		.leds = (struct gpio_led[]) {
			{	/* BLUE */
				.name = "0",
				.gpio = stpio_to_gpio(0, 0),
			},
		},
	},
};
#endif /* CONFIG_LEDS_GPIO */

static struct platform_device *arghda_devices[] __initdata = {
	&physmap_flash,
	&stmmac_phy_device,
	&mcu_bus,
	&i2c_adapter_device,
#if defined(CONFIG_LEDS_GPIO)
	&leds_gpio,
#endif
};

static int __init arghda_device_init(void)
{
	const unsigned long emi_bank1_base = emi_bank_base(1);

	physmap_flash_resource.start = emi_bank_base(0);
	physmap_flash_resource.end = emi_bank1_base - 1UL;

	stx7100_configure_sata();
	stx7100_configure_ssc(&ssc_private_info);
	stx7100_configure_usb(&usb_init);
	stx7100_configure_ethernet(0, 0, 0);
	stx7100_configure_lirc(NULL, lirc_enabled_pios,
	                       ARRAY_SIZE(lirc_enabled_pios));

	return platform_add_devices(arghda_devices,
				    ARRAY_SIZE(arghda_devices));
}

device_initcall(arghda_device_init);
