/*
 * Sagem Bostona 7105 - Sagem Bostona + boot NOR SPI
 * Bostona PCB revision V1
 *
 * Derived from STMicroelectronics STi7105-SDK Reference board support.
 *
 * Copyright (C) 2008 STMicroelectronics Limited
 * Author: Stuart Menefy (stuart.menefy@st.com)
 *
 * Copyright (C) 2009 WyPlay SAS.
 * Aubin Constans <aconstans@wyplay.com>
 *
 * Copyright (C) 2010 WyPlay SAS.
 * Laurent Fazio <lfazio@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/leds.h>
#include <linux/stm/pio.h>
#include <linux/stm/soc.h>
#include <linux/stm/emi.h>
#include <linux/delay.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/partitions.h>
#include <linux/i2c.h>
#include <linux/spi/spi.h>
#include <linux/spi/flash.h>
#include <linux/bpa2.h>
#include <sound/stm.h>
#include <linux/mtd/nand.h>
#include <linux/stm/nand.h>
#include <linux/stm/soc_init.h>
#include <linux/phy.h>
#include <linux/gpio_keys.h>
#include <linux/input.h>
#include <asm/irq-ilc.h>
#include <asm/irl.h>
#include <asm/io.h>

#define ST7105_EMI_BANK_OFFSET 0x00000000
#define ST7105_EMI_BANK_LENGTH 0x07FFFFFF

#ifdef CONFIG_BPA2

#include <linux/bpa2.h>

const char *player_buffers_partalias[] = {
	"BPA2_Region0",
	"v4l2-coded-video-buffers",
	"BPA2_Region1",
	"v4l2-video-buffers",
	"coredisplay-video",
	NULL
};

const char *bigphys_partalias[] = {
	"gfx-memory-1",
	NULL
};

static struct bpa2_partition_desc bpa2_parts_table[] = {
	{
		.name	= "LMI_SYS",		/* 85MiB */
		.start	= 0x45800000,
		.size	= 0x05500000,
		.flags	= 0,
		.aka	= player_buffers_partalias
	},
	{
		.name	= "bigphysarea",	/* 18MiB */
		.start	= 0x4ad00000,
		.size	= 0x01200000,
		.flags	= 0,
		.aka	= bigphys_partalias
	},
	{
		.name	= "gfx-memory-0",	/* 48MiB */
		.start	= 0x4c000000,
		.size	= 0x03000000,
		.flags	= 0,
		.aka	= NULL
	}
};

#endif  /* CONFIG_BPA2 */

static struct stpio_pin* phy_reset_pin = NULL;

static int ascs[] __initdata = {
        1 | (STASC_FLAG_NORTSCTS << 8),
	2 | (STASC_FLAG_NORTSCTS << 8),
};

static void __init bostona_pio_init(void)
{
	struct stpio_pin *pio_ptr = NULL;

        /** Memories **/
	/** Network **/
	/* PIO11_0 = LAN8700 nRST with pull-down */
	/* PHY reset is performed by the phy_reset() callback, below */
	phy_reset_pin = stpio_request_set_pin(11, 0, "eth_phy_reset",
	                                      STPIO_OUT, 0);

        /* RESET */
	/* PIO16_0 = AVM_RESET => Reset Audio Video Matrix */
	/* PIO16_2 = FE_TER_RESET => Reset Frontend DVB-T */
	pio_ptr = stpio_request_set_pin(16, 0, "AVM_RESET", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(16, 2, "FE_TER_RESET", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);

	/** HDD **/
	/* PIO5_1 = HDD power switch active high with pull-down */
	/* PIO11_5 = CDE_ALIM_ANT = Power antenna supply */
	pio_ptr = stpio_request_set_pin(5, 1, "CDE_ALIM_HDD", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(11, 5, "CDE_ALIM_ANT", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);

	/** Front Panel **/
	/* PIO7_0 = FP_CLK_AFF => Display */
	/* PIO7_1 = FP_DIN_AFF => Display */
	/* PIO7_2 = FP_STB_AFF => Display */
	pio_ptr = stpio_request_set_pin(7, 0, "FP_CLK_AFF", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(7, 1, "FP_DIN_AFF", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(7, 2, "FP_STB_AFF", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);
	
}

static __init void bostona_setup(char** cmdline_p)
{
	printk("WyPlay Boston 7105 V1 board initialisation\n");

	stx7105_early_device_init();
	stx7105_configure_asc(ascs, ARRAY_SIZE(ascs), 0);
	bostona_pio_init();
#ifdef CONFIG_BPA2
	bpa2_init(bpa2_parts_table, ARRAY_SIZE(bpa2_parts_table));
#endif
}

#if 0
static struct plat_stm_pwm_data pwm_private_info = {
	.flags		= PLAT_STM_PWM_OUT0,
	.routing	= PWM_OUT0_PIO4_4,
	.pwmclkdiv	= 3, /* 26kHz, for the fan */
};
#endif

static struct plat_ssc_data ssc_private_info = {
	.capability =
	ssc0_has(SSC_I2C_CAPABILITY) |
	ssc1_has(SSC_I2C_CAPABILITY) |
	ssc2_has(SSC_UNCONFIGURED)   |
	ssc3_has(SSC_UNCONFIGURED),
};


/*
 * USB
 */
static struct usb_init_data usb_init[2] __initdata = {
	{
		.oc_en = 1,
		.oc_pinsel = USB0_OC_PIO12_5,
		.oc_actlow = 1,  // @TODO
		.pwr_en = 1,
		.pwr_pinsel = USB0_PWR_PIO12_6,
	}, {
		.oc_en = 1,
		.oc_pinsel = USB1_OC_PIO4_6,
		.oc_actlow = 1,  // @TODO
		.pwr_en = 1,
		.pwr_pinsel = USB1_PWR_PIO4_7,
	}
};

#ifndef CONFIG_SMSC_PHY
#warning stpio locking conflict between phy_reset and stx7105_configure_ethernet
#endif

static int phy_reset(void* priv)
{
	struct stpio_pin *pio_ptr = NULL;

	if (phy_reset_pin == NULL)
		return 0;
	/* Reset SMSC LAN8700 PHY
	 * assert nRST (low) for at least 100usec,
	 * with MII clock fed in at the same time */
	udelay(1);
	stpio_set_pin(phy_reset_pin, 0);
	udelay(100);
	/* It happens that LAN8700:RXD/MODE[2:0] and RXD3/nINTSEL nets are
	 * bootstraps both for the LAN8700 and the STi7105!
	 * They are pulled to suit STi7105 requirements.
	 * Consequently, this software, running after the STi7105 has completed
	 * its own boot sequence, is in charge of forcing applicable bootstrap
	 * values, for the LAN8700 to sense them upon the rising edge of nRST.
	 */
	/* Enable all 10/100 capabilities: MODE[2:0]=0x7 */
	pio_ptr = stpio_request_set_pin(8, 6, "LAN8700_MODE0", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(8, 7, "LAN8700_MODE1", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(9, 0, "LAN8700_MODE2", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	/* Configure nINT/TXER/TXD4 pin as interrupt output: nINTSEL=1 */
	pio_ptr = stpio_request_set_pin(9, 1, "LAN8700_nINTSEL", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	udelay(5);
	stpio_set_pin(phy_reset_pin, 1);
	udelay(10);
	/* Switch back from bootstrap to regular PIO operation */
	pio_ptr = stpio_request_pin(8, 6, "MII_RXD0", STPIO_IN);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_pin(8, 7, "MII_RXD1", STPIO_IN);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_pin(9, 0, "MII_RXD2", STPIO_IN);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_pin(9, 1, "MII_RXD3", STPIO_IN);
	stpio_free_pin(pio_ptr);
	return 0;
}

static struct plat_stmmacphy_data phy_private_data = {
	.bus_id = 0,
	.phy_addr = 0x01,
	.phy_mask = ~(1UL << 0x01),
	.interface = PHY_INTERFACE_MODE_MII,
	.phy_reset = phy_reset,
};

static struct platform_device bostona_phy_device = {
	.name		= "stmmacphy",
	.id		= 0,
	.num_resources	= 1,
	.resource	= (struct resource[]) {
		{
			.name	= "phyirq",
			.start	= ILC_EXT_IRQ(6),
			.end	= ILC_EXT_IRQ(6),
			.flags	= IORESOURCE_IRQ,
		},
	},
	.dev = {
		.platform_data = &phy_private_data,
	}
};

#if defined(CONFIG_LEDS_GPIO)
static struct platform_device bostona_leds = {
	.name = "leds-gpio",
	.id = 0,
	.dev.platform_data = &(struct gpio_led_platform_data) {
		.num_leds = 2,
		.leds = (struct gpio_led[]) {
			{
				.name = "led:0",
				.gpio = stpio_to_gpio(14, 1),
				.active_low = 0,

				.default_trigger = "timerserv",
			},
                        {
				.name = "led:1",
				.gpio = stpio_to_gpio(14, 2),
				.active_low = 0,

				.default_trigger = "timerserv",
			},
		},
	},
};
#endif

static struct mtd_partition mtd_parts_table[1] = {
	{
		.name = "nor0",
		.size = MTDPART_SIZ_FULL,
		.offset = 0x00000000,
	},
};

static struct physmap_flash_data bostona_physmap_flash_data = {
	.width		= 2,
	.set_vpp	= NULL,
	.nr_parts	= ARRAY_SIZE(mtd_parts_table),
	.parts		= mtd_parts_table
};

static struct platform_device bostona_physmap_flash = {
	.name		= "physmap-flash",
	.id		= -1,
	.num_resources	= 1,
	.resource	= (struct resource[]) {
		{
			.start		= 0x00000000,
			.end		= 32*1024*1024 - 1,
			.flags		= IORESOURCE_MEM,
		}
	},
	.dev		= {
		.platform_data	= &bostona_physmap_flash_data,
	},
};

#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
static struct gpio_keys_button nangma_buttons[] = {
	{
		.code = KEY_POWER,
		.gpio = stpio_to_gpio(6, 5),
		.desc = "P+",
	},
	{
		.code = KEY_POWER,
		.gpio = stpio_to_gpio(6, 6),
		.desc = "P-",
	},
	{
		.code = KEY_POWER,
		.gpio = stpio_to_gpio(6, 7),
		.desc = "STANDBY",
	},
};

static struct gpio_keys_platform_data nangma_buttons_data = {
	.buttons = nangma_buttons,
	.nbuttons = ARRAY_SIZE(nangma_buttons),
};

static struct platform_device nangma_buttons_device = {
	.name = "gpio-keys",
	.id = -1,
	.num_resources = 0,
	.dev = {
		.platform_data = &nangma_buttons_data,
	}
};
#endif

static struct platform_device *bostona_devices[] __initdata = {
#if defined(CONFIG_LEDS_GPIO)
	&bostona_leds,
#endif
#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
	&nangma_buttons_device,
#endif
	&bostona_physmap_flash,
	&bostona_phy_device,
};

static unsigned lirc_enabled_pios[] = {
	[0] = 3 * 8 + 0
};

static int __init device_init(void)
{
#if 0
	stx7105_configure_pwm(&pwm_private_info);
#endif

	stx7105_configure_ssc(&ssc_private_info);

	stx7105_configure_usb(0, &usb_init[0]);
	stx7105_configure_usb(1, &usb_init[1]);

	stx7105_configure_lirc(NULL, lirc_enabled_pios,
	                       ARRAY_SIZE(lirc_enabled_pios));

	stx7105_configure_ethernet(0, stx7105_ethernet_mii, 0, 0, 1, 0);

	stx7105_configure_tsin(0, 1);
	stx7105_configure_tsin(1, 1);

	stx7105_configure_audio_pins(0, 0, 1, 0);

	if (ioremap(ST7105_EMI_BANK_OFFSET, ST7105_EMI_BANK_LENGTH) == NULL) {
		printk("BOSTONA: IOREMAP: ERROR\n");
	}

	return platform_add_devices(bostona_devices, ARRAY_SIZE(bostona_devices));
}
arch_initcall(device_init);

static void __iomem *bostona_ioport_map(unsigned long port, unsigned int size)
{
	/* However picking somewhere safe isn't as easy as you might think.
	 * I used to use external ROM, but that can cause problems if you are
	 * in the middle of updating Flash. So I'm now using the processor core
	 * version register, which is guaranted to be available, and non-writable.
	 */

	return (void __iomem *)CCN_PVR;
}

static void __init bostona_init_irq(void)
{
	set_irq_type(ILC_EXT_IRQ(0), IRQ_TYPE_EDGE_RISING);
	set_irq_type(ILC_EXT_IRQ(6), IRQ_TYPE_LEVEL_HIGH);
}

struct sh_machine_vector mv_boston __initmv = {
	.mv_name		= "boston",
	.mv_setup		= bostona_setup,
	.mv_nr_irqs		= NR_IRQS,
	.mv_init_irq		= bostona_init_irq,
	.mv_ioport_map		= bostona_ioport_map,
};
