/*
 * arch/sh/boards/st/hdk5289/setup.c
 *
 * Copyright (C) 2009 STMicroelectronics Limited
 * Author: Pawel Moll (pawel.moll@st.com)
 *
 * May be copied or modified under the terms of the GNU General Public
 * License.  See linux/COPYING for more information.
 */

#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/io.h>
#include <linux/delay.h>
#include <linux/phy.h>
#include <linux/gpio.h>
#include <linux/tm1668.h>
#include <linux/stm/pio.h>
#include <linux/stm/soc.h>
#include <linux/stm/emi.h>
#include "../common/common.h"

#define HDK5289_GPIO_RST stpio_to_gpio(2, 2)
#define HDK5289_POWER_ON stpio_to_gpio(2, 3)

static int hdk5289_ascs[] __initdata = { 2 | (STASC_FLAG_NORTSCTS << 8) };

static void __init hdk5289_setup(char **cmdline_p)
{
	printk(KERN_INFO "STMicroelectronics HDK7105 board initialisation\n");

	stx5206_early_device_init();
	stx5206_configure_asc(hdk5289_ascs, ARRAY_SIZE(hdk5289_ascs), 0);
}

static struct plat_ssc_data hdk5289_ssc_plat_data = {
	.capability =
		ssc0_has(SSC_I2C_CAPABILITY) | /* Internal I2C link */
		ssc1_has(SSC_I2C_CAPABILITY) |
		ssc2_has(SSC_UNCONFIGURED) |
		ssc3_has(SSC_I2C_CAPABILITY),
};



static struct tm1668_key hdk5289_front_panel_keys[] = {
	{ 0x00001000, KEY_UP, "Up (SWF2)" },
	{ 0x00800000, KEY_DOWN, "Down (SWF7)" },
	{ 0x00008000, KEY_LEFT, "Left (SWF6)" },
	{ 0x00000010, KEY_RIGHT, "Right (SWF5)" },
	{ 0x00000080, KEY_ENTER, "Enter (SWF1)" },
	{ 0x00100000, KEY_ESC, "Escape (SWF4)" },
};

static struct tm1668_character hdk5289_front_panel_characters[] = {
	TM1668_7_SEG_HEX_DIGITS,
	TM1668_7_SEG_HEX_DIGITS_WITH_DOT,
	TM1668_7_SEG_SEGMENTS,
};

static struct platform_device hdk5289_front_panel = {
	.name = "tm1668",
	.id = -1,
	.dev.platform_data = &(struct tm1668_platform_data) {
		.gpio_dio = stpio_to_gpio(3, 5),
		.gpio_sclk = stpio_to_gpio(3, 4),
		.gpio_stb = stpio_to_gpio(2, 7),
		.config = tm1668_config_6_digits_12_segments,

		.keys_num = ARRAY_SIZE(hdk5289_front_panel_keys),
		.keys = hdk5289_front_panel_keys,
		.keys_poll_period = DIV_ROUND_UP(HZ, 5),

		.brightness = 8,
		.characters_num = ARRAY_SIZE(hdk5289_front_panel_characters),
		.characters = hdk5289_front_panel_characters,
		.text = "5289",
	},
};



static struct plat_stmmacphy_data hdk5289_phy_plat_data = {
	/* Micrel */
	.bus_id = 0,
	.phy_addr = -1,
	.phy_mask = 0,
	.interface = PHY_INTERFACE_MODE_MII,
};

static struct platform_device hdk5289_phy_device = {
	.name = "stmmacphy",
	.id = -1,
	.dev.platform_data = &hdk5289_phy_plat_data,
};

static struct platform_device *hdk5289_devices[] __initdata = {
	&hdk5289_front_panel,
	&hdk5289_phy_device,
};

static int __init device_init(void)
{
	/* This board has a one PIO line used to reset almost everything:
	 * ethernet PHY, HDMI transmitter chip, PCI... */
	gpio_request(HDK5289_GPIO_RST, "GPIO_RST#");
	gpio_direction_output(HDK5289_GPIO_RST, 0);
	udelay(1000);
	gpio_set_value(HDK5289_GPIO_RST, 1);

	/* Some of the peripherals are powered by regulators
	 * triggered by the following PIO line... */
	gpio_request(HDK5289_POWER_ON, "POWER_ON");
	gpio_direction_output(HDK5289_POWER_ON, 1);

	stx5206_configure_lirc(stx5206_lirc_rx_ir, 0, 0, NULL);
	stx5206_configure_ssc(&hdk5289_ssc_plat_data);

	stx5206_configure_usb();

	stx5206_configure_ethernet(stx5206_ethernet_mii, 0, 0);

	return platform_add_devices(hdk5289_devices,
			ARRAY_SIZE(hdk5289_devices));
}
arch_initcall(device_init);

struct sh_machine_vector mv_hdk5289 __initmv = {
	.mv_name		= "hdk5289",
	.mv_setup		= hdk5289_setup,
	.mv_nr_irqs		= NR_IRQS,
};

