/*
 * Wyplay Nangma 7105 - Nagma + boot NAND
 * Nagma PCB revision V2
 *
 * Derived from STMicroelectronics STi7105-SDK Reference board support.
 *
 * Copyright (C) 2008 STMicroelectronics Limited
 * Author: Stuart Menefy (stuart.menefy@st.com)
 *
 * Copyright (C) 2009 WyPlay SAS.
 * Aubin Constans <aconstans@wyplay.com>
 *
 * Copyright (C) 2010 WyPlay SAS.
 * Laurent Fazio <lfazio@wyplay.com>
 *
 * Copyright (C) 2010 WyPlay SAS.
 * Laurent Masson <lmasson@wyplay.com>
 *
 * Copyright (C) 2010 WyPlay SAS.
 * Lauren Del Giudice <lauren@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/leds.h>
#include <linux/stm/pio.h>
#include <linux/stm/soc.h>
#include <linux/stm/emi.h>
#include <linux/delay.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/partitions.h>
#include <linux/i2c.h>
#include <linux/spi/spi.h>
#include <linux/spi/flash.h>
#include <linux/bpa2.h>
#include <sound/stm.h>
#include <linux/mtd/nand.h>
#include <linux/stm/nand.h>
#include <linux/stm/soc_init.h>
#include <linux/stm/sysconf.h>
#include <linux/phy.h>
#include <linux/gpio_keys.h>
#include <linux/input.h>
#include <linux/interrupt.h>
#include <asm/irq-ilc.h>
#include <asm/irl.h>
#include <asm/io.h>

#define LMI_CFG_BASE		0xfe901000	/* GPLMI0 control registers */
#define LMI_CFG_EXTENT		0x050

#define LMI_SCR_OFFS		0x010
#define LMI_CST_SHIFT		16
#define LMI_CST_MASK		0xfff
#define LMI_BRFSH_SHIFT		4
#define LMI_BRFSH_MASK		0x7
#define LMI_SMS_SHIFT		0
#define LMI_SMS_MASK		0x7
#define LMI_SMS_NOP		0x1		/* NOP command */
#define LMI_SMS_PALL		0x2		/* PRECHARGE ALL banks command */

#define LMI_STR_OFFS		0x018
#define LMI_TCL_SHIFT		19
#define LMI_TCL_MASK		0xf
#define LMI_TWR_SHIFT		16
#define LMI_TWR_MASK		0x7

#define LMI_SDMR_OFFS		0x048

#define ST7105_EMI_BANK_OFFSET 0x00000000
#define ST7105_EMI_BANK_LENGTH 0x07FFFFFF

#ifdef CONFIG_BPA2

#include <linux/bpa2.h>

static const char *LMI_SYS_partalias_256[] = {
	"BPA2_Region0",
	"v4l2-coded-video-buffers",
	"BPA2_Region1",
	"v4l2-video-buffers",
	"coredisplay-video",
	NULL
};

static const char *bigphys_partalias_256[] = {
	"gfx-memory-1",
	NULL
};

static struct bpa2_partition_desc bpa2_parts_table[] = {
	{
		.name	= "LMI_SYS",		/* 73MB */
		.start	= 0,
		.size	= 0x04900000,
		.flags	= 0,
		.aka	= LMI_SYS_partalias_256
	},
	{
		.name	= "bigphysarea",	/* 18MB */
		.start	= 0,
		.size	= 0x01200000,
		.flags	= 0,
		.aka	= bigphys_partalias_256
	},
	{
		.name	= "gfx-memory-0",	/* 50MB */
		.start	= 0x50000000 - 0x03200000,
		.size	= 0x03200000,
		.flags	= 0,
		.aka	= NULL
	},
	{
		.name	= "rt307x",	/* 1MB */
		.start	= 0,
		.size	= 0x00180000,
		.flags	= 0,
		.aka	= NULL
	}
};

#endif  /* CONFIG_BPA2 */

static struct stpio_pin* phy_reset_pin = NULL;

static int ascs[] __initdata = {
	2 | (STASC_FLAG_NORTSCTS << 8),
        /* 1 | (STASC_FLAG_NORTSCTS << 8), */
        3 | (STASC_FLAG_NORTSCTS << 8),
};

static void __init ddr_fixup(void)
{
	u8 __iomem *addr = NULL;
	register u32 reg_mrs;
	u32 reg_lmi, field;

	addr = (u8 __iomem *)ioremap_nocache(LMI_CFG_BASE, LMI_CFG_EXTENT);
	if (!addr)
		return;

	/* Read applicable tWR and tCL values from LMI_STR register */
	reg_lmi = readl(addr + LMI_STR_OFFS);

	/* Fix value of WR parameter in MRS DDR2-SDRAM register.
	 * The MRS register cannot be read back. */
	reg_mrs  = 0x0   << 15;	/* Issue MRS */
	reg_mrs |= 0x0   << 12;	/* PD = Fast exit */
	field = reg_lmi >> LMI_TWR_SHIFT & LMI_TWR_MASK & 0x7;
	reg_mrs |= field <<  9;	/* WR */
	reg_mrs |= 0x0   <<  8;	/* DLL reset = no */
	reg_mrs |= 0x0   <<  7;	/* TM = normal mode */
	field = reg_lmi >> LMI_TCL_SHIFT & LMI_TCL_MASK & 0x7;
	reg_mrs |= field <<  4;	/* nCAS latency */
	reg_mrs |= 0x0   <<  3;	/* Burst type = Sequential */
	reg_mrs |= 0x3;		/* Burst length = 8 */

	reg_lmi = readl(addr + LMI_SCR_OFFS);
	reg_lmi &= LMI_CST_MASK << LMI_CST_SHIFT
	           | LMI_BRFSH_MASK << LMI_BRFSH_SHIFT;

	/* Run the following without accessing the RAM array, hence run it from
	 * cache memory. Make everything fit into a single i-cache line */
	asm volatile (".balign	32\n\t"
	              "mov.l	%[pall], @%[scr]\n\t"
	              "mov.l	%[mrs], @%[sdmr]\n\t"
	              "mov.l	%[nop], @%[scr]\n"
	            : /* no output */
	            : [scr]	"r" (addr + LMI_SCR_OFFS),
	              [sdmr]	"r" (addr + LMI_SDMR_OFFS),
	              [pall]	"r" (reg_lmi | LMI_SMS_PALL),
	              [mrs]	"r" (reg_mrs),
	              [nop]	"r" (reg_lmi | LMI_SMS_NOP));

	iounmap(addr);

	printk("MRS DDR2-SDRAM register rewritten (0x%x)\n", reg_mrs);
}

static void __init nangma_pio_init(void)
{
	struct stpio_pin *pio_ptr = NULL;

        /** Memories **/
	/* PIO4_2 = AT24C08AN WP not pulled
	 * PIO11_3 = K9F1G08U0B nWP with pull-down
	 * PIO15_4 = SST25VF032B nRESET
	 * PIO15_5 = SST25VF032B nWP with pull-up */
	/* EEPROM initially read-only (high) */
	pio_ptr = stpio_request_set_pin(4, 2, "EEPROM_WP_", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	/* NAND Flash initially writable (high) */
	pio_ptr = stpio_request_set_pin(11, 3, "EMI_NAND_nWP", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	/* Reset the SPI Flash */
	pio_ptr = stpio_request_set_pin(15, 4, "SPI_FLASH_nRESET", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	/* Enable Block Protection Lock-Down of SPI Flash
	 * At power-up the Block Protection bits are writable,
	 * it is up to the software to set the Block Protection Lock-Down bit,
	 * if required */
	pio_ptr = stpio_request_set_pin(15, 5, "SPI_FLASH_nWP", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);

#ifdef CONFIG_SPI_STM_SSC
	/* Driving the SPI boot flash with PIO2[4-7] (SSC1).
	 * Put PIO15[0-3] (SPIboot interface) into High-Z state.
	 * Right now we can expect PIO2[4-7] to be in the STPIO_BIDIR_Z1
	 * default state, therefore the lines are not left floating. */
	stpio_request_pin(15, 0, "SPIBoot_SCK", STPIO_IN);
	stpio_request_pin(15, 1, "SPIBoot_MOSI", STPIO_IN);
	stpio_request_pin(15, 2, "SPIBoot_nCS", STPIO_IN);
	stpio_request_pin(15, 3, "SPIBoot_MISO", STPIO_IN);
#else
	/* Driving the SPI boot flash with PIO15[0-3] (GPIO's).
	 * Put PIO2[4-7] (SSC1) into High-Z state.
	 * First enable temporary pull-ups, so the lines are not left
	 * floating. */
	pio_ptr = stpio_request_pin(15, 0, "SPIBoot_SCK", STPIO_BIDIR_Z1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_pin(15, 1, "SPIBoot_MOSI", STPIO_BIDIR_Z1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_pin(15, 2, "SPIBoot_nCS", STPIO_BIDIR_Z1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_pin(15, 3, "SPIBoot_MISO", STPIO_BIDIR_Z1);
	stpio_free_pin(pio_ptr);
	stpio_request_pin(2, 4, "SPI0_nCS", STPIO_IN);
	stpio_request_pin(2, 5, "SPI0_SCK", STPIO_IN);
	stpio_request_pin(2, 6, "SPI0_MOSI", STPIO_IN);
	stpio_request_pin(2, 7, "SPI0_MISO", STPIO_IN);
#endif

	/** Analog audio/video outputs **/
	/* PIO11_1 = Analog audio mute active high with pull-up */
	/* TODO replace these demo-only values */
	pio_ptr = stpio_request_set_pin(11, 1, "ANALOG_AUDIO_MUTE", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);

	/** Network **/
	/* PIO11_6 = LAN8710 nRST with pull-down */
	/* PHY reset is performed by the phy_reset() callback, below */
	phy_reset_pin = stpio_request_set_pin(11, 6, "eth_phy_reset",
	                                      STPIO_OUT, 0);

	/* DVB-T receivers */
	/* PIO10_4 = DVB-T 3V3 power switch active high with pull-down
	 * PIO10_3 = DVB-T antenna power switch active high with pull-down
	 * PIO11_2 = DVB-T nRESET with pull-down
         * PIO5_2 = Enable power on CI with pull-up
         * PIO5_7 = Enable power on CI with pull-down */
	/* Power then reset the DVB-T receivers */
	pio_ptr = stpio_request_set_pin(10, 4, "DVBT_POWER", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	udelay(10);
	pio_ptr = stpio_request_set_pin(11, 2, "DVBT_nRESET", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(5, 2, "CI_POWER", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(5, 7, "CI_RESET", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(10, 3, "ANTENNA_POWER", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);

	/* No CI */
	pio_ptr = stpio_request_set_pin(1, 0, "TS_CTRL_T1_TS0", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(1, 1, "TS_CTRL_T2_TS1", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(1, 2, "TS_CTRL_CI", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);

        /* RESET */
	/*
         * Reset modem reset and setting DTR to high
	 *
	 */
	pio_ptr = stpio_request_pin(1, 6, "MODEM_DTR", STPIO_OUT);
	stpio_set_pin(pio_ptr, 1);
	stpio_free_pin(pio_ptr);
}

static __init void nangma_setup(char** cmdline_p)
{
	printk("WyPlay Nangma 7105 V2 board initialisation\n");

	stx7105_early_device_init();
	stx7105_configure_asc(ascs, ARRAY_SIZE(ascs), 0);
	ddr_fixup();
	nangma_pio_init();
#ifdef CONFIG_BPA2
	bpa2_init(bpa2_parts_table, ARRAY_SIZE(bpa2_parts_table));
#endif
}

#if 0
static struct plat_stm_pwm_data pwm_private_info = {
	.flags		= PLAT_STM_PWM_OUT1,
	.routing	= PWM_OUT1_PIO4_5,
	.pwmclkdiv	= 3, /* 26kHz, for the fan */
};
#endif

#if (defined(CONFIG_SPI_STM_PIO) || defined(CONFIG_SPI_STM_PIO_MODULE)) && \
    (defined(CONFIG_SPI_STM_SSC) || defined(CONFIG_SPI_STM_SSC_MODULE))
#error SPI_STM_PIO plus SPI_STM_SSC results in PIO conflict
#endif

static struct plat_ssc_data ssc_private_info = {
	.capability =
		ssc0_has(SSC_I2C_CAPABILITY) |
#ifdef CONFIG_SPI_STM_SSC
		ssc1_has(SSC_SPI_CAPABILITY) |
#else
		ssc1_has(SSC_UNCONFIGURED)   |
#endif
		ssc2_has(SSC_I2C_CAPABILITY) |
		ssc3_has(SSC_I2C_CAPABILITY),
	.routing =
		SSC2_SCLK_PIO3_4 | SSC2_MTSR_PIO3_5 |
		SSC3_SCLK_PIO3_6 | SSC3_MTSR_PIO3_7,
	.spi_chipselects = {
		[1] = NULL, /* use spi_stpio_chipselect() */
	},
};

#ifdef CONFIG_I2C_BOARDINFO
static struct i2c_board_info nangma_i2c1_devices[] __initdata = {
	{ I2C_BOARD_INFO("rtc-pcf8563", 0x51), .type = "rtc-pcf8563", .irq = IRL2_IRQ },
};
#endif

/*
 * SPI busses
 */
#ifndef CONFIG_SPI_STM_SSC
static struct platform_device spi_gpio_bus_boot_flash = {
	.name           = "spi_st_pio",
	.id             = 8,
	.num_resources  = 0,
	.dev            = {
		.platform_data = &(struct ssc_pio_t) {
			.pio = {
				{ 15, 0 },	/* SCK */
				{ 15, 1 },	/* MOSI */
				{ 15, 3 },	/* MISO */
			},
		},
	},
};
#endif

/*
 * SPI devices
 */
static struct mtd_partition spi_boot_flash_parts[] = {
	{
		.name	= "pbl",
		.offset	= 0x0,
		.size	= 0x10000,		/* 16 * 4KB */
	},{
		.name   = "k1",
		.offset = MTDPART_OFS_APPEND,
		.size   = 0x1F4000,		/* 500 * 4KB */
	},{
		.name   = "k2",
		.offset = MTDPART_OFS_APPEND,
		.size   = 0x1F4000,		/* 500 * 4KB */
	},{
		.name   = "rwdata",
		.offset = MTDPART_OFS_APPEND,
		.size   = 0x1000,		/* 1 * 4KB */
	},{
		.name   = "rodata",
		.offset = MTDPART_OFS_APPEND,
		.size   = 0x1000,		/* 1 * 4KB */
	},{
		.name   = "reserved",
		.offset = MTDPART_OFS_APPEND,
		.size   = MTDPART_SIZ_FULL
	}

};

static struct flash_platform_data spi_boot_flash_data = {
	.name		= "m25p80",
	.parts		= spi_boot_flash_parts,
	.nr_parts	= ARRAY_SIZE(spi_boot_flash_parts),
	.type		= "sst25vf032b",	/* The actual chip model */
};

static struct spi_board_info nangma_spi_boot_flash[] __initdata = {
	{
		.modalias       = "m25p80",
		.platform_data  = &spi_boot_flash_data,
		.max_speed_hz   = 7000000,	/* Max SPI clock frequency
						 * The chip supports 66MHz */
#ifdef CONFIG_SPI_STM_SSC
		.bus_num        = 0,		/* On the first SPI SSC bus */
		.chip_select    = spi_set_cs(2, 4),
#else
		.bus_num        = 8,		/* On the GPIO-emulated
						 * spi_st_pio bus above */
		.chip_select    = spi_set_cs(15, 2),
#endif
		.mode           = SPI_MODE_3,
	},
};

/*
 * USB
 */
static struct usb_init_data usb_init[2] __initdata = {
	/* Both STi7105 USB ports are wired to on-board chips only, hence for
	 * these internal nets neither USB overcurrent nor power control are
	 * implemented.
	 */
	{
		.oc_en = 1,
		.oc_pinsel = USB0_OC_PIO4_4,
		.oc_actlow = 1,
		.pwr_en = 1,
		.pwr_pinsel = USB0_PWR_PIO4_5,
	}, {
		.oc_en = 1,
		.oc_pinsel = USB1_OC_PIO4_6,
		.oc_actlow = 1,
		.pwr_en = 1,
		.pwr_pinsel = USB1_PWR_PIO4_7,
	}
};

#ifndef CONFIG_SMSC_PHY
#warning stpio locking conflict between phy_reset and stx7105_configure_ethernet
#endif

static int phy_reset(void* priv)
{
	struct stpio_pin *pio_rx_d1 = NULL, *pio_col = NULL;

	if (phy_reset_pin == NULL)
		return 0;
	/* Reset SMSC LAN8710 PHY
	 * assert nRST (low) for at least 100usec,
	 * with MII clock fed in at the same time */
	udelay(1);
	stpio_set_pin(phy_reset_pin, 0);
	udelay(100);

	/* It happens that several nets are bootstraps both for the LAN8710
	 * and the STi7105!
	 * They are pulled to suit STi7105 requirements.
	 * Consequently, this software, running after the STi7105 has completed
	 * its own boot sequence, is in charge of forcing applicable bootstrap
	 * values, for the LAN8710 to sense them upon the rising edge of nRST.
	 * Let's review the condition of all LAN8710 configuration straps:
	 *  PHYAD0: PIO7_5 (MII_RX_ER) with on-board 4K7 PD => Hi-Z
	 *  PHYAD1: PIO8_5 (MII_RX_CLK) with on-chip 50uA PD => Hi-Z
	 *  PHYAD2: PIO9_1 (MII_RX_D3) with on-board 4K7 PD => Hi-Z
	 *   MODE0: PIO8_6 (MII_RX_D0) with on-board 4K7 PU => Hi-Z
	 *   MODE1: PIO8_7 (MII_RX_D1) with on-board 4K7 PD => force to 1
	 *   MODE2: PIO9_3 (MII_COL) with on-chip 50uA PU => input weak PU
	 * RMIISEL: PIO9_0 (MII_RX_D2) with on-board 4K7 PD => Hi-Z
	 *  REGOFF: NC
	 * nINTSEL: NC
	 * As of writing this, stx7105_configure_ethernet() configures
	 * the 7 PIO's above in STPIO_IN (Hi-Z) mode. */

	/* Enable all 10/100 capabilities: MODE[2:0]=0x7 */
	pio_col = stpio_request_set_pin(9, 3, "LAN8710_MODE2", STPIO_BIDIR_Z1, 1);
	if (pio_col == NULL)
		return 0;
	pio_rx_d1 = stpio_request_set_pin(8, 7, "LAN8710_MODE1", STPIO_OUT, 1);
	if (pio_rx_d1 == NULL) {
		stpio_configure_pin(pio_col, STPIO_IN);
		stpio_free_pin(pio_col);
		return 0;
	}

	udelay(5);
	local_irq_disable();
	stpio_set_pin(phy_reset_pin, 1);
	/* On one hand, minimum configuration strap pin hold time after
	 * nRST deassertion is 1ns.
	 * On the other hand, no later than 2ns further to nRST deassertion,
	 * the LAN8710 may enable its outputs, resulting in a temporary
	 * HW collision with PIO8_7. Make sure to insert no delay here.
	 * Switch back ASAP from bootstrap to regular PIO operation.
	 * Measurements show that the LAN8710 has RXD1=0 initially;
	 * first clear PIO8_7, as it is faster than reconfiguring it */
	stpio_set_pin(pio_rx_d1, 0);
	stpio_configure_pin(pio_rx_d1, STPIO_IN);
	stpio_configure_pin(pio_col, STPIO_IN);
	local_irq_enable();

	stpio_free_pin(pio_rx_d1);
	stpio_free_pin(pio_col);
	return 0;
}

static struct plat_stmmacphy_data phy_private_data = {
	.bus_id = 0,
	.phy_addr = 0x00,
	.phy_mask = ~(1UL << 0x00),
	.interface = PHY_INTERFACE_MODE_MII,
	.phy_reset = phy_reset,
};

static struct platform_device nangma_phy_device = {
	.name		= "stmmacphy",
	.id		= 0,
	.num_resources	= 1,
	.resource	= (struct resource[]) {
		{
			.name	= "phyirq",
			.start	= -1,
			.end	= -1,
			.flags	= IORESOURCE_IRQ,
		},
	},
	.dev = {
		.platform_data = &phy_private_data,
	}
};

#if defined(CONFIG_LEDS_GPIO)
static struct platform_device nangma_leds_device = {
	.name = "leds-gpio",
	.id = 0,
	.dev.platform_data = &(struct gpio_led_platform_data) {
		.num_leds = 2,
		.leds = (struct gpio_led[]) {
			{
				.name = "led:0",
				.gpio = stpio_to_gpio(6, 0),
				.active_low = 0,

				.default_trigger = "timerserv",
			},
                        {
				.name = "led:1",
				.gpio = stpio_to_gpio(6, 1),
				.active_low = 0,

				.default_trigger = "timerserv",
			},
		},
	},
};
#endif

#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
static struct gpio_keys_button nangma_buttons[] = {
	{
		.code = KEY_POWER,
		.gpio = stpio_to_gpio(4, 3),
		.desc = "POWER",
		.active_low=1,
	},
};

static struct gpio_keys_platform_data nangma_buttons_data = {
	.buttons = nangma_buttons,
	.nbuttons = ARRAY_SIZE(nangma_buttons),
};

static struct platform_device nangma_buttons_device = {
	.name = "gpio-keys",
	.id = -1,
	.num_resources = 0,
	.dev = {
		.platform_data = &nangma_buttons_data,
	}
};
#endif

#if defined(CONFIG_MTD_NAND)
/* Configuration for NAND Flash */
static struct mtd_partition nand_parts[] = {
	{
		.name   = "ubi",
		.offset = 0,
		.size   = 0xAC00000	/* 1376 * 128KiB (/2) */
	},
	{
		.name   = "factory",
		.offset = MTDPART_OFS_APPEND,
		.size   = 0x5380000	/* 84MiB - 4 Blocks = 668 x 128KiB */
	}
};


static struct plat_stmnand_data nand_config = {
	/* STM_NAND_EMI data */
	.emi_withinbankoffset   = 0,
	.rbn_port               = -1,
	.rbn_pin                = -1,

	.timing_data = &(struct nand_timing_data) {
		.sig_setup      = 50,           /* times in ns */
		.sig_hold       = 50,
		.CE_deassert    = 0,
		.WE_to_RBn      = 100,
		.wr_on          = 10,
		.wr_off         = 40,
		.rd_on          = 10,
		.rd_off         = 40,
		.chip_delay     = 50,           /* in us */
	},
	.flex_rbn_connected     = 1,
};

/* Platform data for STM_NAND_EMI/FLEX/AFM. (bank# may be updated later) */
static struct platform_device nand_device =
STM_NAND_DEVICE("stm-nand-afm", 0, &nand_config,
		nand_parts, ARRAY_SIZE(nand_parts), NAND_USE_FLASH_BBT);
#endif /* CONFIG_MTD_NAND */

static struct platform_device *nangma_devices[] __initdata = {
#if defined(CONFIG_LEDS_GPIO)
	&nangma_leds_device,
#endif

#if defined(CONFIG_KEYBOARD_GPIO) || defined(CONFIG_KEYBOARD_GPIO_MODULE)
	&nangma_buttons_device,
#endif
	&nangma_phy_device,
#ifndef CONFIG_SPI_STM_SSC
	&spi_gpio_bus_boot_flash,
#endif
};

static unsigned lirc_enabled_pios[] = {
	[0] = 3 * 8 + 0
};

static int __init device_init(void)
{
#if 0
	stx7105_configure_pwm(&pwm_private_info);
#endif

#ifdef CONFIG_I2C_BOARDINFO
	i2c_register_board_info(1, nangma_i2c1_devices,
	                        ARRAY_SIZE(nangma_i2c1_devices));
#endif
	spi_register_board_info(nangma_spi_boot_flash,
	                        ARRAY_SIZE(nangma_spi_boot_flash));

	stx7105_configure_ssc(&ssc_private_info);

	stx7105_configure_usb(0, &usb_init[0]);
	stx7105_configure_usb(1, &usb_init[1]);

	stx7105_configure_lirc(NULL, lirc_enabled_pios,
	                       ARRAY_SIZE(lirc_enabled_pios));

	stx7105_configure_ethernet(0, stx7105_ethernet_mii, 0, 0, 0, 0);

	/* dual DVB-T, in parallel */
	stx7105_configure_tsin(0, 0);
	stx7105_configure_tsin(1, 0);

	stx7105_configure_audio_pins(0, 0, 1, 0);

#if defined(CONFIG_MTD_NAND)
    stx7105_configure_nand(&nand_device);
#endif

	if (ioremap(ST7105_EMI_BANK_OFFSET, ST7105_EMI_BANK_LENGTH) == NULL) {
		printk("IOREMAP: ERROR\n");
	}

	return platform_add_devices(nangma_devices, ARRAY_SIZE(nangma_devices));
}
arch_initcall(device_init);

static void __iomem *nangma_ioport_map(unsigned long port, unsigned int size)
{
        /* However picking somewhere safe isn't as easy as you might think.
         * I used to use external ROM, but that can cause problems if you are
         * in the middle of updating Flash. So I'm now using the processor core
         * version register, which is guaranted to be available, and non-writable.
         */

	return (void __iomem *)CCN_PVR;
}

static void __init nangma_init_irq(void)
{
	set_irq_type(ILC_EXT_IRQ(0), IRQ_TYPE_LEVEL_LOW);
	set_irq_type(ILC_EXT_IRQ(1), IRQ_TYPE_LEVEL_LOW);
	set_irq_type(ILC_EXT_IRQ(2), IRQ_TYPE_LEVEL_LOW);
	set_irq_type(ILC_EXT_IRQ(6), IRQ_TYPE_LEVEL_LOW);
}

struct sh_machine_vector mv_nangma __initmv = {
	.mv_name		= "nangma",
	.mv_setup		= nangma_setup,
	.mv_nr_irqs		= NR_IRQS,
	.mv_init_irq		= nangma_init_irq,
	.mv_ioport_map		= nangma_ioport_map,
};
