/*
 * arch/sh/boards/st/panda2b/setup.c
 *
 * Copyright (C) 2008 STMicroelectronics Limited
 * Author: Stuart Menefy (stuart.menefy@st.com)
 *
 * Copyright (C) 2009 WyPlay SAS.
 * Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>
 * Jerry Zhang <zhangb@wyplayasia.com>
 *
 * May be copied or modified under the terms of the GNU General Public
 * License.  See linux/COPYING for more information.
 *
 * WyPlay panda2b support.
 */

#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/leds.h>
#include <linux/stm/pio.h>
#include <linux/stm/soc.h>
#include <linux/stm/emi.h>
#include <linux/delay.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/partitions.h>
#include <linux/i2c.h>
#include <linux/i2c-pio.h>
#include <linux/spi/spi.h>
#include <linux/spi/flash.h>
#include <linux/wymcu/mcu.h>
#include <sound/stm.h>
#include <linux/phy.h>
#include <linux/gpio.h>
#include <linux/gpio_keys.h>
#include <linux/input.h>
#include <asm/irq-ilc.h>
#include <asm/irl.h>
#include <asm/io.h>
#include "../common/common.h"

static int ascs[] __initdata = {
	2 | (STASC_FLAG_NORTSCTS << 8)

};

#ifdef CONFIG_BPA2
#include <linux/bpa2.h>

static const char *LMI_SYS_partalias_256[] = {
        "BPA2_Region0",
        "v4l2-coded-video-buffers",
        "BPA2_Region1",
        "v4l2-video-buffers",
        "coredisplay-video",
        NULL
};

static const char *bigphys_partalias_256[] = {
        "gfx-memory-1",
        NULL
};

static struct bpa2_partition_desc bpa2_parts_table[] = {
        {
                .name   = "LMI_SYS",            /* 85MB */
                .start  = 0,
                .size   = 0x05500000,
                .flags  = 0,
                .aka    = LMI_SYS_partalias_256
        },
        {
                .name   = "bigphysarea",        /* 18MB */
                .start  = 0,
                .size   = 0x01200000,
                .flags  = 0,
                .aka    = bigphys_partalias_256
        },
        {
                .name   = "gfx-memory-0",       /* 50MB */
                .start  = 0x4CE00000,
                .size   = 0x03200000,
                .flags  = 0,
                .aka    = NULL
        },
	{
		.name	= "rt307x",	/* 1MB */
		.start	= 0,
		.size	= 0x00180000,
		.flags	= 0,
		.aka	= NULL
	}
};
#endif  /* CONFIG_BPA2 */

static __init void panda2b_setup(char** cmdline_p)
{
	printk("WyPlay PANDA2B board initialisation\n");

	stx7105_early_device_init();
	stx7105_configure_asc(ascs, ARRAY_SIZE(ascs), 0);
#ifdef CONFIG_BPA2
	printk("Use BPA2 partitions table for ");
	switch (memory_end - memory_start) {
	case 0x10000000:
		printk("256MiB DDR2\n");
		bpa2_init(bpa2_parts_table,
			  ARRAY_SIZE(bpa2_parts_table));
		break;
	}
#endif
}

/*
 * ssc
 */
static struct plat_ssc_data ssc_private_info = {
	.capability =
		ssc0_has(SSC_I2C_CAPABILITY) |
		ssc1_has(SSC_UNCONFIGURED) |
		ssc2_has(SSC_I2C_CAPABILITY) |
		ssc3_has(SSC_UNCONFIGURED),
	.routing =
		SSC2_SCLK_PIO3_4 | SSC2_MTSR_PIO3_5 |
		SSC3_SCLK_PIO3_6 | SSC3_MTSR_PIO3_7,
};

static struct usb_init_data usb_init[2] __initdata = {
	{
		.oc_en = 0,
		.pwr_en = 0,
	}, {
		.oc_en = 0,
		.pwr_en = 0,
	}
};

/*
 * phy
 */
static struct plat_stmmacphy_data phy_private_data = {
	/* RTL8201CP */
	.bus_id = 0,
	.phy_addr = 0x01,
	.phy_mask = 0,
	.interface = PHY_INTERFACE_MODE_MII,
};

static struct platform_device panda2b_phy_device = {
	.name		= "stmmacphy",
	.id		= 0,
	.num_resources	= 1,
	.resource	= (struct resource[]) {
		{
			.name	= "phyirq",
			.start	= -1,/*FIXME, should be ILC_EXT_IRQ(6), */
			.end	= -1,
			.flags	= IORESOURCE_IRQ,
		},
	},
	.dev = {
		.platform_data = &phy_private_data,
	}
};

/*
 * Infrared interfaces
 */
static unsigned lirc_enabled_pios[] = {
	[0] = 3 * 8 + 0
};

/* Configuration based on Futarque-RC signals train. */
static lirc_scd_t lirc_scd = {
	.code = 0x3FFFC028,
	.codelen = 0x1e,
	.alt_codelen = 0,
	.nomtime = 0x1f4,
	.noiserecov = 0,
};

/*
 * I2C
 */
static struct i2c_adapter_info i2c_adapt_data_mcu = {
	.pio = {
		{3, 6},	/* SCL port, to mcu */
		{3, 7},	/* SDA port, to mcu */
	},
	.bus_freq = 1000,	/* HZ */
};

static struct platform_device i2c_adapter_device_mcu = {
	.name = "i2c_st_2",
	.id = 3,
	.dev = {
		.platform_data = &i2c_adapt_data_mcu,
	}
};

/*
 * RTC PCF8563
 */
#ifdef CONFIG_RTC_DRV_PCF8563
static struct i2c_board_info panda2b_i2c_devices[] __initdata = {
	{
		I2C_BOARD_INFO("rtc-pcf8563", 0x51),
		.type = "pcf8563",
	},
};
#endif

/*
 * NOR flash
 */
#ifdef CONFIG_MTD_PHYSMAP
/* default partition table, overridden by mtdparts command line option */
static struct mtd_partition mtd_parts_table[] = {
	/* preloader area */
	{
		.name           = "pbl",
		.size           = 0x00010000,
		.offset         = 0x00000000,
	},
	{
		.name           = "k1",
		.size           = 0x001f0000,
		.offset         = 0x00010000,
	},
	{
		.name           = "k2",
		.size           = 0x001f0000,
		.offset         = 0x00200000,
	},

	/* Env */
	{
		.name           = "reserved",
		.size           = 0x00004000,
		.offset         = 0x003f8000,
	},
	/* read-only production data */
	{
		.name           = "rwdata",
		.size           = 0x00002000,
		.offset         = 0x003fc000,
	},
	/* data */
	{
		.name           = "rodata",
		.size           = 0x00002000,
		.offset         = 0x003fe000,
	}
};

static struct physmap_flash_data physmap_flash_data = {
	.width		= 2,
	.set_vpp	= NULL,
	.nr_parts	= ARRAY_SIZE(mtd_parts_table),
	.parts		= mtd_parts_table,
};
#endif

static struct platform_device physmap_flash = {
	.name	= "physmap-flash",
	.id	= -1,
	.num_resources	= 1,
	.resource = (struct resource[]) {
		{
			.start	= 0x00000000,
			.end	= 4 * 1024 * 1024 - 1,
			.flags	= IORESOURCE_MEM,
		}
	},
	.dev	= {
#ifdef CONFIG_MTD_PHYSMAP
		.platform_data = &physmap_flash_data,
#endif
	},
};

#ifdef CONFIG_SPI_STM
/*
 * SPI busses (optional)
 */
static struct platform_device spi_gpio_bus_boot_flash = {
	.name           = "spi_st_pio",
	.id             = 8,
	.num_resources  = 0,
	.dev            = {
		.platform_data = &(struct ssc_pio_t) {
			.pio = {
				{ 15, 0 },	/* SCK */
				{ 15, 1 },	/* MOSI */
				{ 15, 3 },	/* MISO */
			},
		},
	},
};

/*
 * SPI devices (optional)
 */
static struct flash_platform_data spi_boot_flash_data = {
	.name		= "m25p80",
	.parts		= mtd_parts_table,
	.nr_parts	= ARRAY_SIZE(mtd_parts_table),
	.type		= "m25p64",	/* The actual chip model */
};

static struct spi_board_info panda2b_spi_boot_flash[] __initdata = {
	{
		.modalias       = "m25p80",
		.platform_data  = &spi_boot_flash_data,
		.max_speed_hz   = 1000000,	/* Max SPI clock frequency */
		.bus_num        = 8,		/* On the GPIO-emulated
						 * spi_st_pio bus above */
		.chip_select    = spi_set_cs(15, 2),
		.mode           = SPI_MODE_3,
	},
};
#endif /* CONFIG_SPI_STM */

/*
 * MCU
 */
static struct mcu_dev mcu_bus_devices_info[] = {
        {
	        .bus_type = I2C,
	        .bus_data = (void *)0x41,
	        .bus_id   = 3,
	        .name = "mcu_panda2b",
	},
};

static struct mcu_devs mcu_bus_devices = {
	        .nb = ARRAY_SIZE(mcu_bus_devices_info),
	        .devs = mcu_bus_devices_info,
};

static struct platform_device mcu_bus = {
        .name = "mcu",
	.dev = {
		.platform_data = &mcu_bus_devices,
	}
};

/*
 * LED
 */
static struct platform_device panda2b_leds = {
	.name = "leds-gpio",
	.id = 0,
	.dev.platform_data = &(struct gpio_led_platform_data) {
		.num_leds = 2,
		.leds = (struct gpio_led[]) {
			{
				.name = "led:red",
				.default_trigger = "timer",
				.gpio = stpio_to_gpio(7, 1),
			},
			{
				.name = "led:white",
				.default_trigger = "timer",
				.gpio = stpio_to_gpio(7, 0),
			},
		},
	},
};

static struct platform_device *panda2b_devices[] __initdata = {
	&panda2b_phy_device,
#ifdef CONFIG_SPI_STM
	&spi_gpio_bus_boot_flash,
#endif
	&i2c_adapter_device_mcu,
	&physmap_flash,
	&mcu_bus,
	&panda2b_leds,
};

static int __init device_init(void)
{
	struct stpio_pin *pio_ptr = NULL;
	struct stpio_pin *pio_ptr2 = NULL;

	stx7105_configure_sata(0);
	stx7105_configure_ssc(&ssc_private_info);

	stx7105_configure_usb(0, &usb_init[0]);
	stx7105_configure_usb(1, &usb_init[1]);

	/* ETH POWER ON */
	pio_ptr = stpio_request_set_pin(15, 5, "ETH_POWER_ON", STPIO_OUT, 1);
	stpio_free_pin(pio_ptr);

	/* Audio */
	stx7105_configure_audio_pins(0, 0, 1, 0);

	/* MUTE1 */
	pio_ptr2 = stpio_request_set_pin(4, 3, "MUTE1", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr2);

	/* RTC_INT */
	pio_ptr2 = stpio_request_set_pin(4, 2, "RTC_INT", STPIO_BIDIR, 0);
	stpio_free_pin(pio_ptr2);

	/* Tuner power supply */
	pio_ptr2 = stpio_request_set_pin(7, 2, "POWER ON", STPIO_OUT, 1);

	stx7105_configure_lirc(&lirc_scd, lirc_enabled_pios,
			       ARRAY_SIZE(lirc_enabled_pios));

	stx7105_configure_ethernet(0, stx7105_ethernet_mii, 0, 0, 0, 0);

	stx7105_configure_tsin(0, 0);
	stx7105_configure_tsin(1, 0);

#ifdef CONFIG_RTC_DRV_PCF8563
#if 0
	panda2b_i2c_devices[0].irq = gpio_to_irq(stpio_to_gpio(4, 2));
#endif
	i2c_register_board_info(1, panda2b_i2c_devices,
	                        ARRAY_SIZE(panda2b_i2c_devices));
#endif

#ifdef CONFIG_SPI_STM
	spi_register_board_info(panda2b_spi_boot_flash, ARRAY_SIZE(panda2b_spi_boot_flash));
#endif

	return platform_add_devices(panda2b_devices, ARRAY_SIZE(panda2b_devices));
}
arch_initcall(device_init);

static void __iomem *panda2b_ioport_map(unsigned long port, unsigned int size)
{
	/* Shouldn't be here! */
	BUG();
	return NULL;
}

static void __init panda2b_init_irq(void)
{
	set_irq_type(ILC_EXT_IRQ(0), IRQ_TYPE_LEVEL_LOW);
	set_irq_type(ILC_EXT_IRQ(1), IRQ_TYPE_LEVEL_LOW);
}

struct sh_machine_vector mv_panda2b __initmv = {
	.mv_name		= "panda2b",
	.mv_setup		= panda2b_setup,
	.mv_nr_irqs		= NR_IRQS,
	.mv_init_irq		= panda2b_init_irq,
	.mv_ioport_map		= panda2b_ioport_map,
};
