/*
 * arch/sh/boards/st/saturna/setup.c
 *
 * Copyright (C) 2008 STMicroelectronics Limited
 * Author: Stuart Menefy (stuart.menefy@st.com)
 *
 * Copyright (C) 2009 WyPlay SAS.
 * Marek Skuczynski <mskuczynski@wyplay.com>
 *
 * May be copied or modified under the terms of the GNU General Public
 * License.  See linux/COPYING for more information.
 *
 * WyPlay Dolmena support.
 */

#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/leds.h>
#include <linux/stm/pio.h>
#include <linux/stm/soc.h>
#include <linux/stm/emi.h>
#include <linux/stm/soc_init.h>
#include <linux/delay.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/nand.h>
#include <linux/stm/nand.h>
#include <linux/bpa2.h>
#include <sound/stm.h>
#include <linux/phy.h>
#include <linux/gpio_keys.h>
#include <linux/input.h>
#include <asm/irq-ilc.h>
#include <asm/irl.h>
#include <asm/io.h>
#include "../common/common.h"

static int ascs[] __initdata = { 
	2 | (STASC_FLAG_NORTSCTS << 8) 
};


static const char *LMI_SYS_partalias_256[] = {
	"BPA2_Region0",
	"v4l2-coded-video-buffers",
	"BPA2_Region1",
	"v4l2-video-buffers",
	"coredisplay-video",
	"bigphysarea",
	NULL
};

static struct bpa2_partition_desc bpa2_parts_table[] = {
	{
		.name	= "LMI_SYS",		/* 39.25MB */
		.start	= 0,
		.size	= 0x2a40000, /* 41.25MiB */
		.flags	= 0,
		.aka	= LMI_SYS_partalias_256
	},
	{
		.name	= "gfx-memory-0",	/* 18MB */
		.start	= 0x46E00000,
		.size	= 0x011a0000,
		.flags	= 0,
		.aka	= NULL
	}
};

static void __init saturna_setup(char** cmdline_p)
{
	printk("Wyplay DOLMENA board initialisation\n");

	stx7105_early_device_init();
	stx7105_configure_asc(ascs, ARRAY_SIZE(ascs), 0);
	bpa2_init(bpa2_parts_table, ARRAY_SIZE(bpa2_parts_table));
}

static struct plat_ssc_data ssc_private_info = {
	.capability  =
		ssc0_has(SSC_I2C_CAPABILITY) |
		ssc1_has(SSC_UNCONFIGURED)   |
		ssc2_has(SSC_UNCONFIGURED)   |
		ssc3_has(SSC_UNCONFIGURED)
};

static struct usb_init_data usb_init[2] __initdata = {
	{
		.oc_en  = 0,
		.pwr_en = 0,
	}, {
		.oc_en  = 0,
		.pwr_en = 0,
	}
};

static struct plat_stmmacphy_data phy_private_data = {
	.bus_id = 0,
	.phy_addr = -1,
	.phy_mask = 0,
	.interface = PHY_INTERFACE_MODE_MII,
};

static struct platform_device saturna_phy_device = {
	.name		= "stmmacphy",
	.id		= 0,
	.num_resources	= 1,
	.resource	= (struct resource[]) {
		{
			.name	= "phyirq",
			.start	= -1,/* no IRQ line (?) */
			.end	= -1,
			.flags	= IORESOURCE_IRQ,
		},
	},
	.dev = {
		.platform_data = &phy_private_data,
	}
};

/* Configuration for NAND Flash */
static struct mtd_partition saturna_nand_parts[] = {
	{
		.name   = "rootfs",
		.offset = 0,
		.size   = MTDPART_SIZ_FULL
	},
};

static struct plat_stmnand_data saturna_nand_config = {
	/* STM_NAND_EMI data */
	.emi_withinbankoffset   = 0,
	.rbn_port		= -1, /* 11 */
	.rbn_pin		= -1, /* 1 */

	.timing_data = &(struct nand_timing_data) {
		.sig_setup      = 50,	   /* times in ns */
		.sig_hold       = 50,
		.CE_deassert    = 0,
		.WE_to_RBn      = 100,
		.wr_on		= 10,
		.wr_off		= 40,
		.rd_on		= 10,
		.rd_off		= 40,
		.chip_delay     = 50,	   /* in us */
	},
	.flex_rbn_connected     = 1,
};

/* Platform data for STM_NAND_EMI/FLEX/AFM */
static struct platform_device saturna_nand_device =
	STM_NAND_DEVICE("stm-nand-flex", 0, &saturna_nand_config,
		saturna_nand_parts, ARRAY_SIZE(saturna_nand_parts),
		NAND_USE_FLASH_BBT);


#if 0 /* multicolor leds need dedicated handling */
static struct platform_device saturna_leds = {
	.name = "leds-gpio",
	.id = 0,
	.dev.platform_data = &(struct gpio_led_platform_data) {
		.num_leds = 4,
		.leds = (struct gpio_led[]) {
			{
				.name = "led:0",
				.gpio = stpio_to_gpio(5, 6),
				.active_low = 0,
			},
			{
				.name = "led:1",
				.gpio = stpio_to_gpio(5, 7),
				.active_low = 0,
			},
			{
				.name = "led:2",
				.gpio = stpio_to_gpio(2, 0),
				.active_low = 0,
			},
			{
				.name = "led:3",
				.gpio = stpio_to_gpio(2, 1),
				.active_low = 0,
			},
		},
	},
};
#endif

static struct platform_device *saturna_devices[] __initdata = {
//	&saturna_leds,
	&saturna_phy_device,
};

static unsigned lirc_enabled_pios[] = {
	[0] = 3 * 8 + 0
};

/* Configuration based on ??? signals train. */
static lirc_scd_t lirc_scd = {
	.code = 0x3FFFC028,
	.codelen = 0x1e,
	.alt_codelen = 0,
	.nomtime = 0x1f4,
	.noiserecov = 0,
};

static int __init device_init(void)
{
#if 0
    struct stpio_pin *hub_pin, *reader_pin;
    struct stpio_pin *audio_pin;
//    struct stpio_pin *flashwp_pin;

    /* I2C */
    stx7105_configure_ssc(&ssc_private_info);

    /* USB card reader (nRST_READER) - set in reset state */
    reader_pin = stpio_request_set_pin(4, 2, "usb_creader_reset", STPIO_OUT, 0);
    /* USB hub (nRST_USBHUB) */
    hub_pin = stpio_request_set_pin(4, 3, "usb_hub_reset", STPIO_OUT, 0);
    udelay(2000);
    stpio_set_pin (hub_pin, 1);
    stpio_free_pin (hub_pin);
    /* USB card reader (nRST_READER) - release reset state */
    stpio_set_pin (reader_pin, 1);
    stpio_free_pin (reader_pin);
    udelay(2000);


    /* ethernet */
    stx7105_configure_ethernet(0, 0, 0, 0, 1 /*ext_clk*/, 0, 0);

    /* audio (AUDIO_EN) - disable output on startup - should be moved to setup() ?*/
    audio_pin = stpio_request_set_pin(6, 5, "audio_en", STPIO_OUT, 0);
    stpio_free_pin (audio_pin);

    /* audio */
    stx7105_configure_audio_pins(3, 1, 0);

    /* flash write protect (nFLASH_WP) */
    /*
     * FLASH_WP is used by NAND. Since MTD NAND has no
     * concept of WP/VPP, we must permanently enable it
     */
//  flashwp_pin = stpio_request_set_pin(6, 4, "FLASH_WP", STPIO_OUT, 1);
//    stpio_free_pin (flashwp_pin);
#else
	struct stpio_pin *wp_pio;

	/* nand */
	wp_pio = stpio_request_pin(11, 0, "NAND WP", STPIO_OUT);
	stpio_set_pin(wp_pio, 0);
	stx7105_configure_nand(&saturna_nand_device);

	/* usb */
	stx7105_configure_usb(0, &usb_init[0]);
	stx7105_configure_usb(1, &usb_init[1]);

	/* lirc */
	stx7105_configure_lirc(&lirc_scd, lirc_enabled_pios,
		       ARRAY_SIZE(lirc_enabled_pios));

#endif
    return platform_add_devices(saturna_devices, ARRAY_SIZE(saturna_devices));
}
arch_initcall(device_init);

static void __iomem *saturna_ioport_map(unsigned long port, unsigned int size)
{
	/* However picking somewhere safe isn't as easy as you might think.
	 * I used to use external ROM, but that can cause problems if you are
	 * in the middle of updating Flash. So I'm now using the processor core
	 * version register, which is guaranted to be available, and non-writable.
	 */
	return (void __iomem *)CCN_PVR;
}

static void __init saturna_init_irq(void)
{
}

struct sh_machine_vector mv_saturna __initmv = {
	.mv_name		= "saturna",
	.mv_setup		= saturna_setup,
	.mv_nr_irqs		= NR_IRQS,
	.mv_init_irq		= saturna_init_irq,
	.mv_ioport_map		= saturna_ioport_map,
};
