/*
 * arch/sh/boards/st/silver/setup.c
 *
 * Copyright (C) 2008 STMicroelectronics Limited
 * Author: Stuart Menefy (stuart.menefy@st.com)
 *
 * Copyright (C) 2010 WyPlay SAS.
 * Marek Skuczynski <mskuczynski@wyplay.com>
 *
 * May be copied or modified under the terms of the GNU General Public
 * License.  See linux/COPYING for more information.
 *
 * Silver support.
 */

#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/stm/pio.h>
#include <linux/stm/soc.h>
#include <linux/stm/emi.h>
#include <linux/delay.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/physmap.h>
#include <linux/mtd/partitions.h>
#include <sound/stm.h>
#include <linux/mtd/nand.h>
#include <linux/stm/nand.h>
#include <linux/stm/soc_init.h>
#include <linux/phy.h>
#include <linux/gpio_keys.h>
#include <linux/input.h>
#include <asm/irq-ilc.h>
#include <asm/irl.h>
#include <asm/io.h>

static int ascs[] __initdata = { 
	2 | (STASC_FLAG_NORTSCTS << 8) 
};


#if defined(CONFIG_BPA2)

#include <linux/bpa2.h>

static const char *LMI_SYS_partalias_256[] = {
	"BPA2_Region0",
	"v4l2-coded-video-buffers",
	"BPA2_Region1",
	"v4l2-video-buffers",
	"coredisplay-video",
	NULL
};

static const char *bigphys_partalias_256[] = {
	"gfx-memory-1",
	NULL
};

static struct bpa2_partition_desc bpa2_parts_table[] = {
	{
		.name	= "LMI_SYS",		/* 85MB */
		.start	= 0,
		.size	= 0x05500000,
		.flags	= 0,
		.aka	= LMI_SYS_partalias_256
	},
	{
		.name	= "bigphysarea",	/* 18MB */
		.start	= 0,
		.size	= 0x01200000,
		.flags	= 0,
		.aka	= bigphys_partalias_256
	},
	{
		.name	= "gfx-memory-0",	/* 50MB */
		.start	= 0x4CE00000,
		.size	= 0x03200000,
		.flags	= 0,
		.aka	= NULL
	}
};

#endif


static void __init silvera_setup(char** cmdline_p)
{
	printk("Wyplay SILVERA board initialisation\n");

	stx7105_early_device_init();
	stx7105_configure_asc(ascs, ARRAY_SIZE(ascs), 0);

#if defined(CONFIG_BPA2)
	bpa2_init(bpa2_parts_table, ARRAY_SIZE(bpa2_parts_table));
#endif
}

static struct plat_ssc_data ssc_private_info = {
	.capability  =
		ssc0_has(SSC_I2C_CAPABILITY) |
		ssc1_has(SSC_UNCONFIGURED)   |
		ssc2_has(SSC_UNCONFIGURED)   |
		ssc3_has(SSC_I2C_CAPABILITY),
	.routing =
		SSC3_SCLK_PIO3_6 | SSC3_MTSR_PIO3_7,
};

#if defined(CONFIG_I2C_BOARDINFO)
static struct i2c_board_info silvera_i2c1_devices[] __initdata = {
	{ I2C_BOARD_INFO("stv6432", 0x4A), .type = "stv6432" },
};
#endif


static struct usb_init_data usb_init[2] __initdata = {
	{
		.oc_en  = 0,
		.pwr_en = 0,
	}, {
		.oc_en  = 0,
		.pwr_en = 0,
	}
};

#if defined(CONFIG_STMMAC_ETH)
static struct plat_stmmacphy_data phy_private_data = {
	/* ANY */
	.bus_id = 0,
	.phy_addr = -1,
	.phy_mask = 0,
	.interface = PHY_INTERFACE_MODE_MII,
};

static struct platform_device silvera_phy_device = {
	.name		= "stmmacphy",
	.id		= 0,
	.num_resources	= 1,
	.resource	= (struct resource[]) {
		{
			.name	= "phyirq",
			.start	= -1,/* no IRQ line (?) */
			.end	= -1,
			.flags	= IORESOURCE_IRQ,
		},
	},
	.dev = {
		.platform_data = &phy_private_data,
	}
};
#endif /* CONFIG_STMMAC_ETH */

static struct mtd_partition mtd_nor_parts_table[] = {
	{
		.name = "pbl",
		.offset = 0,
		.size = 64 * 1024,
	}, {
		.name = "k1",
		.offset = MTDPART_OFS_APPEND,
		.size = 1920 * 1024,
	}, {
		.name = "k2",
		.offset = MTDPART_OFS_APPEND,
		.size = 1920 * 1024,
	}, {
		.name = "rwdata",
		.offset = MTDPART_OFS_APPEND,
		.size = 64 * 1024,
	}, {
		.name = "rodata",
		.offset = MTDPART_OFS_APPEND,
		.size = 64 * 1024,
	}, {
		.name = "reserved",
		.offset = MTDPART_OFS_APPEND,
		.size = 64 * 1024,
	}
};

static struct physmap_flash_data silvera_physmap_flash_data = {
	.width		= 2,
	.set_vpp	= NULL,
	.nr_parts	= ARRAY_SIZE(mtd_nor_parts_table),
	.parts		= mtd_nor_parts_table
};

static struct platform_device silvera_physmap_flash = {
	.name		= "physmap-flash",
	.id		= -1,
	.num_resources	= 1,
	.resource	= (struct resource[]) {
		{
			.start		= 0x00000000,
			.end		= 4*1024*1024 - 1,
			.flags		= IORESOURCE_MEM,
		}
	},
	.dev		= {
		.platform_data	= &silvera_physmap_flash_data,
	},
};

#if defined(CONFIG_MTD_NAND)
/* Configuration for NAND Flash */
static struct mtd_partition mtd_nand_parts_table[] = {
	{
		.name   = "ubi",
		.offset = 0,
		.size   = 0x7F80000
	}
};

static struct plat_stmnand_data nand_config = {
	/* STM_NAND_EMI data */
	.emi_withinbankoffset   = 0,
	.rbn_port               = -1,
	.rbn_pin                = -1,

	.timing_data = &(struct nand_timing_data) {
		.sig_setup      = 50,           /* times in ns */
		.sig_hold       = 50,
		.CE_deassert    = 0,
		.WE_to_RBn      = 100,
		.wr_on          = 10,
		.wr_off         = 40,
		.rd_on          = 10,
		.rd_off         = 40,
		.chip_delay     = 50,           /* in us */
	},
	.flex_rbn_connected     = 1,
};

/* Platform data for STM_NAND_EMI/FLEX/AFM. (bank# may be updated later) */
static struct platform_device nand_device =
STM_NAND_DEVICE("stm-nand-afm", 2, &nand_config,
		mtd_nand_parts_table, ARRAY_SIZE(mtd_nand_parts_table), NAND_USE_FLASH_BBT);
#endif /* CONFIG_MTD_NAND */

static struct platform_device *silvera_devices[] __initdata = {
	&silvera_physmap_flash,
#if defined(CONFIG_STMMAC_ETH)
	&silvera_phy_device,
#endif
};

#if defined(CONFIG_LIRC_STM)
static unsigned lirc_enabled_pios[] = {
	[0] = 3 * 8 + 0
};
#endif /* CONFIG_LIRC_STM */

static int __init device_init(void)
{
#if defined(CONFIG_STMMAC_ETH)
    struct stpio_pin *pwr_on_eth_pin;
#endif
    struct stpio_pin *flashwp_pin;

    /* I2C */
#if defined(CONFIG_I2C_BOARDINFO)
    i2c_register_board_info(1, silvera_i2c1_devices, ARRAY_SIZE(silvera_i2c1_devices));
#endif
    stx7105_configure_ssc(&ssc_private_info);

    /* usb */
    stx7105_configure_usb(0, &usb_init[0]);
    stx7105_configure_usb(1, &usb_init[1]);

#if defined(CONFIG_LIRC_STM)
    /* lirc */
    stx7105_configure_lirc(NULL, lirc_enabled_pios, ARRAY_SIZE(lirc_enabled_pios));
#endif /* CONFIG_LIRC_STM */

#if defined(CONFIG_STMMAC_ETH)
    /* ethernet */
    pwr_on_eth_pin = stpio_request_set_pin(15, 5, "POWER_ON_ETH", STPIO_OUT, 1);
    stpio_free_pin (pwr_on_eth_pin);

    stx7105_configure_ethernet(0, stx7105_ethernet_mii, 0, 0, 0, 0);
#endif /* CONFIG_STMMAC_ETH */

    /* audio */
    stx7105_configure_audio_pins(1, 1, 1, 0);

    /* flash write protect (nFLASH_WP) */
    /*
     * FLASH_WP is used by NAND. Since MTD NAND has no
     * concept of WP/VPP, we must permanently enable it
     */
    flashwp_pin = stpio_request_set_pin(6, 4, "FLASH_WP", STPIO_OUT, 1);
    stpio_free_pin (flashwp_pin);

#if defined(CONFIG_MTD_NAND)
    stx7105_configure_nand(&nand_device);
#endif

    return platform_add_devices(silvera_devices, ARRAY_SIZE(silvera_devices));
}
arch_initcall(device_init);

static void __iomem *silvera_ioport_map(unsigned long port, unsigned int size)
{
	/* However picking somewhere safe isn't as easy as you might think.
	 * I used to use external ROM, but that can cause problems if you are
	 * in the middle of updating Flash. So I'm now using the processor core
	 * version register, which is guaranted to be available, and non-writable.
	 */
	return (void __iomem *)CCN_PVR;
}

static void __init silvera_init_irq(void)
{
}

struct sh_machine_vector mv_silvera __initmv = {
	.mv_name		= "silvera",
	.mv_setup		= silvera_setup,
	.mv_nr_irqs		= NR_IRQS,
	.mv_init_irq		= silvera_init_irq,
	.mv_ioport_map		= silvera_ioport_map,
};
