/*
 * T2
 *
 * (C) Copyright 2006-2009 WyPlay SAS.
 * Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>
 * Aubin Constans <aconstans@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/stm/pio.h>
#include <linux/stm/soc.h>
#include <linux/stm/sysconf.h>

#include <asm/io.h>

void tango2a_pio_init(void)
{
	struct stpio_pin *pio_ptr = NULL, *tmp_pio_ptr = NULL;
	u64 sysconf;
	struct sysconf_field *sc;

	/** configure SYS_CFG7 register, "PIOs alternate" **/
	sc = sysconf_claim(SYS_CFG, 7, 0, 20, __func__);
	sysconf = sysconf_read(sc);
	/* PIO2_1 (I2C0 SDA) = SSC0_MTSR, and not SSC0_MRST */
	sysconf &= ~(1 << 1);
	/* PIO3_1 (I2C1 SDA) = SSC1_MTSR, and not SSC1_MRST */
	sysconf &= ~(1 << 2);
	/* PIO4_1 (SSC2 SDA) = SSC2_MTSR, and not SSC2_MRST */
	sysconf &= ~(1 << 3);
	/* have ASC0 (SmartCard interface) clock derive from CLK_DSS,
	 * not from SC0_CLK_OUT */
	sysconf &= ~(1 << 4);
	/* ensure ASC1 (RS-232 UART) clock derives from CLK_DSS */
	sysconf &= ~(1 << 5);
	/* configure SC0_nSETVCC: derived from SC0_DETECT input */
	sysconf &= ~(1 << 7);
	/* set polarity of SC0_nSETVCC: SC0_nSETVCC = NOT(SC0_DETECT) */
	sysconf |= (1 << 8);
	/* use SSC0 and SSC1, and not the video horizontal and vertical synchro
	 * VTG_X outputs, nor the DVO_BLANK digital video blanking output */
	sysconf &= ~(1 << 10);
	/* enable Synopsys Ethernet MAC Subsystem, and Media Independent Interface */
	/* select MII interface as function of the shared pad, and not DVO output */
	sysconf |= (1 << 16);
	/* enable MII pad */
	sysconf |= (1 << 17);
	/* select MII interface, and not Reduced MII */
	sysconf &= ~(1 << 18);
	/* PHY clock is provided by STx7109 */
	sysconf &= ~(1 << 19);
	/* for now have the MAC run at 10Mbps speed */
	sysconf &= ~(1 << 20);
	sysconf_write(sc, sysconf);

	/* MMI */
	/* PIO0_0 = MCU or IR Activity LED: blue, active high */
	/* Turn the LED off, let the MCU control it */
	pio_ptr = stpio_request_set_pin(0, 0, "LED_IR_ACT", STPIO_IN, 0);
	stpio_free_pin(pio_ptr);

	/* REC_LED */
	pio_ptr = stpio_request_set_pin(0, 5, "REC_LED", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);

	/* 1.1v DUMMY */
	pio_ptr = stpio_request_set_pin(0, 7, "11V2_DUMMY", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);

	/** RS-232 **/
	/* PIO4_2 = ASC2_RXD
	 * PIO4_3 = ASC2_TXD */
	/* The STM Asynchronous Serial Controller driver configures
	 * TXD and RXD PIO's itself, see asc_set_termios_cflag() */

	/** I2C busses **/
	/* PIO0_3 = MCU I2C SDA
	 * PIO0_4 = MCU I2C SCL
	 * PIO2_0 = I2C0 SCL "TS0"
	 * PIO2_1 = I2C0 SDA "TS0"
	 * PIO3_0 = HDMI SCL (I2C1) "TS1"
	 * PIO3_1 = HDMI SDA (I2C1) "TS1" */
	/* The STM Synchronous Serial Controller driver configures
	 * SCL and SDA PIO's itself, see st_i2c.c */
	pio_ptr = stpio_request_set_pin(0, 3, "MCU_SDA", STPIO_BIDIR, 1);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_set_pin(0, 4, "MCU_SCL", STPIO_BIDIR, 1);
	stpio_free_pin(pio_ptr);

	/* Power Controller (Microcontroller) */
	/* PIO4_0 = SN8P2613:nINT (Interrupt output) */

	/** Ethernet **/
	/* PIO3_7 = LAN8700:CLKIN (MII PHY clock)
	 * PIO5_3 = LAN8700:nRST */
	/* Reset the Ethernet PHY SMSC LAN8700
	 * assert nRST (low) for at least 100usec,
	 * with MII clock fed in at the same time */
	pio_ptr = stpio_request_set_pin(5, 3, "ETH_PHY_RESET", STPIO_OUT, 0);
	tmp_pio_ptr = stpio_request_pin(3, 7, "MII_PHYCLK", STPIO_ALT_OUT);
	stpio_free_pin(tmp_pio_ptr);
	udelay(110);
	stpio_set_pin(pio_ptr, 1);
	stpio_free_pin(pio_ptr);
	udelay(1);

	/** InfraRed **/
	/* PIO3_3 = InfraRed IN */
	/* The LIRC driver configures IN and OUT PIOs itself */

	/** HDMI **/
	/* PIO2_2 = HDMI Hot Plug Detect: input, on-board pull-up (BUG) (1K pull-up in typical HDMI receiver)
	 * PIO2_3 = HDMI Consumer Electronics Control: open-drain, no on-board pull-up (BUG) */
	/* HPD PIO configured by stgfb itself (>= 2.0_stm22_0007-alpha)
	 * because the pull-up on CEC is missing, configure this GPIO as
	 * open-drain with weak pull-up */
	pio_ptr = stpio_request_set_pin(2, 3, "HDMI_CEC", STPIO_BIDIR_Z1, 0);
	stpio_free_pin(pio_ptr);

	/** DVB-T receivers **/
	/* PIO0_6 = Tuner daughter board ON/OFF, supposed active high
	 * PIO4_7 = Transport Stream 0 Error, on-board pull-down */
	pio_ptr = stpio_request_set_pin(0, 6, "TUNER_ENABLE", STPIO_OUT, 0);
	stpio_free_pin(pio_ptr);
	pio_ptr = stpio_request_pin(4, 7, "TS0_ERROR", STPIO_IN);
	stpio_free_pin(pio_ptr);

	/** Analog Audio/Video outputs **/
	/* PIO3_2 = Mute of analog audio output (CINCH, stereo), open-drain, active high */
	/* Set audio mute high to let the MCU control it */
	pio_ptr = stpio_request_set_pin(3, 2, "AUDIO_MUTE_CINCH", STPIO_BIDIR, 1);
	stpio_free_pin(pio_ptr);
}
