/*
 * SATA MIPHY initialization hack(s)
 *
 * Copyright (C) 2007 STMicroelectronics Limited
 * Author: Stuart Menefy <stuart.menefy@st.com>
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */

#include <asm/processor.h>
#include <linux/bug.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/stm/soc.h>
#include <linux/stm/sysconf.h>

#define OUT(value) sysconf_write(m->out, value)
#define IN() sysconf_read(m->in_tdo)

static void __init SATA_JTAG_IR_Select_MIPHY(struct stm_sata_miphy *m,
		int lane)
{
	/* Flush trst synchronizer with two tck clocks */
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	/* Set tap into idle state */
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	/* Set tap into shift ir state */
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);

	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	if (!m->dual_not_single) {
		/* Shift in MacroMicroBusAccess TDI = 101 */
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);
		OUT(m->trstn | m->tdi_low | m->tck_low);
		OUT(m->trstn | m->tdi_low | m->tck_high);
	} else if (lane == 0) {
		/* Lane 1: Shift in BYPASS mode, TDI = 111 */
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);

		/* Lane 0: Shift in MacroMicroBusAccess TDI = 101 */
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);
		OUT(m->trstn | m->tdi_low | m->tck_low);
		OUT(m->trstn | m->tdi_low | m->tck_high);
	} else if (lane == 1) {
		/* Lane 1: Shift in MacroMicroBusAccess TDI = 101 */
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);
		OUT(m->trstn | m->tdi_low | m->tck_low);
		OUT(m->trstn | m->tdi_low | m->tck_high);
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);

		/* Lane 0: Shift in BYPASS mode, TDI = 111 */
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);
		OUT(m->trstn | m->tdi_high | m->tck_low);
		OUT(m->trstn | m->tdi_high | m->tck_high);
	} else {
		BUG();
	}

	/* Set tap into idle mode */
	OUT(m->tms | m->trstn | m->tdi_high | m->tck_low);
	OUT(m->tms | m->trstn | m->tdi_high | m->tck_high);
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
}

static __init u32 SATA_JTAG_DR_Read_MIPHY(struct stm_sata_miphy *m,
		int lane, u8 regno)
{
	int k;
	u8 x;
	u32 regvalue = 0;

	/* Set TAP into shift DR state */
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);

	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	/* Shift in DR=[17:10]dummy_data;[9:2]address=regno;[1]rd=1;[0]wr=0 x */
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tdi_high | m->tck_low);
	OUT(m->trstn | m->tdi_high | m->tck_high);

	/*  push in the register address */
	for (k = 0; k < 8; k++) {
		x = ((regno >> k) & 0x1);
		x = x << 1;
		OUT(m->trstn | x | m->tck_low);
		OUT(m->trstn | x | m->tck_high);
	}

	/* Push in 0 in the data field */
	for (k = 0; k < 7; k++) {
		OUT(m->trstn | m->tck_low);
		OUT(m->trstn | m->tck_high);
	}

	if (m->dual_not_single && lane == 1) {
		/*Shift out 1 bit content of Bypass register of Lane 0 */
		OUT(m->trstn | m->tck_low);
		OUT(m->trstn | m->tck_high);
	}

	/* Set TAP back round into SHIFT DR STATE
	 * (exit1_dr,update_dr,idle,idle,select_dr,capture_dr) */
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	/* Shift out the MIPHY register contents */

	if (m->dual_not_single && lane == 0) {
		/*Shift out 1 bit content of Bypass register of Lane 1 */
		OUT(m->trstn | m->tck_low);
		OUT(m->trstn | m->tck_high);
	}

	/* Discard first 10 bits */
	for (k = 0; k < 10; k++) {
		OUT(m->trstn | m->tck_low);
		OUT(m->trstn | m->tck_high);
	}

	for (k = 0; k < 8; k++) {
		OUT(m->trstn | m->tck_low);
		OUT(m->trstn | m->tck_high);
		regvalue = (IN() << k) | regvalue;
	}

	/* Set TAP into idle mode */
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	return regvalue;
}

static void __init SATA_JTAG_DR_Write_MIPHY(struct stm_sata_miphy *m,
		int lane, u8 regno, u8 data)
{
	int k;
	u8 x;

	/* Set TAP into shift dr state */
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	/* Shift in DR=[17:10]=data;[9:2]address=regno;[1]rd=0;[0]wr=1  */
	OUT(m->trstn | m->tdi_high | m->tck_low);
	OUT(m->trstn | m->tdi_high | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	/* Push in the register address */
	for (k = 0; k < 8; k++) {
		x = ((regno >> k) & 0x1);
		x = x << 1;
		OUT(m->trstn | x | m->tck_low);
		OUT(m->trstn | x | m->tck_high);
	}

	/* Push in the data to be written */
	x = 0;
	for (k = 0; k < 7; k++) {
		x = ((data >> k) & 0x1);
		x = x << 1;
		OUT(m->trstn | x | m->tck_low);
		OUT(m->trstn | x | m->tck_high);
	}
	x = ((data >> 7) & 0x1);
	x = x << 1;

	if (m->dual_not_single && lane == 1) {
		/*Shift out 1 bit content of Bypass register of Lane 0 */
		OUT(m->trstn | x | m->tck_low);
		OUT(m->trstn | x | m->tck_high);
	}

	/* Set TAP back round into SHIFT DR STATE
	 * (exit1_dr,update_dr,idle,idle,select_dr,capture_dr) */
	if (!m->dual_not_single || lane == 0) {
		OUT(m->tms | m->trstn | x | m->tck_low);
		OUT(m->tms | m->trstn | x | m->tck_high);
	} else if (lane == 1) {
		OUT(m->tms | m->trstn | m->tck_low);
		OUT(m->tms | m->trstn | m->tck_high);
	} else {
		BUG();
	}
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);

	/* Shift out MiPhy Control reg , shift in TDI = 0x00000000 ----> Why? */
	/* Shift out 1 bit content of Bypassed Lane*/
	if (m->dual_not_single) {
		/*Shift out 1 bit content of Bypassed Lane  */
		OUT(m->trstn | m->tck_low);
		OUT(m->trstn | m->tck_high);
		if (lane == 0) {
			/* Shift out 1 bit content of Bypass register
			 * of Lane 1 */
			OUT(m->trstn | m->tck_low);
			OUT(m->trstn | m->tck_high);
		}

		/*Discard first 10 bits */
		for (k = 0; k < 10; k++) {
			OUT(m->trstn | m->tck_low);
			OUT(m->trstn | m->tck_high);
		}
	}

	/*Set TAP into idle mode */
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->tms | m->trstn | m->tck_low);
	OUT(m->tms | m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
	OUT(m->trstn | m->tck_low);
	OUT(m->trstn | m->tck_high);
}

static void __init SATA_JTAG_MIPHY_Start_Lane0(struct stm_sata_miphy *miphy)
{
	SATA_JTAG_IR_Select_MIPHY(miphy, 0);

	/* Force Macro1 in reset and request PLL calibration reset */

	/* Force PLL calibration reset, PLL reset and assert
	 * Deserializer Reset */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x00, 0x16);
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x11, 0x0);
	/* Force macro1 to use rx_lspd, tx_lspd (by default rx_lspd
	 * and tx_lspd set for Gen1)  */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x10, 0x1);

	/* Force Recovered clock on first I-DLL phase & all
	 * Deserializers in HP mode */

	/* Force Rx_Clock on first I-DLL phase on macro1 */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x72, 0x40);
	/* Force Des in HP mode on macro1 */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x12, 0x00);

	/* Wait for HFC_READY = 0 */
	while (SATA_JTAG_DR_Read_MIPHY(miphy, 0, 0x1) & 0x3)
		;

	/* Restart properly Process compensation & PLL Calibration */

	/* Set properly comsr definition for 30 MHz ref clock */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x41, 0x1E);
	/* comsr compensation reference */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x42, 0x28);
	/* Set properly comsr definition for 30 MHz ref clock */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x41, 0x1E);
	/* comsr cal gives more suitable results in fast PVT for comsr
	   used by TX buffer to build slopes making TX rise/fall fall
	   times. */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x42, 0x33);
	/* Force VCO current to value defined by address 0x5A */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x51, 0x2);
	/* Force VCO current to value defined by address 0x5A */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x5A, 0xF);
	/* Enable auto load compensation for pll_i_bias */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x47, 0x2A);
	/* Force restart compensation and enable auto load for
	 * Comzc_Tx, Comzc_Rx & Comsr on macro1 */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x40, 0x13);

	/* Wait for comzc & comsr done */
	while ((SATA_JTAG_DR_Read_MIPHY(miphy, 0, 0x40) & 0xC) != 0xC)
		;

	/* Recommended settings for swing & slew rate FOR SATA GEN 1
	 * from CPG */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x20, 0x00);
	/* (Tx Swing target 500-550mV peak-to-peak diff) */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x21, 0x2);
	/* (Tx Slew target120-140 ps rising/falling time) */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x22, 0x4);

	/* Force Macro1 in partial mode & release pll cal reset */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x00, 0x10);
	udelay(10);

	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x50, 0x8D);
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x50, 0x8D);

	/*  Wait for phy_ready */
	/*  When phy is in ready state ( register 0x01 of macro1 to 0x13) */
	while ((SATA_JTAG_DR_Read_MIPHY(miphy, 0, 0x01) & 0x03) != 0x03)
		;

	/* Enable macro1 to use rx_lspd  & tx_lspd from link interface */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x10, 0x00);
	/* Release Rx_Clock on first I-DLL phase on macro1 */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x72, 0x00);

	/* Deassert deserializer reset */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x00, 0x00);
	/* des_bit_lock_en is set */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x02, 0x08);

	/* bit lock detection strength */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x86, 0x61);
}

static void __init SATA_JTAG_MIPHY_Start_Lane1(struct stm_sata_miphy *miphy)
{
	/* Select MacroMicroBus for Lane 1*/
	SATA_JTAG_IR_Select_MIPHY(miphy, 1);
	/* Force PLL calibration reset, PLL reset and assert Deserializer
	 * Reset */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x00, 0x2);
	/* Force restart compensation and enable auto load for Comzc_Tx,
	 * Comzc_Rx & Comsr on macro2 */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x40, 0x13);

	/* Select MacroMicroBus for Lane 0 */
	SATA_JTAG_IR_Select_MIPHY(miphy, 0);
	/* Force PLL reset  */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x00, 0x2);
	/* Set properly comsr definition for 30 MHz ref clock */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x41, 0x1E);
	/* to get more optimum result on comsr calibration giving faster
	 * rise/fall time in SATA spec Gen1 useful for some corner case.*/
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x42, 0x33);
	/* Force restart compensation and enable auto load for Comzc_Tx,
	 * Comzc_Rx & Comsr on macro1 */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x40, 0x13);

	/*Wait for HFC_READY = 0*/
	while (SATA_JTAG_DR_Read_MIPHY(miphy, 0, 0x1) & 0x3)
		;

	/* Select MacroMicroBus for Lane 1*/
	SATA_JTAG_IR_Select_MIPHY(miphy, 1);
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x11, 0x0);
	/* Force macro2 to use rx_lspd, tx_lspd  (by default rx_lspd and
	 * tx_lspd set for Gen1) */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x10, 0x1);
	/* Force Rx_Clock on first I-DLL phase on macro2*/
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x72, 0x40);
	/* Force Des in HP mode on macro2 */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x12, 0x00);

	while ((SATA_JTAG_DR_Read_MIPHY(miphy, 1, 0x40) & 0xC) != 0xC)
		;

	/*RECOMMENDED SETTINGS for Swing & slew rate FOR SATA GEN 1 from CPG*/
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x20, 0x00);
	/*(Tx Swing target 500-550mV peak-to-peak diff) */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x21, 0x2);
	/*(Tx Slew target120-140 ps rising/falling time) */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x22, 0x4);
	/*Force Macr21 in partial mode & release pll cal reset */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x00, 0x10);
	udelay(10);
	/* Select MacroMicroBus for Lane 0 */
	SATA_JTAG_IR_Select_MIPHY(miphy, 0);
	/* Release PLL reset  */
	SATA_JTAG_DR_Write_MIPHY(miphy, 0, 0x00, 0x0);

	/* Select MacroMicroBus for Lane 1*/
	SATA_JTAG_IR_Select_MIPHY(miphy, 1);

	/*  Wait for phy_ready */
	/*  When phy is in ready state ( register 0x01 of macro1 to 0x13)*/
	while ((SATA_JTAG_DR_Read_MIPHY(miphy, 1, 0x01) & 0x03) != 0x03)
		;

	/* Enable macro1 to use rx_lspd  & tx_lspd from link interface */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x10, 0x00);
	/* Release Rx_Clock on first I-DLL phase on macro1 */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x72, 0x00);

	/* Deassert deserializer reset */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x00, 0x00);
	/*des_bit_lock_en is set */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x02, 0x08);

	/*bit lock detection strength */
	SATA_JTAG_DR_Write_MIPHY(miphy, 1, 0x86, 0x61);
}



void __init stm_sata_miphy_init(struct stm_sata_miphy *miphy, int lane)
{
	BUG_ON(!miphy);
	BUG_ON(lane < 0 || lane > 1);
	BUG_ON(!miphy->dual_not_single && lane != 0);

	switch (lane) {
	case 0:
		SATA_JTAG_MIPHY_Start_Lane0(miphy);
		break;
	case 1:
		SATA_JTAG_MIPHY_Start_Lane1(miphy);
		break;
	default:
		BUG();
		break;
	}
}
