/*
 * (C) Copyright 2009 WyPlay SAS.
 * Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/autoconf.h>
#include <linux/wymcu/mcu.h>
#include <linux/wymcu/mcu_service.h>

static struct mcu_service_dev demo = {
	.name = "demo",
	.service_type = SERVICE_DEMO,
};

static int mcu_example_probe(struct mcu_dev *dev)
{
	demo.mcu_dev = dev;
	printk("%s\n", __FUNCTION__);
	return mcu_service_device_register(&demo);
}

static int mcu_example_remove(struct mcu_dev *dev)
{
	printk("%s\n", __FUNCTION__);
	mcu_service_device_unregister(&demo);
	return 0;
}

static int mcu_example_suspend(struct mcu_dev *dev, pm_message_t pm)
{
	printk("%s\n", __FUNCTION__);
	return 0;
}

static int mcu_example_resume(struct mcu_dev *dev)
{
	printk("%s\n", __FUNCTION__);
	return 0;
}

static struct mcu_driver mcu_example_driver = {
	.driver = {
		.owner	= THIS_MODULE,
		.name	= "mcu_example",
	},
	.bus_type = DEMO,
	.bus_data = 0,
	.probe = mcu_example_probe,
	.remove = mcu_example_remove,
	.suspend = mcu_example_suspend,
	.resume = mcu_example_resume,
};

static int __init mcu_example_init(void)
{
	return mcu_driver_register(&mcu_example_driver);
}

static void __exit mcu_example_exit(void)
{
	mcu_driver_unregister(&mcu_example_driver);
}

module_init(mcu_example_init);
module_exit(mcu_example_exit);

MODULE_DESCRIPTION("MCU Example Driver");
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>");
