/*
 * (C) Copyright 2009 WyPlay SAS.
 * Frederic Mazuel <fmazuel@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _MCU_1_H_
#define _MCU_1_H_

/* MCU version supported by this driver */
#define MCU_V1		0x01
#define MCU_V7		0x07
#define MCU_V10		0x10

/* MCU firmware version supported by this driver */
#define MCU_ESS6461	0x10
#define MCU_RT107x	0x20
#define MCU_ST7109_X1	0x90
#define MCU_ST7109_T2	0x91
#define MCU_ST7105_P2	0x92

/* i2c address list supported by this driver */
#define I2C_MCU_ADDR	0x41

/* MCU sn8p2613 command */
/* Addres	R/W	Initial State	Description */
/* 0x0		R			uP version code */
/* 0x1		R			uP application code */
/*					0x10:ESS6461  */
/*					0x20:RT1071/1073 */
/*					0x90:ST7109  */
/*					0x92:ST7105  */
/* 0x2		R/W		0x0	Host state */
/*					0: idle	 */
/*					1: boot up */
/*					2: player. */
/*					3: shut down ready */
/*					4: power down */
/*					5: recording */
/*					6-0xff: reserved  */
/* 0x3		R		0x0	uP State */
/*					0: Idle */
/*					1: self test  */
/*					2. system standby */
/*					3: shut down */
/*					4: USB standby	 */
/*					5: player standby  */
/*					6: player */
/*					7. USB */
/*					8. Auto play for p2 only*/
/*					9:-0xff: reserved */
/* 0x4		R/W		0x0	uP CNTL0 */
/*					bit0: 1: mute on, 0: mute off */
/*					bit1: 1:Fan on,	 0: Fan off */
/*					bit2: 1: LED flash, 0:Disable flash */
/* 0x5		R		0x0	uP CNTL 1 */
/*					bit0: RTC INT state for p2 only*/
/* 0x6		R		0x0	Host Status0 */
/*					bit0: 1,HDD error */
/*					bit1: 1,Lan has connected */
/* 0x7		R		0x0	Host Status1 */
/* 0x8		R/W		0x0	encryption byte 0 */
/* 0x9		R/W		0x0	encryption byte 1 */
/* 0xA		R/W		0x0	encryption byte 2 */
/* 0xB		R/W		0x0	encryption byte 3 */
/* 0xC		R		0x0	encryption result byte 0 */
/* 0xD		R		0x0	encryption result byte 1 */
/* 0xE		R		0x0	encryption result byte 2 */
/* 0xF		R		0x0	encryption result byte 3 */

#define I2C_MCU_VERSION		0x00
#define I2C_MCU_APP_CODE	0x01
#define I2C_MCU_HOST_STATE	0x02
#define I2C_MCU_STATE		0x03
#define I2C_MCU_CNTL0		0x04
#define I2C_MCU_CNTL1		0x05
#define I2C_MCU_HOST_STATUS0	0x06
#define I2C_MCU_HOST_STATUS1	0x07
#define I2C_MCU_CRYPT_B0	0x08
#define I2C_MCU_CRYPT_B1	0x09
#define I2C_MCU_CRYPT_B2	0x0A
#define I2C_MCU_CRYPT_B3	0x0B
#define I2C_MCU_CRYPT_RES_B0	0x0C
#define I2C_MCU_CRYPT_RES_B1	0x0D
#define I2C_MCU_CRYPT_RES_B2	0x0E
#define I2C_MCU_CRYPT_RES_B3	0x0F

/* MCU CNTL0 */
#define MUTE			0x01
#define FAN			0x02
#define LED			0x04
#define MUTE_POS		0x00
#define FAN_POS			0x01
#define LED_POS			0x02

/* MCU CNTL1 */
#define CNTL1_MSK_RTC_INT	0x01

/* HOST status 0 */
#define HDD_ERROR		0x01
#define LAN_STATUS		0x02
#define HDD_ERROR_POS		0x00
#define LAN_STATUS_POS		0x01

/* CPU STATE */
#define C_IDLE			0
#define C_BOOT_UP		1
#define C_PLAYER		2
#define C_SHUT_DOWN_READY	3
#define C_POWER_DOWN		4
#define C_RECORDING		5

/* MCU STATE */
#define M_IDLE			0
#define M_SELF_TEST		1
#define M_SYS_STANDBY		2
#define M_SHUT_DOWN		3
#define M_USB_STANDBY		4
#define M_PLAYER_STANDBY       	5
#define M_PLAYER	       	6
#define M_USB			7
#define M_AUTO_PLAY		8

struct state {
	char *name;
	int val;
};

#endif /* _MCU_1_H_ */
