/*
 * (C) Copyright 2009 WyPlay SAS.
 * Frederic Mazuel <fmazuel@wyplay.com>
 * Jerry Zhang <zhangb@wyplayasia.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/errno.h>
#include <linux/device.h>
#include <linux/tty.h>
#include <linux/tty_driver.h>
#include <linux/tty_ldisc.h>
#include <linux/spinlock.h>
#include <linux/wait.h>
#include <linux/wymcu/mcu.h>
#include <linux/wymcu/mcu_service.h>
#include <linux/wymcu/mcu_ldisc.h>
#include "mcu_serial.h"

/* Global table */
struct pkg_params
{
	unsigned char cmd;
	int tx_lengh;
	int rx_lengh;
};

static struct pkg_params params_table[] = {
	{0x00,  0,  0},				/* NONE */
	{0x00,  1,  5},				/* VER_GET */
	{0x01,  3,  0},				/* LED_SET_GRP_STA */
	{0x02,  1,  2},				/* LED_GET_GRP_STA */
	{0x03,  1,  0},				/* LED_SET_1_ON, */
	{0x04,  1,  0},				/* LED_SET_1_OFF */
	{0x05,  1,  0},				/* LED_SET_2_ON, */
	{0x06,  1,  0},				/* LED_SET_2_OFF, */
	{0x07,  1,  0},				/* LED_SET_BI_BLK, */
	{0x00,  0,  0},				/* LED_SET_BI_OFF, */
	{0x00,  0,  0},				/* LED_SET_BI_RED, */
	{0x00,  0,  0},				/* LED_SET_BI_GRN, */
	{0x00,  0,  0},				/* ED_SET_BI_ORG, */
	{0x0A,  3,  0},				/* PWR_SET_GRP_STA, */
	{0x0B,  1,  2},				/* PWR_GET_GRP_STA, */
	{0x0C,  1,  0},				/* PWR_SET_SOC_OFF, */
	{0x0D,  1,  0},				/* PWR_SET_SOC_PWR_CYC, */
	{0x0E,  1,  0},				/* PWR_SET_VFD_ON, */
	{0x0F,  1,  0},				/* PWR_SET_VFD_OFF, */
	{0x10,  1,  0},				/* PWR_SET_DVT_TUN_ON, */
	{0x11,  1,  0},				/* PWR_SET_DVT_TUN_OFF, */
	{0x12,  1,  0},				/* PWR_SET_HDD_ON, */
	{0x13,  1,  0},				/* PWR_SET_HDD_OFF, */
	{0x14,  1,  0},				/* PWR_SET_WFI_CRD_ON, */
	{0x15,  1,  0},				/* PWR_SET_WFI_CRD_OFF, */
	{0x16,  1,  0},				/* PWR_SET_DS2_NIM_ON, */
	{0x17,  1,  0},				/* PWR_SET_DS2_NIM_OFF, */
	{0x18,  1,  0},				/* PWR_SET_ETH_ON, */
	{0x19,  1,  0},				/* PWR_SET_ETH_OFF, */
	{0x1A,  1,  0},				/* PWR_SET_TRC_FUN_ON, */
	{0x1B,  1,  0},				/* PWR_SET_TRC_FUN_OFF, */
	{0x1C,  1,  0},				/* PWR_SET_PLC_ON, */
	{0x1D,  1,  0},				/* PWR_SET_PLC_OFF, */
	{0x1E,  1,  0},				/* PWR_SET_RST_VFD, */
	{0x1F,  1,  0},				/* PWR_SET_RST_RF, */
	{0x20,  1,  0},				/* PWR_SET_RST_BOD, */
	{0x23,  4,  0},				/* TMR_SET_ARM, */
	{0x24,  1,  4},				/* TMR_GET_ARM, */
	{0x25,  1,  0},				/* TMR_SET_STP, */
	{0x51,  3,  0},				/* WUP_SET_IPT_SRC_MSK, */
	{0x26,  1,  2},				/* WUP_GET_IPT_SRC_MSK, */
	{0x27,  1,  2},				/* WUP_GET_LST_SRC, */
	{0x29,  2,  0},				/* WUP_SET_ADD_AUT_KEY_COD, */
	{0x2A,  1,  0},				/* WUP_SET_RST_AUT_KEY_COD, */
	{0x2B,  1,  0},				/* EVT_SET_DIS, */
	{0x2C,  1,  0},				/* EVT_SET_ENA, */
	{0x2D,  1,  1},				/* EVT_GET_STA, */
	{0x2E,  5,  0},				/* VFD_SET_FIX_SYM_CON, */
	{0x2F,  1,  0},				/* VFD_SET_FIX_SYM_PLAY_ON, */
	{0x30,  1,  0},				/* VFD_SET_FIX_SYM_PLAY_OFF, */
	{0x31,  1,  0},				/* VFD_SET_FIX_SYM_PAUSE_ON, */
	{0x32,  1,  0},				/* VFD_SET_FIX_SYM_PAUSE_OFF, */
	{0x33,  1,  0},				/* VFD_SET_FIX_SYM_FASTFORWARD_ON, */
	{0x34,  1,  0},				/* VFD_SET_FIX_SYM_FASTFORWARD_OFF, */
	{0x35,  1,  0},				/* VFD_SET_FIX_SYM_REVERSE_ON, */
	{0x36,  1,  0},				/* VFD_SET_FIX_SYM_REVERSE_OFF, */
	{0x37,  1,  0},				/* VFD_SET_FIX_SYM_LOCK_ON, */
	{0x38,  1,  0},				/* VFD_SET_FIX_SYM_LOCK_OFF, */
	{0x39,  1,  0},				/* VFD_SET_FIX_SYM_REC_ON, */
	{0x3A,  1,  0},				/* VFD_SET_FIX_SYM_REC_OFF, */
	{0x3B,  1,  0},				/* VFD_SET_FIX_SYM_CLOCK_ON, */
	{0x3C,  1,  0},				/* VFD_SET_FIX_SYM_CLOCK_OFF, */
	{0x3D,  1,  0},				/* VFD_SET_FIX_SYM_POWERIO_ON, */
	{0x3E,  1,  0},				/* VFD_SET_FIX_SYM_POWERIO_OFF, */
	{0x3F,  1,  0},				/* VFD_SET_FIX_SYM_USB_ON, */
	{0x40,  1,  0},				/* VFD_SET_FIX_SYM_USB_OFF, */
	{0x41,  1,  0},				/* VFD_SET_FIX_SYM_HDD_ON, */
	{0x42,  1,  0},				/* VFD_SET_FIX_SYM_HDD_OFF, */
	{0x43,  1,  0},				/* VFD_SET_FIX_SYM_HD_ON, */
	{0x44,  1,  0},				/* VFD_SET_FIX_SYM_HD_OFF, */
	{0x45,  1,  0},				/* VFD_SET_FIX_SYM_EURO_ON, */
	{0x46,  1,  0},				/* VFD_SET_FIX_SYM_EURO_OFF, */
	{0x47,  1,  0},				/* VFD_SET_FIX_SYM_NOTES_ON, */
	{0x48,  1,  0},				/* VFD_SET_FIX_SYM_NOTES_OFF, */
	{0x49,  1,  0},				/* VFD_SET_FIX_SYM_TV_ON, */
	{0x4A,  1,  0},				/* VFD_SET_FIX_SYM_TV_OFF, */
	{0x4B,  1,  0},				/* VFD_SET_FIX_SYM_ENVELOPE_ON, */
	{0x4C,  1,  0},				/* VFD_SET_FIX_SYM_ENVELOPE_OFF, */
	{0x4D,  1,  0},				/* VFD_SET_FIX_SYM_MUTEON_ON, */
	{0x4E,  1,  0},				/* VFD_SET_FIX_SYM_MUTEON_OFF, */
	{0x4F,  1,  0},				/* VFD_SET_OUT_STR,, tx_len is set dynamiclly */
	{0x50,  4,  0},				/* RCU_SET_SYC_WRD, */
	{0x00,  0,  3},				/* RCU_EVT_COD */

	/* Submode package */
	{0x21,  1,  1},				/* SUB_UPG_FRM_ENT, */
	{0x00,  1,  1},				/* SUB_UPG_FRM_GET_BUF_SIZ, */
	{0x01,  0,  1},				/* SUB_UPG_FRM_SND_PLD, tx_len is set dynamiclly. */
	{0x02,  1,  0},				/* SUB_UPG_FRM_APL */
	{0x22,  1,  1},				/* SUB_BUS_ENT, */
	{0x00,  1,  1},				/* SUB_BUS_GET_BUF_SIZ, */
	{0x01,  1,  1},				/* SUB_BUS_SET_SLV_VFD, */
	{0x02,  1,  1},				/* SUB_BUS_SET_SLV_RF */
	{0x03,  0,  1},				/* SUB_BUS_WRT_SLV, tx_len is set dynamiclly */
	{0x04,  2,  0},				/* SUB_BUS_RED_SLV, rx_len is set dynamiclly.*/
	{0x05,  1,  0},				/* SUB_BUS_QUT */

	/* new added */
	{0x28,	1,	0},			/* WUP_SET_RST_LST_SRC */
	{0x08,	9,	0},			/* DTH_LET_PST */
	{0X09,	1,	9},			/* DTH_LET_RCV */
};
/* for icon table */
static struct state icon_state_on[] = {
	{"icon:play", VFD_SET_FIX_SYM_PLAY_ON},
	{"icon:pause", VFD_SET_FIX_SYM_PAUSE_ON},
	{"icon:fastforward", VFD_SET_FIX_SYM_FASTFORWARD_ON},
	{"icon:reverse", VFD_SET_FIX_SYM_REVERSE_ON},
	{"icon:lock", VFD_SET_FIX_SYM_LOCK_ON},
	{"icon:Record", VFD_SET_FIX_SYM_REC_ON},
	{"icon:clock", VFD_SET_FIX_SYM_CLOCK_ON},
	{"icon:power", VFD_SET_FIX_SYM_POWERIO_ON},
	{"icon:usb", VFD_SET_FIX_SYM_USB_ON},
	{"icon:HDD", VFD_SET_FIX_SYM_HDD_ON},
	{"icon:HD", VFD_SET_FIX_SYM_HD_ON},
	{"icon:euro", VFD_SET_FIX_SYM_EURO_ON},
	{"icon:notes", VFD_SET_FIX_SYM_NOTES_ON},
	{"icon:TV", VFD_SET_FIX_SYM_TV_ON},
	{"icon:mail", VFD_SET_FIX_SYM_ENVELOPE_ON},
	{"icon:mute", VFD_SET_FIX_SYM_MUTEON_ON},
	{NULL, 0}
};
static struct state icon_state_off[] = {
	{"icon:play", VFD_SET_FIX_SYM_PLAY_OFF},
	{"icon:pause", VFD_SET_FIX_SYM_PAUSE_OFF},
	{"icon:fastforward", VFD_SET_FIX_SYM_FASTFORWARD_OFF},
	{"icon:reverse", VFD_SET_FIX_SYM_REVERSE_OFF},
	{"icon:lock", VFD_SET_FIX_SYM_LOCK_OFF},
	{"icon:Record", VFD_SET_FIX_SYM_REC_OFF},
	{"icon:clock", VFD_SET_FIX_SYM_CLOCK_OFF},
	{"icon:power", VFD_SET_FIX_SYM_POWERIO_OFF},
	{"icon:usb", VFD_SET_FIX_SYM_USB_OFF},
	{"icon:HDD", VFD_SET_FIX_SYM_HDD_OFF},
	{"icon:HD", VFD_SET_FIX_SYM_HD_OFF},
	{"icon:euro", VFD_SET_FIX_SYM_EURO_OFF},
	{"icon:notes", VFD_SET_FIX_SYM_NOTES_OFF},
	{"icon:TV", VFD_SET_FIX_SYM_TV_OFF},
	{"icon:mail", VFD_SET_FIX_SYM_ENVELOPE_OFF},
	{"icon:mute", VFD_SET_FIX_SYM_MUTEON_OFF},
	{NULL, 0}
};
static int get_st_value(struct state *st, char *st_name)
{
	struct state *pt = st;
	int val = -1;

	while (pt->name != NULL) {
		if(!strcmp(st_name, pt->name)) {
			val = pt->val;
			break;
		}
		pt++;
	}

	return val;
}

/* to smcu main block */
struct smcu **smcu_ptr = NULL;

static void init_pkg(pkg_general_t *pkg);
static int smcu_pkg_transfer(pkg_general_t *pkg_gen, smcu_mode_t mode);
static int rcu_handler_register(void *parsing_fun);

static struct mcu_service mcu_dev_callback = {
	.smcu_transfer = smcu_pkg_transfer,
};
/* timer callback */
static struct timer_service timer_dev_callback = {
	.smcu_transfer = smcu_pkg_transfer,
};
/* rcu callback */
static struct rcu_service rcu_dev_callback = {
	.smcu_transfer = smcu_pkg_transfer,
	.rcu_handler_register = rcu_handler_register,
};
/* vfd callback */
static struct vfd_service vfd_dev_callback = {
	.smcu_transfer = smcu_pkg_transfer,
};
static int set_led_brightness(struct mcu_service_dev *dev, int brightness);
static int set_led_flash(struct mcu_service_dev *dev, int mode);
static int get_led_flash(struct mcu_service_dev *dev);

static struct led_service led_dev_callback = {
	.set_brightness		= set_led_brightness,
	.set_flash		= set_led_flash,
	.get_flash		= get_led_flash,
	.set_frequency		= NULL,
	.get_frequency		= NULL,
	.set_duty_cycle		= NULL,
	.get_duty_cycle		= NULL,
};

static int set_icon_brightness(struct mcu_service_dev *dev, int brightness);

static struct led_service icon_dev_callback = {
	.set_brightness		= set_icon_brightness,
	.set_flash		= NULL,
	.get_flash		= NULL,
	.set_frequency		= NULL,
	.get_frequency		= NULL,
	.set_duty_cycle		= NULL,
	.get_duty_cycle		= NULL,
};

int led_dev_registration(struct mcu_dev *dev, char *name)
{
	struct mcu_service_dev *led_dev;
	led_dev = kzalloc(sizeof(struct mcu_service_dev), GFP_KERNEL);
	if (led_dev != NULL) {
		led_dev->name = name;
		led_dev->mcu_dev = dev;
		led_dev->service_type = SERVICE_LED;
		led_dev->service = &led_dev_callback;
		led_dev->priv = NULL;
		return mcu_service_device_register(led_dev);
	}
	return -ENOMEM;
}
/* icon */
int icon_dev_registration(struct mcu_dev *dev, char *name)
{
	struct mcu_service_dev *led_dev;
	led_dev = kzalloc(sizeof(struct mcu_service_dev), GFP_KERNEL);
	if (led_dev != NULL) {
		led_dev->name = name;
		led_dev->mcu_dev = dev;
		led_dev->service_type = SERVICE_LED;
		led_dev->service = &icon_dev_callback;
		led_dev->priv = NULL;
		return mcu_service_device_register(led_dev);
	}
	return -ENOMEM;
}
/* mcu */
static struct mcu_service_dev mcu_dev;
static int mcu_dev_registration(struct mcu_dev *dev){
	mcu_dev.name = NULL;
	mcu_dev.name = "mcu";
	mcu_dev.mcu_dev = dev;
	mcu_dev.service_type = SERVICE_MCU;
	mcu_dev.service = &mcu_dev_callback;
	mcu_dev.priv = NULL;
	return mcu_service_device_register(&mcu_dev);
}
/* timer */
static struct mcu_service_dev timer_dev;
static int timer_dev_registration(struct mcu_dev *dev){
	timer_dev.name = NULL;
	timer_dev.name = "timer";
	timer_dev.mcu_dev = dev;
	timer_dev.service_type = SERVICE_TIMER;
	timer_dev.service = &timer_dev_callback;
	timer_dev.priv = NULL;
	return mcu_service_device_register(&timer_dev);
}
/* rcu */
static struct mcu_service_dev rcu_dev;
static int rcu_dev_registration(struct mcu_dev *dev){
	rcu_dev.name = NULL;
	rcu_dev.name = "RCU";
	rcu_dev.mcu_dev = dev;
	rcu_dev.service_type = SERVICE_RCU;
	rcu_dev.service = &rcu_dev_callback;
	rcu_dev.priv = NULL;
	return mcu_service_device_register(&rcu_dev);
}
/* VFD */
static struct mcu_service_dev vfd_dev;
static int vfd_dev_registration(struct mcu_dev *dev){
	vfd_dev.name = NULL;
	vfd_dev.name = "VFD";
	vfd_dev.mcu_dev = dev;
	vfd_dev.service_type = SERVICE_VFD;
	vfd_dev.service = &vfd_dev_callback;
	vfd_dev.priv = NULL;
	return mcu_service_device_register(&vfd_dev);
}

/* Some definition used by local parsing, it is put here temperally, you should
 * move them to where you want to */
#define REG_MSK_LED_GREEN	0X01
#define REG_MSK_LED_RED		0x02

/*
 *
 */
static int set_led_brightness(struct mcu_service_dev *dev, int brightness)
{
	pkg_general_t pkg;
	char tx_buffer[5], rx_buffer[5];
	smcu_mode_t smcu_mode;
	char led_mask, led_state;
	int ret, res;

	ret = 0;
	smcu_mode = MODE_NORMAL;

	/* Firstly, Check the params */
	if(brightness < 0 || brightness > 255 ) {
		ret = -1;
		goto exit;
	}
	/* Check which led, 2 leds in hardware currently */
	if(!strcmp("led:green", dev->name)) {
		led_mask = REG_MSK_LED_GREEN;
	}else if(!strcmp("led:red", dev->name)) {
		led_mask = REG_MSK_LED_RED;
	}else {
		ret = -1;
		goto exit;
	}
	led_state = brightness ? 0xFF : 0;

	pkg.tx_buf = tx_buffer;
	pkg.rx_buf = rx_buffer;

	tx_buffer[1] = led_mask;
	tx_buffer[2] = led_state;

	res = smcu_pkg_transfer(&pkg, smcu_mode);
	if(res < 0) {
		ret = -1;
		goto exit;
	}
exit:
	return ret;
}

static int set_led_flash(struct mcu_service_dev *dev, int mode)
{
	pkg_general_t pkg;
	char tx_buffer[5], rx_buffer[10];
	smcu_mode_t smcu_mode;
	char led_state, led_mask;
	int ret, res;

	ret = 0;
	smcu_mode = MODE_NORMAL;

	/* Check which led, 2 leds in hardware currently */
	if(!strcmp("led:green", dev->name)) {
		led_mask = REG_MSK_LED_GREEN;
	}else if(!strcmp("led:red", dev->name)) {
		led_mask = REG_MSK_LED_RED;
	}else {
		ret = -1;
		goto exit;
	}
	/* led mode */
	switch(mode) {
	case LED_FLASH_OFF:
		led_state = 0x00;
		pkg.type = LED_SET_GRP_STA;
		tx_buffer[1] = led_mask;
		tx_buffer[2] = led_state;
		break;
	case LED_FLASH_ON:
		led_state = 0xFF;
		pkg.type = LED_SET_GRP_STA;
		tx_buffer[1] = led_mask;
		tx_buffer[2] = led_state;
		break;
	case LED_FLASH_RCU:
		/* Only the red led supports blinking in current hardware */
		if(led_mask == REG_MSK_LED_RED) {
			pkg.type = LED_SET_BI_BLK;
		}else {
			pkg.type = -1;
		}
		break;
	default:
		pkg.type = -1;
		break;
	}
	pkg.tx_buf = tx_buffer;
	pkg.rx_buf = rx_buffer;
	if(pkg.type >= 0) {
		res = smcu_pkg_transfer(&pkg, smcu_mode);
		if(res < 0) {
			ret = -1;
			goto exit;
		}
	}else {
		ret = -1;
		goto exit;
	}

exit:

	return ret;
}

static int get_led_flash(struct mcu_service_dev *dev)
{
	pkg_general_t pkg;
	char tx_buffer[5], rx_buffer[5];
	smcu_mode_t mode;
	char led_mask;
	int ret, res;

	ret = -1;
	mode = MODE_NORMAL;

	/* Check which led, 2 leds in hardware currently */
	if(!strcmp("led:green", dev->name)) {
		led_mask = REG_MSK_LED_GREEN;
	}else if(!strcmp("led:red", dev->name)) {
		led_mask = REG_MSK_LED_RED;
	}else {
		ret = -1;
		goto exit;
	}

	/* Only red led supports blink in current hardware */
	if(led_mask == REG_MSK_LED_RED) {
		spin_lock_bh((*smcu_ptr)->lock);
		if((*smcu_ptr)->flag & (1 << F_LED_IN_BLINK)) {
			/* LED2 is in blinking */
			ret = LED_FLASH_RCU;
			spin_unlock_bh((*smcu_ptr)->lock);
			goto exit;
		}
		spin_unlock_bh((*smcu_ptr)->lock);
	}
	pkg.type = LED_GET_GRP_STA;
	pkg.tx_buf = tx_buffer;
	pkg.rx_buf = rx_buffer;
	res = smcu_pkg_transfer(&pkg, mode);
	if(res < 0) {
		ret = -1;
		goto exit;
	}
	ret = (rx_buffer[1] & led_mask) ? LED_FLASH_ON : LED_FLASH_OFF;

exit:
	return ret;
}

static int set_icon_brightness(struct mcu_service_dev *dev, int brightness)
{
	pkg_general_t pkg;
	char tx_buffer[5], rx_buffer[5];
	smcu_mode_t mode;
	int ret, res;

	mode = MODE_NORMAL;
	if(brightness < 0 || brightness > 255 ) {
		ret = -EINVAL;
		goto exit;
	}
	if(brightness) {
		res = get_st_value(icon_state_on, dev->name);
	}else {
		res = get_st_value(icon_state_off, dev->name);
	}
	pkg.type = res ? res : -1;
	if(pkg.type > 0) {
		pkg.tx_buf = tx_buffer;
		pkg.rx_buf = rx_buffer;
		res = smcu_pkg_transfer(&pkg, mode);
		if(res < 0) {
			ret = -1;
			goto exit;
		}
	}else {
		ret = -EINVAL;
		goto exit;
	}
	/* we are success here */
	ret = 0;

exit:
	return ret;
}

static int check_version(struct mcu_dev *dev)
{
	/* Check the firmware version and the MCU RX fifo size */
	return 0;
}

/*
 * Init the pkg
 */
static void init_pkg(pkg_general_t *pkg)
{
	int index = pkg->type;

	/* for raw package, cmd, tx_len, rx_len initialized by mcu services*/
	if(!index) {
		return;
	}
	pkg->err = 0;
	pkg->rx_ct = 0;
	pkg->tx_buf[0] = params_table[index].cmd;
	pkg->tx_len = params_table[index].tx_lengh;
	pkg->rx_len = params_table[index].rx_lengh;
}

/*
 * It is used to read/write the virtual table, it could be shared by all
 * mcu_service_dev.
 * @ID The reg id in the table, defined in <linux/wymcu/mcu_serv_table.h>.
 * @mode READ or WRITE the reg.
 * @reg The value.
 */
static int smcu_pkg_transfer(pkg_general_t *pkg_gen, smcu_mode_t mode)
{
	int ret;

	ret = -1;
	/* first check the smcu has been attached */
	if(!(*smcu_ptr)) {
		printk(KERN_ERR "The ttySn should be opened with N_MCU before transfer.\n");
		return -1;
	}
	/* We got 2 modes:
	 * MODE_NORMAL
	 * MODE_SUB
	 */
	(*smcu_ptr)->mode = mode;
	init_pkg(pkg_gen);
	/* acquire the semaphore */
	if(down_interruptible(&(*smcu_ptr)->sem)) {
		return -ERESTARTSYS;
	}
	printk(KERN_DEBUG "%s-- type == %d; tx_cmd == %d\n; rx_len = %d\n", __FUNCTION__, \
		pkg_gen->type, pkg_gen->tx_buf[0], pkg_gen->rx_len);
	ret = (*smcu_ptr)->smcu_write((*smcu_ptr), pkg_gen);
	up(&(*smcu_ptr)->sem);

	return ret;
}
static int rcu_handler_register(void *parsing_fun)
{
	int ret;

	ret = -1;
	/* first check the smcu has been attached */
	if(!(*smcu_ptr)) {
	printk(KERN_ERR "The ttySn should be opened with N_MCU firstly.\n");
	return -1;
	}
	ret = (*smcu_ptr)->smcu_rcu_handler_register((*smcu_ptr),
						(int (*)(struct smcu *))parsing_fun);
	return ret;
}
static int mcu_s1_probe(struct mcu_dev *dev)
{
	int ret = 0;

	printk(KERN_DEBUG "driver MCU_s1 instance is starting\n");

	/* devices registration */
	if((ret = check_version(dev)) < 0) {
		goto exit;
	}
	/* get smcu attached */
	smcu_ptr = smcu_attach();

	/* Device registrations */
	mcu_dev_registration(dev);
	timer_dev_registration(dev);
	rcu_dev_registration(dev);
	vfd_dev_registration(dev);

	led_dev_registration(dev, "led:green");
	led_dev_registration(dev, "led:red");
	led_dev_registration(dev, "led:orange");

	icon_dev_registration(dev, "icon:play");
	icon_dev_registration(dev, "icon:pause");
	icon_dev_registration(dev, "icon:fastforward");
	icon_dev_registration(dev, "icon:reverse");
	icon_dev_registration(dev, "icon:lock");
	icon_dev_registration(dev, "icon:Record");
	icon_dev_registration(dev, "icon:clock");
	icon_dev_registration(dev, "icon:power");
	icon_dev_registration(dev, "icon:usb");
	icon_dev_registration(dev, "icon:HDD");
	icon_dev_registration(dev, "icon:HD");
	icon_dev_registration(dev, "icon:euro");
	icon_dev_registration(dev, "icon:notes");
	icon_dev_registration(dev, "icon:TV");
	icon_dev_registration(dev, "icon:mail");
	icon_dev_registration(dev, "icon:mute");

exit:
	return ret;
}

static int mcu_s1_remove(struct mcu_dev *dev)
{
	printk(KERN_INFO "driver MCU_s1 instance is stopping\n");

	mcu_service_device_unregister(&mcu_dev);
	return 0;
}

static struct mcu_driver mcu_s1_drv = {
	.driver = {
		.owner	= THIS_MODULE,
		.name = "mcu_serial",
	},
	.bus_type	= SERIAL,
	.bus_data	= (void *)0x02,
	.probe		= mcu_s1_probe,
	.remove		= mcu_s1_remove,
};

static int __init mcu_serial_init(void)
{
	int ret;

	/* Register the line discipline N_MCU */
	printk(KERN_INFO "Line discipline N_MCU is registering\n");
	if ((ret = tty_register_ldisc(N_MCU, &smcu_ldisc))) {
		printk(KERN_ERR "SMCU: can't register line discipline N_MCU(err = %d)\n",
			ret);
		goto exit;
	}
	/* Register the driver of mcu_serial_1 */
	printk(KERN_INFO "Driver MCU_SERIAL_1 is registering\n");
	if((ret = mcu_driver_register(&mcu_s1_drv))) {
		printk(KERN_ERR "SMCU: can't register driver mcu_serial_1(err = %d)\n",
				ret);
		goto exit_1;
	}

	goto exit;

exit_1:
	printk(KERN_INFO "Driver MCU_SERIAL_1 is unregistring\n");
	mcu_driver_unregister(&mcu_s1_drv);
exit:
	return ret;
}

static void __exit mcu_serial_exit(void)
{
	int i;

	/* Unregister the driver of mcu_serial_1 */
	printk(KERN_DEBUG "Driver MCU_SERIAL_1 is unregistring\n");
	mcu_driver_unregister(&mcu_s1_drv);
	/* Unregister the line discipline N_MCU */
	printk(KERN_DEBUG "Line discipline N_MCU is unregistering\n");
	if ((i = tty_unregister_ldisc(N_MCU)))
		printk(KERN_ERR "SMCU: can't unregister line discipline (err = %d)\n", i);
}

module_init(mcu_serial_init);
module_exit(mcu_serial_exit);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Frederic MAZUEL <fmazuel@wyplay.com>, Jerry Zhang <zhangb@wyplayasia.com");
MODULE_DESCRIPTION("Serial MCU driver");
