/*
 * (C) Copyright 2009 WyPlay SAS.
 * Frederic Mazuel <fmazuel@wyplay.com>
 * Jerry Zhang <zhangb@wyplayasia.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _MCU_SERIAL_H_
#define _MCU_SERIAL_H_

/* MCU version supported by this driver */
#define MCU_V1			0x01
#define MCU_V7			0x07
#define MCU_V10			0x10

/* MCU firmware version supported by this driver */
#define MCU_ESS6461		0x10
#define MCU_RT107x		0x20
#define MCU_ST7109		0x90

/* MCU CNTL0 */
#define MUTE			0x01
#define FAN			0x02
#define LED			0x04
#define MUTE_POS		0x00
#define FAN_POS			0x01
#define LED_POS			0x02

/* HOST status 0 */
#define HDD_ERROR		0x01
#define LAN_STATUS		0x02
#define HDD_ERROR_POS		0x00
#define LAN_STATUS_POS		0x01

/* CPU STATE */
#define C_IDLE			0
#define C_BOOT_UP		1
#define C_PLAYER		2
#define C_SHUT_DOWN_READY	3
#define C_POWER_DOWN		4
#define C_RECORDING		5

/* MCU STATE */
#define M_IDLE			0
#define M_SELF_TEST		1
#define M_SYS_STANDBY		2
#define M_SHUT_DOWN		3
#define M_USB_STANDBY		4
#define M_PLAYER_STANDBY	5
#define M_PLAYER		6
#define M_USB			7

struct state {
	char *name;
	int val;
};

/* local reg value */
#define REG_RST_BOD		0x01
#define REG_RST_DIS		0x01
#define REG_RST_RCU		0x01
#define REG_STA_EVT_ENA		0x01         
#define REG_STA_EVT_DSA		0x00
#define REG_WUP_KEY_COD_CLR	0X00
#define REG_WUP_MSK_MSK_KEY	0x01
#define REG_WUP_MSK_MSK_WOL	0x02
#define REG_WUP_MSK_MSK_RF	0x04
#define REG_WUP_MSK_MSK_TMR	0x08
#define REG_WUP_MSK_MSK_IR	0x10
#define REG_WUP_MSK_MSK_CEC	0x20
#define REG_WUP_SRC_MSK_ALL	0X3F
#define REG_WUP_SRC_KEY		0x01
#define REG_WUP_SRC_WOL		0x02
#define REG_WUP_SRC_RF		0x04
#define REG_WUP_SRC_TMR		0x08
#define REG_WUP_SRC_IR		0x10
#define REG_WUP_SRC_CEC		0x20

/* power reg value*/
#define REG_PWR_OFF		0x00
#define REG_PWR_ON		0x01
/* power reg mask */
#define REG_PWR_MSK_VFD		0x01
#define REG_PWR_MSK_DVT		0x02
#define REG_PWR_MSK_HDD		0x04
#define REG_PWR_MSK_WIF		0x08
#define REG_PWR_MSK_DS2		0x10
#define REG_PWR_MSK_ETH		0x20
#define REG_PWR_MSK_TRC		0x40
#define REG_PWR_MSK_PLC		0x80

#endif /* _MCU_SERIAL_H_ */
