/*
 * (C) Copyright 2009 WyPlay SAS.
 * Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/sysfs.h>
#include <linux/device.h>
#include <linux/wymcu/mcu.h>
#include <linux/wymcu/mcu_service.h>

static dev_t demo_dev_number;
static struct class *mcu_service_demo_class;
static struct class_device *mcu_service_demo_class_dev;

static ssize_t show_demo(struct class_device *cdev, char *buf)
{
	char *pos = buf;

	pos += sprintf(pos, "demo\n");

	return pos - buf;
}

static CLASS_DEVICE_ATTR(demo, S_IRUGO, show_demo, NULL);

static int mcu_service_demo_probe(struct mcu_service_dev *dev)
{
	int ret = 0;

	ret = alloc_chrdev_region (&demo_dev_number, 0, 1, "mcu_service_demo");
	if (ret < 0) {
		printk (KERN_ERR
			"%s: mcu_service_demo can't register device err: 0x%x\n",
			__FUNCTION__, ret);
		goto out;
	}

	mcu_service_demo_class = class_create (THIS_MODULE, "mcu_service_demo");
	mcu_service_demo_class_dev = class_device_create (mcu_service_demo_class,NULL,
							  demo_dev_number, NULL, "demo0");
	ret = class_device_create_file(mcu_service_demo_class_dev, &class_device_attr_demo);
	if (ret) {
		printk( KERN_ERR
			"%s: ERROR: Creating mcu_service_demo demo file: 0x%x)\n",
			__FUNCTION__, ret);
	}
out:
	return ret;
}

static int mcu_service_demo_remove(struct mcu_service_dev *dev)
{
	class_device_remove_file(mcu_service_demo_class_dev, &class_device_attr_demo);
	class_device_destroy(mcu_service_demo_class, demo_dev_number);
	class_destroy(mcu_service_demo_class);
	return 0;
}

static int mcu_service_demo_suspend(struct mcu_service_dev *dev, pm_message_t pm)
{
	return 0;
}

static int mcu_service_demo_resume(struct mcu_service_dev *dev)
{
	return 0;
}

struct mcu_service_driver mcu_service_demo_driver = {
	.driver = {
		.owner	= THIS_MODULE,
		.name	= "mcu_service_demo",
	},
	.service_type = SERVICE_DEMO,
	.probe = mcu_service_demo_probe,
	.remove = mcu_service_demo_remove,
	.suspend = mcu_service_demo_suspend,
	.resume = mcu_service_demo_resume,
};

static int __init mcu_service_demo_init(void)
{
	return mcu_service_driver_register(&mcu_service_demo_driver);
}

static void __exit mcu_service_demo_exit(void)
{
	mcu_service_driver_unregister(&mcu_service_demo_driver);
}

module_init(mcu_service_demo_init);
module_exit(mcu_service_demo_exit);

MODULE_DESCRIPTION("MCU Service Demo");
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>");
