/*
 * (C) Copyright 2009 WyPlay SAS.
 * Frederic Mazuel <fmazuel@wyplay.com>
 * Jerry Zhang <zhangb@wyplayasia.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/device.h>
#include <linux/ctype.h>
#include <linux/sysfs.h>
#include <linux/input.h>
#include <linux/wymcu/mcu_service.h>

static struct rcu_service *services;
static dev_t rcuserv_dev_number;
static struct class *rcuserv_class;
static struct class_device *rcuserv_class_dev;
/* input device */
static struct input_dev *wyir_input_dev, *wyrf_input_dev;

/* rcu receive */
static int rcu_data_parsing(struct smcu *smcu_info)
{
	/* Parsing the data, and report to the input core.
	 * Note, Can not sleep */
	printk(KERN_DEBUG "received RCU data\n");
	return 0;
}

/* local variable for read */
static int rcu_sync_word;

/* RCU. sync word */
ssize_t store_rcu_sync_word(struct class_device *cdev, const char *buf, size_t count)
{
	char status[10];
	int res, value;
	pkg_general_t pkg;
	char tx_buffer[5];
	smcu_mode_t mode = MODE_NORMAL;

	res = sscanf(buf, "%s", status);
	if(res != 1) {
		res = -EINVAL;
		goto exit;
	}
	value = simple_strtoul(status, NULL, 10);
	/* store it to a local variable */
	rcu_sync_word = value;
	if(value < 0) {
		res = -EINVAL;
		goto exit;
	}
	/* call */
	pkg.type = RCU_SET_SYC_WRD;
	pkg.tx_buf = tx_buffer;
	pkg.rx_buf = NULL;
	tx_buffer[3] = value & 0xff;
	tx_buffer[2] = (value >> 8) & 0xff;
	tx_buffer[1] = (value >> 16) & 0xff;
	res = services->smcu_transfer(&pkg, mode);
	if(res < 0) {
		goto exit;
	}
	res = strnlen(buf, count);
exit:
	return res;
}

ssize_t show_rcu_sync_word(struct class_device *cdev, char *buf)
{
	char *pos = buf;

	/* Update the status buffer */
	pos += sprintf(pos, "%d\n", rcu_sync_word);

	return pos - buf;
}

static CLASS_DEVICE_ATTR(rcu_sync_word, S_IRUGO | S_IWUSR,
			 show_rcu_sync_word, store_rcu_sync_word);
/* RCU. register rcu handler to the smcu */
ssize_t store_rcu_handler_status(struct class_device *cdev, const char *buf,
				size_t count)
{
	char status[10];
	int res;

	res = sscanf(buf, "%s", status);
	if(res != 1) {
		res = -EINVAL;
		goto exit;
	}
	if(!strcmp(status, "attach")) {
		res = services->rcu_handler_register((void *)rcu_data_parsing);
		if(res < 0) {
			goto exit;
		}
	}else {
		res = -EINVAL;
		goto exit;
	}
	res = strnlen(buf, count);
exit:
	return res;
}

static CLASS_DEVICE_ATTR(attach_rcu_handler, S_IWUSR,
			 NULL, store_rcu_handler_status);

static int mcu_service_rcu_probe(struct mcu_service_dev *dev)
{
	int ret = 0;

	printk(KERN_DEBUG "RCU service drv instance is starting\n");

	services = (struct rcu_service *)(dev->service);
	if(services == NULL) {
		printk(KERN_WARNING "no rcu services\n");
		goto exit;
	}

	rcuserv_class_dev = class_device_create (rcuserv_class, NULL,
						 rcuserv_dev_number, NULL,
						 "rcuserv0");
	if (IS_ERR(rcuserv_class_dev)) {
		printk( KERN_ERR
			"%s: ERROR: Creating rcuserv0 class device: %ld)\n",
			__FUNCTION__, PTR_ERR(rcuserv_class_dev));
		ret = PTR_ERR(rcuserv_class_dev);
		goto exit;
	}

	ret = class_device_create_file(rcuserv_class_dev,
					&class_device_attr_rcu_sync_word);
	if (ret < 0) {
		printk (KERN_ERR
			"%s: Can't Creating rcu service sync word file (%d)\n",
			__FUNCTION__, ret);
		goto err1;
	}
	ret = class_device_create_file(rcuserv_class_dev,
					&class_device_attr_attach_rcu_handler);
	if (ret < 0) {
		printk (KERN_ERR
			"%s: Can't Creating rcu service attach_rcu_handler file (%d)\n",
			__FUNCTION__, ret);
		goto err2;
	}
	goto exit;
err2:
	class_device_remove_file(rcuserv_class_dev,
				&class_device_attr_rcu_sync_word);
err1:
	class_device_del(rcuserv_class_dev);
exit:
	return ret;
}

static int mcu_service_rcu_remove(struct mcu_service_dev *dev)
{
	printk(KERN_DEBUG "RCU service drv instance is stopping\n");

	class_device_remove_file(rcuserv_class_dev,
				 &class_device_attr_attach_rcu_handler);
	class_device_remove_file(rcuserv_class_dev,
				 &class_device_attr_rcu_sync_word);
	class_device_destroy(rcuserv_class, rcuserv_dev_number);
	return 0;
}

struct mcu_service_driver mcu_service_rcu_driver = {
	.driver = {
		.owner	= THIS_MODULE,
		.name	= "service_RCU",
	},
	.service_type = SERVICE_RCU,
	.probe = mcu_service_rcu_probe,
	.remove = mcu_service_rcu_remove,
};

static int __init rcu_service_init(void)
{
	int ret;

	printk(KERN_INFO "RCU service is registering\n");
	ret = alloc_chrdev_region (&rcuserv_dev_number, 0, 1, "rcuservice");
	if (ret < 0) {
		printk (KERN_ERR
			"%s: rcu service can't register device err: 0x%x\n",
			__FUNCTION__, ret);
		goto exit;
	}

	rcuserv_class = class_create (THIS_MODULE, "rcuserv");
	if (IS_ERR(rcuserv_class)) {
		printk( KERN_ERR
			"%s: ERROR: Creating rcuserv class: %ld)\n",
			__FUNCTION__, PTR_ERR(rcuserv_class));
		ret = PTR_ERR(rcuserv_class);
		goto err1;
	}
	ret = mcu_service_driver_register(&mcu_service_rcu_driver);
	if (ret < 0) {
		printk (KERN_ERR
			"%s: Can't register the MCU service rcu driver: 0x%x\n",
			__FUNCTION__, ret);
		goto err2;
	}
	/* Allocate the input device data structure */
	wyir_input_dev = input_allocate_device();
	if (!wyir_input_dev) {
		printk(KERN_DEBUG "WyIR_rcu: Bad input_alloc_device()\n");
		goto err3;
	}
	wyrf_input_dev = input_allocate_device();
	if (!wyrf_input_dev) {
		printk(KERN_DEBUG "WyRF_rcu: Bad input_alloc_device()\n");
		goto err4;
	}
	/* Announce that the wysrcu will generate
	 * a key event for each button */
	/* For IR RCU*/
	set_bit(EV_KEY, wyir_input_dev->evbit);
	set_bit(BTN_0, wyir_input_dev->keybit);	/* Click button	*/
	set_bit(BTN_1, wyir_input_dev->keybit);	/* Button 1 	*/
	set_bit(BTN_2, wyir_input_dev->keybit);	/* Button 2	*/
	set_bit(BTN_3, wyir_input_dev->keybit);	/* Button 3	*/
	set_bit(BTN_4, wyir_input_dev->keybit);	/* Button 4	*/
	set_bit(KEY_MODE, wyir_input_dev->keybit);	/* Mode		*/
	set_bit(KEY_LEFT, wyir_input_dev->keybit);	/* Left		*/
	set_bit(KEY_RIGHT, wyir_input_dev->keybit);	/* Right	*/
	wyir_input_dev->name = "ir_rcu";

	/* For RF RCU*/
	set_bit(EV_KEY, wyrf_input_dev->evbit);
	set_bit(BTN_0, wyrf_input_dev->keybit);	/* Click button	*/
	set_bit(BTN_1, wyrf_input_dev->keybit);	/* Button 1 	*/
	set_bit(BTN_2, wyrf_input_dev->keybit);	/* Button 2	*/
	set_bit(BTN_3, wyrf_input_dev->keybit);	/* Button 3	*/
	set_bit(BTN_4, wyrf_input_dev->keybit);	/* Button 4	*/
	set_bit(KEY_MODE, wyrf_input_dev->keybit);	/* Mode		*/
	set_bit(KEY_LEFT, wyrf_input_dev->keybit);	/* Left		*/
	set_bit(KEY_RIGHT, wyrf_input_dev->keybit);	/* Right	*/
	wyrf_input_dev->name = "rf_rcu";

	/* Register with the input subsystem */
	input_register_device(wyir_input_dev);
	input_register_device(wyrf_input_dev);
	goto exit;

err4:
	input_unregister_device(wyir_input_dev);
err3:
	mcu_service_driver_unregister(&mcu_service_rcu_driver);
err2:
	class_destroy(rcuserv_class);
err1:
	unregister_chrdev_region(rcuserv_dev_number, 1);
exit:
	return ret;
}

static void __exit rcu_service_exit(void)
{
	input_unregister_device(wyrf_input_dev);
	input_unregister_device(wyir_input_dev);
	printk(KERN_DEBUG "RCU service is unregistering\n");
	mcu_service_driver_unregister(&mcu_service_rcu_driver);
	class_destroy(rcuserv_class);
	unregister_chrdev_region(rcuserv_dev_number, 1);
}

module_init(rcu_service_init);
module_exit(rcu_service_exit);

EXPORT_SYMBOL(rcu_data_parsing);
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("RCU service device support");
MODULE_AUTHOR("Frederic Mazuel <fmazuel@wyplay.com>, \
		Jerry Zhang <zhangb@wyplayasia.com");

