/*
 * (C) Copyright 2009 WyPlay SAS.
 * Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _LINUX_MCU_H_
#define _LINUX_MCU_H_

#include <linux/device.h>

#define MAX_MCU_NB	2

extern struct bus_type mcu_bus_type;

struct mcu_dev;

struct mcu_devs {
	unsigned int nb;
	struct mcu_dev *devs;
};

struct mcu_dev {
	struct device	dev;
	unsigned int	bus_type;
	unsigned int	bus_id;
	void		*bus_data;
	void		*bus_client;
	char		*name;

	void		*priv;
	struct list_head list;
};

#define to_mcu_dev(_d)	container_of((_d), struct mcu_dev, dev)

#define mcu_get_drvdata(d)	dev_get_drvdata(&(d)->dev)
#define mcu_set_drvdata(d,p)	dev_set_drvdata(&(d)->dev, p)

struct mcu_driver {
	struct device_driver	driver;
	unsigned int		bus_type;
	void			*bus_data;
	int (*probe)(struct mcu_dev *);
	int (*remove)(struct mcu_dev *);
	int (*suspend)(struct mcu_dev *, pm_message_t);
	int (*resume)(struct mcu_dev *);
};

#define to_mcu_drv(_d)	container_of((_d), struct mcu_driver, driver)

#define MCU_DRIVER_NAME(_ldev) ((_ldev)->dev.driver->name)

int mcu_driver_register(struct mcu_driver *);
void mcu_driver_unregister(struct mcu_driver *);

enum MCU_BUS_TYPE {
	DEMO,
	I2C,
	SPI,
	USB,
	SERIAL,
	UNKNOWN,
};
#endif /* _LINUX_MCU_H_ */
