/*
 * (C) Copyright 2009 WyPlay SAS.
 * Frederic Mazuel <fmazuel@wyplay.com>
 * Jerry Zhang <zhangb@wyplayasia.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _MCU_LDISC_H_
#define _MCU_LDISC_H_

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/tty.h>
#include <linux/tty_driver.h>
#include <linux/tty_ldisc.h>
#include <linux/spinlock.h>
#include <asm/semaphore.h>
#include <linux/wait.h>

/* message package
 * There are 2 kind of packages
 * @pkg_general_t.
 * @pkg_evt_t.
 */
typedef int smcu_err_t;

#define INITIALIZED_PKG				0
#define	VER_GET					1
#define	LED_SET_GRP_STA				2
#define	LED_GET_GRP_STA				3
#define	LED_SET_1_ON				4
#define	LED_SET_1_OFF				5
#define	LED_SET_2_ON				6
#define	LED_SET_2_OFF				7
#define	LED_SET_BI_BLK				8
#define	LED_SET_BI_OFF				9
#define	LED_SET_BI_RED				10
#define	LED_SET_BI_GRN				11
#define	LED_SET_BI_ORG				12
#define	PWR_SET_GRP_STA				13
#define	PWR_GET_GRP_STA				14
#define	PWR_SET_SOC_OFF				15
#define	PWR_SET_SOC_PWR_CYC			16
#define	PWR_SET_VFD_ON				17
#define	PWR_SET_VFD_OFF				18
#define	PWR_SET_DVT_TUN_ON			19
#define	PWR_SET_DVT_TUN_OFF			20
#define	PWR_SET_HDD_ON				21
#define	PWR_SET_HDD_OFF				22
#define	PWR_SET_WFI_CRD_ON			23	/* Wifi card */
#define	PWR_SET_WFI_CRD_OFF			24
#define	PWR_SET_DS2_NIM_ON			25
#define	PWR_SET_DS2_NIM_OFF			26
#define	PWR_SET_ETH_ON				27
#define	PWR_SET_ETH_OFF				28
#define	PWR_SET_TRC_FUN_ON			29	/* Transcoder function */
#define	PWR_SET_TRC_FUN_OFF			30
#define	PWR_SET_PLC_ON				31
#define	PWR_SET_PLC_OFF				32
#define	PWR_SET_RST_VFD				33
#define	PWR_SET_RST_RF				34
#define	PWR_SET_RST_BOD				35
#define	TMR_SET_ARM				36
#define	TMR_GET_ARM				37
#define	TMR_SET_STP				38
#define	WUP_SET_IPT_SRC_MSK			39
#define	WUP_GET_IPT_SRC_MSK			40
#define	WUP_GET_LST_SRC				41	/* last wake up source */
#define	WUP_SET_ADD_AUT_KEY_COD			42	/* Add authorized keycode to wakeup */
#define	WUP_SET_RST_AUT_KEY_COD			43	/* Cancle all added authorized keycode */
#define	EVT_SET_DIS				44
#define	EVT_SET_ENA				45
#define	EVT_GET_STA				46
#define	VFD_SET_FIX_SYM_CON			47	/* Fixed symbol control */
#define	VFD_SET_FIX_SYM_PLAY_ON			48
#define	VFD_SET_FIX_SYM_PLAY_OFF		49
#define	VFD_SET_FIX_SYM_PAUSE_ON		50
#define	VFD_SET_FIX_SYM_PAUSE_OFF		51
#define	VFD_SET_FIX_SYM_FASTFORWARD_ON		52
#define	VFD_SET_FIX_SYM_FASTFORWARD_OFF		53
#define	VFD_SET_FIX_SYM_REVERSE_ON		54
#define	VFD_SET_FIX_SYM_REVERSE_OFF		55
#define	VFD_SET_FIX_SYM_LOCK_ON			56
#define	VFD_SET_FIX_SYM_LOCK_OFF		57
#define	VFD_SET_FIX_SYM_REC_ON			58
#define	VFD_SET_FIX_SYM_REC_OFF			59
#define	VFD_SET_FIX_SYM_CLOCK_ON		60
#define	VFD_SET_FIX_SYM_CLOCK_OFF		61
#define	VFD_SET_FIX_SYM_POWERIO_ON		62
#define	VFD_SET_FIX_SYM_POWERIO_OFF		63
#define	VFD_SET_FIX_SYM_USB_ON			64
#define	VFD_SET_FIX_SYM_USB_OFF			65
#define	VFD_SET_FIX_SYM_HDD_ON			66
#define	VFD_SET_FIX_SYM_HDD_OFF			67
#define	VFD_SET_FIX_SYM_HD_ON			68
#define	VFD_SET_FIX_SYM_HD_OFF			69
#define	VFD_SET_FIX_SYM_EURO_ON			70
#define	VFD_SET_FIX_SYM_EURO_OFF		71
#define	VFD_SET_FIX_SYM_NOTES_ON		72
#define	VFD_SET_FIX_SYM_NOTES_OFF		73
#define	VFD_SET_FIX_SYM_TV_ON			74
#define	VFD_SET_FIX_SYM_TV_OFF			75
#define	VFD_SET_FIX_SYM_ENVELOPE_ON		76
#define	VFD_SET_FIX_SYM_ENVELOPE_OFF		77
#define	VFD_SET_FIX_SYM_MUTEON_ON		78
#define	VFD_SET_FIX_SYM_MUTEON_OFF		79
#define	VFD_SET_OUT_STR				80
#define	RCU_SET_SYC_WRD				81
#define	RCU_EVT_COD				82

/* Submode package */
#define	SUB_UPG_FRM_ENT				83	/* Upgrade firmware enter */
#define	SUB_UPG_FRM_GET_BUF_SIZ			84
#define	SUB_UPG_FRM_SND_PLD			85		/* Send payload */
#define	SUB_UPG_FRM_APL				86		/* Upgrade apply */
#define	SUB_BUS_ENT				87
#define	SUB_BUS_GET_BUF_SIZ			88
#define	SUB_BUS_SET_SLV_VFD			89
#define	SUB_BUS_SET_SLV_RF			90
#define	SUB_BUS_WRT_SLV				91
#define	SUB_BUS_RED_SLV				92
#define	SUB_BUS_QUT				93

/* new added */
#define WUP_SET_RST_LST_SRC			94
#define DTH_LET_PST				95
#define DTH_LET_RCV				96

/* SMCU */
#define SMCU_MAGIC	(2356)
#define RX_BUF_SIZE	(20)
#define TX_BUF_SIZE	(20)

/* flag definition */
#define F_HOLD_TX		0
#define F_HOLD_RX		1
#define F_ACK			2
#define F_PKG_RX		3
#define F_LED_IN_BLINK		4

/* SMCU state, smcu_info->current_state */
#define STATE_FREE		1
#define STATE_PKG_RXING		2
#define STATE_PARSING		3
/* protocol specific code */
#define CODE_ACK		(0x00)
#define CODE_NACK		(0X01)
#define PKG_RX_BUF_SIZE		(10)
#define PKG_TX_BUF_SIZE		(10)
#define MCU_FIFO_SIZE		(2)
#define MCU_FLASH_PAGE_SIZE	(16)
#define MCU_RAM_BUF_SIZE	(80)

typedef struct
{
	smcu_err_t		err;
	int			type;
	char			*tx_buf;
	int			tx_len;
	char			*rx_buf;
	int			rx_len;
	int			rx_ct;
}pkg_general_t;

typedef struct
{
	smcu_err_t		err;
	int			type;
	unsigned char		rx_buf[5];
	int			rx_len;
	int			rx_ct;
}pkg_asyn_evt_t;

typedef enum
{
	MODE_NORMAL,
	MODE_SUB
}smcu_mode_t;

typedef enum
{
	STA_MONITOR,
	STA_PARSING_GEN,
	STA_PARSING_EVT_RCU
}smcu_rx_state;

struct smcu {
	int		magic;
	spinlock_t	lock;
	spinlock_t	rcu_lock;
	/* used to access the smcu between mcu services */
	struct semaphore	sem;
	smcu_mode_t		mode;
	unsigned int		flag;
	wait_queue_head_t	read_queue_t;
	wait_queue_head_t	write_queue_t;
	smcu_rx_state		current_state;
	/*
	 * These are pointers to the frame buffers.
	 */
	unsigned char		rx_buff[RX_BUF_SIZE];	/* buffer for received serial data */
	int			rx_in_count;
	int			rx_out_count;
	unsigned char		tx_buff[TX_BUF_SIZE];	/* transmitter buffer */
	unsigned char		*tx_head;
	int			tx_left;
	int			rx_errors;
	/* There are pointers to the pkg */
	pkg_general_t		*pkg_general;
	pkg_general_t		*pkg_led_bi_blk;
	pkg_asyn_evt_t		pkg_evt_rcu;
	/* callback, rcu handler */
	int (*rcu_handler)(struct smcu *);
	/*
	 * Some pointers to API Writing/reading the smcu.
	 */
	int (*smcu_write)(struct smcu * smcu_info, pkg_general_t *pkg);
	int (*smcu_flush)(struct smcu * smcu_info);
	int (*smcu_rcu_handler_register)(struct smcu * smcu_info,
					 int (*handler)(struct smcu *));
	/* ptr to TTY structure */
	struct tty_struct *tty;
};

extern struct tty_ldisc smcu_ldisc;
extern struct smcu **smcu_attach(void);
//extern void smcu_detach(struct smcu *smcu_info);

#endif
