/*
 * (C) Copyright 2009 WyPlay SAS.
 * Jean-Christophe PLAGNIOL-VILLARD <jcplagniol@wyplay.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _LINUX_MCU_SERVICE_H_
#define _LINUX_MCU_SERVICE_H_

#include <linux/device.h>
#include <linux/wymcu/mcu_ldisc.h>
extern struct bus_type mcu_service_bus_type;

struct mcu_service_dev {
	struct device	dev;
	char		*name;
	struct mcu_dev	*mcu_dev;
	unsigned int	service_type;
	void		*service;
	void		*priv;
	struct list_head list;
};

#define to_mcu_service_dev(_d)	container_of((_d), struct mcu_service_dev, dev)

#define mcu_service_get_drvdata(d)	dev_get_drvdata(&(d)->dev)
#define mcu_service_set_drvdata(d,p)	dev_set_drvdata(&(d)->dev, p)

struct input_service {
	int (*get_status)(void);
};

struct fan_service {
	void	(*set_frequency)(struct mcu_service_dev *, int);
	int	(*get_frequency)(struct mcu_service_dev *);
	void	(*set_duty_cycle)(struct mcu_service_dev *, int);
	int	(*get_duty_cycle)(struct mcu_service_dev *);
	int	(*set_mode)(struct mcu_service_dev *, int);
	int	(*get_mode)(struct mcu_service_dev *);
};

struct led_service {
	int (*set_brightness)(struct mcu_service_dev *, int);
	int (*set_flash)(struct mcu_service_dev *, int);
	int (*get_flash)(struct mcu_service_dev *);
	void (*set_frequency)(struct mcu_service_dev *, int);
	int (*get_frequency)(struct mcu_service_dev *);
	void (*set_duty_cycle)(struct mcu_service_dev *,int);
	int (*get_duty_cycle)(struct mcu_service_dev *);
};

struct sound_service {
	int (*set_mode)(struct mcu_service_dev *, int);
	int (*get_mode)(struct mcu_service_dev *);
};

struct mcu_service {
	int (*set_cpu_state)(int);
	int (*get_cpu_state)(void);
	int (*get_mcu_state)(void);
	int (*get_mcu_vers)(void);
	int (*get_firm_vers)(void);
	int (*smcu_transfer)(pkg_general_t *, smcu_mode_t);
};
struct timer_service {
	int (*smcu_transfer)(pkg_general_t *, smcu_mode_t);
	int (*get_rtc_state)(void);
};
struct rcu_service {
	int (*smcu_transfer)(pkg_general_t *, smcu_mode_t);
	int (*rcu_handler_register)(void *parsing_fun);
};
struct vfd_service {
	int (*smcu_transfer)(pkg_general_t *, smcu_mode_t);
};
/* for led, lcd, button driver */
struct mcu_service_driver {
	struct device_driver	driver;
	unsigned int		service_type;
	int (*probe)(struct mcu_service_dev *);
	int (*remove)(struct mcu_service_dev *);
	int (*suspend)(struct mcu_service_dev *, pm_message_t);
	int (*resume)(struct mcu_service_dev *);
};

#define to_mcu_service_drv(_d)	container_of((_d), struct mcu_service_driver, driver)

#define MCU_SERVICE_DRIVER_NAME(_ldev) ((_ldev)->dev.driver->name)

int mcu_service_device_register(struct mcu_service_dev*);
int mcu_service_device_unregister(struct mcu_service_dev*);

int mcu_service_driver_register(struct mcu_service_driver *);
void mcu_service_driver_unregister(struct mcu_service_driver *);

enum mcu_service_type {
	SERVICE_DEMO,
	SERVICE_INPUT,
	SERVICE_FAN,
	SERVICE_LED,
	SERVICE_SND,
	SERVICE_TIMER,
	SERVICE_RCU,
	SERVICE_MCU, /* MCU SPECIFIC */
	SERVICE_VFD,
};

#define MAX_MCU_LED	20
enum led_flash {
	LED_FLASH_OFF,
	LED_FLASH_ON,
	LED_FLASH_RCU,
};

enum mcu_snd_mode {
	MUTE_OFF,
	MUTE_ON,
};

struct serv_tag {
	char *name;
	int val;
};

#endif /* _LINUX_MCU_SERVICE_H_ */
