/*******************************************************************************
 *
 * File: Gamma/STRefDisplayFilters.h
 * Copyright (c) 2002 by STMicroelectronics. All rights reserved.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file COPYING in the main directory of this archive for
 * more details.
 * 
 * Description : Contains filter coeffs for Horizontal & vertical format
 *               (HAL video display sddispo2 family) taken from the
 *               ST Reference drivers.
 * 
 ******************************************************************************/

#ifndef _STREF_DISPLAY_FILTER_H
#define _STREF_DISPLAY_FILTER_H


/* Note :                                 */
/* -------------------------------------- */
/* 1    < Zoom < 10     -> use coeffs 'A' */
/*        Zoom = 1      -> use coeffs 'B' */
/* 0.69 < Zoom < 0.99   -> use coeffs 'C' */
/* 0.50 < Zoom < 0.69   -> use coeffs 'D' */
/* 0.33 < Zoom < 0.5    -> use coeffs 'E' */
/* 0.25 < Zoom < 0.33   -> use coeffs 'F' */
/* 0    < Zoom < 0.25   -> use coeffs 'G' */

typedef ULONG U32;

#define HFC_NB_COEFS 35
#define VFC_NB_COEFS 22

/* Filter coeffs available for Horizontal zoom >1 (zoom in) */
/*----------------------------------------------------------*/
static const U32 HorizontalCoefficientsA[HFC_NB_COEFS] =
   {((U32)(-7 & 0xFF)  << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)  << 8) | (-7 & 0xFF),
    ((U32)(-7 & 0xFF)  << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)  << 8) | (-7 & 0xFF),
    ((U32)(-6 & 0xFF)  << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)  << 8) | (-7 & 0xFF),
    ((U32)(-5 & 0xFF)  << 24) | ((U32)(-6 & 0xFF)   << 16) | ((U32)(-6 & 0xFF)  << 8) | (-6 & 0xFF),
    ((U32)(-4 & 0xFF)  << 24) | ((U32)(-4 & 0xFF)   << 16) | ((U32)(-5 & 0xFF)  << 8) | (-5 & 0xFF),
    ((U32)(-1 & 0xFF)  << 24) | ((U32)(-2 & 0xFF)   << 16) | ((U32)(-2 & 0xFF)  << 8) | (-3 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(1 & 0xFF)    << 16) | ((U32)(0 & 0xFF)   << 8) | (0 & 0xFF),
    ((U32)(6 & 0xFF)   << 24) | ((U32)(5 & 0xFF)    << 16) | ((U32)(4 & 0xFF)   << 8) | (3 & 0xFF),
    ((U32)(20 & 0xFF)  << 24) | ((U32)(19 & 0xFF)   << 16) | ((U32)(18 & 0xFF)  << 8) | (18 & 0xFF),
    ((U32)(22 & 0xFF)  << 24) | ((U32)(22 & 0xFF)   << 16) | ((U32)(21 & 0xFF)  << 8) | (21 & 0xFF),
    ((U32)(23 & 0xFF)  << 24) | ((U32)(23 & 0xFF)   << 16) | ((U32)(23 & 0xFF)  << 8) | (23 & 0xFF),
    ((U32)(21 & 0xFF)  << 24) | ((U32)(22 & 0xFF)   << 16) | ((U32)(22 & 0xFF)  << 8) | (23 & 0xFF),
    ((U32)(16 & 0xFF)  << 24) | ((U32)(17 & 0xFF)   << 16) | ((U32)(19 & 0xFF)  << 8) | (20 & 0xFF),
    ((U32)(7 & 0xFF)   << 24) | ((U32)(9 & 0xFF)    << 16) | ((U32)(12 & 0xFF)  << 8) | (14 & 0xFF),
    ((U32)(-6 & 0xFF)  << 24) | ((U32)(-2 & 0xFF)   << 16) | ((U32)(1 & 0xFF)   << 8) | (4 & 0xFF),
    ((U32)(-20 & 0xFF) << 24) | ((U32)(-16 & 0xFF)  << 16) | ((U32)(-13 & 0xFF) << 8) | (-9 & 0xFF),
    ((U32)(-36 & 0xFF) << 24) | ((U32)(-33 & 0xFF)  << 16) | ((U32)(-29 & 0xFF) << 8) | (-25 & 0xFF),
    ((U32)(-50 & 0xFF) << 24) | ((U32)(-46 & 0xFF)  << 16) | ((U32)(-43 & 0xFF) << 8) | (-40 & 0xFF),
    ((U32)(-58 & 0xFF) << 24) | ((U32)(-57 & 0xFF)  << 16) | ((U32)(-55 & 0xFF) << 8) | (-52 & 0xFF),
    ((U32)(-60 & 0xFF) << 24) | ((U32)(-60 & 0xFF)  << 16) | ((U32)(-60 & 0xFF) << 8) | (-59 & 0xFF),
    ((U32)(-51 & 0xFF) << 24) | ((U32)(-55 & 0xFF)  << 16) | ((U32)(-57 & 0xFF) << 8) | (-59 & 0xFF),
    ((U32)(-31 & 0xFF) << 24) | ((U32)(-37 & 0xFF)  << 16) | ((U32)(-43 & 0xFF) << 8) | (-48 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(-8 & 0xFF)   << 16) | ((U32)(-16 & 0xFF) << 8) | (-24 & 0xFF),
    ((U32)(47 & 0xFF)  << 24) | ((U32)(35 & 0xFF)   << 16) | ((U32)(23 & 0xFF)  << 8) | (12 & 0xFF),
    ((U32)(-78 & 0xFF) << 24) | ((U32)(-86 & 0xFF)  << 16) | ((U32)(-93 & 0xFF) << 8) | (-100 & 0xFF),
    ((U32)(-46 & 0xFF) << 24) | ((U32)(-54 & 0xFF)  << 16) | ((U32)(-63 & 0xFF) << 8) | (-70 & 0xFF),
    ((U32)(-12 & 0xFF) << 24) | ((U32)(-21 & 0xFF)  << 16) | ((U32)(-29 & 0xFF) << 8) | (-38 & 0xFF),
    ((U32)(21 & 0xFF)  << 24) | ((U32)(13 & 0xFF)   << 16) | ((U32)(5 & 0xFF)   << 8) | (-4 & 0xFF),
    ((U32)(51 & 0xFF)  << 24) | ((U32)(44 & 0xFF)   << 16) | ((U32)(37 & 0xFF)  << 8) | (29 & 0xFF),
    ((U32)(75 & 0xFF)  << 24) | ((U32)(70 & 0xFF)   << 16) | ((U32)(64 & 0xFF)  << 8) | (58 & 0xFF),
    ((U32)(92 & 0xFF)  << 24) | ((U32)(89 & 0xFF)   << 16) | ((U32)(85 & 0xFF)  << 8) | (80 & 0xFF),
    ((U32)(99 & 0xFF)  << 24) | ((U32)(98 & 0xFF)   << 16) | ((U32)(97 & 0xFF)  << 8) | (95 & 0xFF),
    ((U32)(7 & 0xFF)   << 24) | ((U32)(-24 & 0xFF)  << 16) | ((U32)(60 & 0xFF)  << 8) | (100 & 0xFF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(-7 & 0x3FF)  << 16) | ((U32)(1 & 0x7)    << 11) | ((U32)(0 & 0x1) << 10) | (4 & 0x3FF),
    ((U32)(1 & 0x1)    << 26) | ((U32)(253 & 0x3FF) << 16) | ((U32)(0 & 0x1)    << 10) | (-6 & 0x3FF)};


/* Filter coeffs available for Horizontal zoom 1 (no zoom) */
/*---------------------------------------------------------*/
static const U32 HorizontalCoefficientsB[HFC_NB_COEFS] =
   {((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF) << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0x1)  << 26) | ((U32)(128 & 0x3FF) << 16) | ((U32)(0 & 0x1) << 10) | (0 & 0x3FF)};

/* Filter coeffs available from Horizontal zoom 0.69 to zoom 0.99 (zoom out) */
/*---------------------------------------------------------------------------*/
static const U32 HorizontalCoefficientsC[HFC_NB_COEFS] =
   {((U32)(1 & 0xFF)   << 24) | ((U32)(1 & 0xFF)    << 16) | ((U32)(1 & 0xFF)   << 8) | (1 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(2 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (2 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(2 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (2 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(2 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (2 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(2 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (2 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(2 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (2 & 0xFF),
    ((U32)(0 & 0xFF)   << 24) | ((U32)(1 & 0xFF)    << 16) | ((U32)(1 & 0xFF)   << 8) | (1 & 0xFF),
    ((U32)(-2 & 0xFF)  << 24) | ((U32)(-2 & 0xFF)   << 16) | ((U32)(-1 & 0xFF)  << 8) | (0 & 0xFF),
    ((U32)(15 & 0xFF)  << 24) | ((U32)(16 & 0xFF)   << 16) | ((U32)(17 & 0xFF)  << 8) | (18 & 0xFF),
    ((U32)(10 & 0xFF)  << 24) | ((U32)(12 & 0xFF)   << 16) | ((U32)(13 & 0xFF)  << 8) | (14 & 0xFF),
    ((U32)(5 & 0xFF)   << 24) | ((U32)(6 & 0xFF)    << 16) | ((U32)(8 & 0xFF)   << 8) | (9 & 0xFF),
    ((U32)(-1 & 0xFF)  << 24) | ((U32)(0 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (4 & 0xFF),
    ((U32)(-7 & 0xFF)  << 24) | ((U32)(-6 & 0xFF)   << 16) | ((U32)(-4 & 0xFF)  << 8) | (-3 & 0xFF),
    ((U32)(-13 & 0xFF) << 24) | ((U32)(-11 & 0xFF)  << 16) | ((U32)(-10 & 0xFF) << 8) | (-9 & 0xFF),
    ((U32)(-16 & 0xFF) << 24) | ((U32)(-16 & 0xFF)  << 16) | ((U32)(-15 & 0xFF) << 8) | (-14 & 0xFF),
    ((U32)(-18 & 0xFF) << 24) | ((U32)(-18 & 0xFF)  << 16) | ((U32)(-18 & 0xFF) << 8) | (-17 & 0xFF),
    ((U32)(-79 & 0xFF) << 24) | ((U32)(-80 & 0xFF)  << 16) | ((U32)(-80 & 0xFF) << 8) | (-81 & 0xFF),
    ((U32)(-73 & 0xFF) << 24) | ((U32)(-75 & 0xFF)  << 16) | ((U32)(-76 & 0xFF) << 8) | (-78 & 0xFF),
    ((U32)(-62 & 0xFF) << 24) | ((U32)(-65 & 0xFF)  << 16) | ((U32)(-68 & 0xFF) << 8) | (-70 & 0xFF),
    ((U32)(-45 & 0xFF) << 24) | ((U32)(-50 & 0xFF)  << 16) | ((U32)(-54 & 0xFF) << 8) | (-58 & 0xFF),
    ((U32)(-23 & 0xFF) << 24) | ((U32)(-29 & 0xFF)  << 16) | ((U32)(-35 & 0xFF) << 8) | (-40 & 0xFF),
    ((U32)(4 & 0xFF)   << 24) | ((U32)(-3 & 0xFF)   << 16) | ((U32)(-10 & 0xFF) << 8) | (-17 & 0xFF),
    ((U32)(36 & 0xFF)  << 24) | ((U32)(27 & 0xFF)   << 16) | ((U32)(19 & 0xFF)  << 8) | (12 & 0xFF),
    ((U32)(71 & 0xFF)  << 24) | ((U32)(62 & 0xFF)   << 16) | ((U32)(53 & 0xFF)  << 8) | (44 & 0xFF),
    ((U32)(-78 & 0xFF) << 24) | ((U32)(-88 & 0xFF)  << 16) | ((U32)(-97 & 0xFF) << 8) | (-106 & 0xFF),
    ((U32)(-40 & 0xFF) << 24) | ((U32)(-50 & 0xFF)  << 16) | ((U32)(-59 & 0xFF) << 8) | (-69 & 0xFF),
    ((U32)(-3 & 0xFF)  << 24) | ((U32)(-12 & 0xFF)  << 16) | ((U32)(-21 & 0xFF) << 8) | (-31 & 0xFF),
    ((U32)(31 & 0xFF)  << 24) | ((U32)(23 & 0xFF)   << 16) | ((U32)(14 & 0xFF)  << 8) | (6 & 0xFF),
    ((U32)(60 & 0xFF)  << 24) | ((U32)(53 & 0xFF)   << 16) | ((U32)(46 & 0xFF)  << 8) | (38 & 0xFF),
    ((U32)(82 & 0xFF)  << 24) | ((U32)(77 & 0xFF)   << 16) | ((U32)(72 & 0xFF)  << 8) | (66 & 0xFF),
    ((U32)(98 & 0xFF)  << 24) | ((U32)(95 & 0xFF)   << 16) | ((U32)(91 & 0xFF)  << 8) | (87 & 0xFF),
    ((U32)(104 & 0xFF) << 24) | ((U32)(104 & 0xFF)  << 16) | ((U32)(102 & 0xFF) << 8) | (100 & 0xFF),
    ((U32)(-3 & 0xFF)  << 24) | ((U32)(-18 & 0xFF)  << 16) | ((U32)(81 & 0xFF)  << 8) | (105 & 0xFF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(-19 & 0x3FF) << 16) | ((U32)(1 & 0x7)    << 11)| ((U32)(0 & 0x1) << 10) | (2 & 0x3FF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(231 & 0x3FF) << 16) | ((U32)(0 & 0x1)    << 1) | (44 & 0x3FF)};


/* Filter coeffs available from Horizontal zoom 0.5 to zoom 0.69 (zoom out) */
/*--------------------------------------------------------------------------*/
static const U32 HorizontalCoefficientsD[HFC_NB_COEFS] =
   {((U32)(3 & 0xFF)   << 24) | ((U32)(3 & 0xFF)    << 16) | ((U32)(4 & 0xFF)   << 8) | (4 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(3 & 0xFF)    << 16) | ((U32)(3 & 0xFF)   << 8) | (3 & 0xFF),
    ((U32)(1 & 0xFF)   << 24) | ((U32)(1 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (2 & 0xFF),
    ((U32)(0 & 0xFF)   << 24) | ((U32)(0 & 0xFF)    << 16) | ((U32)(0 & 0xFF)   << 8) | (1 & 0xFF),
    ((U32)(-2 & 0xFF)  << 24) | ((U32)(-1 & 0xFF)   << 16) | ((U32)(-1 & 0xFF)  << 8) | (-1 & 0xFF),
    ((U32)(-3 & 0xFF)  << 24) | ((U32)(-3 & 0xFF)   << 16) | ((U32)(-3 & 0xFF)  << 8) | (-2 & 0xFF),
    ((U32)(-5 & 0xFF)  << 24) | ((U32)(-4 & 0xFF)   << 16) | ((U32)(-4 & 0xFF)  << 8) | (-4 & 0xFF),
    ((U32)(-5 & 0xFF)  << 24) | ((U32)(-5 & 0xFF)   << 16) | ((U32)(-5 & 0xFF)  << 8) | (-5 & 0xFF),
    ((U32)(-33 & 0xFF) << 24) | ((U32)(-33 & 0xFF)  << 16) | ((U32)(-33 & 0xFF) << 8) | (-33 & 0xFF),
    ((U32)(-31 & 0xFF) << 24) | ((U32)(-32 & 0xFF)  << 16) | ((U32)(-32 & 0xFF) << 8) | (-33 & 0xFF),
    ((U32)(-28 & 0xFF) << 24) | ((U32)(-29 & 0xFF)  << 16) | ((U32)(-30 & 0xFF) << 8) | (-30 & 0xFF),
    ((U32)(-22 & 0xFF) << 24) | ((U32)(-24 & 0xFF)  << 16) | ((U32)(-25 & 0xFF) << 8) | (-26 & 0xFF),
    ((U32)(-14 & 0xFF) << 24) | ((U32)(-16 & 0xFF)  << 16) | ((U32)(-18 & 0xFF) << 8) | (-20 & 0xFF),
    ((U32)(-3 & 0xFF)  << 24) | ((U32)(-6 & 0xFF)   << 16) | ((U32)(-9 & 0xFF)  << 8) | (-12 & 0xFF),
    ((U32)(10 & 0xFF)  << 24) | ((U32)(7 & 0xFF)    << 16) | ((U32)(3 & 0xFF)   << 8) | (0 & 0xFF),
    ((U32)(27 & 0xFF)  << 24) | ((U32)(23 & 0xFF)   << 16) | ((U32)(18 & 0xFF)  << 8) | (14 & 0xFF),
    ((U32)(-97 & 0xFF) << 24) | ((U32)(-102 & 0xFF) << 16) | ((U32)(-107 & 0xFF)<< 8) | (-112 & 0xFF),
    ((U32)(-74 & 0xFF) << 24) | ((U32)(-80 & 0xFF)  << 16) | ((U32)(-86 & 0xFF) << 8) | (-92 & 0xFF),
    ((U32)(-49 & 0xFF) << 24) | ((U32)(-55 & 0xFF)  << 16) | ((U32)(-62 & 0xFF) << 8) | (-68 & 0xFF),
    ((U32)(-21 & 0xFF) << 24) | ((U32)(-28 & 0xFF)  << 16) | ((U32)(-35 & 0xFF) << 8) | (-42 & 0xFF),
    ((U32)(9 & 0xFF)   << 24) | ((U32)(1 & 0xFF)    << 16) | ((U32)(-6 & 0xFF)  << 8) | (-13 & 0xFF),
    ((U32)(40 & 0xFF)  << 24) | ((U32)(32 & 0xFF)   << 16) | ((U32)(24 & 0xFF)  << 8) | (17 & 0xFF),
    ((U32)(72 & 0xFF)  << 24) | ((U32)(64 & 0xFF)   << 16) | ((U32)(56 & 0xFF)  << 8) | (48 & 0xFF),
    ((U32)(103 & 0xFF) << 24) | ((U32)(95 & 0xFF)   << 16) | ((U32)(88 & 0xFF)  << 8) | (80 & 0xFF),
    ((U32)(-51 & 0xFF) << 24) | ((U32)(-59 & 0xFF)  << 16) | ((U32)(-66 & 0xFF) << 8) | (-74 & 0xFF),
    ((U32)(-22 & 0xFF) << 24) | ((U32)(-29 & 0xFF)  << 16) | ((U32)(-37 & 0xFF) << 8) | (-44 & 0xFF),
    ((U32)(4 & 0xFF)   << 24) | ((U32)(-3 & 0xFF)   << 16) | ((U32)(-9 & 0xFF)  << 8) | (-16 & 0xFF),
    ((U32)(27 & 0xFF)  << 24) | ((U32)(21 & 0xFF)   << 16) | ((U32)(16 & 0xFF)  << 8) | (10 & 0xFF),
    ((U32)(45 & 0xFF)  << 24) | ((U32)(41 & 0xFF)   << 16) | ((U32)(37 & 0xFF)  << 8) | (32 & 0xFF),
    ((U32)(60 & 0xFF)  << 24) | ((U32)(57 & 0xFF)   << 16) | ((U32)(53 & 0xFF)  << 8) | (49 & 0xFF),
    ((U32)(69 & 0xFF)  << 24) | ((U32)(67 & 0xFF)   << 16) | ((U32)(65 & 0xFF)  << 8) | (62 & 0xFF),
    ((U32)(73 & 0xFF)  << 24) | ((U32)(73 & 0xFF)   << 16) | ((U32)(72 & 0xFF)  << 8) | (71 & 0xFF),
    ((U32)(-5 & 0xFF)  << 24) | ((U32)(32 & 0xFF)   << 16) | ((U32)(111 & 0xFF) << 8) | (73 & 0xFF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(18 & 0x3FF)  << 16) | ((U32)(2 & 0x7)    << 11)| ((U32)(0 & 0x1) << 10) | (-10 & 0x3FF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(347 & 0x3FF) << 16) | ((U32)(0 & 0x1)    << 10)| (162 & 0x3FF)};


/* Filter coeffs available from Horizontal zoom 0.33 to zoom 0.5 (zoom out) */
/*--------------------------------------------------------------------------*/
static const U32 HorizontalCoefficientsE[HFC_NB_COEFS] =
   {((U32)(-7 & 0xFF)  << 24) | ((U32)(-8 & 0xFF)   << 16) | ((U32)(-8 & 0xFF)  << 8) | (-8 & 0xFF),
    ((U32)(-7 & 0xFF)  << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)  << 8) | (-7 & 0xFF),
    ((U32)(-6 & 0xFF)  << 24) | ((U32)(-6 & 0xFF)   << 16) | ((U32)(-6 & 0xFF)  << 8) | (-7 & 0xFF),
    ((U32)(-5 & 0xFF)  << 24) | ((U32)(-5 & 0xFF)   << 16) | ((U32)(-5 & 0xFF)  << 8) | (-6 & 0xFF),
    ((U32)(-3 & 0xFF)  << 24) | ((U32)(-4 & 0xFF)   << 16) | ((U32)(-4 & 0xFF)  << 8) | (-4 & 0xFF),
    ((U32)(-1 & 0xFF)  << 24) | ((U32)(-2 & 0xFF)   << 16) | ((U32)(-2 & 0xFF)  << 8) | (-3 & 0xFF),
    ((U32)(2 & 0xFF)   << 24) | ((U32)(1 & 0xFF)    << 16) | ((U32)(1 & 0xFF)   << 8) | (0 & 0xFF),
    ((U32)(6 & 0xFF)   << 24) | ((U32)(5 & 0xFF)    << 16) | ((U32)(4 & 0xFF)   << 8) | (3 & 0xFF),
    ((U32)(-38 & 0xFF) << 24) | ((U32)(-40 & 0xFF)  << 16) | ((U32)(-41 & 0xFF) << 8) | (-42 & 0xFF),
    ((U32)(-32 & 0xFF) << 24) | ((U32)(-34 & 0xFF)  << 16) | ((U32)(-35 & 0xFF) << 8) | (-37 & 0xFF),
    ((U32)(-24 & 0xFF) << 24) | ((U32)(-26 & 0xFF)  << 16) | ((U32)(-28 & 0xFF) << 8) | (-30 & 0xFF),
    ((U32)(-14 & 0xFF) << 24) | ((U32)(-17 & 0xFF)  << 16) | ((U32)(-19 & 0xFF) << 8) | (-22 & 0xFF),
    ((U32)(-3 & 0xFF)  << 24) | ((U32)(-6 & 0xFF)   << 16) | ((U32)(-9 & 0xFF)  << 8) | (-12 & 0xFF),
    ((U32)(9 & 0xFF)   << 24) | ((U32)(6 & 0xFF)    << 16) | ((U32)(3 & 0xFF)   << 8) | (-1 & 0xFF),
    ((U32)(23 & 0xFF)  << 24) | ((U32)(19 & 0xFF)   << 16) | ((U32)(16 & 0xFF)  << 8) | (12 & 0xFF),
    ((U32)(38 & 0xFF)  << 24) | ((U32)(34 & 0xFF)   << 16) | ((U32)(30 & 0xFF)  << 8) | (27 & 0xFF),
    ((U32)(-61 & 0xFF) << 24) | ((U32)(-65 & 0xFF)  << 16) | ((U32)(-70 & 0xFF) << 8) | (-74 & 0xFF),
    ((U32)(-44 & 0xFF) << 24) | ((U32)(-48 & 0xFF)  << 16) | ((U32)(-52 & 0xFF) << 8) | (-57 & 0xFF),
    ((U32)(-25 & 0xFF) << 24) | ((U32)(-30 & 0xFF)  << 16) | ((U32)(-34 & 0xFF) << 8) | (-39 & 0xFF),
    ((U32)(-6 & 0xFF)  << 24) | ((U32)(-11 & 0xFF)  << 16) | ((U32)(-16 & 0xFF) << 8) | (-20 & 0xFF),
    ((U32)(13 & 0xFF)  << 24) | ((U32)(8 & 0xFF)    << 16) | ((U32)(4 & 0xFF)   << 8) | (-1 & 0xFF),
    ((U32)(32 & 0xFF)  << 24) | ((U32)(28 & 0xFF)   << 16) | ((U32)(23 & 0xFF)  << 8) | (18 & 0xFF),
    ((U32)(51 & 0xFF)  << 24) | ((U32)(47 & 0xFF)   << 16) | ((U32)(42 & 0xFF)  << 8) | (37 & 0xFF),
    ((U32)(69 & 0xFF)  << 24) | ((U32)(65 & 0xFF)   << 16) | ((U32)(60 & 0xFF)  << 8) | (56 & 0xFF),
    ((U32)(-27 & 0xFF) << 24) | ((U32)(-31 & 0xFF)  << 16) | ((U32)(-35 & 0xFF) << 8) | (-39 & 0xFF),
    ((U32)(-11 & 0xFF) << 24) | ((U32)(-15 & 0xFF)  << 16) | ((U32)(-19 & 0xFF) << 8) | (-23 & 0xFF),
    ((U32)(3 & 0xFF)   << 24) | ((U32)(0 & 0xFF)    << 16) | ((U32)(-4 & 0xFF)  << 8) | (-7 & 0xFF),
    ((U32)(15 & 0xFF)  << 24) | ((U32)(12 & 0xFF)   << 16) | ((U32)(9 & 0xFF)   << 8) | (6 & 0xFF),
    ((U32)(25 & 0xFF)  << 24) | ((U32)(22 & 0xFF)   << 16) | ((U32)(20 & 0xFF)  << 8) | (17 & 0xFF),
    ((U32)(32 & 0xFF)  << 24) | ((U32)(30 & 0xFF)   << 16) | ((U32)(29 & 0xFF)  << 8) | (27 & 0xFF),
    ((U32)(37 & 0xFF)  << 24) | ((U32)(36 & 0xFF)   << 16) | ((U32)(35 & 0xFF)  << 8) | (33 & 0xFF),
    ((U32)(39 & 0xFF)  << 24) | ((U32)(38 & 0xFF)   << 16) | ((U32)(38 & 0xFF)  << 8) | (37 & 0xFF),
    ((U32)(8 & 0xFF)   << 24) | ((U32)(42 & 0xFF)   << 16) | ((U32)(74 & 0xFF)  << 8) | (39 & 0xFF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(55 & 0x3FF)  << 16) | ((U32)(2 & 0x7)    << 11)| ((U32)(0 & 0x1) << 10) | (5 & 0x3FF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(284 & 0x3FF) << 16) | ((U32)(0 & 0x1)    << 10)| (171 & 0x3FF)};


/* Filter coeffs available from Horizontal zoom 0.25 to zoom 0.33 (zoom out) */
/*---------------------------------------------------------------------------*/
static const U32 HorizontalCoefficientsF[HFC_NB_COEFS] =
   {((U32)(-11 & 0xFF) << 24) | ((U32)(-11 & 0xFF)  << 16) | ((U32)(-12 & 0xFF) << 8) | (-12 & 0xFF),
    ((U32)(-10 & 0xFF) << 24) | ((U32)(-10 & 0xFF)  << 16) | ((U32)(-11 & 0xFF) << 8) | (-11 & 0xFF),
    ((U32)(-8 & 0xFF)  << 24) | ((U32)(-9 & 0xFF)   << 16) | ((U32)(-9 & 0xFF)  << 8) | (-10 & 0xFF),
    ((U32)(-6 & 0xFF)  << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)  << 8) | (-8 & 0xFF),
    ((U32)(-3 & 0xFF)  << 24) | ((U32)(-4 & 0xFF)   << 16) | ((U32)(-5 & 0xFF)  << 8) | (-6 & 0xFF),
    ((U32)(0 & 0xFF)   << 24) | ((U32)(-1 & 0xFF)   << 16) | ((U32)(-2 & 0xFF)  << 8) | (-3 & 0xFF),
    ((U32)(5 & 0xFF)   << 24) | ((U32)(4 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (1 & 0xFF),
    ((U32)(10 & 0xFF)  << 24) | ((U32)(9 & 0xFF)    << 16) | ((U32)(7 & 0xFF)   << 8) | (6 & 0xFF),
    ((U32)(-38 & 0xFF) << 24) | ((U32)(-40 & 0xFF)  << 16) | ((U32)(-42 & 0xFF) << 8) | (-43 & 0xFF),
    ((U32)(-30 & 0xFF) << 24) | ((U32)(-32 & 0xFF)  << 16) | ((U32)(-34 & 0xFF) << 8) | (-36 & 0xFF),
    ((U32)(-22 & 0xFF) << 24) | ((U32)(-24 & 0xFF)  << 16) | ((U32)(-26 & 0xFF) << 8) | (-28 & 0xFF),
    ((U32)(-12 & 0xFF) << 24) | ((U32)(-14 & 0xFF)  << 16) | ((U32)(-17 & 0xFF) << 8) | (-19 & 0xFF),
    ((U32)(-1 & 0xFF)  << 24) | ((U32)(-4 & 0xFF)   << 16) | ((U32)(-6 & 0xFF)  << 8) | (-9 & 0xFF),
    ((U32)(11 & 0xFF)  << 24) | ((U32)(8 & 0xFF)    << 16) | ((U32)(5 & 0xFF)   << 8) | (2 & 0xFF),
    ((U32)(24 & 0xFF)  << 24) | ((U32)(21 & 0xFF)   << 16) | ((U32)(18 & 0xFF)  << 8) | (14 & 0xFF),
    ((U32)(38 & 0xFF)  << 24) | ((U32)(35 & 0xFF)   << 16) | ((U32)(31 & 0xFF)  << 8) | (28 & 0xFF),
    ((U32)(-48 & 0xFF) << 24) | ((U32)(-52 & 0xFF)  << 16) | ((U32)(-56 & 0xFF) << 8) | (-59 & 0xFF),
    ((U32)(-33 & 0xFF) << 24) | ((U32)(-37 & 0xFF)  << 16) | ((U32)(-41 & 0xFF) << 8) | (-45 & 0xFF),
    ((U32)(-18 & 0xFF) << 24) | ((U32)(-22 & 0xFF)  << 16) | ((U32)(-26 & 0xFF) << 8) | (-30 & 0xFF),
    ((U32)(-3 & 0xFF)  << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-10 & 0xFF) << 8) | (-14 & 0xFF),
    ((U32)(13 & 0xFF)  << 24) | ((U32)(9 & 0xFF)    << 16) | ((U32)(5 & 0xFF)   << 8) | (1 & 0xFF),
    ((U32)(28 & 0xFF)  << 24) | ((U32)(24 & 0xFF)   << 16) | ((U32)(20 & 0xFF)  << 8) | (16 & 0xFF),
    ((U32)(42 & 0xFF)  << 24) | ((U32)(39 & 0xFF)   << 16) | ((U32)(35 & 0xFF)  << 8) | (31 & 0xFF),
    ((U32)(56 & 0xFF)  << 24) | ((U32)(53 & 0xFF)   << 16) | ((U32)(49 & 0xFF)  << 8) | (46 & 0xFF),
    ((U32)(-19 & 0xFF) << 24) | ((U32)(-22 & 0xFF)  << 16) | ((U32)(-25 & 0xFF) << 8) | (-29 & 0xFF),
    ((U32)(-8 & 0xFF)  << 24) | ((U32)(-10 & 0xFF)  << 16) | ((U32)(-13 & 0xFF) << 8) | (-16 & 0xFF),
    ((U32)(3 & 0xFF)   << 24) | ((U32)(0 & 0xFF)    << 16) | ((U32)(-2 & 0xFF)  << 8) | (-5 & 0xFF),
    ((U32)(12 & 0xFF)  << 24) | ((U32)(10 & 0xFF)   << 16) | ((U32)(7 & 0xFF)   << 8) | (5 & 0xFF),
    ((U32)(19 & 0xFF)  << 24) | ((U32)(17 & 0xFF)   << 16) | ((U32)(15 & 0xFF)  << 8) | (13 & 0xFF),
    ((U32)(24 & 0xFF)  << 24) | ((U32)(23 & 0xFF)   << 16) | ((U32)(22 & 0xFF)  << 8) | (20 & 0xFF),
    ((U32)(27 & 0xFF)  << 24) | ((U32)(27 & 0xFF)   << 16) | ((U32)(26 & 0xFF)  << 8) | (25 & 0xFF),
    ((U32)(29 & 0xFF)  << 24) | ((U32)(29 & 0xFF)   << 16) | ((U32)(28 & 0xFF)  << 8) | (28 & 0xFF),
    ((U32)(12 & 0xFF)  << 24) | ((U32)(42 & 0xFF)   << 16) | ((U32)(59 & 0xFF)  << 8) | (29 & 0xFF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(70 & 0x3FF)  << 16) | ((U32)(2 & 0x7)    << 11)| ((U32)(0 & 0x1) << 10) | (15 & 0x3FF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(259 & 0x3FF) << 16) | ((U32)(0 & 0x1)    << 10)| (171 & 0x3FF)};


/* Filter coeffs available until Horizontal zoom 0.25 (zoom out) */
/*---------------------------------------------------------------*/
static const U32 HorizontalCoefficientsG[HFC_NB_COEFS] =
   {((U32)(4 & 0xFF)   << 24) | ((U32)(5 & 0xFF)    << 16) | ((U32)(6 & 0xFF)   << 8) | (8 & 0xFF),
    ((U32)(1 & 0xFF)   << 24) | ((U32)(1 & 0xFF)    << 16) | ((U32)(2 & 0xFF)   << 8) | (3 & 0xFF),
    ((U32)(-2 & 0xFF)  << 24) | ((U32)(-2 & 0xFF)   << 16) | ((U32)(-1 & 0xFF)  << 8) | (0 & 0xFF),
    ((U32)(-4 & 0xFF)  << 24) | ((U32)(-4 & 0xFF)   << 16) | ((U32)(-3 & 0xFF)  << 8) | (-3 & 0xFF),
    ((U32)(-5 & 0xFF)  << 24) | ((U32)(-5 & 0xFF)   << 16) | ((U32)(-5 & 0xFF)  << 8) | (-4 & 0xFF),
    ((U32)(-6 & 0xFF)  << 24) | ((U32)(-6 & 0xFF)   << 16) | ((U32)(-6 & 0xFF)  << 8) | (-6 & 0xFF),
    ((U32)(-7 & 0xFF)  << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)  << 8) | (-7 & 0xFF),
    ((U32)(-8 & 0xFF)  << 24) | ((U32)(-8 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)  << 8) | (-7 & 0xFF),
    ((U32)(30 & 0xFF)  << 24) | ((U32)(34 & 0xFF)   << 16) | ((U32)(38 & 0xFF)  << 8) | (42 & 0xFF),
    ((U32)(26 & 0xFF)  << 24) | ((U32)(19 & 0xFF)   << 16) | ((U32)(23 & 0xFF)  << 8) | (27 & 0xFF),
    ((U32)(3 & 0xFF)   << 24) | ((U32)(6 & 0xFF)    << 16) | ((U32)(9 & 0xFF)   << 8) | (12 & 0xFF),
    ((U32)(-9 & 0xFF)  << 24) | ((U32)(-6 & 0xFF)   << 16) | ((U32)(-3 & 0xFF)  << 8) | (-1 & 0xFF),
    ((U32)(-19 & 0xFF) << 24) | ((U32)(-17 & 0xFF)  << 16) | ((U32)(-14 & 0xFF) << 8) | (-12 & 0xFF),
    ((U32)(-28 & 0xFF) << 24) | ((U32)(-26 & 0xFF)  << 16) | ((U32)(-24 & 0xFF) << 8) | (-22 & 0xFF),
    ((U32)(-35 & 0xFF) << 24) | ((U32)(-34 & 0xFF)  << 16) | ((U32)(-32 & 0xFF) << 8) | (-30 & 0xFF),
    ((U32)(-41 & 0xFF) << 24) | ((U32)(-40 & 0xFF)  << 16) | ((U32)(-38 & 0xFF) << 8) | (-37 & 0xFF),
    ((U32)(60 & 0xFF)  << 24) | ((U32)(65 & 0xFF)   << 16) | ((U32)(69 & 0xFF)  << 8) | (74 & 0xFF),
    ((U32)(42 & 0xFF)  << 24) | ((U32)(47 & 0xFF)   << 16) | ((U32)(51 & 0xFF)  << 8) | (56 & 0xFF),
    ((U32)(23 & 0xFF)  << 24) | ((U32)(28 & 0xFF)   << 16) | ((U32)(32 & 0xFF)  << 8) | (37 & 0xFF),
    ((U32)(4 & 0xFF)   << 24) | ((U32)(8 & 0xFF)    << 16) | ((U32)(13 & 0xFF)  << 8) | (18 & 0xFF),
    ((U32)(-16 & 0xFF) << 24) | ((U32)(-11 & 0xFF)  << 16) | ((U32)(-6 & 0xFF)  << 8) | (-1 & 0xFF),
    ((U32)(-34 & 0xFF) << 24) | ((U32)(-30 & 0xFF)  << 16) | ((U32)(-25 & 0xFF) << 8) | (-20 & 0xFF),
    ((U32)(-52 & 0xFF) << 24) | ((U32)(-48 & 0xFF)  << 16) | ((U32)(-44 & 0xFF) << 8) | (-39 & 0xFF),
    ((U32)(-70 & 0xFF) << 24) | ((U32)(-65 & 0xFF)  << 16) | ((U32)(-61 & 0xFF) << 8) | (-57 & 0xFF),
    ((U32)(38 & 0xFF)  << 24) | ((U32)(38 & 0xFF)   << 16) | ((U32)(39 & 0xFF)  << 8) | (39 & 0xFF),
    ((U32)(35 & 0xFF)  << 24) | ((U32)(36 & 0xFF)   << 16) | ((U32)(37 & 0xFF)  << 8) | (37 & 0xFF),
    ((U32)(29 & 0xFF)  << 24) | ((U32)(30 & 0xFF)   << 16) | ((U32)(32 & 0xFF)  << 8) | (33 & 0xFF),
    ((U32)(20 & 0xFF)  << 24) | ((U32)(22 & 0xFF)   << 16) | ((U32)(25 & 0xFF)  << 8) | (27 & 0xFF),
    ((U32)(9 & 0xFF)   << 24) | ((U32)(12 & 0xFF)   << 16) | ((U32)(15 & 0xFF)  << 8) | (17 & 0xFF),
    ((U32)(-4 & 0xFF)  << 24) | ((U32)(0 & 0xFF)    << 16) | ((U32)(3 & 0xFF)   << 8) | (6 & 0xFF),
    ((U32)(-19 & 0xFF) << 24) | ((U32)(-15 & 0xFF)  << 16) | ((U32)(-11 & 0xFF) << 8) | (-7 & 0xFF),
    ((U32)(-35 & 0xFF) << 24) | ((U32)(-31 & 0xFF)  << 16) | ((U32)(-27 & 0xFF) << 8) | (-23 & 0xFF),
    ((U32)(-8 & 0xFF)  << 24) | ((U32)(-42 & 0xFF)  << 16) | ((U32)(-74 & 0xFF) << 8) | (-39 & 0xFF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(55 & 0x3FF)  << 16) | ((U32)(1 & 0x7)    << 11)| ((U32)(0 & 0x1) << 10) | (5 & 0x3FF),
    ((U32)(0 & 0x1)    << 26) | ((U32)(284 & 0x3FF) << 16) | ((U32)(0 & 0x1)    << 10)| (171 & 0x3FF)};



/* Filter coeffs available for Vertical zoom >1 (zoom in) */
/*--------------------------------------------------------*/
static const U32 VerticalCoefficientsA[VFC_NB_COEFS] =
   {((U32)(18 & 0xFF)   << 24) | ((U32)(18 & 0xFF)  << 16) | ((U32)(19 & 0xFF)   << 8) | (19 & 0xFF),
    ((U32)(16 & 0xFF)   << 24) | ((U32)(16 & 0xFF)  << 16) | ((U32)(17 & 0xFF)   << 8) | (17 & 0xFF),
    ((U32)(13 & 0xFF)   << 24) | ((U32)(14 & 0xFF)  << 16) | ((U32)(15 & 0xFF)   << 8) | (15 & 0xFF),
    ((U32)(8 & 0xFF)    << 24) | ((U32)(9 & 0xFF)   << 16) | ((U32)(11 & 0xFF)   << 8) | (12 & 0xFF),
    ((U32)(1 & 0xFF)    << 24) | ((U32)(3 & 0xFF)   << 16) | ((U32)(5 & 0xFF)    << 8) | (6 & 0xFF),
    ((U32)(-7 & 0xFF)   << 24) | ((U32)(-5 & 0xFF)  << 16) | ((U32)(-3 & 0xFF)   << 8) | (-1 & 0xFF),
    ((U32)(-14 & 0xFF)  << 24) | ((U32)(-13 & 0xFF) << 16) | ((U32)(-11 & 0xFF)  << 8) | (-9 & 0xFF),
    ((U32)(-19 & 0xFF)  << 24) | ((U32)(-19 & 0xFF) << 16) | ((U32)(-17 & 0xFF)  << 8) | (-16 & 0xFF),
    ((U32)(-81 & 0xFF)  << 24) | ((U32)(-82 & 0xFF) << 16) | ((U32)(-82 & 0xFF)  << 8) | (-82 & 0xFF),
    ((U32)(-77 & 0xFF)  << 24) | ((U32)(-78 & 0xFF) << 16) | ((U32)(-80 & 0xFF)  << 8) | (-81 & 0xFF),
    ((U32)(-66 & 0xFF)  << 24) | ((U32)(-69 & 0xFF) << 16) | ((U32)(-72 & 0xFF)  << 8) | (-74 & 0xFF),
    ((U32)(-48 & 0xFF)  << 24) | ((U32)(-53 & 0xFF) << 16) | ((U32)(-58 & 0xFF)  << 8) | (-62 & 0xFF),
    ((U32)(-24 & 0xFF)  << 24) | ((U32)(-31 & 0xFF) << 16) | ((U32)(-37 & 0xFF)  << 8) | (-43 & 0xFF),
    ((U32)(6 & 0xFF)    << 24) | ((U32)(-2 & 0xFF)  << 16) | ((U32)(-10 & 0xFF)  << 8) | (-17 & 0xFF),
    ((U32)(39 & 0xFF)   << 24) | ((U32)(31 & 0xFF)  << 16) | ((U32)(22 & 0xFF)   << 8) | (14 & 0xFF),
    ((U32)(74 & 0xFF)   << 24) | ((U32)(66 & 0xFF)  << 16) | ((U32)(57 & 0xFF)   << 8) | (48 & 0xFF),
    ((U32)(-31 & 0xFF)  << 24) | ((U32)(-47 & 0xFF) << 16) | ((U32)(-63 & 0xFF)  << 8) | (-79 & 0xFF),
    ((U32)(23 & 0xFF)   << 24) | ((U32)(10 & 0xFF)  << 16) | ((U32)(-3 & 0xFF)   << 8) | (-17 & 0xFF),
    ((U32)(60 & 0xFF)   << 24) | ((U32)(53 & 0xFF)  << 16) | ((U32)(44 & 0xFF)   << 8) | (34 & 0xFF),
    ((U32)(77 & 0xFF)   << 24) | ((U32)(75 & 0xFF)  << 16) | ((U32)(71 & 0xFF)   << 8) | (66 & 0xFF),
    ((U32)(1 & 0x3)     << 25) | ((U32)(0 & 0x1)    << 24) | ((U32)(-20 & 0xFF)  << 16)| ((U32)(83 & 0xFF) << 8) | (78 & 0xFF),
    ((U32)(372 & 0x3FF) << 22) | ((U32)(1 & 0x1)    << 21) | ((U32)(128 & 0x3FF) << 11)| ((U32)(0 & 0x1) << 10) | (-16 & 0x3FF)};


/* Filter coeffs available for Vertical zoom 1 (no zoom) */
/*-------------------------------------------------------*/
static const U32 VerticalCoefficientsB[VFC_NB_COEFS] =
   {((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(0 & 0xFF) << 16) | ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(0 & 0x3)     << 25) | ((U32)(0 & 0x1)  << 24) | ((U32)(0 & 0xFF) << 16)| ((U32)(0 & 0xFF) << 8) | (0 & 0xFF),
    ((U32)(256 & 0x3FF) << 22) | ((U32)(0 & 0x1)  << 21) | ((U32)(0 & 0x3FF)<< 11)| ((U32)(0 & 0x1) << 10) | (0 & 0x3FF)};

/* Filter coeffs available for Vertical zoom 0.69 to 0.99 (zoom out) */
/*-------------------------------------------------------------------*/
static const U32 VerticalCoefficientsC[VFC_NB_COEFS] =
   {((U32)(-1 & 0xFF)   << 24) | ((U32)(0 & 0xFF)    << 16) | ((U32)(0 & 0xFF)    << 8) | (0 & 0xFF),
    ((U32)(-2 & 0xFF)   << 24) | ((U32)(-2 & 0xFF)   << 16) | ((U32)(-1 & 0xFF)   << 8) | (-1 & 0xFF),
    ((U32)(-4 & 0xFF)   << 24) | ((U32)(-4 & 0xFF)   << 16) | ((U32)(-3 & 0xFF)   << 8) | (-3 & 0xFF),
    ((U32)(-6 & 0xFF)   << 24) | ((U32)(-6 & 0xFF)   << 16) | ((U32)(-5 & 0xFF)   << 8) | (-5 & 0xFF),
    ((U32)(-7 & 0xFF)   << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)   << 8) | (-7 & 0xFF),
    ((U32)(-7 & 0xFF)   << 24) | ((U32)(-7 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)   << 8) | (-7 & 0xFF),
    ((U32)(-3 & 0xFF)   << 24) | ((U32)(-5 & 0xFF)   << 16) | ((U32)(-6 & 0xFF)   << 8) | (-6 & 0xFF),
    ((U32)(4 & 0xFF)    << 24) | ((U32)(2 & 0xFF)    << 16) | ((U32)(0 & 0xFF)    << 8) | (-2 & 0xFF),
    ((U32)(-116 & 0xFF) << 24) | ((U32)(-120 & 0xFF) << 16) | ((U32)(-124 & 0xFF) << 8) | (-127 & 0xFF),
    ((U32)(-97 & 0xFF)  << 24) | ((U32)(-102 & 0xFF) << 16) | ((U32)(-107 & 0xFF) << 8) | (-112 & 0xFF),
    ((U32)(-71 & 0xFF)  << 24) | ((U32)(-78 & 0xFF)  << 16) | ((U32)(-85 & 0xFF)  << 8) | (-91 & 0xFF),
    ((U32)(-39 & 0xFF)  << 24) | ((U32)(-48 & 0xFF)  << 16) | ((U32)(-56 & 0xFF)  << 8) | (-64 & 0xFF),
    ((U32)(-2 & 0xFF)   << 24) | ((U32)(-12 & 0xFF)  << 16) | ((U32)(-21 & 0xFF)  << 8) | (-30 & 0xFF),
    ((U32)(38 & 0xFF)   << 24) | ((U32)(28 & 0xFF)   << 16) | ((U32)(18 & 0xFF)   << 8) | (8 & 0xFF),
    ((U32)(79 & 0xFF)   << 24) | ((U32)(69 & 0xFF)   << 16) | ((U32)(58 & 0xFF)   << 8) | (48 & 0xFF),
    ((U32)(118 & 0xFF)  << 24) | ((U32)(109 & 0xFF)  << 16) | ((U32)(99 & 0xFF)   << 8) | (89 & 0xFF),
    ((U32)(-15 & 0xFF)  << 24) | ((U32)(-23 & 0xFF)  << 16) | ((U32)(-32 & 0xFF)  << 8) | (-41 & 0xFF),
    ((U32)(13 & 0xFF)   << 24) | ((U32)(7 & 0xFF)    << 16) | ((U32)(0 & 0xFF)    << 8) | (-7 & 0xFF),
    ((U32)(32 & 0xFF)   << 24) | ((U32)(28 & 0xFF)   << 16) | ((U32)(24 & 0xFF)   << 8) | (19 & 0xFF),
    ((U32)(40 & 0xFF)   << 24) | ((U32)(39 & 0xFF)   << 16) | ((U32)(38 & 0xFF)   << 8) | (35 & 0xFF),
    ((U32)(1 & 0x3)     << 25) | ((U32)(0 & 0x1)     << 24) | ((U32)(7 & 0xFF)    << 16)| ((U32)(127 & 0xFF) << 8)  | (41 & 0xFF),
    ((U32)(297 & 0x3FF) << 22) | ((U32)(0 & 0x1)     << 21) | ((U32)(129 & 0x3FF) << 11)| ((U32)(0 & 0x1)    << 10) | (-5 & 0x3FF)};

/* Filter coeffs available for Vertical zoom 0.5 to 0.69 (zoom out) */
/*------------------------------------------------------------------*/
static const U32 VerticalCoefficientsD[VFC_NB_COEFS] =
   {((U32)(-47 & 0xFF)  << 24) | ((U32)(-48 & 0xFF) << 16) | ((U32)(-48 & 0xFF)  << 8) | (-49 & 0xFF),
    ((U32)(-44 & 0xFF)  << 24) | ((U32)(-45 & 0xFF) << 16) | ((U32)(-46 & 0xFF)  << 8) | (-46 & 0xFF),
    ((U32)(-40 & 0xFF)  << 24) | ((U32)(-41 & 0xFF) << 16) | ((U32)(-42 & 0xFF)  << 8) | (-43 & 0xFF),
    ((U32)(-32 & 0xFF)  << 24) | ((U32)(-34 & 0xFF) << 16) | ((U32)(-36 & 0xFF)  << 8) | (-38 & 0xFF),
    ((U32)(-21 & 0xFF)  << 24) | ((U32)(-24 & 0xFF) << 16) | ((U32)(-27 & 0xFF)  << 8) | (-30 & 0xFF),
    ((U32)(-6 & 0xFF)   << 24) | ((U32)(-10 & 0xFF) << 16) | ((U32)(-14 & 0xFF)  << 8) | (-18 & 0xFF),
    ((U32)(15 & 0xFF)   << 24) | ((U32)(9 & 0xFF)   << 16) | ((U32)(4 & 0xFF)    << 8) | (-1 & 0xFF),
    ((U32)(41 & 0xFF)   << 24) | ((U32)(34 & 0xFF)  << 16) | ((U32)(27 & 0xFF)   << 8) | (21 & 0xFF),
    ((U32)(-69 & 0xFF)  << 24) | ((U32)(-73 & 0xFF) << 16) | ((U32)(-77 & 0xFF)  << 8) | (-81 & 0xFF),
    ((U32)(-50 & 0xFF)  << 24) | ((U32)(-55 & 0xFF) << 16) | ((U32)(-60 & 0xFF)  << 8) | (-64 & 0xFF),
    ((U32)(-30 & 0xFF)  << 24) | ((U32)(-35 & 0xFF) << 16) | ((U32)(-40 & 0xFF)  << 8) | (-45 & 0xFF),
    ((U32)(-9 & 0xFF)   << 24) | ((U32)(-14 & 0xFF) << 16) | ((U32)(-19 & 0xFF)  << 8) | (-25 & 0xFF),
    ((U32)(14 & 0xFF)   << 24) | ((U32)(9 & 0xFF)   << 16) | ((U32)(3 & 0xFF)    << 8) | (-3 & 0xFF),
    ((U32)(36 & 0xFF)   << 24) | ((U32)(31 & 0xFF)  << 16) | ((U32)(25 & 0xFF)   << 8) | (20 & 0xFF),
    ((U32)(57 & 0xFF)   << 24) | ((U32)(52 & 0xFF)  << 16) | ((U32)(47 & 0xFF)   << 8) | (42 & 0xFF),
    ((U32)(76 & 0xFF)   << 24) | ((U32)(72 & 0xFF)  << 16) | ((U32)(67 & 0xFF)   << 8) | (62 & 0xFF),
    ((U32)(-13 & 0xFF)  << 24) | ((U32)(-21 & 0xFF) << 16) | ((U32)(-29 & 0xFF)  << 8) | (-37 & 0xFF),
    ((U32)(12 & 0xFF)   << 24) | ((U32)(6 & 0xFF)   << 16) | ((U32)(0 & 0xFF)    << 8) | (-6 & 0xFF),
    ((U32)(28 & 0xFF)   << 24) | ((U32)(25 & 0xFF)  << 16) | ((U32)(21 & 0xFF)   << 8) | (17 & 0xFF),
    ((U32)(36 & 0xFF)   << 24) | ((U32)(35 & 0xFF)  << 16) | ((U32)(33 & 0xFF)   << 8) | (31 & 0xFF),
    ((U32)(2 & 0x3)     << 25) | ((U32)(0 & 0x1)    << 24) | ((U32)(48 & 0xFF)   << 16)| ((U32)(81 & 0xFF) << 8) | (36 & 0xFF),
    ((U32)(455 & 0x3FF) << 22) | ((U32)(1 & 0x1)    << 21) | ((U32)(257 & 0x3FF) << 11)| ((U32)(0 & 0x1) << 10)  | (48 & 0x3FF)};

/* Filter coeffs available for Vertical zoom 0.33 to 0.5 (zoom out) */
/*------------------------------------------------------------------*/
static const U32 VerticalCoefficientsE[VFC_NB_COEFS] =
   {((U32)(-55 & 0xFF)  << 24) | ((U32)(-56 & 0xFF) << 16) | ((U32)(-57 & 0xFF)  << 8) | (-57 & 0xFF),
    ((U32)(-51 & 0xFF)  << 24) | ((U32)(-52 & 0xFF) << 16) | ((U32)(-54 & 0xFF)  << 8) | (-55 & 0xFF),
    ((U32)(-44 & 0xFF)  << 24) | ((U32)(-46 & 0xFF) << 16) | ((U32)(-48 & 0xFF)  << 8) | (-49 & 0xFF),
    ((U32)(-33 & 0xFF)  << 24) | ((U32)(-36 & 0xFF) << 16) | ((U32)(-39 & 0xFF)  << 8) | (-41 & 0xFF),
    ((U32)(-18 & 0xFF)  << 24) | ((U32)(-22 & 0xFF) << 16) | ((U32)(-26 & 0xFF)  << 8) | (-29 & 0xFF),
    ((U32)(0 & 0xFF)    << 24) | ((U32)(-5 & 0xFF)  << 16) | ((U32)(-9 & 0xFF)   << 8) | (-14 & 0xFF),
    ((U32)(23 & 0xFF)   << 24) | ((U32)(17 & 0xFF)  << 16) | ((U32)(11 & 0xFF)   << 8) | (6 & 0xFF),
    ((U32)(49 & 0xFF)   << 24) | ((U32)(42 & 0xFF)  << 16) | ((U32)(36 & 0xFF)   << 8) | (29 & 0xFF),
    ((U32)(-52 & 0xFF)  << 24) | ((U32)(-56 & 0xFF) << 16) | ((U32)(-60 & 0xFF)  << 8) | (-64 & 0xFF),
    ((U32)(-36 & 0xFF)  << 24) | ((U32)(-40 & 0xFF) << 16) | ((U32)(-44 & 0xFF)  << 8) | (-48 & 0xFF),
    ((U32)(-19 & 0xFF)  << 24) | ((U32)(-24 & 0xFF) << 16) | ((U32)(-28 & 0xFF)  << 8) | (-32 & 0xFF),
    ((U32)(-2 & 0xFF)   << 24) | ((U32)(-6 & 0xFF)  << 16) | ((U32)(-11 & 0xFF)  << 8) | (-15 & 0xFF),
    ((U32)(16 & 0xFF)   << 24) | ((U32)(12 & 0xFF)  << 16) | ((U32)(7 & 0xFF)    << 8) | (3 & 0xFF),
    ((U32)(32 & 0xFF)   << 24) | ((U32)(28 & 0xFF)  << 16) | ((U32)(24 & 0xFF)   << 8) | (20 & 0xFF),
    ((U32)(48 & 0xFF)   << 24) | ((U32)(44 & 0xFF)  << 16) | ((U32)(40 & 0xFF)   << 8) | (36 & 0xFF),
    ((U32)(62 & 0xFF)   << 24) | ((U32)(58 & 0xFF)  << 16) | ((U32)(55 & 0xFF)   << 8) | (51 & 0xFF),
    ((U32)(-9 & 0xFF)   << 24) | ((U32)(-14 & 0xFF) << 16) | ((U32)(-19 & 0xFF)  << 8) | (-25 & 0xFF),
    ((U32)(9 & 0xFF)    << 24) | ((U32)(5 & 0xFF)   << 16) | ((U32)(1 & 0xFF)    << 8) | (-4 & 0xFF),
    ((U32)(20 & 0xFF)   << 24) | ((U32)(18 & 0xFF)  << 16) | ((U32)(15 & 0xFF)   << 8) | (12 & 0xFF),
    ((U32)(25 & 0xFF)   << 24) | ((U32)(24 & 0xFF)  << 16) | ((U32)(23 & 0xFF)   << 8) | (22 & 0xFF),
    ((U32)(2 & 0x3)     << 25) | ((U32)(0 & 0x1)    << 24) | ((U32)(56 & 0xFF)   << 16)| ((U32)(65 & 0xFF) << 8) | (25 & 0xFF),
    ((U32)(407 & 0x3FF) << 22) | ((U32)(1 & 0x1)    << 21) | ((U32)(253 & 0x3FF) << 11)| ((U32)(0 & 0x1) << 10)  | (70 & 0x3FF)};


/* Filter coeffs available for Vertical zoom 0.02 to 0.33 (zoom out) */
/*-------------------------------------------------------------------*/
static const U32 VerticalCoefficientsF[VFC_NB_COEFS] =
   {((U32)(-57 & 0xFF)  << 24) | ((U32)(-58 & 0xFF)  << 16) | ((U32)(-58 & 0xFF)  << 8) | (-59 & 0xFF),
    ((U32)(-52 & 0xFF)  << 24) | ((U32)(-54 & 0xFF)  << 16) | ((U32)(-55 & 0xFF)  << 8) | (-56 & 0xFF),
    ((U32)(-44 & 0xFF)  << 24) | ((U32)(-46 & 0xFF)  << 16) | ((U32)(-49 & 0xFF)  << 8) | (-51 & 0xFF),
    ((U32)(-32 & 0xFF)  << 24) | ((U32)(-35 & 0xFF)  << 16) | ((U32)(-38 & 0xFF)  << 8) | (-41 & 0xFF),
    ((U32)(-16 & 0xFF)  << 24) | ((U32)(-21 & 0xFF)  << 16) | ((U32)(-25 & 0xFF)  << 8) | (-28 & 0xFF),
    ((U32)(3 & 0xFF)    << 24) | ((U32)(-2 & 0xFF)   << 16) | ((U32)(-7 & 0xFF)   << 8) | (-12 & 0xFF),
    ((U32)(26 & 0xFF)   << 24) | ((U32)(20 & 0xFF)   << 16) | ((U32)(14 & 0xFF)   << 8) | (8 & 0xFF),
    ((U32)(52 & 0xFF)   << 24) | ((U32)(45 & 0xFF)   << 16) | ((U32)(39 & 0xFF)   << 8) | (32 & 0xFF),
    ((U32)(-95 & 0xFF)  << 24) | ((U32)(-102 & 0xFF) << 16) | ((U32)(-109 & 0xFF) << 8) | (-116 & 0xFF),
    ((U32)(-64 & 0xFF)  << 24) | ((U32)(-72 & 0xFF)  << 16) | ((U32)(-80 & 0xFF)  << 8) | (-87 & 0xFF),
    ((U32)(-33 & 0xFF)  << 24) | ((U32)(-41 & 0xFF)  << 16) | ((U32)(-49 & 0xFF)  << 8) | (-57 & 0xFF),
    ((U32)(-1 & 0xFF)   << 24) | ((U32)(-9 & 0xFF)   << 16) | ((U32)(-17 & 0xFF)  << 8) | (-25 & 0xFF),
    ((U32)(30 & 0xFF)   << 24) | ((U32)(22 & 0xFF)   << 16) | ((U32)(14 & 0xFF)   << 8) | (7 & 0xFF),
    ((U32)(59 & 0xFF)   << 24) | ((U32)(52 & 0xFF)   << 16) | ((U32)(45 & 0xFF)   << 8) | (37 & 0xFF),
    ((U32)(86 & 0xFF)   << 24) | ((U32)(80 & 0xFF)   << 16) | ((U32)(73 & 0xFF)   << 8) | (66 & 0xFF),
    ((U32)(110 & 0xFF)  << 24) | ((U32)(104 & 0xFF)  << 16) | ((U32)(98 & 0xFF)   << 8) | (92 & 0xFF),
    ((U32)(-8 & 0xFF)   << 24) | ((U32)(-12 & 0xFF)  << 16) | ((U32)(-17 & 0xFF)  << 8) | (-22 & 0xFF),
    ((U32)(7 & 0xFF)    << 24) | ((U32)(4 & 0xFF)    << 16) | ((U32)(0 & 0xFF)    << 8) | (-3 & 0xFF),
    ((U32)(17 & 0xFF)   << 24) | ((U32)(15 & 0xFF)   << 16) | ((U32)(13 & 0xFF)   << 8) | (10 & 0xFF),
    ((U32)(21 & 0xFF)   << 24) | ((U32)(21 & 0xFF)   << 16) | ((U32)(20 & 0xFF)   << 8) | (19 & 0xFF),
    ((U32)(2 & 0x3)     << 25) | ((U32)(0 & 0x1)     << 24) | ((U32)(59 & 0xFF)   << 16)| ((U32)(115 & 0xFF) << 8) | (21 & 0xFF),
    ((U32)(390 & 0x3FF) << 22) | ((U32)(0 & 0x1)     << 21) | ((U32)(253 & 0x3FF) << 11)| ((U32)(0 & 0x1) << 10)   | (78 & 0x3FF)};

/* Filter coeffs available for Vertical zoom until 0.02 to 0.33 (zoom out) */
/*-------------------------------------------------------------------------*/
static const U32 VerticalCoefficientsG[VFC_NB_COEFS] =
   {((U32)(36 & 0xFF)   << 24) | ((U32)(42 & 0xFF)  << 16) | ((U32)(49 & 0xFF)   << 8) | (56 & 0xFF),
    ((U32)(11 & 0xFF)   << 24) | ((U32)(17 & 0xFF)  << 16) | ((U32)(23 & 0xFF)   << 8) | (29 & 0xFF),
    ((U32)(-9 & 0xFF)   << 24) | ((U32)(-5 & 0xFF)  << 16) | ((U32)(0 & 0xFF)    << 8) | (6 & 0xFF),
    ((U32)(-26 & 0xFF)  << 24) | ((U32)(-22 & 0xFF) << 16) | ((U32)(-18 & 0xFF)  << 8) | (-14 & 0xFF),
    ((U32)(-39 & 0xFF)  << 24) | ((U32)(-36 & 0xFF) << 16) | ((U32)(-33 & 0xFF)  << 8) | (-29 & 0xFF),
    ((U32)(-48 & 0xFF)  << 24) | ((U32)(-46 & 0xFF) << 16) | ((U32)(-44 & 0xFF)  << 8) | (-41 & 0xFF),
    ((U32)(-54 & 0xFF)  << 24) | ((U32)(-52 & 0xFF) << 16) | ((U32)(-51 & 0xFF)  << 8) | (-49 & 0xFF),
    ((U32)(-57 & 0xFF)  << 24) | ((U32)(-56 & 0xFF) << 16) | ((U32)(-55 & 0xFF)  << 8) | (-55 & 0xFF),
    ((U32)(55 & 0xFF)   << 24) | ((U32)(58 & 0xFF)  << 16) | ((U32)(62 & 0xFF)   << 8) | (65 & 0xFF),
    ((U32)(40 & 0xFF)   << 24) | ((U32)(44 & 0xFF)  << 16) | ((U32)(48 & 0xFF)   << 8) | (51 & 0xFF),
    ((U32)(24 & 0xFF)   << 24) | ((U32)(28 & 0xFF)  << 16) | ((U32)(32 & 0xFF)   << 8) | (36 & 0xFF),
    ((U32)(7 & 0xFF)    << 24) | ((U32)(12 & 0xFF)  << 16) | ((U32)(16 & 0xFF)   << 8) | (20 & 0xFF),
    ((U32)(-11 & 0xFF)  << 24) | ((U32)(-6 & 0xFF)  << 16) | ((U32)(-2 & 0xFF)   << 8) | (3 & 0xFF),
    ((U32)(-28 & 0xFF)  << 24) | ((U32)(-24 & 0xFF) << 16) | ((U32)(-19 & 0xFF)  << 8) | (-15 & 0xFF),
    ((U32)(-44 & 0xFF)  << 24) | ((U32)(-40 & 0xFF) << 16) | ((U32)(-36 & 0xFF)  << 8) | (-32 & 0xFF),
    ((U32)(-60 & 0xFF)  << 24) | ((U32)(-56 & 0xFF) << 16) | ((U32)(-52 & 0xFF)  << 8) | (-48 & 0xFF),
    ((U32)(-9 & 0xFF)   << 24) | ((U32)(-14 & 0xFF) << 16) | ((U32)(-19 & 0xFF)  << 8) | (-25 & 0xFF),
    ((U32)(9 & 0xFF)    << 24) | ((U32)(5 & 0xFF)   << 16) | ((U32)(1 & 0xFF)    << 8) | (-4 & 0xFF),
    ((U32)(20 & 0xFF)   << 24) | ((U32)(18 & 0xFF)  << 16) | ((U32)(15 & 0xFF)   << 8) | (12 & 0xFF),
    ((U32)(25 & 0xFF)   << 24) | ((U32)(24 & 0xFF)  << 16) | ((U32)(23 & 0xFF)   << 8) | (22 & 0xFF),
    ((U32)(1 & 0x3)     << 25) | ((U32)(0 & 0x1)    << 24) | ((U32)(-57 & 0xFF)  << 16)| ((U32)(-64 & 0xFF) << 8) | (25 & 0xFF),
    ((U32)(407 & 0x3FF) << 22) | ((U32)(1 & 0x1)    << 21) | ((U32)(253 & 0x3FF) << 11)| ((U32)(0 & 0x1) << 10)   | (706 & 0x3FF)};

#endif // _STREF_DISPLAY_FILTER_H
