;; VR1x000 pipeline description.
;;   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
;;
;; This file is part of GCC.

;; GCC is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 2, or (at your
;; option) any later version.

;; GCC is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
;; License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GCC; see the file COPYING.  If not, write to the
;; Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
;; MA 02110-1301, USA.


;; This file overrides parts of generic.md.  It is derived from the
;; old define_function_unit description.



;; R12K/R14K/R16K are derivatives of R10K, thus copy its description
;; until specific tuning for each is added


;; R10000 has int queue, fp queue, address queue
(define_automaton "r10k_int, r10k_fp, r10k_addr")

;; R10000 has 2 integer ALUs, fp-adder and fp-multiplier, load/store
(define_cpu_unit "r10k_alu1" "r10k_int")
(define_cpu_unit "r10k_alu2" "r10k_int")
(define_cpu_unit "r10k_fpadd" "r10k_fp")
(define_cpu_unit "r10k_fpmpy" "r10k_fp")
(define_cpu_unit "r10k_loadstore" "r10k_addr")

;; R10000 has separate fp-div and fp-sqrt units as well and these can
;; execute in parallel, however their issue & completion logic is shared
;; by the fp-multiplier
(define_cpu_unit "r10k_fpdiv" "r10k_fp")
(define_cpu_unit "r10k_fpsqrt" "r10k_fp")




;; loader
(define_insn_reservation "r10k_load" 2
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "load,prefetch,prefetchx"))
  "r10k_loadstore")

(define_insn_reservation "r10k_store" 0
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "store,fpstore,fpidxstore"))
  "r10k_loadstore")

(define_insn_reservation "r10k_fpload" 3
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "fpload,fpidxload"))
  "r10k_loadstore")




;; Integer add/sub + logic ops, and mf/mt hi/lo can be done by alu1 or alu2
;; Miscellaneous arith goes here too (this is a guess)
(define_insn_reservation "r10k_arith" 1
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "arith,mfhilo,mthilo,slt,clz,const,nop,trap"))
  "r10k_alu1 | r10k_alu2")




;; ALU1 handles shifts, branch eval, and condmove
;;
;; Brancher is separate, but part of ALU1, but can only
;; do one branch per cycle (needs implementing??)
;;
;; jump, call - unsure if brancher handles these too (added for now)
(define_insn_reservation "r10k_shift" 1
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "shift,branch,jump,call"))
  "r10k_alu1")

(define_insn_reservation "r10k_int_cmove" 1
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "condmove")
            (eq_attr "mode" "SI,DI")))
  "r10k_alu1")




;; Coprocessor Moves
;; mtc1/dmtc1 are handled by ALU1
;; mfc1/dmfc1 are handled by the fp-multiplier
(define_insn_reservation "r10k_mt_xfer" 3
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "xfer")
            (not (match_operand 0 "fpr_operand"))))
  "r10k_alu1")

(define_insn_reservation "r10k_mf_xfer" 2
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "xfer")
            (match_operand 0 "fpr_operand")))
  "r10k_fpmpy")




;; Only ALU2 does int multiplications and divisions
;; R10K allows an int insn using register Lo to be issued
;; one cycle earlier than an insn using register Hi for
;; the insns below, however, we skip on doing this
;; for now until correct usage of lo_operand() is figured
;; out.
;;
;; Divides keep ALU2 busy, but this isn't expressed here (I think...?)
(define_insn_reservation "r10k_imul_single" 6
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "imul,imul3,imadd")
            (eq_attr "mode" "SI")))
  "r10k_alu2 * 6")

(define_insn_reservation "r10k_imul_double" 10
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "imul,imul3,imadd")
            (eq_attr "mode" "DI")))
  "r10k_alu2 * 10")

(define_insn_reservation "r10k_idiv_single" 35
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "idiv")
            (eq_attr "mode" "SI")))
  "r10k_alu2 * 35")

(define_insn_reservation "r10k_idiv_double" 67
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "idiv")
            (eq_attr "mode" "DI")))
  "r10k_alu2 * 67")




;; FP add/sub, mul, abs value, neg, comp, & moves
(define_insn_reservation "r10k_fp_miscadd" 2
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "fadd,fabs,fneg,fcmp"))
  "r10k_fpadd")

(define_insn_reservation "r10k_fp_miscmul" 2
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "fmul,fmove"))
  "r10k_fpmpy")

(define_insn_reservation "r10k_fp_cmove" 2
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "condmove")
            (eq_attr "mode" "SF,DF")))
  "r10k_fpmpy")




;; fcvt.s.[wl] has latency 4, repeat 2
;; All other fcvt have latency 2, repeat 1
(define_insn_reservation "r10k_fcvt_single" 4
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "fcvt")
            (eq_attr "cnv_mode" "I2S")))
  "r10k_fpadd * 2")

(define_insn_reservation "r10k_fcvt_other" 2
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "fcvt")
            (eq_attr "cnv_mode" "!I2S")))
  "r10k_fpadd")




;; fmadd -  Runs through fp-adder first, then fp-multiplier
;;
;; The latency for fmadd is 2 cycles if the result is used
;; by another fmadd instruction
(define_insn_reservation "r10k_fmadd" 4
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "fmadd"))
  "r10k_fpadd, r10k_fpmpy")

(define_bypass 2 "r10k_fmadd" "r10k_fmadd")




;; fp Divisions & square roots 
(define_insn_reservation "r10k_fdiv_single" 12
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "fdiv,frdiv")
            (eq_attr "mode" "SF")))
  "r10k_fpdiv * 14")

(define_insn_reservation "r10k_fdiv_double" 19
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "fdiv,frdiv")
            (eq_attr "mode" "DF")))
  "r10k_fpdiv * 21")

(define_insn_reservation "r10k_fsqrt_single" 18
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "fsqrt")
            (eq_attr "mode" "SF")))
  "r10k_fpsqrt * 20")

(define_insn_reservation "r10k_fsqrt_double" 33
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "fsqrt")
            (eq_attr "mode" "DF")))
  "r10k_fpsqrt * 35")

(define_insn_reservation "r10k_frsqrt_single" 30
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "frsqrt")
            (eq_attr "mode" "SF")))
  "r10k_fpsqrt * 20")

(define_insn_reservation "r10k_frsqrt_double" 52
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (and (eq_attr "type" "frsqrt")
            (eq_attr "mode" "DF")))
  "r10k_fpsqrt * 35")




;; Unknown/multi (this is a guess)
(define_insn_reservation "r10k_unknown" 1
  (and (eq_attr "cpu" "r10000,r12000,r14000,r16000")
       (eq_attr "type" "unknown,multi"))
  "r10k_alu1 + r10k_alu2")

