#ifndef __MXL5007_API_H
#define __MXL5007_API_H

/******************************/
/*	MxL5007 Err message  	  */
/******************************/
typedef enum{
	MXL_OK				=   0,
	MXL_ERR_INIT		=   1,
	MXL_ERR_RFTUNE		=   2,
	MXL_ERR_SET_REG		=   3,
	MXL_ERR_GET_REG		=	4,
	MXL_ERR_OTHERS		=   10
}mxl_err_msg;

/******************************/
/*	MxL5007 Chip verstion     */
/******************************/
typedef enum{
	MXL_UNKNOWN_ID		= 0x00,
	MXL_5007T_V4		= 0x14,
	MXL_GET_ID_FAIL		= 0xFF
}mxl5007_version;

#ifndef MHz
	#define MHz 1000000
#endif

#define MAX_ARRAY_SIZE 100


// Enumeration of Mode
// Enumeration of Mode
typedef enum 
{
	MXL_MODE_ISDBT = 0,
	MXL_MODE_DVBT = 1,
	MXL_MODE_ATSC = 2,	
	MXL_MODE_CABLE = 0x10
} mxl5007_mode ;

typedef enum
{
	MXL_IF_4_MHZ	  = 4000000,
	MXL_IF_4_5_MHZ	  = 4500000,
	MXL_IF_4_57_MHZ	  =	4570000,
	MXL_IF_5_MHZ	  =	5000000,
	MXL_IF_5_38_MHZ	  =	5380000,
	MXL_IF_6_MHZ	  =	6000000,
	MXL_IF_6_28_MHZ	  =	6280000,
	MXL_IF_9_1915_MHZ =	9191500,
	MXL_IF_35_25_MHZ  = 35250000,
	MXL_IF_36_15_MHZ  = 36150000,
	MXL_IF_44_MHZ	  = 44000000
} mxm5007_if_frequency ;

typedef enum
{
	MXL_XTAL_16_MHZ		= 16000000,
	MXL_XTAL_20_MHZ		= 20000000,
	MXL_XTAL_20_25_MHZ	= 20250000,
	MXL_XTAL_20_48_MHZ	= 20480000,
	MXL_XTAL_24_MHZ		= 24000000,
	MXL_XTAL_25_MHZ		= 25000000,
	MXL_XTAL_25_14_MHZ	= 25140000,
	MXL_XTAL_27_MHZ		= 27000000,
	MXL_XTAL_28_8_MHZ	= 28800000,
	MXL_XTAL_32_MHZ		= 32000000,
	MXL_XTAL_40_MHZ		= 40000000,
	MXL_XTAL_44_MHZ		= 44000000,
	MXL_XTAL_48_MHZ		= 48000000,
	MXL_XTAL_49_3811_MHZ = 49381100	
} mxl5007_xtal_frequency ;

typedef enum
{
	MXL_BW_6MHz = 6,
	MXL_BW_7MHz = 7,
	MXL_BW_8MHz = 8
} mxl5007_bandwidth;

typedef enum
{
	MXL_NORMAL_IF = 0 ,
	MXL_INVERT_IF
} mxl5007_if_spectrum ;

typedef enum
{
	MXL_LT_DISABLE = 0 ,
	MXL_LT_ENABLE
} mxl5007_lt ;

typedef enum
{
	MXL_CLKOUT_DISABLE = 0 ,
	MXL_CLKOUT_ENABLE
} mxl5007_clock_out;

typedef enum
{
	MXL_CLKOUT_AMP_0 = 0 ,
	MXL_CLKOUT_AMP_1,
	MXL_CLKOUT_AMP_2,
	MXL_CLKOUT_AMP_3,
	MXL_CLKOUT_AMP_4,
	MXL_CLKOUT_AMP_5,
	MXL_CLKOUT_AMP_6,
	MXL_CLKOUT_AMP_7
} mxl5007_clk_out_amp;

typedef enum
{
	MXL_I2C_ADDR_96 = 0xC0,
	MXL_I2C_ADDR_97 = 0xC2 ,
	MXL_I2C_ADDR_98 = 0xC4 ,
	MXL_I2C_ADDR_99 = 0xC6 	
} mxl5007_i2c_addr ;

typedef struct _mxl5007_tuner_configs
{
	mxl5007_i2c_addr		i2c_addr;
	mxl5007_mode			mode;
	int						if_level;
	mxl5007_xtal_frequency	xtal;
	mxm5007_if_frequency	if_frequency;
	mxl5007_if_spectrum		if_spectrum;
	mxl5007_clock_out		clock_out;
	mxl5007_clk_out_amp		amp;
	mxl5007_bandwidth		bandwidth;
	int						rf_frequency;
	mxl5007_lt				lt_setting;
} mxl5007_tuner_configs;


int mxl5007_init(u8* array,				// a array pointer that store the addr and data pairs for I2C write
					int* array_size,			// a integer pointer that store the number of element in above array
					u8 mode,				
					int if_level,		 
					int xtal_freq,			
					int if_freq,				
					u8 invert_if,											
					u8 clk_out_en,    
					u8 clk_out_amp													
					);
					
int mxl5007_rf_tune(u8* array, int* array_size, int rf_frequency, u8 bandwidth);

mxl_err_msg mxl_set_register(mxl5007_tuner_configs* configs, u8 addr, u8 val);

mxl_err_msg mxl_get_register(mxl5007_tuner_configs* configs, u8 addr, u8 *val);

mxl_err_msg mxl_soft_reset(mxl5007_tuner_configs* configs);

mxl_err_msg mxl_loop_through_on(mxl5007_tuner_configs* configs, mxl5007_lt isOn);

mxl_err_msg mxl_standby(mxl5007_tuner_configs* configs);

mxl_err_msg mxl_wakeup(mxl5007_tuner_configs* configs);

mxl_err_msg mxl_tuner_init(mxl5007_tuner_configs* configs);

mxl_err_msg mxl_tuner_rftune(mxl5007_tuner_configs* configs, int rf_frequency, mxl5007_bandwidth bandwidth);

mxl5007_version mxl_check_version(mxl5007_tuner_configs* configs);

mxl_err_msg mxl_rf_syn_lockstatus(mxl5007_tuner_configs* configs, u8* isLock);

mxl_err_msg mxl_ref_syn_lockstatus(mxl5007_tuner_configs* configs, u8* isLock);

#endif
