/*
    Wyplay Asia s5h1432 DVB OFDM demodulator driver

    Copyright (C) 2006 ST Microelectronics
	Peter Bennett <peter.bennett@st.com>
	
    Copyright (C) 2009 Wyplay Asia 
    Hu Gang <hug@wyplayasia.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/string.h>
#include <linux/slab.h>
#include "dvb_frontend.h"


#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17)
#include <linux/stm/pio.h>
#else	/* STLinux 2.2 or older */
#include <linux/stpio.h>        /*!< for stpio_set_pin */
#endif

#include "s5h1432.h"
#include <linux/time.h>
struct s5h1432_state {
	struct s5h1432_config* config;
    u8 demod_address;
	struct i2c_adapter* i2c;
	struct dvb_frontend_ops ops;
	struct dvb_frontend frontend;
};

mxl5007_tuner_configs mxl5007_config;

static int debug = 1;

#define dprintk(fmt, args...) \
	do { \
		if (debug) printk(KERN_DEBUG "%s: "fmt, __FUNCTION__, ## args); \
	} while (0)
	
#define INRANGE( x,  y, z) (((x)<=(y) && (y)<=(z))||((z)<=(y) && (y)<=(x)))

struct i2c_opt
{
	u8	addr;
	u8	dat;
};

struct i2c_opt init_tab[] = {
	{0x03, 0x00},
	{0x04, 0xa8},
	{0x05, 0x01},
	{0x06, 0x06},
	{0x07, 0x70},
	{0x0a, 0x80},
	{0x0b, 0xc0},
	{0x10, 0x08},
	{0x12, 0xfa},
	{0x19, 0x80},
	{0x1a, 0xc4},
	{0x1b, 0x9b},
	{0x1c, 0x30},
	{0x1d, 0x20},
	{0x1e, 0x29}, 
	{0x1f, 0x26},
	{0x28, 0x80},
	{0x2a, 0x35},
	{0x2e, 0x40},
	{0x30, 0x65},
	{0x31, 0x65},
	{0x35, 0xee},
	{0x38, 0x81},
	{0x3a, 0x80},
	{0x3b, 0x40},
	{0x3d, 0x40},
	{0x3e, 0x00},
	{0x42, 0x04}, /*<- MPEG Clock positive edge*/
	{0x44, 0x45},
	{0x50, 0x5a},
	{0x51, 0x01},
	{0x54, 0x08},
	{0x55, 0x0b},
	{0x58, 0x64},
	{0x59, 0x05},
	{0x5a, 0xd3},
	{0x5f, 0x27},
	{0x61, 0x37},
	{0x63, 0x27},
	{0x64, 0x0f},
	{0x65, 0xa0},
	{0x66, 0xa4},
	{0x67, 0x67},
	{0x68, 0x50},
	{0x69, 0x30},
	{0x6a, 0xa0},
	{0x6b, 0x10},
	{0x6c, 0x44},
	{0x6d, 0x00},
	{0x6e, 0x40},
	{0x6f, 0x77},
	{0xb8, 0x3c},
	{0xc4, 0x10}, //080412 0x20 -> 0x10
	{0xc5, 0x20},
	{0xc6, 0x20},
	{0xc7, 0x20},
	{0xc8, 0x80}, //080406 0x80 -> 0x20 (final France Stream) -> 0x80(080810 : locking time issue)
	{0xc9, 0x20},
	{0xca, 0xbe}, //080410 0xbe -> 0xbf -> 0xbe(sampling offset  Ŭ lock time )
	{0xcb, 0x33},
	{0xcc, 0x9c},
	{0xcd, 0xC0},
	{0xce, 0x38},
	{0xcf, 0xce},
	{0xd8, 0x91},
	{0xd9, 0x30}, //090408 0x70 -> 0x30 (co-channel resync)
	{0xda, 0x00}, //090408 0xa0 -> 0x00 (co-channel resync)
	{0xdb, 0x15},
	{0xde, 0x48},
	{0xdf, 0xab}, //080406 0xb2 -> 0xab (co-channel threshold)
	{0xe0, 0x51},
	{0xe1, 0x94},
	{0xe2, 0x09},
	{0xe3, 0x06},
	{0xe6, 0x08},

	//S5H1432 IF : 4.57	08.12.04 JILEE
	{0xe7, 0xE7},
	{0xe4, 0xA0},
	{0xe5, 0x6D},
	{0xe8, 0x00},
	{0xe9, 0x10},
	{0xf4, 0x93}, //0xa1 -> 0xa3(080329) -> 0x93(080406) (co-channel threshold)
	{0xf5, 0xff},
	{0xf6, 0x40}, //0x90 -> 0x40(080406) (co-channel threshold)
	{0xf7, 0x80},
	{0xf8, 0x04}, //0x04 -> 0x01(080406) (to speec up SWPATCH), no swpatch -> 0x04
	{0xf9, 0x00}, //0x11 -> 0x00(080406)
	{0xfa, 0x21},
	{0xfb, 0x57},
	{0xfc, 0x00},
	{0xfd, 0x40},
	{0xfe, 0x08},
};

static int _strength_lookup[][2]=
{
    {4000,  0},
    {3900,  5},
    {3850, 10},
    {3800, 20},
    {3700, 50},
    {3600, 70},
    {3480, 80},
    {3240, 90},
    {3120, 95},
    {3000,100},
};

static int _quality_lookup[][2]=
{
    {3100,100},
    {2713, 85},
    {2326, 70},
    {1939, 55},
    {1552, 40},
    {1165, 25},
    { 778, 10},
    {   0,  0}
};

static int _ber_lookup[][2]=
{
    {200000,100},                               //2E-3 
    {100000, 95},                               //1E-3
    { 50000, 90},                               //5E-4
    { 10000, 85},                               //1E-4
    {  9000, 80},                               //9E-5
    {  7700, 75},                               //7.71E-5
    {  7000, 70},                               
    {  5900, 65},                               
    {  5800, 60},                               
    {  5700, 55},                               
    {  5600, 50},                               
    {  5600, 45},                               
    {  5500, 40},                               
    {  5400, 35},                               
    {  5300, 30},                               
    {  5200, 27},                               
    {  5100, 25},                               
    {  5000, 22},                               
    {  4900, 20},                               
    {  4800, 15},
    {  4700, 12},
    {  4500, 10},
    {  4300,  5}
};

static int _lookup (int val, int table[][2], int size)
{
    int ret = 0;
    int i;
    
    size = (size / (sizeof(int) << 1)) - 1;
    if (INRANGE(table[size][0], val, table[0][0]))
    {
        for (i=0; i<size; ++i)
        {
            if (INRANGE(table[i+1][0], val, table[i][0]))
            {
                ret = table[i][1];
                break;
            }
        }	
    }
    else 
    {
        ret = (val > table[0][0]) ? table[0][1] : table[size][1];
    }
    
    return ret;
    
}
static int s5h1432_writereg (struct s5h1432_state* state, u8 reg, u8 data)
{
	int ret  = 0;
	u8 buf [] = { reg, data };
	struct i2c_msg msg = { .addr = state->demod_address, .flags = 0, .buf = buf, .len = 2 };

	ret = i2c_transfer (state->i2c, &msg, 1); 

	if (ret != 1)
		printk("%s: error (addr == 0x%02x,reg == 0x%02x, val == 0x%02x, ret == %i)\n",
			__FUNCTION__, msg.addr,reg, data, ret);

	return (ret != 1) ? -1 : 0;
}

static int s5h1432_readreg (struct s5h1432_state* state, u8 reg)
{
	int ret;
	u8 b0 [] = { reg };
	u8 b1 [] = { 0 };
	struct i2c_msg msg [] = { { .addr = state->demod_address, .flags = 0, .buf = b0, .len = 1 },
			   { .addr = state->demod_address, .flags = I2C_M_RD, .buf = b1, .len = 1 } };

	ret = i2c_transfer (state->i2c, msg, 1);
	ret |= i2c_transfer (state->i2c, msg+1, 1);

	if (ret != 1) {
		printk("%s: error (reg == 0x%02x,  ret == %i)\n",
			__FUNCTION__, reg, ret);
	  return -EIO;
	}
	return (int)b1[0];
}

static void s5h1432_wait(int millisecond)
{
	msleep(millisecond);
} 

static int s5h1432_soft_reset (struct s5h1432_state* state)
{
	if (s5h1432_writereg(state, 0xda, 0x60))
		goto failed;
	
	if (s5h1432_writereg(state, 0x09, 0x1a))
		goto failed;
		
	s5h1432_wait(30);
		
	if (s5h1432_writereg(state, 0x09, 0x1b))
		goto failed;

	return 0;
		
failed:
	dprintk("%s failed\n", __FUNCTION__);
	return -1;
}

static int s5h1432_soft_sleep(struct s5h1432_state* state, u8 val)
{	
	u8 dat = 0;

	dat = s5h1432_readreg(state, 0x06);	
	if(val == 1) 
	{
		s5h1432_writereg(state,	0x06, dat & 0xFB);
	}
	else 
	{
		s5h1432_writereg(state,	0x06, dat | 0x04);
	}
	
	dat = s5h1432_readreg(state, 0x03);
	
	return s5h1432_writereg(state, 0x03, ((dat & 0xFE) | (val)));
}

static int s5h1432_set_hplp(struct s5h1432_state* state, u8 value)
{
	//value	->	0:hp	1:lp
	u8 datax = 0;
	u8 datay  = 0;
	
	datax = s5h1432_readreg(state, 0x42);
	datay = ((datax & 0xf8) | (value << 2) | (datax & 0x3));	
	
	return s5h1432_writereg(state, 0x42, datay);
}

static int s5h1432_set_bandwidth(struct s5h1432_state* state, fe_bandwidth_t bandwidth)
{
		u8 datax;
		u8 datay;
	
		datax = s5h1432_readreg(state, 0x2e);
		datay = ((datax & 0xf0) | ((u8)bandwidth << 2) | (datax & 0x03));
		
		if (s5h1432_writereg(state, 0x2e, datay))
			return -EINVAL;

		return 0;
}

static int s5h1432_set_inversion (struct s5h1432_state* state, int inversion)
{
	u8 val;

	switch (inversion) {
	case INVERSION_AUTO:
	  return -EOPNOTSUPP;
	case INVERSION_ON:
	  val = s5h1432_readreg (state, 0x42);
		return s5h1432_writereg (state, 0x42, val | 0x20);
	case INVERSION_OFF:
		val = s5h1432_readreg (state, 0x42);
		return s5h1432_writereg (state, 0x42, val & ~0x20);
	default:
		return -EINVAL;
	}
}

static int s5h1432_set_tps (struct s5h1432_state *state, struct dvb_ofdm_parameters *p)
{
	if ((p->code_rate_HP < FEC_1_2) || ((p->code_rate_HP > FEC_7_8) && (p->code_rate_HP != FEC_AUTO)))
		return -EINVAL;

	if ((p->code_rate_LP < FEC_1_2) || ((p->code_rate_LP > FEC_7_8) && (p->code_rate_LP != FEC_AUTO)))

	if ((p->code_rate_HP == FEC_4_5) || (p->code_rate_LP == FEC_4_5))
		return -EINVAL;

	if ((p->guard_interval < GUARD_INTERVAL_1_32) ||
	    ((p->guard_interval > GUARD_INTERVAL_1_4) && (p->guard_interval != GUARD_INTERVAL_AUTO)))
		return -EINVAL;

	if ((p->transmission_mode != TRANSMISSION_MODE_2K) &&
	    (p->transmission_mode != TRANSMISSION_MODE_8K) &&
	    (p->transmission_mode != TRANSMISSION_MODE_AUTO))
		return -EINVAL;

	if ((p->constellation != QPSK) &&
	    (p->constellation != QAM_16) &&
	    (p->constellation != QAM_64))
		return -EINVAL;

	if ((p->hierarchy_information < HIERARCHY_NONE) ||
	    (p->hierarchy_information > HIERARCHY_4))
		return -EINVAL;

	if ((p->bandwidth > BANDWIDTH_6_MHZ) || (p->bandwidth < BANDWIDTH_8_MHZ))
	{
		printk(KERN_INFO "we don't support %d M bandwidth(only 6MHZ, 7MHZ and 8MHZ\n", p->bandwidth);
		return -EINVAL;
	}
	
	if (s5h1432_set_bandwidth(state, p->bandwidth))
		return -EINVAL;
		
	return 0;
}

static int s5h1432_get_tps (struct s5h1432_state* state, struct dvb_ofdm_parameters *p)
{
	static const fe_modulation_t qam_tab [3] = { QPSK, QAM_16, QAM_64 };
	static const fe_code_rate_t fec_tab [5] = { FEC_1_2, FEC_2_3, FEC_3_4,
						    FEC_5_6, FEC_7_8 };
	u8 val;
	int datax, datay;
	
	if (!(s5h1432_readreg(state, 0x9f) & 0x10))  /*  tps valid? */
		return -EAGAIN;
	
	val = s5h1432_readreg (state, 0xd3);
	
	if (((val >> 2) & 0x7) > 3)
		printk(KERN_ERR "%s: Invalid Hierarchy detected\n",__FUNCTION__);
	else
		p->hierarchy_information = HIERARCHY_NONE + ((val >> 2) & 0x7);
	
	if (((val >> 5)& 0x3) > 2)
		printk(KERN_ERR "%s: Invalid Modulation detected\n",__FUNCTION__);
	else
		p->constellation = qam_tab[(val >> 5) & 0x3];
	
	datax = s5h1432_readreg(state, 0xd3);
	datay = s5h1432_readreg(state, 0xd0);

	val= (((u8)datax & 0x03) << 1) | (((u8)datay & 0x80) >> 7);
	
	if (val > 4)
		printk(KERN_ERR "%s: Invalid FEC rate 0x%02x detected\n", __FUNCTION__, val);
	else
		p->code_rate_HP = fec_tab[val];
	
	val = s5h1432_readreg (state, 0xd0);
	if (((val >> 4) & 0x07) > 4)
		printk(KERN_ERR "%s: Invalid FEC rate detected\n",__FUNCTION__);
	else
		p->code_rate_LP = fec_tab[(val>>4) & 0x07];
	
	p->guard_interval = GUARD_INTERVAL_1_32 + ((val >> 2) & 0x3);
	
	if ((val & 0x03) > 1)
		printk(KERN_ERR "%s: Invalid transmission mode %d detected\n",__FUNCTION__, (val & 0x03));
	else
		p->transmission_mode = TRANSMISSION_MODE_2K + (val & 0x03);
	
	return 0;
}

static int s5h1432_set_i2c_bypass (struct s5h1432_state* state, int val)
{
	u8 dat;
	
	dat = s5h1432_readreg(state, 0x07);
	
	return s5h1432_writereg(state, 0x07, (dat & 0xFE) | val);
}

static int initialize(struct s5h1432_state *state)
{
	struct i2c_opt *p = &init_tab[0];
	int i=0;

	for (; i<sizeof(init_tab)/sizeof(init_tab[0]); i++, p++)
	{
		if (s5h1432_writereg(state, p->addr, p->dat))
			return -1;
	}

	return 0;
}

static int s5h1432_init (struct dvb_frontend* fe)
{
    struct s5h1432_state* state = (struct s5h1432_state*) fe->demodulator_priv;
	int err=0;
	int b=0;
	u8 val;
	
	mxl5007_config.i2c_addr = state->config->tuner_address;
	mxl5007_config.if_frequency = MXL_IF_4_57_MHZ;
	mxl5007_config.if_spectrum = MXL_INVERT_IF;
	mxl5007_config.mode = MXL_MODE_DVBT;
	mxl5007_config.lt_setting = MXL_CLKOUT_ENABLE;
	mxl5007_config.xtal = MXL_XTAL_24_MHZ;
	mxl5007_config.clock_out = MXL_CLKOUT_ENABLE;
	mxl5007_config.amp = MXL_CLKOUT_AMP_0;
	mxl5007_config.bandwidth = MXL_BW_8MHz;
	mxl5007_config.rf_frequency = 666000000;
	
 	if (initialize(state))
	{
		err = -1;
		goto failed;
	}

	if ((err = s5h1432_soft_reset(state)) != 0)	goto failed;
	
	s5h1432_set_i2c_bypass(state, 1);
	b=1;

	if (mxl_tuner_init(&mxl5007_config) != MXL_OK)
	{
		printk(KERN_ERR "failed to initialize tuner\n");
		err = -1;
		goto failed;
	}
	
	if (mxl_loop_through_on(&mxl5007_config, mxl5007_config.lt_setting) != MXL_OK)
	{
		printk(KERN_ERR "mxl_loop_through_on failed\n");
		err = -1;
		goto failed;
	}
	
	s5h1432_set_i2c_bypass(state, 0);
	
	val = s5h1432_readreg(state, 0x42);
	if (state->config->serial_not_parallel)
		val |= 0x80;
	else
		val &= ~0x80;
		
	s5h1432_writereg(state, 0x42, val);
	
	return err;

failed:
	if (b)
		s5h1432_set_i2c_bypass(state, 0);
	dprintk("initialize failed\n");
	return err;
}

static int s5h1432_read_tps_lock(struct s5h1432_state *state)
{
	int ret;

	ret = s5h1432_readreg(state, 0x9f);
	return ((ret >> 4) & 0x01);
}

static int s5h1432_read_rsframe_lock(struct s5h1432_state *state)
{
	int dat;
	int b;

	dat = s5h1432_readreg(state, 0x9f);
	
	b = ((dat >> 4) & 0x01);
	dat = ((dat >> 2) & 0x01);

	if (b)
		s5h1432_writereg(state, 0xda, 0x80);
	else
		s5h1432_writereg(state, 0xda, 0x00);
	
	return dat;
}

static int s5h1432_read_int_lock(struct s5h1432_state *state)
{
	int ret;

	ret = s5h1432_readreg(state, 0x9f);
	return ((ret >> 7) & 0x01);
}

static int s5h1432_read_status(struct dvb_frontend* fe, fe_status_t* status)
{
	struct s5h1432_state* state = (struct s5h1432_state*)fe->demodulator_priv;

#if 0
	int result = (FE_HAS_SIGNAL | FE_HAS_CARRIER | FE_HAS_SYNC | FE_HAS_VITERBI);

	*status = 0;

	if (s5h1432_read_tps_lock(state))
		*status |= (FE_HAS_SIGNAL | FE_HAS_CARRIER);
	else
		printk("FE_HAS_SIGNAL and FE_HAS_CARRIER not found\n");

	if (s5h1432_read_rsframe_lock(state))
		*status |= (FE_HAS_SYNC | FE_HAS_VITERBI);
	else
		printk("FE_HAS_SYNC and FE_HAS_VITERBI not found\n");

	if (s5h1432_read_int_lock(state) && ((*status & result) == result))
		*status |= FE_HAS_LOCK;
	else
		printk("FE_HAS_LOCK not found!\n");
#else
	int result;
	int datax, datay;

	*status = 0;

	datax = s5h1432_readreg(state, 0x9e);
	datay = s5h1432_readreg(state, 0x9f); 

	result = (datax << 8) | datay;

	if (0x600 == (result & 0x600))
		*status |=  FE_HAS_SIGNAL;
	else
		printk("FE_HAS_SIGNAL not found\n");

	if (0x100 == (result & 0x100))
		*status |= FE_HAS_CARRIER;
	else
		printk("FE_HAS_CARRIER not found\n");

	if (0x80 == (result & 0x80))
		*status |= FE_HAS_VITERBI;
	else
		printk("FE_HAS_VITERBI not found\n");

	if (0x40 == (result & 0x40))
		*status |= FE_HAS_SYNC;
	else
		printk("FE_HAS_SYNC not found\n");

	if (0xAFFD == result)
		*status |= FE_HAS_LOCK;
	else
		printk("FE_HAS_LOCK not found\n");
#endif

	return 0;
}

u16 s5h1432_read_dacif(struct dvb_frontend *fe)
{
	struct s5h1432_state * state = (struct s5h1432_state *)fe->demodulator_priv;
	u16 result = 0;
	
	result = (((s5h1432_readreg(state, 0x72) << 8) | (s5h1432_readreg(state, 0x73))) & 0xffff);
	return result;
}

static int s5h1432_read_preber(struct dvb_frontend* fe, u32* ber)
{
	struct s5h1432_state* state = (struct s5h1432_state*) fe->demodulator_priv;	
	int datax = 0;
	int datay = 0; 

	datax = (s5h1432_readreg(state, 0xa8) << 8) | (s5h1432_readreg(state, 0xa9));
	datay = 1 << (s5h1432_readreg(state, 0x49) & 0x0f);

	if(datay==0)
	{
		*ber =0;
	}
	else
	{
		*ber = ((48828 * datax) / datay);
		*ber <<= 1;
		*ber *= 100000000;
	}
	
	return 0;
}

static int s5h1432_read_signal_strength(struct dvb_frontend* fe, u16* signal_strength)
{
	unsigned int agc;
	
	agc = (s5h1432_read_dacif(fe) + s5h1432_read_dacif(fe)) >> 1;
	*signal_strength = (u16)(_lookup(agc, _strength_lookup, sizeof(_strength_lookup)));
	
	return 0;
}

static int s5h1432_read_snr(struct dvb_frontend* fe, u16* snr)
{
	struct s5h1432_state* state = (struct s5h1432_state*) fe->demodulator_priv;

	*snr = ((u16)(s5h1432_readreg(state, 0x93)) * 25);

	return 0;
}

static int s5h1432_read_ucblocks(struct dvb_frontend* fe, u32* ucblocks)
{
	struct s5h1432_state* state = (struct s5h1432_state*) fe->demodulator_priv;

	*ucblocks = (s5h1432_readreg(state, 0xbe) << 8) | (s5h1432_readreg(state, 0xbf));

	return 0;
}

static int s5h1432_set_frontend(struct dvb_frontend* fe, struct dvb_frontend_parameters *p)
{
	struct s5h1432_state* state = (struct s5h1432_state*) fe->demodulator_priv;
	
	dprintk("current frequency is %d\n", p->frequency);

	if (s5h1432_set_tps(state, &p->u.ofdm))
		return -1;
		
	s5h1432_set_i2c_bypass(state, 1);
	if (mxl_tuner_rftune(&mxl5007_config, p->frequency, p->u.ofdm.bandwidth) != MXL_OK)
	{
		s5h1432_set_i2c_bypass(state, 0);
		return -1;
	}

	s5h1432_set_i2c_bypass(state, 0);
	
	if (s5h1432_soft_reset(state))
		return -1;
	
	if (s5h1432_set_hplp(state, 1))
		return -1;
	
	return 0;
}

static int s5h1432_get_frontend(struct dvb_frontend* fe, struct dvb_frontend_parameters *p)
{
	struct s5h1432_state* state;
	u8 reg;
	
	state = (struct s5h1432_state*) fe->demodulator_priv;
	reg = s5h1432_readreg (state, 0x42);
	
	p->inversion = reg & 0x20 ? INVERSION_ON : INVERSION_OFF;
	
	return s5h1432_get_tps (state, &p->u.ofdm);
}

static int s5h1432_get_tune_settings(struct dvb_frontend* fe, struct dvb_frontend_tune_settings* fesettings)
{
	fesettings->min_delay_ms = 250;
	fesettings->step_size = 0;
	fesettings->max_drift = 166667 * 2;
	
	return 0;
}

static void s5h1432_release(struct dvb_frontend* fe)
{
	struct s5h1432_state* state;
	
	state = (struct s5h1432_state*) fe->demodulator_priv;	
	kfree(state);
}

static struct dvb_frontend_ops s5h1432_ops;

struct dvb_frontend* s5h1432_attach(const struct s5h1432_config* config,
				    struct i2c_adapter* i2c, u8 demod_address)
{
	struct s5h1432_state* state = NULL;

	/* allocate memory for the internal state */
	state = (struct s5h1432_state*) kmalloc(sizeof(struct s5h1432_state), GFP_KERNEL);
	if (state == NULL) goto error;

	/* setup the state */
	state->config = config;
	state->i2c = i2c;
	state->demod_address = demod_address;
	memcpy(&state->ops, &s5h1432_ops, sizeof(struct dvb_frontend_ops));

	/* create dvb_frontend */
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17)
	state->frontend.ops = state->ops;
	state->frontend.demodulator_priv = state;
#else
	state->frontend.ops = &state->ops;
	state->frontend.demodulator_priv = state;
#endif

	mxl5007_config.i2c_addr = config->tuner_address;
	mxl5007_config.if_frequency = MXL_IF_4_57_MHZ;
	mxl5007_config.if_spectrum = MXL_INVERT_IF;
	mxl5007_config.mode = MXL_MODE_DVBT;
	mxl5007_config.lt_setting = MXL_CLKOUT_ENABLE;
	mxl5007_config.xtal = MXL_XTAL_24_MHZ;
	mxl5007_config.clock_out = MXL_CLKOUT_ENABLE;
	mxl5007_config.amp = MXL_CLKOUT_AMP_0;
	mxl5007_config.bandwidth = MXL_BW_8MHz;
	mxl5007_config.rf_frequency = 666000000;
	
	s5h1432_set_i2c_bypass(state, 1);
 	if (mxl_check_version(&mxl5007_config) != MXL_5007T_V4)
 	{
		printk(KERN_INFO "mxl5007 with s5h1432 not found!\n");

		s5h1432_set_i2c_bypass(state, 0);
		goto error;
	}
	s5h1432_set_i2c_bypass(state, 0);
	 
	return &state->frontend;

error:
	if (state) kfree(state);
	return NULL;
}

static struct dvb_frontend_ops s5h1432_ops = {

	.info = {
		.name 			= "SAMSUNG s5h1432 DVB-T",
		.type 			= FE_OFDM,
		.frequency_min 		= 470000000,
		.frequency_max 		= 860000000,
		.frequency_stepsize 	= 166667,
		.caps = FE_CAN_FEC_1_2 | FE_CAN_FEC_2_3 | FE_CAN_FEC_3_4 |
		FE_CAN_FEC_5_6 | FE_CAN_FEC_7_8 | FE_CAN_FEC_AUTO |
		FE_CAN_QPSK | FE_CAN_QAM_16 | FE_CAN_QAM_64 | FE_CAN_TRANSMISSION_MODE_AUTO |
		FE_CAN_GUARD_INTERVAL_AUTO |
		FE_CAN_RECOVER
	},

	.release = s5h1432_release,
	
	.init = s5h1432_init,
	
	.set_frontend = s5h1432_set_frontend,
	.get_frontend = s5h1432_get_frontend,
	.get_tune_settings = s5h1432_get_tune_settings,
	
	.read_status = s5h1432_read_status,
	.read_ber = s5h1432_read_preber,
	.read_signal_strength = s5h1432_read_signal_strength,
	.read_snr = s5h1432_read_snr,
	.read_ucblocks = s5h1432_read_ucblocks,
};

module_param(debug, int, 0644);
MODULE_PARM_DESC(debug, "Turn on/off frontend debugging (default:off).");

MODULE_DESCRIPTION("WyplayAsia s5h1432 DVB-T Demodulator driver");
MODULE_AUTHOR("Hu Gang");
MODULE_LICENSE("GPL");

EXPORT_SYMBOL(s5h1432_attach);
