/*
    ST Microelectronics stv036x DVB OFDM demodulator driver

    Copyright (C) 2006 ST Microelectronics
	Peter Bennett <peter.bennett@st.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/string.h>
#include <linux/slab.h>
#include "dvb_frontend.h"


#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17)
#include <linux/stm/pio.h>
#else	/* STLinux 2.2 or older */
#include <linux/stpio.h>        /*!< for stpio_set_pin */
#endif

#include "stv036x.h"
#include <linux/time.h>
struct stv036x_state {
	struct stv036x_config* config;
    	u8 demod_address;
	struct i2c_adapter* i2c;
	struct dvb_frontend_ops ops;
	struct dvb_frontend frontend;
	u8 *init_tab;
	u8 type;
	u8 version;
};

/* #if 1 to set serial, #if 0 to set parallel */
#ifdef CONFIG_SH_ST_PDK7105
#define __SET_SERIAL
#endif
#ifdef CONFIG_SH_WYMDBOX_01
#warning CONFIG_SH_WYMDBOX_01 defined
#define __SWITCH_ON_POWER__
#endif

#define __SWITCH_ON_POWER__

#ifdef CONFIG_SH_WYGAIABOX_01
/*#warning CONFIG_SH_WYGAIABOX_01 defined*/
/*#define __SWITCH_ON_POWER__*/
#endif
static int debug = 1;
/*#define __TIME_DEBUG*/
#ifdef __TIME_DEBUG
struct timeval a,b,diff;
static int is_tune = 0;
static u32 l_frequency = 0;
# define timersub(a, b, result)                                               \
    do {                                                                  \
        (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;                 \
        (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;              \
        if ((result)->tv_usec < 0) {                                  \
            --(result)->tv_sec;                                   \
            (result)->tv_usec += 1000000;                         \
        }                                                             \
    } while (0)
#endif
#define dprintk(fmt,args...) \
	do { \
		if (debug) printk(KERN_DEBUG "%s: " fmt,__FUNCTION__, ## args); \
	} while (0)
#define wyprintk(fmt,args...) \
	do { \
		if (debug) printk(fmt,## args); \
	} while (0)

static u8 init_tab [] = {
R_ID            ,0x21, R_I2CRPT        ,0x53, R_TOPCTRL    ,0x2,  R_IOCFG0        ,0x40, R_DAC0R         ,0x00,
R_IOCFG1        ,0x40, R_DAC1R         ,0x00, R_IOCFG2     ,0x60, R_PWMFR         ,0x00, R_STATUS        ,0xf9,
R_AUX_CLK       ,0x1c, R_FREESYS1      ,0x00, R_FREESYS2   ,0x00, R_FREESYS3      ,0x00, R_AGC2MAX       ,0xff,
R_AGC2MIN       ,0x0,  R_AGC1MAX       ,0x0,  R_AGC1MIN    ,0x0,  R_AGCR          ,0xbc, R_AGC2TH        ,0xfe,
R_AGC12C3       ,0x00, R_AGCCTRL1      ,0x85, R_AGCCTRL2   ,0x1f, R_AGC1VAL1      ,0x00, R_AGC1VAL2      ,0x0,
R_AGC2VAL1      ,0x2,  R_AGC2VAL2      ,0x08, R_AGC2PGA    ,0x00, R_OVF_RATE1     ,0x00, R_OVF_RATE2     ,0x00,
R_GAIN_SRC1     ,0xce, R_GAIN_SRC2     ,0x10, R_INC_DEROT1 ,0x55, R_INC_DEROT2    ,0x4b, R_FREESTFE_1    ,0x03,
R_SYR_THR       ,0x1c, R_INR           ,0xff, R_EN_PROCESS ,0x1,  R_SDI_SMOOTHER  ,0xff, R_FE_LOOP_OPEN  ,0x00,
R_EPQ           ,0x11, R_EPQ2          ,0xd,  R_COR_CTL    ,0x20, R_COR_STAT      ,0xf6, R_COR_INTEN     ,0x00,
R_COR_INTSTAT   ,0x3d, R_COR_MODEGUARD ,0x3,  R_AGC_CTL    ,0x18, R_AGC_MANUAL1   ,0x00, R_AGC_MANUAL2   ,0x00,
R_AGC_TARGET    ,0x20, R_AGC_GAIN1     ,0x9,  R_AGC_GAIN2  ,0x10, R_ITB_CTL       ,0x00, R_ITB_FREQ1     ,0x00,
R_ITB_FREQ2     ,0x00, R_CAS_CTL       ,0x85, R_CAS_FREQ   ,0xB3, R_CAS_DAGCGAIN  ,0x10, R_SYR_CTL       ,0x00,
R_SYR_STAT      ,0x17, R_SYR_NC01      ,0x00, R_SYR_NC02   ,0x00, R_SYR_OFFSET1   ,0x00,R_SYR_OFFSET2   ,0x00,
R_FFT_CTL       ,0x00, R_SCR_CTL       ,0x00, R_PPM_CTL1    ,0x30, R_TRL_CTL       ,0x94,R_TRL_NOMRATE1  ,0xb0,
R_TRL_NOMRATE2  ,0x56, R_TRL_TIME1     ,0xee, R_TRL_TIME2   ,0xfd, R_CRL_CTL       ,0x4F,R_CRL_FREQ1     ,0x90,
R_CRL_FREQ2     ,0x9b, R_CRL_FREQ3     ,0x0,  R_CHC_CTL1     ,0xb1, R_CHC_SNR       ,0xd7,R_BDI_CTL       ,0x00,
R_DMP_CTL       ,0x00, R_TPS_RCVD1     ,0x32, R_TPS_RCVD2   ,0x2,  R_TPS_RCVD3     ,0x01,R_TPS_RCVD4     ,0x31,
R_TPS_CELLID    ,0x00, R_TPS_FREE2     ,0x00, R_TPS_SET1    ,0x01, R_TPS_SET2      ,0x02,R_TPS_SET3      ,0x4,
R_TPS_CTL       ,0x00, R_CTL_FFTOSNUM  ,0x2b, R_CAR_DISP_SEL,0x0C, R_MSC_REV       ,0x0A,R_PIR_CTL       ,0x00,
R_SNR_CARRIER1  ,0xA8, R_SNR_CARRIER2  ,0x86, R_PPM_CPAMP   ,0xcc, R_TSM_AP0       ,0x00,R_TSM_AP1       ,0x00,
R_TSM_AP2       ,0x00, R_TSM_AP3       ,0x00, R_TSM_AP4     ,0x00, R_TSM_AP5       ,0x00,R_TSM_AP6       ,0x00,
R_TSM_AP7       ,0x00, R_CONSTMODE     ,0x02, R_CONSTCARR1  ,0xD2, R_CONSTCARR2    ,0x04,R_ICONSTEL      ,0xf7,
R_QCONSTEL      ,0xf8, R_AGC1RF        ,0xff, R_EN_RF_AGC1  ,0x83, R_FECM          ,0x00,R_VTH0  ,0x1E,
R_VTH1          ,0x1e, R_VTH2          ,0x0F, R_VTH3        ,0x09, R_VTH4          ,0x00,R_VTH5  ,0x05,
R_FREEVIT       ,0x00, R_VITPROG       ,0x92, R_PR          ,0x2,  R_VSEARCH       ,0xb0,R_RS    ,0xfc, /*0xbc*/
R_RSOUT         /* 0x15*/,0x1f, R_ERRCTRL1      ,0x12, R_ERRCNTM1    ,0x00, R_ERRCNTL1      ,0x00,R_ERRCTRL2      ,0x32,
R_ERRCNTM2      ,0x00, R_ERRCNTL2      ,0x00, R_ERRCTRL3    ,0x12, R_ERRCNTM3      ,0x00,R_ERRCNTL3      ,0x00,
R_DILSTK1       ,0x00, R_DILSTK0       ,0x03, R_DILBWSTK1   ,0x00, R_DILBWST0      ,0x03,R_LNBRX ,0x80,
R_RSTC          ,0xB0, R_VIT_BIST      ,0x07, R_FREEDRS     ,0x00, R_VERROR        ,0x00,R_TSTRES        ,0x00,
R_ANACTRL       ,0x00, R_TSTBUS        ,0x00, R_TSTCK       ,0x00, R_TSTI2C        ,0x00,R_TSTRAM1       ,0x00,
R_TSTRATE       ,0x00, R_SELOUT        ,0x00, R_FORCEIN     ,0x00, R_TSTFIFO       ,0x00,R_TSTRS ,0x00,
R_TSTBISTRES0   ,0x00, R_TSTBISTRES1   ,0x00, R_TSTBISTRES2 ,0x00, R_TSTBISTRES3   ,0x00 ,0xff,0xff
};

static u8 init_tab_362[] = {
0x00d9,0x0f,
0x0000,0x40,
0x0001,0x02,
0x0002,0x00,
0x0003,0x00,
0x0004,0x00,
0x0005,0x00,
0x0006,0x00,
#ifdef __SWITCH_ON_POWER__
#warning __SWITCH_ON_POWER__ defined
0x0007,0xF8,
#else
0x0007,0x00,
#endif
0x0008,0x00,
0x0009,0xf9,
0x000a,0x0b,
0x000b,0x00,
0x000c,0x00,
0x000d,0x00,
0x0010,0xff,
0x0011,0x00,
0x0012,0xff,
0x0013,0x00,
0x0014,0xbc,
0x0015,0x00,
0x0016,0x40,
0x0017,0x85,
0x0018,0x18,
0x0019,0xff,
0x001a,0x0f,
0x001b,0xA5,
0x001c,0x09,
0x001d,0x00,
0x001e,0x00,
0x001f,0x00,
0x0020,0xFB,
0x0021,0xB8,
0x0022,0x55,
0x0023,0x33,
0x0024,0xbc,
0x0025,0x00,
0x0026,0x03,
0x0027,0x1c,
0x0028,0x7D,
0x0029,0xb1,
0x002a,0xff,
0x002b,0x00,
0x002c,0x00,
0x002d,0x00,
0x002e,0x00,
0x002f,0x00,
0x0030,0x00,
0x0031,0x01,
0x0032,0x01,
0x0033,0x01,
0x0034,0x02,
0x0035,0x23,
0x0036,0xf4,
0x0037,0x89,
0x0038,0x00,
0x0039,0x01,
0x0040,0x00,
0x0041,0x1e,
0x0042,0x14,
0x0043,0x0f,
0x0044,0x09,
0x0045,0x00,
0x0046,0x05,
0x0047,0x00,
0x0049,0x92,
0x004a,0xAF,
0x004b,0x30,
#ifdef __SET_SERIAL
0x004c,0xfc, //0xbc = parallel 0xfc = serial
#else
0x004c,0xbc, //0xbc = parallel 0xfc = serial
#endif
0x004d,0x05,
0x004e,0x12,
0x004f,0x00,
0x0050,0x00,
0x0051,0xb3,
0x0052,0x00,
0x0053,0x00,
0x0054,0x00,
0x0055,0x00,
0x0056,0x12,
0x0057,0x00,
0x0058,0x00,
0x0059,0x00,
0x005a,0x03,
0x005b,0x00,
0x005c,0x03,
0x005d,0x80,
0x005e,0xb0,
0x005f,0x07,
/*0x0060,0x80,*/
/*0x0061,0x00,*/
/*0x0062,0x00,*/
/*0x0063,0x0e,*/
/*0x0064,0xcc,*/
/*0x0065,0x0e,*/
/*0x0066,0xcc,*/
/*0x0067,0x00,*/
/*0x0068,0x00,*/
/*0x0069,0x36,*/
/*0x006a,0x47,*/
0x006b,0x01,
0x006c,0x25,
0x006d,0x00,
0x006e,0x00,
0x0070,0xf8,
0x0071,0x0d,
0x0072,0xc1,
/*0x0073,0x3b,*/
0x0073,0x3a,
0x0074,0x39,
0x0075,0x10,
/*0x0076,0x01,*/
0x0076,0x06,
/*0x0077,0x00,*/
0x0077,0x40,
0x0078,0xf4,
0x0079,0xf0,
/*0x007a,0x23,*/
0x007a,0x20,
0x007b,0x00,
0x007c,0x05,
0x007d,0x60,
0x007e,0xc0,
0x007f,0x01,
0x006f,0xec,
0x0080,0x20,
0x0081,0xf6,
0x0082,0x00,
0x0083,0x39,
0x0084,0x04,
0x0085,0x18,
0x0086,0x00,
0x0087,0x00,
0x0088,0x1e,
0x0089,0x97,
0x008a,0x1a,
0x008b,0x00,
0x008c,0x00,
0x008d,0x00,
0x008e,0x80,
0x008f,0xb3,
0x0090,0x0f,
0x0091,0x84,
0x0092,0x13,
0x0093,0x00,
0x0094,0x00,
0x0095,0x00,
0x0096,0x00,
0x0097,0x00,
0x0098,0x00,
0x0099,0x38,
0x009a,0x14,
0x009b,0xac,
0x009c,0x56,
0x009d,0x87,
0x009e,0xfd,
0x009f,0x4f,
0x00a0,0xa0,
0x00a1,0xb6,
0x00a2,0xff,
0x00a3,0x11,
0x00a4,0xee,
0x00a5,0x00,
0x00a6,0x00,
0x00a7,0x33,
0x00a8,0x02,
0x00a9,0x02,
0x00aa,0x30,
0x00ab,0x00,
0x00ac,0x00,
0x00ad,0x00,
0x00ae,0x02,
0x00af,0x02,
0x00b0,0x00,
0x00b1,0x2b,
0x00b2,0x09,
0x00b3,0x0a,
0x00b4,0x00,
0x00b5,0xa9,
0x00b6,0x86,
0x00b7,0x31,
0x00b8,0x00,
0x00b9,0x00,
0x00ba,0x00,
0x00bb,0x00,
0x00bc,0x00,
0x00bd,0x00,
0x00be,0x00,
0x00bf,0x00,
0x00c0,0x00,
0x00c1,0x00,
0x00c2,0x00,
0x00c6,0x00,
0x00cb,0x00,
0x00cc,0x00,
0x00cd,0x00,
0x00ce,0x00,
0x00cf,0x00,
0x00d0,0x02,
0x00d1,0x00,
0x00d2,0x00,
0x00d3,0x00,
0x00d4,0xe6,
0x00d5,0x81,
0x00d7,0x01,
0x00d8,0x00,
0x00da,0x10,
0x00db,0x08,
0x00dc,0x00,
0x00dd,0x00,
0x00de,0x00,
0x00df,0xe0,
0x0000,0x40,
0xff,0xff
};

static int stv036x_writereg (struct stv036x_state* state, u8 reg, u8 data)
{
	int ret  = 0;
	u8 buf [] = { reg, data };
	struct i2c_msg msg = { .addr = state->demod_address, .flags = 0, .buf = buf, .len = 2 };

	ret = i2c_transfer (state->i2c, &msg, 1); 

	if (ret != 1)
		printk("%s: error (addr == 0x%02x,reg == 0x%02x, val == 0x%02x, ret == %i)\n",
			__FUNCTION__, msg.addr,reg, data, ret);

	return (ret != 1) ? -1 : 0;
}

static int stv036x_readreg (struct stv036x_state* state, u8 reg)
{
	int ret;
	u8 b0 [] = { reg };
	u8 b1 [] = { 0 };
	struct i2c_msg msg [] = { { .addr = state->demod_address, .flags = 0, .buf = b0, .len = 1 },
			   { .addr = state->demod_address, .flags = I2C_M_RD, .buf = b1, .len = 1 } };

	//dprintk ("%s %x %x %x\n", __FUNCTION__,state->demod_address,reg,state->i2c);

	ret = i2c_transfer (state->i2c, msg, 1);
	ret |= i2c_transfer (state->i2c, msg+1, 1);

	if (ret != 1) {
		printk("%s: error (reg == 0x%02x,  ret == %i)\n",
			__FUNCTION__, reg, ret);
	  return -EIO;
	}
	return b1[0];
}

static int stv036x_set_inversion (struct stv036x_state* state, int inversion)
{
	u8 val;

	dprintk ("%s\n", __FUNCTION__);

	switch (inversion) {
	case INVERSION_AUTO:
	  return -EOPNOTSUPP;
	case INVERSION_ON:
	  val = stv036x_readreg (state, R_GAIN_SRC1);
		return stv036x_writereg (state, R_GAIN_SRC1, val | 0x80);
	case INVERSION_OFF:
		val = stv036x_readreg (state, R_GAIN_SRC1);
		return stv036x_writereg (state, R_GAIN_SRC1, val & ~0x80);
	default:
		return -EINVAL;
	}
}

static int stv036x_set_tps (struct stv036x_state *state, struct dvb_ofdm_parameters *p)
{
        u8 val;

	dprintk ("%s\n", __FUNCTION__);

	if ((p->code_rate_HP < FEC_1_2) || ((p->code_rate_HP > FEC_7_8) && (p->code_rate_HP != FEC_AUTO)))
		return -EINVAL;

	if ((p->code_rate_LP < FEC_1_2) || ((p->code_rate_LP > FEC_7_8) && (p->code_rate_LP != FEC_AUTO)))

	if ((p->code_rate_HP == FEC_4_5) || (p->code_rate_LP == FEC_4_5))
		return -EINVAL;

	if ((p->guard_interval < GUARD_INTERVAL_1_32) ||
	    ((p->guard_interval > GUARD_INTERVAL_1_4) && (p->guard_interval != GUARD_INTERVAL_AUTO)))
		return -EINVAL;

	if ((p->transmission_mode != TRANSMISSION_MODE_2K) &&
	    (p->transmission_mode != TRANSMISSION_MODE_8K) &&
	    (p->transmission_mode != TRANSMISSION_MODE_AUTO))
		return -EINVAL;

	if ((p->constellation != QPSK) &&
	    (p->constellation != QAM_16) &&
	    (p->constellation != QAM_64))
		return -EINVAL;

	if ((p->hierarchy_information < HIERARCHY_NONE) ||
	    (p->hierarchy_information > HIERARCHY_4))
		return -EINVAL;

#if 0
	if ((p->bandwidth < BANDWIDTH_8_MHZ) && (p->bandwidth > BANDWIDTH_6_MHZ))
		return -EINVAL;
#endif

#warning we currently only support 8MHz and 7MHz
	if ((p->bandwidth != BANDWIDTH_8_MHZ)&&(p->bandwidth != BANDWIDTH_7_MHZ)) 
        {
                printk("bandwidth is not supported (only 8MHz and 7MHz)");
                return -EINVAL;
        }

#if 0
	if (p->bandwidth == BANDWIDTH_7_MHZ)
		stv036x_writereg (state, 0x09, stv036x_readreg (state, 0x09 | 0x10));
	else
		stv036x_writereg (state, 0x09, stv036x_readreg (state, 0x09 & ~0x10));
#endif
#if 1
        {
                int com_n, com_rst;

                com_n = stv036x_readreg(state,R_COM_AGC_CFG);
                wyprintk("com_n = 0x%02X \n",com_n);
                stv036x_writereg(state,R_COM_AGC_CFG,0x7<<3|(com_n&0x7));
                com_rst = stv036x_readreg(state,R_DIG_AGC_R);
                stv036x_writereg(state,R_DIG_AGC_R,com_rst&0x3F);
                stv036x_writereg(state,R_DIG_AGC_R,com_rst|0xC0);
                stv036x_writereg(state,R_COM_AGC_CFG,com_n);
        }
#endif
        val  = (p->transmission_mode == TRANSMISSION_MODE_8K) ? (1<<2) : 0;
	val |= p->guard_interval & 3;

        /*    val |= 1<<4; |+ Force it added PB 20/11/2006 +|*/
        stv036x_writereg (state, R_COR_CTL, 0);

        stv036x_writereg (state, R_COR_MODEGUARD, val);
	//stv036x_writereg (state, R_CHP_TAPS,(p->transmission_mode == TRANSMISSION_MODE_8K) ? 0x3 : 0x1);

        /*    stv036x_writereg (state, R_COR_CTL, 0);*/

	// should be 5
        /*    msleep(5);*/

	stv036x_writereg (state, R_COR_CTL, (1<<5));

	if (p->transmission_mode == TRANSMISSION_MODE_2K)
                stv036x_writereg(state, R_CHP_TAPS, 0x1);
	else
                stv036x_writereg(state, R_CHP_TAPS, 0x3);
				    
	return 0;
}

static int stv036x_get_tps (struct stv036x_state* state, struct dvb_ofdm_parameters *p)
{
	static const fe_modulation_t qam_tab [3] = { QPSK, QAM_16, QAM_64 };
	static const fe_code_rate_t fec_tab [5] = { FEC_1_2, FEC_2_3, FEC_3_4,
						    FEC_5_6, FEC_7_8 };
	u8 val;
    wyprintk("%s\n",__FUNCTION__);

	if (!(stv036x_readreg(state, R_STATUS) & 0x80))  /*  tps valid? */
		return -EAGAIN;

	val = stv036x_readreg (state, R_TPS_RCVD2);

	if (((val>>4) & 0x7) > 3)
	  printk(KERN_ERR "%s: Invalid Hierarchy detected\n",__FUNCTION__);
	else
	  p->hierarchy_information = HIERARCHY_NONE + ((val>>4) & 0x7);

	if ((val & 0x3) > 2)
	  printk(KERN_ERR "%s: Invalid Modulation detected\n",__FUNCTION__);
	else
		p->constellation = qam_tab[val & 0x3];

	val = stv036x_readreg (state, R_TPS_RCVD3);

	if ((val & 0x07) > 4)
	  printk(KERN_ERR "%s: Invalid FEC rate detected\n",__FUNCTION__);
	else
	  p->code_rate_HP = fec_tab[val & 0x07];

	if (((val >> 4) & 0x07) > 4)
	  printk(KERN_ERR "%s: Invalid FEC rate detected\n",__FUNCTION__);
	else
	  p->code_rate_LP = fec_tab[(val>>4) & 0x07];

	val = stv036x_readreg (state, R_TPS_RCVD4);

	p->guard_interval = GUARD_INTERVAL_1_32 + ((val >> 4) & 0x3);
	p->transmission_mode = TRANSMISSION_MODE_2K + (val & 0x1);

	return 0;
}

#if 0
#define M8_F_GAIN_SRC_HI        (0x0A+1)
#define M8_F_GAIN_SRC_LO        (0xB4+4)

#define M8_F_TRL_NOMRATE0       (0x00+1)
#define M8_F_TRL_NOMRATE8_1     (0xAC+1)
#define M8_F_TRL_NOMRATE16_9    (0x56)

#define M8_GAIN_SRC_HI        	0x0B
#define M8_GAIN_SRC_LO        	0xB8 	/*For normal IF Mode*/ 

#define M8_E_GAIN_SRC_HI                0x08
#define M8_E_GAIN_SRC_LO                0x86    /* For long path IF / IQ modes */
#endif

static unsigned short int CellsCoeffs_8MHz[6][5]={
                                                {0x10EF,0xE205,0x10EF,0xCE49,0x6DA7},  /* CELL 1 COEFFS */
                                                {0x2151,0xc557,0x2151,0xc705,0x6f93},  /* CELL 2 COEFFS */
                                                {0x2503,0xc000,0x2503,0xc375,0x7194},  /* CELL 3 COEFFS */
                                                {0x20E9,0xca94,0x20e9,0xc153,0x7194},  /* CELL 4 COEFFS */
                                                {0x06EF,0xF852,0x06EF,0xC057,0x7207},  /* CELL 5 COEFFS */
                                                {0x0000,0x0ECC,0x0ECC,0x0000,0x3647}   /* CELL 6 COEFFS */
                                          } ;

static unsigned short int CellsCoeffs_7MHz[6][5]={
                {0x12CA,0xDDAF,0x12CA,0xCCEB,0x6FB1},  /* CELL 1 COEFFS */
                {0x2329,0xC000,0x2329,0xC6B0,0x725F},  /* CELL 2 COEFFS */
                {0x2394,0xC000,0x2394,0xC2C7,0x7410},  /* CELL 3 COEFFS */
                {0x251C,0xC000,0x251C,0xC103,0x74D9},  /* CELL 4 COEFFS */
                {0x0804,0xF546,0x0804,0xC040,0x7544},  /* CELL 5 COEFFS */
                {0x0000,0x0CD9,0x0CD9,0x0000,0x370A}   /* CELL 6 COEFFS */
              } ;

void bypass_360(struct dvb_frontend* fe, int n)
{
  struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
  if (n) 
  {
      stv036x_writereg (state, R_I2CRPT, 0x80);
  }
  else 
  {
      stv036x_writereg (state, R_I2CRPT, 0x0);
  }
}
void bypass_362(struct dvb_frontend* fe,int n)
{
  struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
  stv036x_writereg (state, R_I2CRPT, 0xFA);
}


#ifdef __TIME_DEBUG
void dump_chip(struct dvb_frontend* fe,int status)
{
  struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
  int val,reg;
  if(status)
  {
      printk("tune succeed\n");
  }
  else
  {
      printk("tune failed\n");
  }
  printk("dump chip :\n");
  printk("--------------------------------------------------------------------------------\n");
  printk("   |");
  for(reg = 0;reg <0x10;reg++)
  {
    printk("%02X ",reg);
  }
  printk("\n--------------------------------------------------------------------------------");
  for(reg = 0;reg<0xE0;reg++)
  {

     if(!(reg%0x10))
     {
         printk("\n   |");
     }
     val = stv036x_readreg(state,reg);
     printk("%02X ",val);
  }
  printk("\n--------------------------------------------------------------------------------\n");

}
#endif

static int stv036x_coeff_set(struct dvb_frontend* fe,unsigned short int CellsCoeffs[][5])
{
    struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
    int i,j;    
    for(i=1;i<=6;i++)
    {
        stv036x_writereg(state,R_IIR_CELL_NB,i-1);
        for(j=1;j<=5;j++)
        {
            stv036x_writereg(state,(R_IIR_CX_COEFF1_MSB+2*(j-1)),CellsCoeffs[i-1][j-1]>>8);
            stv036x_writereg(state,(R_IIR_CX_COEFF1_LSB+2*(j-1)),CellsCoeffs[i-1][j-1]);

        }

    }
/*#define ___DEBUG_CHECK_COEFF*/
#ifdef ___DEBUG_CHECK_COEFF
    //check Coef.
    for(i = 0;i<6;i++)
    {
        int reg_msb =0;
        int reg_lsb =0;
        stv036x_writereg(state,R_IIR_CELL_NB,i);
        reg_msb = stv036x_readreg(state,R_IIR_CX_COEFF1_MSB);
        reg_lsb = stv036x_readreg(state,R_IIR_CX_COEFF1_LSB);
        printk("0x%02X%02X ",reg_msb,reg_lsb);
        reg_msb = stv036x_readreg(state,R_IIR_CX_COEFF2_MSB);
        reg_lsb = stv036x_readreg(state,R_IIR_CX_COEFF2_LSB);
        printk("0x%02X%02X ",reg_msb,reg_lsb);
        reg_msb = stv036x_readreg(state,R_IIR_CX_COEFF3_MSB);
        reg_lsb = stv036x_readreg(state,R_IIR_CX_COEFF3_LSB);
        printk("0x%02X%02X ",reg_msb,reg_lsb);
        reg_msb = stv036x_readreg(state,R_IIR_CX_COEFF4_MSB);
        reg_lsb = stv036x_readreg(state,R_IIR_CX_COEFF4_LSB);
        printk("0x%02X%02X ",reg_msb,reg_lsb);
        reg_msb = stv036x_readreg(state,R_IIR_CX_COEFF5_MSB);
        reg_lsb = stv036x_readreg(state,R_IIR_CX_COEFF5_LSB);
        printk("0x%02X%02X ",reg_msb,reg_lsb);
        printk("\n");

    }
#endif

    return 0;
}



static int stv036x_iir_filter_init(struct dvb_frontend* fe,fe_bandwidth_t bandwidth)
{
    struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
    int ret = -1;

    stv036x_writereg(state,R_IIR_CELL_NB,0x00);
    switch(bandwidth)
    {
        case BANDWIDTH_8_MHZ:
            ret = stv036x_coeff_set(fe,CellsCoeffs_8MHz);
            break;
        case BANDWIDTH_7_MHZ:
            ret = stv036x_coeff_set(fe,CellsCoeffs_7MHz);
            break;
        default:
            break;
    }
    stv036x_writereg(state,R_IIR_CELL_NB,0x80);
    return ret;
}

static int stv036x_init (struct dvb_frontend* fe)
{
    struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
    int i=0;

	printk("init chip\n");
#ifdef CONFIG_SH_WYBOX_03
#warning CONFIG_SH_WYBOX03 defined
    if(state->demod_address == 0x1C)
    {
        printk("demod 1 --> ");
        struct stpio_pin *reset_ptr = NULL;
#if 0
        reset_ptr =  wybox_pio[PIO_TUNER_SEL];
        printk("switch to DVB-T \n");
        if(reset_ptr!=NULL)
        {
            stpio_set_pin(reset_ptr, 0);
        }
#else
        printk("switch to DVB-T \n");
        reset_ptr = stpio_request_set_pin(2, 3, "TUNER_SEL", STPIO_OUT, 0);
        stpio_free_pin(reset_ptr);
#endif
    }
    else
    {
        printk("demod 2 --> no need to switch\n");
    }
#endif
	while (!(state->init_tab[i] == 0xff && state->init_tab[i+1]==0xff))
	{
	  stv036x_writereg (state, state->init_tab[i], state->init_tab[i+1]);
	  i+=2;
	}

	stv036x_writereg(state, R_RS, (stv036x_readreg(state, R_RS) & ~(1<<6)) | 
			 ( state->config->serial_not_parallel ? (1<<6) : 0 ));
	if(state->type == 0)
    {
        state->config->bypass = bypass_360;
    }
    else if(state->type == 2)
    {
        state->config->bypass = bypass_362;
    }
    else
    {
        state->config->bypass = bypass_360;
    }

	if (state->config->pll_init) {
/*            stv036x_writereg (state, R_I2CRPT, 0x80);  |+ d1 open i2c bus switch +|*/
		state->config->pll_init(fe);
/*        stv036x_writereg (state, R_I2CRPT, 0x0);  |+ 53 close i2c bus switch +|*/
	}
	return 0;
}

static int stv036x_read_status(struct dvb_frontend* fe, fe_status_t* status)
{
	struct stv036x_state* state;
	u8 cellid;
        int loop = 1;
        int sync;
        
        state = (struct stv036x_state*) fe->demodulator_priv;
        wyprintk("%s\n",__FUNCTION__);
        wyprintk("********************************************\n");
        wyprintk("demod address 0x%02X\n", state->demod_address);
        cellid  = stv036x_readreg (state, R_TPS_CELLID);
        wyprintk("cellid lsb 0x%02X \n",cellid);
	cellid  = stv036x_readreg (state, R_TPS_CELLID+1);
        wyprintk("cellid msb 0x%02X \n",cellid);

begin:
        wyprintk("loop %d \n",loop);
	*status = 0;
	sync = stv036x_readreg (state, R_STATUS);
/*    printk("R_STATUS = 0x%02X\n",sync);*/
	if (sync & (1<<5))
        {
                wyprintk("FE_HAS_SIGNAL\n");
		*status |= FE_HAS_SIGNAL;
        }
        else
        {
        msleep(100);
        sync = stv036x_readreg (state, R_STATUS);
        if (sync & (1<<5))
        {
            wyprintk("FE_HAS_SIGNAL in retry\n");
            *status |= FE_HAS_SIGNAL;
        }
        else
        {
            wyprintk("FE_HAS_SIGNAL not found\n");
            goto end;
        }
    }

	if (sync & (1<<6))
    {
        wyprintk("FE_HAS_CARRIER\n");
		*status |= FE_HAS_CARRIER;
    }
    else
    {
        msleep(100);
        sync = stv036x_readreg (state, R_STATUS);
        if (sync & (1<<6))
        {
            wyprintk("FE_HAS_CARRIER in retry\n");
            *status |= FE_HAS_CARRIER;
        }
        else
        {
            wyprintk("FE_HAS_CARRIER not found\n");
            goto end;
        }
    }
    msleep(50);
#if __CHECK_NOISE
    int syr_stat = (stv036x_readreg(state,R_SYR_STAT)>>2)&0x3;
    printk("syr_stat =0x%02x \n",syr_stat);
    int ppm = stv036x_readreg (state, R_PPM_CPAMP_DIR);
    printk("ppm dir value = %d\n",ppm);
    ppm = stv036x_readreg (state, R_PPM_CPAMP_INV);
    printk("ppm inv value = %d\n",ppm);
    if(syr_stat == 0)
    {
/*        printk("expected 2K -> ppm>20\n");*/
        if(ppm<20)
        {
            printk("signal have noise\n");
/*            goto end;*/
        }
    }
    else
    {
/*        printk("expected 8K -> ppm>80\n");*/
        if(ppm<80)
        {
            printk("signal have noise\n");
/*            goto end;*/
        }
    }
#endif
	if (sync & (1<<7))
    {
        wyprintk("FE_HAS_SYNC\n");
		*status |= FE_HAS_SYNC;
    }
    else
    {
        msleep(260);
        sync = stv036x_readreg (state, R_STATUS);
        if (sync & (1<<7))
        {
            wyprintk("FE_HAS_SYNC in retry\n");
            *status |= FE_HAS_SYNC;
        }
        else
        {
            wyprintk("FE_HAS_SYNC not found\n");
/*            dump_chip(fe,0);*/
            goto end;
        }
    }
	if (sync & (1<<4))
    {
        wyprintk("FE_HAS_VITERBI\n");
		*status |= FE_HAS_VITERBI;  
    }
    else
    {
        msleep(144);
        sync = stv036x_readreg (state, R_STATUS);
        if (sync & (1<<4))
        {
            wyprintk("FE_HAS_VITERBI in retry\n");
            *status |= FE_HAS_VITERBI;  
        }
        else
        {
            wyprintk("FE_HAS_VITERBI not found\n");
            goto end;
        }
    }
    if (sync & (1<<3))
    {
        wyprintk("FE_HAS_LOCK\n");
		*status |= FE_HAS_LOCK;
    }
    else
    {
        msleep(300);
        sync = stv036x_readreg (state, R_STATUS);
        if (sync & (1<<3))
        {
            wyprintk("FE_HAS_LOCK in retry\n");
            *status |= FE_HAS_LOCK;
        }
        else
        {
            wyprintk("FE_HAS_LOCK not found\n");
            goto end;
        }
    }
#ifdef __TIME_DEBUG
    if(!is_tune)
    {
        do_gettimeofday(&b);
        timersub(&b, &a, &diff);
        printk("tune successful\n");
        printk("b time : [%u.%06u]\n",b.tv_sec, b.tv_usec);
        printk("tune in: [%u.%06u]\n",diff.tv_sec, diff.tv_usec);
        is_tune = 1;
/*        dump_chip(fe,1);*/
        
    }
#endif
    wyprintk("********************************************\n");
    wyprintk("-------------------> tune successful ? \n");
    wyprintk("********************************************\n");
    return 0;
end:
#if 0 // only need to do this when we found lock
    stat = stv036x_readreg(state, R_STATUS );

	if (stat & (1<<7)) {
	  int hp,lp,vth,punc;

	  hp = stv036x_readreg(state, R_TPS_RCVD3) & 0x7;
	  lp = (stv036x_readreg(state, R_TPS_RCVD3) >> 4) & 0x7;

	  /* Is it a LP or HP rate */
	  if ((stv036x_readreg(state, R_TPS_RCVD2) >> 4))
	    punc = lp; else punc = hp;

	  if (punc==4) hp = 5;

	  switch(punc)
	  {
	  case 0:
	      vth = stv036x_readreg(state, R_VTH0);
	      if (vth <60) stv036x_writereg(state,R_VTH0,45);
	      break;
	  case 1:
	      vth = stv036x_readreg(state, R_VTH1);
	      if (vth <40) stv036x_writereg(state,R_VTH1,30);
	      break;
	  case 2:
	      vth = stv036x_readreg(state, R_VTH2);
	      if (vth <30) stv036x_writereg(state,R_VTH2,23);
	      break;
	  case 3:
	      vth = stv036x_readreg(state, R_VTH3);
	      if (vth <18) stv036x_writereg(state,R_VTH3,14);
	      break;
	  case 5:
	      vth = stv036x_readreg(state, R_VTH5);
	      if (vth <10) stv036x_writereg(state,R_VTH5,8);
	      break;
	  }

	}
#endif
    if(!loop)
    {
        loop++;
        goto begin;
    }
    wyprintk("********************************************\n");
    wyprintk("-------------------> tune failed ?\n");
    wyprintk("********************************************\n");
	return 0;
}

static int stv036x_read_ber(struct dvb_frontend* fe, u32* ber)
{
	struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;

	*ber = stv036x_readreg(state, R_ERRCNTL2) | stv036x_readreg(state, R_ERRCNTM2) << 8;
	wyprintk("%s:\n", __FUNCTION__);

	return 0;
}

static int stv036x_read_signal_strength(struct dvb_frontend* fe, u16* signal_strength)
{
	*signal_strength = 0xdead;

	return 0;
}

static int stv036x_read_snr(struct dvb_frontend* fe, u16* snr)
{
	struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;

/*    *snr = stv036x_readreg(state, R_SNR_CARRIER1) | (stv036x_readreg(state, R_SNR_CARRIER2) & 0x1f) << 8;*/
        *snr = stv036x_readreg(state,R_CHC_SNR);
	dprintk("*snr = 0x%x\n", *snr);

	return 0;
}
#ifdef FE_READ_CELLID
#warning FE_READ_CELLID is defined
static int stv036x_read_cellid(struct dvb_frontend* fe,u16 *cellid)
{
	struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
	*cellid  = stv036x_readreg (state, R_TPS_CELLID)| (stv036x_readreg (state, R_TPS_CELLID+1)&0xFF)<<8;
    return 0;
}
#endif
static int stv036x_enable_high_lnb_voltage(struct dvb_frontend* fe,long arg )
{
	struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
    
	wyprintk("%s:\n", __FUNCTION__);
	if(arg == 0)
    {
        stv036x_writereg(state, R_IOCFG0, 0x0);
    }
    else
    {
        stv036x_writereg(state, R_IOCFG0, 0x40);
    }
    return 0;
}
static int stv036x_read_ucblocks(struct dvb_frontend* fe, u32* ucblocks)
{
	struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;

	*ucblocks = stv036x_readreg(state, R_ERRCNTL1) | stv036x_readreg(state, R_ERRCNTM1) << 8;
	wyprintk("%s:\n", __FUNCTION__);

	return 0;
}

#define M8_F_TRL_NOMRATE0       0x01            /* ok */
#define M8_F_TRL_NOMRATE8_1     0xAB    /* ok */
#define M8_F_TRL_NOMRATE16_9    0x56    /* ok */

#define M7_F_TRL_NOMRATE0       0x0     /* ok */
#define M7_F_TRL_NOMRATE8_1     0xD6    /* ok */
#define M7_F_TRL_NOMRATE16_9    0x4B    /* ok */

/*#define M8_GAIN_SRC_HI          0x0B*/
#define M8_GAIN_SRC_HI          0x58
/*#define M8_GAIN_SRC_LO          0xB8    |+For normal IF Mode+|*/
#define M8_GAIN_SRC_LO          0xCA    /*For normal IF Mode*/

#define M7_GAIN_SRC_HI          0xC
#define M7_GAIN_SRC_LO          0xE4    /*For normal IF Mode*/
static int stv036x_set_frontend(struct dvb_frontend* fe, struct dvb_frontend_parameters *p)
{
	struct stv036x_state* state = (struct stv036x_state*) fe->demodulator_priv;
	int ctrl = 0, gain = 0;
        int ret = 0;

        wyprintk("%s\n",__FUNCTION__);
#ifdef __TIME_DEBUG
        if(l_frequency != p->frequency)
        {
                l_frequency = p->frequency;
                is_tune = 0;
                do_gettimeofday(&a);
                printk("tune on %d\n",l_frequency);
                printk("a time : [%u.%06u]\n",a.tv_sec, a.tv_usec);
        }
#endif
        stv036x_writereg(state, R_TOPCTRL, 0x0);
	stv036x_writereg(state, R_FEPATH_CFG, 0x0);


	ctrl = stv036x_readreg(state, R_TRL_CTL);
        if(p->u.ofdm.bandwidth == BANDWIDTH_7_MHZ)
        {
                stv036x_writereg(state, R_TRL_CTL,      (ctrl & 0x7f)| (M7_F_TRL_NOMRATE0<<7));
                stv036x_writereg(state, R_TRL_NOMRATE1, M7_F_TRL_NOMRATE8_1);
                stv036x_writereg(state, R_TRL_NOMRATE2, M7_F_TRL_NOMRATE16_9);
        }
        else
        {
                stv036x_writereg(state, R_TRL_CTL,      (ctrl & 0x7f)| (M8_F_TRL_NOMRATE0<<7));
                stv036x_writereg(state, R_TRL_NOMRATE1, M8_F_TRL_NOMRATE8_1);
                stv036x_writereg(state, R_TRL_NOMRATE2, M8_F_TRL_NOMRATE16_9);
        }

        ret = stv036x_iir_filter_init(fe,p->u.ofdm.bandwidth);
        if(ret<0)
        {
                printk("error in iir_filter_init\n");
                return -EINVAL;
        }


	gain = stv036x_readreg(state, R_GAIN_SRC1);

        if(p->u.ofdm.bandwidth == BANDWIDTH_7_MHZ)
        {
                stv036x_writereg(state, R_GAIN_SRC1, (gain & 0xF0) |M7_GAIN_SRC_HI);
                stv036x_writereg(state, R_GAIN_SRC2, M7_GAIN_SRC_LO);
        }
        else
        {
                stv036x_writereg(state, R_GAIN_SRC1, (gain & 0xF0) |M8_GAIN_SRC_HI);
                stv036x_writereg(state, R_GAIN_SRC2, M8_GAIN_SRC_LO);
        }

/*    stv036x_writereg (state, R_I2CRPT, 0xd3);  |+ open i2c bus switch d3 +|*/
	ret = state->config->pll_set(fe, p);
        if(ret<0)
        {
                printk("error in pll set\n");
                return -EINVAL;
        }
/*    stv036x_writereg (state, R_I2CRPT, 0x53);   |+ close i2c bus switch 53 +|*/

	stv036x_writereg(state, R_VTH0,0x1E);
	stv036x_writereg(state, R_VTH1,0x14);
	stv036x_writereg(state, R_VTH2,0x0F);
	stv036x_writereg(state, R_VTH3,0x09);
        stv036x_writereg(state, R_VTH4,0x0);
	stv036x_writereg(state, R_VTH5,0x05);

	// it 0x4 should be last read value???
	//stv036x_writereg(state, R_PR, 0x4 | 0x30);
	stv036x_writereg(state, R_PR, 0xaf);

	stv036x_writereg(state, R_VSEARCH, 0x30);

	//stv036x_writereg(state, R_VSEARCH, 0x4 | 0x30);

	stv036x_writereg(state, R_PR, stv036x_readreg(state, R_PR) | 0x80);
	//msleep(5);

	stv036x_set_inversion (state, p->inversion);
	//msleep(5);

	stv036x_set_tps (state, &p->u.ofdm);

	//msleep(5);
	
	return 0;
}

static int stv036x_get_frontend(struct dvb_frontend* fe, struct dvb_frontend_parameters *p)
{
	struct stv036x_state* state;
	u8 reg;

        wyprintk("%s\n",__FUNCTION__);
        
        state = (struct stv036x_state*) fe->demodulator_priv;
	reg = stv036x_readreg (state, R_GAIN_SRC1);

	p->inversion = reg & 0x80 ? INVERSION_ON : INVERSION_OFF;
        
	return stv036x_get_tps (state, &p->u.ofdm);
}

static int stv036x_get_tune_settings(struct dvb_frontend* fe, struct dvb_frontend_tune_settings* fesettings)
{
        wyprintk("%s\n",__FUNCTION__);
/*        fesettings->min_delay_ms = 500;*/
/*        fesettings->step_size = 166667;*/
        fesettings->min_delay_ms = 250;
        fesettings->step_size = 0;
        fesettings->max_drift = 166667*2;
        
        return 0;
}

static void stv036x_release(struct dvb_frontend* fe)
{
	struct stv036x_state* state;
        
	wyprintk("stv036x_release \n");

        state = (struct stv036x_state*) fe->demodulator_priv;

        kfree(state);
}

static struct dvb_frontend_ops stv036x_ops;

struct dvb_frontend* stv036x_attach(const struct stv036x_config* config,
				    struct i2c_adapter* i2c, u8 demod_address)
{
	struct stv036x_state* state = NULL;
	int reg;

        wyprintk("stv036x_attach \n");
	/* allocate memory for the internal state */
	state = (struct stv036x_state*) kmalloc(sizeof(struct stv036x_state), GFP_KERNEL);
	if (state == NULL) goto error;

	/* setup the state */
	state->config = config;
	state->i2c = i2c;
	state->demod_address = demod_address;
	memcpy(&state->ops, &stv036x_ops, sizeof(struct dvb_frontend_ops));

	/* check if the demod is there */
	reg = stv036x_readreg(state, R_ID); 
        state->version = reg;
        dprintk("found %x at %x\n", reg,state->demod_address);
	if (reg != 0x10 && reg != 0x21  && reg != 0x41) goto error;

	if (reg == 0x10 || reg == 0x21) 
        {
                printk("STV36x detected\n");
                state->init_tab = init_tab;
                state->type = 0;
        }
	else 
        {
                printk("STV362 detected\n");
                state->init_tab = init_tab_362;
                state->type = 2;
        }

	/* create dvb_frontend */
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17)
	state->frontend.ops = state->ops;
	state->frontend.demodulator_priv = state;
#else
	state->frontend.ops = &state->ops;
	state->frontend.demodulator_priv = state;
#endif

#ifdef __SWITCH_ON_POWER__
#warning __SWITCH_ON_POWER__(2) defined
/*    Switch on the alimentation*/
    printk("Switch on the alimentation\n");
    stv036x_writereg(state,R_IOCFG2,0xF8);
#endif
	return &state->frontend;

error:
	if (state) kfree(state);
	return NULL;
}

static struct dvb_frontend_ops stv036x_ops = {

	.info = {
		.name 			= "ST STV036x DVB-T",
		.type 			= FE_OFDM,
		.frequency_min 		= 470000000,
		.frequency_max 		= 860000000,
                .frequency_stepsize 	= 166667,
		.caps = FE_CAN_FEC_1_2 | FE_CAN_FEC_2_3 | FE_CAN_FEC_3_4 |
		      FE_CAN_FEC_5_6 | FE_CAN_FEC_7_8 | FE_CAN_FEC_AUTO |
		      FE_CAN_QPSK | FE_CAN_QAM_16 | FE_CAN_QAM_64 | FE_CAN_TRANSMISSION_MODE_AUTO |
		      FE_CAN_GUARD_INTERVAL_AUTO |
	              FE_CAN_RECOVER
	},

	.release = stv036x_release,

	.init = stv036x_init,

	.set_frontend = stv036x_set_frontend,
	.get_frontend = stv036x_get_frontend,
	.get_tune_settings = stv036x_get_tune_settings,

	.read_status = stv036x_read_status,
	.read_ber = stv036x_read_ber,
	.read_signal_strength = stv036x_read_signal_strength,
	.read_snr = stv036x_read_snr,
	.read_ucblocks = stv036x_read_ucblocks,

        .enable_high_lnb_voltage = stv036x_enable_high_lnb_voltage,
#ifdef FE_READ_CELLID 
        .read_cellid = stv036x_read_cellid,
#endif
};

module_param(debug, int, 0644);
MODULE_PARM_DESC(debug, "Turn on/off frontend debugging (default:off).");

MODULE_DESCRIPTION("ST STV036x DVB-T Demodulator driver");
MODULE_AUTHOR("Peter Bennett");
MODULE_LICENSE("GPL");

EXPORT_SYMBOL(stv036x_attach);
