/* 
 *  Driver for Dummy Frontend 
 *
 *  Written by Emard <emard@softhome.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.=
 */    

#include <linux/version.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>

#include "dvb_frontend.h"
#include "wyvideocapture_dvb_fe.h"



struct wyvideocapture_dvb_fe_state {

	struct dvb_frontend_ops ops;

	struct dvb_frontend frontend;
};

static int wyvideocapture_dvb_fe_read_status(struct dvb_frontend* fe, fe_status_t* status)
	{
		*status = FE_HAS_SIGNAL
			| FE_HAS_CARRIER
			| FE_HAS_VITERBI
			| FE_HAS_SYNC
			| FE_HAS_LOCK;

	return 0;
	}

static int wyvideocapture_dvb_fe_read_ber(struct dvb_frontend* fe, u32* ber)
	{
		*ber = 0;
	return 0;
	}

static int wyvideocapture_dvb_fe_read_signal_strength(struct dvb_frontend* fe, u16* strength)
	{
	*strength = 0;
	return 0;
	}

static int wyvideocapture_dvb_fe_read_snr(struct dvb_frontend* fe, u16* snr)
	{
	*snr = 0;
	return 0;
	}

static int wyvideocapture_dvb_fe_read_ucblocks(struct dvb_frontend* fe, u32* ucblocks)
{
	*ucblocks = 0;
		return 0;
}

static int wyvideocapture_dvb_fe_get_frontend(struct dvb_frontend* fe, struct dvb_frontend_parameters *p)
{
		return 0;
}

static int wyvideocapture_dvb_fe_set_frontend(struct dvb_frontend* fe, struct dvb_frontend_parameters *p)
{
	return 0;
}

static int wyvideocapture_dvb_fe_sleep(struct dvb_frontend* fe)
{
		return 0;
}

static int wyvideocapture_dvb_fe_init(struct dvb_frontend* fe)
{
	return 0;
}

static int wyvideocapture_dvb_fe_set_tone(struct dvb_frontend* fe, fe_sec_tone_mode_t tone)
{
        return 0;
} 

static int wyvideocapture_dvb_fe_set_voltage(struct dvb_frontend* fe, fe_sec_voltage_t voltage)
{
	return 0;
}

static void wyvideocapture_dvb_fe_release(struct dvb_frontend* fe)
{
	struct wyvideocapture_dvb_fe_state* state = (struct wyvideocapture_dvb_fe_state*) fe->demodulator_priv;
	kfree(state);
}


static struct dvb_frontend_ops wyvideocapture_dvb_fe_ops;

struct dvb_frontend* wyvideocapture_dvb_fe_attach()
{
        struct wyvideocapture_dvb_fe_state* state = NULL;

        /* allocate memory for the internal state */
        state = (struct wyvideocapture_dvb_fe_state*) kmalloc(sizeof(struct wyvideocapture_dvb_fe_state), GFP_KERNEL);
        if (state == NULL) goto error;

        /* setup the state */
        memcpy(&state->ops, &wyvideocapture_dvb_fe_ops, sizeof(struct dvb_frontend_ops));

        /* create dvb_frontend */
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17)
        state->frontend.ops = state->ops;
        state->frontend.demodulator_priv = state;
#else
        state->frontend.ops = &state->ops;
        state->frontend.demodulator_priv = state;
#endif
        return &state->frontend;

error:
        if (state) kfree(state);
        return NULL;
}

static struct dvb_frontend_ops wyvideocapture_dvb_fe_ops = {

        .info = {
                .name                   = "WYPLAY VIDEO CAPTURE FRONTEND",
                .type                   = 0x10,
                .frequency_min          = 0,
                .frequency_max          = 863250000,
                .frequency_stepsize     = 62500,
                .caps = FE_CAN_FEC_1_2 | FE_CAN_FEC_2_3 | FE_CAN_FEC_3_4 |
                                FE_CAN_FEC_4_5 | FE_CAN_FEC_5_6 | FE_CAN_FEC_6_7 |
                                FE_CAN_FEC_7_8 | FE_CAN_FEC_8_9 | FE_CAN_FEC_AUTO |
                                FE_CAN_QAM_16 | FE_CAN_QAM_64 | FE_CAN_QAM_AUTO |
                                FE_CAN_TRANSMISSION_MODE_AUTO |
                                FE_CAN_GUARD_INTERVAL_AUTO |
                                FE_CAN_HIERARCHY_AUTO,
        },

        .release = wyvideocapture_dvb_fe_release,

        .init = wyvideocapture_dvb_fe_init,
        .sleep = wyvideocapture_dvb_fe_sleep,

        .set_frontend = wyvideocapture_dvb_fe_set_frontend,
        .get_frontend = wyvideocapture_dvb_fe_get_frontend,

        .read_status = wyvideocapture_dvb_fe_read_status,
        .read_ber = wyvideocapture_dvb_fe_read_ber,
        .read_signal_strength = wyvideocapture_dvb_fe_read_signal_strength,
        .read_snr = wyvideocapture_dvb_fe_read_snr,
        .read_ucblocks = wyvideocapture_dvb_fe_read_ucblocks,
};

MODULE_DESCRIPTION("WYPLAY Video Capture DVB Frontend");
MODULE_AUTHOR("Emard");
MODULE_LICENSE("LGPL");

EXPORT_SYMBOL(wyvideocapture_dvb_fe_attach);

