/* $Id: pdk7105-stv0362-dtt73200.c lfazio $ */

#include <linux/module.h>
#include <linux/platform_device.h>
#include "../include/stm-frontend.h"

static struct plat_frontend_channel frontend_channels[] = {
	{       
                .option            = STM_TSM_CHANNEL_1 | STM_DEMOD_360 | STM_TUNER_DTT73200 | STM_DISEQC_STONCHIP | STM_LNB_AUTO | STM_SERIAL_NOT_PARALLEL,
	 	.demod_i2c_bus     = 2,
		.demod_i2c_address = 0x1e,	/* Auto detect address */
		.demod_i2c_ad01    = 0x0,

		.tuner_i2c_bus     = 2,
		.tuner_i2c_address = 0x60,

		.lnb_i2c_bus       = 2,
		.lnb_i2c_address   = 0xff,

		.lock              = 0,
		.drop              = 0,

		.pio_reset_bank    = 0x3,
		.pio_reset_pin     = 0x3
	}
};


static struct plat_frontend_config frontend_config = {
	.diseqc_address       = 0x18068000,
	.diseqc_irq           = 0x81,
	.diseqc_pio_bank      = 5,
	.diseqc_pio_pin       = 5,

        /* From havana2-modules: stm/platform/platform_7105.h */
        .tsm_base_address     = 0xfe242000,

	.tsm_sram_buffer_size = 0xc00,
	.tsm_num_pti_alt_out  = 1,
	.tsm_num_1394_alt_out = 1,

	.nr_channels          = 1,
	.channels             = frontend_channels
};

static struct resource pti_resource = {
    /* From havana2-modules: stm/platform/platform_7105.h */
        .start                = 0xfe230000,
        .end                  = 0xfe240000 - 1,
	.flags                = IORESOURCE_MEM,
};

static struct platform_device frontend_device = {
	.name = "pti",
	.id = -1,
	.dev = {
		.platform_data = &frontend_config,
		},

	.num_resources = 1,
	.resource = &pti_resource,
};


static int __init frontend_init(void)
{
        int error;

	error = platform_device_register(&frontend_device);
        if (error != 0) {
                printk("PTI: Error %d\n", error);
        }
	return error;
}

static void __exit frontend_exit(void)
{
	platform_device_unregister(&frontend_device);
}

module_init(frontend_init);
module_exit(frontend_exit);

/* XXX: module needs to be GPL in order to use platform_device
 * registering functions */
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Gabriel FORTE <gforte@wyplay.com>");
MODULE_DESCRIPTION("platform_device for stv0362 registration on pdk7105");

