/* $Id: wybox-a-v0100-sate-v0200.c 785 2007-08-23 15:17:10Z gforte $ */

/* 
 * Platform setup for tuner daughter-boards on
 * WYBOX_A_MBRD_V0100 with WYBOX_A_SATE_V0200.
 */

/* the BSBE1 NIM is only supported in pti-2611. */
#define CONFIG_PTI_2611


#include <linux/module.h>
#include <linux/platform_device.h>
#include "../include/stm-frontend.h"


/* platform setup for PTI driver from 2.6.11.12-wybox branch */

static struct plat_frontend_channel frontend_channels[] = {
	/* DVB-S tuner 0 (NIM 0) */
	{ STM_TSM_CHANNEL_0 | STM_DEMOD_299 | STM_TUNER_STB6000 | STM_DISEQC_STONCHIP | STM_LNB_LNBH221  ,
		.demod_i2c_bus     = 0,
		.demod_i2c_address = 0x68,
		.demod_i2c_ad01    = 0x1,

		.tuner_i2c_bus     = 0,
		.tuner_i2c_address = 0xff,

		.lnb_i2c_bus       = 0,
		.lnb_i2c_address   = 0x08,

		.lock = 0,
		.drop = 0,

		.pio_reset_bank    = 0x2,
		.pio_reset_pin     = 0x4,
		.active            = 0
	},

#if 0
	/* DVB-S tuner 1 (NIM 1) */
	{ STM_TSM_CHANNEL_1 | STM_DEMOD_299 | STM_TUNER_STB6000 | STM_DISEQC_STONCHIP | STM_LNB_LNBH221  ,
		.demod_i2c_bus     = 0,
		.demod_i2c_address = 0x69,
		.demod_i2c_ad01    = 0x1,

		.tuner_i2c_bus     = 0,
		.tuner_i2c_address = 0xff,

		.lnb_i2c_bus       = 0,
		.lnb_i2c_address   = 0x09,

		.lock = 0,
		.drop = 0,

		.pio_reset_bank    = 0x2,
		.pio_reset_pin     = 0x5,
		.active            = 0
	}
#endif
};

static struct plat_frontend_config frontend_config = {
	.diseqc_address = 0x18068000,
	.diseqc_irq = 0x81,
	.diseqc_pio_bank = 5,
	.diseqc_pio_pin = 5,

	.tsm_base_address = 0x19242000,

	.tsm_sram_buffer_size = 0xc00,
	.tsm_num_pti_alt_out = 1,
	.tsm_num_1394_alt_out = 1,

	.nr_channels = 1,
	.channels = frontend_channels
};

static struct resource pti_resource = {
	.start = 0x19230000,
	.end = 0x19240000 - 1,
	.flags = IORESOURCE_MEM,
};

static struct platform_device frontend_device = {
	.name = "pti",
	.id = -1,
	.dev = {
		.platform_data = &frontend_config,
	},

	.num_resources = 1,
	.resource = &pti_resource,
};

static int __init wybox_setup_init(void)
{
	platform_device_register(&frontend_device);
	return 0;
}

static void __exit wybox_setup_exit(void)
{
	platform_device_unregister(&frontend_device);
}

module_init(wybox_setup_init);
module_exit(wybox_setup_exit);

/* XXX: module needs to be GPL in order to use platform_device
 * registering functions */
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Gabriel FORTE <gforte@wyplay.com>");
MODULE_DESCRIPTION("platform_devices for Wybox tuner NIMs registration");

