/*
 * ST-FEI DVB driver
 *
 * Copyright (c) 2006 ST Microelectroncs 
 *   
 *   Author: Peter Bennett <peter.bennett@st.com>
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation; either version 2 of
 *	the License, or (at your option) any later version.
 */
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/wait.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/delay.h>
#include <linux/time.h>
#include <linux/errno.h>
#include <asm/semaphore.h>

#include "dvb_frontend.h"
#include "dmxdev.h"
#include "dvb_demux.h"
#include "stv0299.h"
#include "stv0360.h"

#include <linux/dvb/frontend.h>
#include <linux/dvb/dmx.h>
#include <linux/pci.h>

#include <linux/interrupt.h>

#include <linux/stm/8010_fdma.h>

static int debug = 0;

module_param(debug, int, 0644);
MODULE_PARM_DESC(debug, "Turn on/off debugging (default:off).");

#define dprintk(x...) do { if (debug) printk(KERN_DEBUG x); } while (0)

#define STFEI_MAXCHANNEL 32

/* FEI STUFF START */
#define FEI_SOFT_RST 0x0
#define FEI_ENABLE 0x8
#define FEI_CONFIG   0x10
#define FEI_STATUS   0x18
#define FEI_STATUS_CLR 0x20
#define FEI_OUT_DATA 0x28
#define FEI_OUT_ERROR 0x30
#define FEI_SECTOR_SIZE 0x38

unsigned long fei1;

#define fei1_writereg( reg, val) \
  writel( val, reg + fei1)

#define fei1_readreg( reg) \
  readl( reg + fei1)


struct fdma_channel *fec_data;

/**
 *  since we're casting (struct ttusb*) <-> (struct dvb_demux*) around
 *  the dvb_demux field must be the first in struct!!
 */
struct stfei {
	struct dvb_demux dvb_demux;
	struct dmxdev dmxdev;

	/* our semaphore, for channel allocation/deallocation */
	struct semaphore sem;

	struct dvb_adapter *adapter;
	struct i2c_adapter i2c_adap;

	void *buffer;
	dma_addr_t dma_handle;

        int running_feed_count;

        spinlock_t timer_lock;
        struct timer_list timer;        /* timer interrupts for outputs */


	struct stfei_channel {
		struct stfei *stfei;
		struct dvb_demux_feed *dvbdmxfeed;

		int active;
		int id;
		int pid;
		int type;	/* 1 - TS, 2 - Filter */
	} channel[STFEI_MAXCHANNEL];

	struct dvb_frontend* fe;
};

struct stfei *stfei;
struct stfei *stfei2;

/* ugly workaround ... don't know why it's neccessary to read */

static int stfei_set_channel(struct stfei *stfei, int chan_id, int filter_type,
		      int pid)
{
	int err;
	/* SetChannel */

	return err;
}

static int stfei_del_channel(struct stfei *stfei, int channel_id)
{
	int err;
	/* DelChannel */

	return err;
}


/*****************************************************************************/

static struct stfei_channel *stfei_channel_allocate(struct stfei *stfei)
{
	int i;

#if 0
	if (down_interruptible(&stfei->sem))
		return NULL;
#endif

	/* lock! */
	for (i = 0; i < STFEI_MAXCHANNEL; ++i) {
		if (!stfei->channel[i].active) {
			stfei->channel[i].active = 1;
			//			up(&stfei->sem);
			return stfei->channel + i;
		}
	}

	//	up(&stfei->sem);

	return NULL;
}

static int feed = 0;

static int stfei_start_feed(struct dvb_demux_feed *dvbdmxfeed)
{
	struct stfei *stfei = (struct stfei *) dvbdmxfeed->demux;
	struct stfei_channel *channel;
	static int b=0;

	dprintk("stfei_start_feed\n");

	switch (dvbdmxfeed->type) {
	case DMX_TYPE_TS:
		break;
	case DMX_TYPE_SEC:
		break;
	default:
		return -EINVAL;
	}

	if (dvbdmxfeed->type == DMX_TYPE_TS) {
		switch (dvbdmxfeed->pes_type) {
		case DMX_TS_PES_VIDEO:
		case DMX_TS_PES_AUDIO:
		case DMX_TS_PES_TELETEXT:
		case DMX_TS_PES_PCR:
		case DMX_TS_PES_OTHER:
			channel = stfei_channel_allocate(stfei);
			break;
		default:
			return -EINVAL;
		}
	} else {
		channel = stfei_channel_allocate(stfei);
	}

	if (!channel)
		return -EBUSY;

	dvbdmxfeed->priv = channel;
	channel->dvbdmxfeed = dvbdmxfeed;

	channel->pid = dvbdmxfeed->pid;

	channel->type = 1;

	stfei_set_channel(stfei, channel->id, channel->type, channel->pid);

	if (!b) 
	  //if (0 == stfei->running_feed_count++)
	{
	  fdma_channel_start(fec_data); 
	  fei1_writereg (FEI_ENABLE, 0x1);
	  b++; 
	}
	fdma_fec_set_pid(fec_data,channel->pid,1);

#if 0
       
		stfei_start_iso_xfer(stfei);
#endif

	feed++;

	return 0;
}

static int stfei_stop_feed(struct dvb_demux_feed *dvbdmxfeed)
{
	struct stfei_channel *channel =
	    (struct stfei_channel *) dvbdmxfeed->priv;
	struct stfei *stfei = (struct stfei *) dvbdmxfeed->demux;

	feed--;

	fdma_fec_clear_pid(fec_data,channel->pid);

	stfei_del_channel(channel->stfei, channel->id);

#if 0
	if (--stfei->running_feed_count == 0)
	  fdma_channel_stop(fec_data);
#endif


	channel->active = 0;

	return 0;
}

static void stfei_disconnect(void *intf)
{
	struct stfei *stfei = intf;

	stfei->dvb_demux.dmx.close(&stfei->dvb_demux.dmx);
	//	dvb_net_release(&stfei->dvbnet);
	dvb_dmxdev_release(&stfei->dmxdev);
	dvb_dmx_release(&stfei->dvb_demux);
	if (stfei->fe != NULL) dvb_unregister_frontend(stfei->fe);
	i2c_del_adapter(&stfei->i2c_adap);
	dvb_unregister_adapter(stfei->adapter);

	kfree(stfei);

	dprintk("%s: STFEI DVB disconnected\n", __FUNCTION__);
}

#define PRIV fe->dvb->priv

#include "st-frontend.c"

static int
stm_i2c_attach(struct i2c_adapter *adapter)
{
    static int b = 0;
    u8 addr = 0x0;
    u8 buf[2];
    struct i2c_msg msg[] = { { .addr = 0x1c, .flags = 0, .buf = &addr, .len = 1 },
			     { .addr = 0x1c, .flags = I2C_M_RD, .buf = buf, .len = 1 } };
    
    dprintk("%s: here\n",__FUNCTION__);

    b = i2c_transfer(adapter,msg,1);
    b |= i2c_transfer(adapter,msg+1,1);

    if (b==1)
    {
      tmm360.tuner_i2c = adapter;
      stfei->fe = stv0360_attach(&tmm360,adapter,0x1e);

      if (stfei->fe)
	if (dvb_register_frontend(stfei->adapter,stfei->fe))
	  { printk(KERN_WARNING "%s: Failed attaching frontend\n",__FUNCTION__); return 0; }
    }

}

static int
stm_i2c_detatch(struct i2c_adapter *adapter)
{
  
}


static struct i2c_driver stm_i2c_driver = {
        .owner          = THIS_MODULE,
        .name           = "pti_i2c",
        .flags          = I2C_DF_NOTIFY,
        .attach_adapter = stm_i2c_attach,
        .detach_adapter = stm_i2c_detatch
};

#if 0
static irqreturn_t fei_irq(int irq, void *dev_id, struct pt_regs *regs)
{

}
#endif


static int callback(struct fdma_fec_buffer *buffer,struct stfei *stfei)
{
  int n;
  char *data = buffer->data;

  for (n=0;n<buffer->packet_count;n++)
    dvb_dmx_swfilter_packets(&stfei->dvb_demux,&data[n*192],1);

}

/* FEI stuff */

#include <linux/stpio.h>

struct stpio_pin *clk;
struct stpio_pin *data;
struct stpio_pin *packet;
struct stpio_pin *valid;

struct stpio_pin *nime;

static int stfei_setup(unsigned long address,int irq)
{
  int n;

  fei1 = ioremap(address,0x1000);

  if (!fei1) return -ENODEV;

  clk    = stpio_request_pin(3,0,"clk",STPIO_IN);
  data   = stpio_request_pin(3,2,"data",STPIO_IN);
  packet = stpio_request_pin(3,3,"packet",STPIO_IN);
  valid  = stpio_request_pin(3,1,"valid",STPIO_IN);
  
  nime  = stpio_request_pin(3,1,"nim_enable",STPIO_OUT);

  stpio_set_pin(nime,0);

  fei1_writereg (FEI_SOFT_RST, 0x0);
  fei1_writereg (FEI_ENABLE, 0x0);
  fei1_writereg (FEI_SOFT_RST, 0x1);
  fei1_writereg (FEI_ENABLE, 0x0);
  fei1_writereg (FEI_CONFIG, 0xE);
  fei1_writereg (FEI_SECTOR_SIZE, 192);  

  fei1_writereg (FEI_STATUS_CLR, (7<<8));

  fec_data = fdma_fec_setup(0,FEC1,FEI_OUT_DATA + address,callback,stfei);

  if (!fec_data)
  {
    printk("%s: Couldn't setup FDMA channel\n",__FUNCTION__);
    goto err2;
  }

#if 0
  if (request_irq(irq , fei_irq, SA_INTERRUPT , "fei1", fei1)) goto err1;
#endif

  return 0;

 err2:
  // add code to deassign pins clear up etc...

 err1:
  iounmap(address);
  return -ENODEV;
}

#ifndef CONFIG_HAVANA_USE_PORTWAY
static int __init stfei_init(void)
{
  void *write_to_decoder = NULL;
  void *private_data     = NULL;
  struct dvb_adapter *adapter = NULL;
#else
int stpti_attach(void *write_to_decoder,void *private_data,struct dvb_adapter *adapter)
{
#endif
	int err;
	int result;
	int channel;


	if (!(stfei = kmalloc(sizeof(struct stfei), GFP_KERNEL)))
                return -ENOMEM;

	stfei2 = stfei;

	dprintk("%s: Allocated stfei @ 0x%x\n",__FUNCTION__,stfei);

        memset(stfei, 0, sizeof(struct stfei));

        for (channel = 0; channel < STFEI_MAXCHANNEL; ++channel) {
                stfei->channel[channel].id    = channel;
                stfei->channel[channel].stfei = stfei;
		//stfei->channel[channel].havana_id = private_data;
        }

	if (!adapter)
	{
	  dvb_register_adapter(&stfei->adapter, "ST Generic FEI Driver", THIS_MODULE);
	  stfei->adapter->priv = stfei;
	  adapter = stfei->adapter;
	}

	stfei->adapter = adapter;

	memset(&stfei->dvb_demux, 0, sizeof(stfei->dvb_demux));

	stfei->dvb_demux.dmx.capabilities =
	  DMX_TS_FILTERING | DMX_SECTION_FILTERING/* | DMX_MEMORY_BASED_FILTERING*/;
	stfei->dvb_demux.priv = private_data;

	stfei->dvb_demux.filternum = 32;

	stfei->dvb_demux.feednum = STFEI_MAXCHANNEL;
	stfei->dvb_demux.start_feed = stfei_start_feed;
	stfei->dvb_demux.stop_feed = stfei_stop_feed;
	stfei->dvb_demux.write_to_decoder = write_to_decoder;
	//	stfei->dvb_demux.write_to_decoder = NULL;

	if ((result = dvb_dmx_init(&stfei->dvb_demux)) < 0) {
		printk("stfei_dvb: dvb_dmx_init failed (errno = %d)\n",
		       result);
		goto err;
	}
//FIXME dmxdev (nur WAS?)
	stfei->dmxdev.filternum = stfei->dvb_demux.filternum;
	stfei->dmxdev.demux = &stfei->dvb_demux.dmx;
	stfei->dmxdev.capabilities = 0;

	if ((result = dvb_dmxdev_init(&stfei->dmxdev, stfei->adapter)) < 0) {
		printk("stfei_dvb: dvb_dmxdev_init failed (errno = %d)\n",
		       result);
		dvb_dmx_release(&stfei->dvb_demux);
		goto err;
	}

	// Attach the correct front-end
	dprintk("%s: here we are\n",__FUNCTION__);
	//i2c_set_adapdata(&stfei_i2c_adap,&stfei);
	i2c_add_driver(&stm_i2c_driver);	

	stfei_setup(0x57000000,7);

	return 0;

 err:
	return result;

}

static void __exit stfei_exit(void)
{

}

#ifndef CONFIG_HAVANA_USE_PORTWAY
module_init(stfei_init);
module_exit(stfei_exit);
#else
EXPORT_SYMBOL(stpti_attach);
#endif

MODULE_AUTHOR("Peter Bennett <peter.bennett@st.com>");
MODULE_DESCRIPTION("STFEI DVB Driver");
MODULE_LICENSE("GPL");
