/*
 * STM TSMerger driver
 *
 * Copyright (c) STMicroelectronics 2006
 *
 *   Author:Peter Bennett <peter.bennett@st.com>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 */

#include <asm/io.h>

//#include <linux/stm/stm-frontend.h>
#include "../include/stm-frontend.h"
#include "tsmerger.h"

#include <linux/delay.h>

#ifndef TSM_NUM_PTI_ALT_OUT
unsigned long TSM_NUM_PTI_ALT_OUT;
unsigned long TSM_NUM_1394_ALT_OUT;
#define LOAD_TSM_DATA
#endif

unsigned long tsm_io;

#if 0
void stm_tsm_int ( void )
{
  int n;

  for (n=0;n<2;n++)
  {
    int status = readl(tsm_io + TSM_STREAM_STATUS(n));

    if (readl(tsm_io + TSM_STREAM_CONF(n)) & TSM_STREAM_ON) 
    {
      writel( readl(tsm_io + TSM_STREAM_CONF(n)) & ~TSM_STREAM_ON,
	      tsm_io + TSM_STREAM_CONF(n));

      udelay(500);

      printk(KERN_WARNING"%s: TSMerger RAM Overflow on channel %d(%x), deploying work around\n",__FUNCTION__,n,status);
      //writel( TSM_CHANNEL_RESET, tsm_io + TSM_STREAM_CONF2(n) );
      
      udelay(500);

      writel( readl(tsm_io + TSM_STREAM_CONF(n)) | TSM_STREAM_ON,
	      tsm_io + TSM_STREAM_CONF(n));
  }
  }
}
#endif

void stm_tsm_reset_channel ( int n, int status )
{
  if (status == -1)
    status = readl(tsm_io + TSM_STREAM_STATUS(n));

  writel( readl(tsm_io + TSM_STREAM_CONF(n)) & ~TSM_STREAM_ON,
	  tsm_io + TSM_STREAM_CONF(n));

  udelay(500);

  writel( status & ~TSM_RAM_OVERFLOW, tsm_io + TSM_STREAM_STATUS(n) );
  udelay(500);
  writel( TSM_CHANNEL_RESET, tsm_io + TSM_STREAM_CONF2(n) );

  udelay(500);

  writel( 0, tsm_io + TSM_STREAM_CONF2(n) );

  udelay(500);

  writel( readl(tsm_io + TSM_STREAM_CONF(n)) | TSM_STREAM_ON,
	  tsm_io + TSM_STREAM_CONF(n));

  printk("Resetting Channel %d\n",n);
}

void stm_tsm_interrupt ( void )
{
  int n;
  for (n=0;n<3;n++)
    {
      int status = readl(tsm_io + TSM_STREAM_STATUS(n));

      if (status & (TSM_RAM_OVERFLOW | TSM_INPUTFIFO_OVERFLOW ))
      {
	stm_tsm_reset_channel(n,status);
	printk(KERN_WARNING"%s: TSMerger RAM Overflow on channel %d(%x), deploying work around\n",__FUNCTION__,n,status);
      }
    }
}


int read_tsm(int a)
{
  return readl(tsm_io + TSM_STREAM_STATUS(a));
}

void stm_tsm_init ( struct plat_frontend_config *config )
{
  int n;
  unsigned long size;

  tsm_io = ioremap (config->tsm_base_address,0x1000);

#ifdef LOAD_TSM_DATA
  TSM_NUM_PTI_ALT_OUT  = config->tsm_num_pti_alt_out;
  TSM_NUM_1394_ALT_OUT = config->tsm_num_1394_alt_out;
#endif

  writel( 0x0, tsm_io + TSM_SW_RESET);
  writel( 0x6, tsm_io + TSM_SW_RESET);

  size = (config->tsm_sram_buffer_size / config->nr_channels) & ~31;

  for (n=0;n<5;n++)
    writel( TSM_RAM_ALLOC_START( 0x3 *n ), tsm_io + TSM_STREAM_CONF(n));

  for (n=0;n<config->nr_channels;n++)
  {
    enum plat_frontend_options options = config->channels[n].option; 
    int chan = STM_GET_CHANNEL(options);// / STM_TSM_CHANNEL_1) & 0xf;

    writel( readl(tsm_io + TSM_DESTINATION(0)) | (1 << chan), tsm_io + TSM_DESTINATION(0));

    writel( (readl(tsm_io + TSM_STREAM_CONF(chan)) & TSM_RAM_ALLOC_START(0xff)) |
	    (options & STM_SERIAL_NOT_PARALLEL ? TSM_SERIAL_NOT_PARALLEL : 0 ) |
	    (options & STM_INVERT_CLOCK        ? TSM_INVERT_BYTECLK : 0 ) |
	    (options & STM_PACKET_CLOCK_VALID  ? TSM_SYNC_NOT_ASYNC : 0 ) |
	    TSM_ALIGN_BYTE_SOP |
	    TSM_PRIORITY(0xf) | TSM_STREAM_ON | TSM_ADD_TAG_BYTES ,
	    tsm_io + TSM_STREAM_CONF(chan));

    writel( TSM_SYNC(config->channels[n].lock) |      
	    TSM_DROP(config->channels[n].drop) |
	    TSM_SOP_TOKEN(0x47)                |
	    TSM_PACKET_LENGTH(188) 
	    ,tsm_io + TSM_STREAM_SYNC(chan));
  }

  /* Put TSMERGER into normal mode */
  writel( TSM_CFG_BYPASS_NORMAL, tsm_io + TSM_SYSTEM_CFG);

}

void stm_tsm_release ( )
{
  iounmap( tsm_io );
}
