/*
 * ST Tuner driver
 *
 * Copyright (c) STMicroelectronics 2005
 *
 *   Author:Peter Bennett <peter.bennett@st.com>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 */

/* This adds 299 and/or 360 support with the various tuners ST uses */

#include <linux/version.h>
#include <linux/module.h>

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17)
#include <linux/stm/pio.h>
#else	/* STLinux 2.2 or older */
#include <linux/stpio.h>
#endif
#include <asm/io.h>

#ifdef CONFIG_DVB_HAVANA
/* #define CONFIG_DVB_STV0360 */
/* #define CONFIG_DVB_STV0299 */
/*#define CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION*/
#endif

#ifdef CONFIG_DVB_STV0360_MODULE
#define CONFIG_DVB_STV0360
#endif

#ifdef CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION_MODULE
#define CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION
#endif

#ifdef CONFIG_DVB_STV0299
#include "../frontends/stv0299.h"
#endif

#ifdef CONFIG_DVB_STV0360
#include "../frontends/stv036x.h"
#endif
//#define CONFIG_DVB_S5H1432
//#define CONFIG_DVB_LGS8G52
#ifdef CONFIG_DVB_LGS8G52
#include "../frontends/lgs8g52.h"
#endif
#ifdef CONFIG_DVB_S5H1432
#include "../frontends/s5h1432/s5h1432.h"
#endif
#include "../frontends/intel/mxl5007-ce6353.h"
#include "st-common.h"
#include "st-tuner.h"

#define READ_BIT2(array,offset,bitstart,bitsize)	\
    ((array[offset] >> bitstart) & ((1<<bitsize)-1))

#define WRITE_BIT2(array,offset,bitstart,bitsize,value) \
    array[offset] = (array[offset] & (~(((1<<bitsize)-1) << bitstart))) | (((value) & ((1<<bitsize)-1) )<< bitstart)

#define READ_BIT(array,pos) READ_BIT2(array,pos)
#define WRITE_BIT(array,pos,value) WRITE_BIT2(array,pos,value)


#ifdef CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION
#warning ""
#warning ""
#warning ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>> CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
#warning ""
#warning ""
#include "../frontends/wyvideocapture_dvb_fe.h"
#endif



/**** Internal structure for tuners *********************/

struct st_tuner {
        int type;    // 0 = None, 1 = i2c, 2 = spi (not supported yet)

        struct i2c_adapter *i2c_bus;

        u8     i2c_addr_demod;
        u8     i2c_addr_tuner;
        u8     i2c_addr_lnb;

        int srate;
};



/**** Tuner stuff ***************************************/

static u8 sharp_stem299_inittab[] = {
    0x00, 0xa1,
    0x01, 0x15,
    0x02, 0x00,   // 8 = serial clock, 0= none
    0x03, 0x00,
    0x04, 0x7d,             /* F22FR = 0x7d, F22 = f_VCO / 128 / 0x7d = 22 kHz */
    0x05, 0x05,             /* I2CT = 0, SCLT = 1, SDAT = 1 */
    0x06, 0x02,             /* DAC not used, set to high impendance mode */
    0x07, 0x00,             /* DAC LSB */
    0x08, 0x40,             /* DiSEqC off, LNB power on OP2/LOCK pin on */
    0x09, 0x00,             /* FIFO */

    0x0a, 0x82,
    0x0b, 0x00,
    0x0c, 0x51,             /* OP1 ctl = Normal, OP1 val = 1 (LNB Power ON) */
    //0x0c, 0x40,             /* OP1 ctl = Normal, OP1 val = 1 (LNB Power ON) */
    0x0d, 0x81,             /* DC offset compensation = ON, beta_agc1 = 2 */
    0x0e, 0x23,             /* alpha_tmg = 2, beta_tmg = 3 */
    0x0f, 0x1a,
    //0x0f, 0x9a,
    0x10, 0x34,             // AGC2  0x3d
    0x11, 0x84,
    0x12, 0xb9,
    0x13, 0x9b, // Lock detect: -64  Carrier freq detect:on

    0x14, 0x9e,
    0x15, 0xe3,             // lock detector threshold
    0x16, 0x80,
    0x17, 0x18,
    0x18, 0xff,
    0x19, 0xff,
    0x1a, 0x82,
    0x1b ,0x00,
    0x1c ,0x7f,
    0x1d ,0x00,

    0x1e,0x00,
    0x1f, 0x00,
    0x20, 0x00,
    0x21, 0x00,
    0x22, 0x80,
    0x23, 0x0b,
    0x24, 0x2b,
    0x25, 0x75,
    0x26, 0x1a,
    0x28, 0x00, // 2 = serial, 0 = parallel

    0x29, 0x1e,             // 1/2 threshold
    0x2a, 0x14,             // 2/3 threshold
    0x2b, 0x0f,             // 3/4 threshold
    0x2c, 0x09,             // 5/6 threshold
    0x2d, 0x05,             // 7/8 threshold
    0x31, 0x1f,             // test all FECs
    0x32, 0x19,             // viterbi and synchro search
    0x33, 0xfc,             // rs control
    0x34, 0x13,             // error control
    0x40, 0x00,

    0x41, 0x00,
    0x42, 0x00,
    0x43, 0x00,
    0x44, 0x00,
    0x45, 0x00,
    0x46, 0x00,
    0x47, 0x00,
    0x48, 0x00,
    0x49, 0x00,
    0x4a, 0x00,

    0x4b, 0x00,
    0x4c, 0x00,
    0x4d, 0x00,
    0x4e, 0x00,
    0x4f, 0x00,

    0xff, 0xff
};

static u8 stb6000_nim299_inittab_serial[] = {
    0x00, 0xa1,
    0x01, 0x15,
    0x02, 0x08,   // 8 = serial clock, 0= none
    0x03, 0x00,
    0x04, 0x7d,             /* F22FR = 0x7d, F22 = f_VCO / 128 / 0x7d = 22 kHz */
    0x05, 0x05,             /* I2CT = 0, SCLT = 1, SDAT = 1 */
    0x06, 0x02,             /* DAC not used, set to high impendance mode */
    0x07, 0x00,             /* DAC LSB */
    0x08, 0x40,             /* DiSEqC off, LNB power on OP2/LOCK pin on */
    0x09, 0x00,             /* FIFO */

    0x0a, 0x82,
    0x0b, 0x00,
    //0x0c, 0x51,             /* OP1 ctl = Normal, OP1 val = 1 (LNB Power ON) */
    0x0c, 0x40,             /* OP1 ctl = Normal, OP1 val = 1 (LNB Power ON) */
    0x0d, 0x81,             /* DC offset compensation = ON, beta_agc1 = 2 */
    0x0e, 0x23,             /* alpha_tmg = 2, beta_tmg = 3 */
    0x0f, 0x9a,
    //0x0f, 0x1a,
    0x10, 0x34,             // AGC2  0x3d
    0x11, 0x84,
    0x12, 0xb9,
    0x13, 0x9b, // Lock detect: -64  Carrier freq detect:on

    0x14, 0x9e,
    0x15, 0xe3,             // lock detector threshold
    0x16, 0x80,
    0x17, 0x18,
    0x18, 0xff,
    0x19, 0xff,
    0x1a, 0x82,
    0x1b ,0x00,
    0x1c ,0x7f,
    0x1d ,0x00,

    0x1e,0x00,
    0x1f, 0x00,
    0x20, 0x00,
    0x21, 0x00,
    0x22, 0x80,
    0x23, 0x0b,
    0x24, 0x2b,
    0x25, 0x75,
    0x26, 0x1a,
    0x28, 0x02, // 2 = serial, 0 = parallel

    0x29, 0x1e,             // 1/2 threshold
    0x2a, 0x14,             // 2/3 threshold
    0x2b, 0x0f,             // 3/4 threshold
    0x2c, 0x09,             // 5/6 threshold
    0x2d, 0x05,             // 7/8 threshold
    0x31, 0x1f,             // test all FECs
    0x32, 0x19,             // viterbi and synchro search
    0x33, 0xfc,             // rs control
    0x34, 0x13,             // error control
    0x40, 0x00,

    0x41, 0x00,
    0x42, 0x00,
    0x43, 0x00,
    0x44, 0x00,
    0x45, 0x00,
    0x46, 0x00,
    0x47, 0x00,
    0x48, 0x00,
    0x49, 0x00,
    0x4a, 0x00,

    0x4b, 0x00,
    0x4c, 0x00,
    0x4d, 0x00,
    0x4e, 0x00,
    0x4f, 0x00,

    0xff, 0xff
};

static u8 stb6000_nim299_inittab_parallel[] = {
    0x00, 0xa1,
    0x01, 0x15,
    0x02, 0x08, //0x08,   // 8 = serial clock, 0= none
    0x03, 0x00,
    0x04, 0x7d,             /* F22FR = 0x7d, F22 = f_VCO / 128 / 0x7d = 22 kHz */
    0x05, 0x05,             /* I2CT = 0, SCLT = 1, SDAT = 1 */
    0x06, 0x02,             /* DAC not used, set to high impendance mode */
    0x07, 0x00,             /* DAC LSB */
    0x08, 0x40,             /* DiSEqC off, LNB power on OP2/LOCK pin on */
    0x09, 0x00,             /* FIFO */

    0x0a, 0x82,
    0x0b, 0x00,
    //0x0c, 0x51,             /* OP1 ctl = Normal, OP1 val = 1 (LNB Power ON) */
    0x0c, 0x40,             /* OP1 ctl = Normal, OP1 val = 1 (LNB Power ON) */
    0x0d, 0x81,             /* DC offset compensation = ON, beta_agc1 = 2 */
    0x0e, 0x23,             /* alpha_tmg = 2, beta_tmg = 3 */
    0x0f, 0x9a, //0x15, //0x9a,
    //0x0f, 0x1a,
    0x10, 0x34,             // AGC2  0x3d
    0x11, 0x84,
    0x12, 0xb9,
    0x13, 0x9b, // Lock detect: -64  Carrier freq detect:on

    0x14, 0x9e,
    0x15, 0xe3,             // lock detector threshold
    0x16, 0x80,
    0x17, 0x18,
    0x18, 0xff,
    0x19, 0xff,
    0x1a, 0x82,
    0x1b ,0x00,
    0x1c ,0x7f,
    0x1d ,0x00,

    0x1e,0x00,
    0x1f, 0x00,
    0x20, 0x00,
    0x21, 0x00,
    0x22, 0x80,
    0x23, 0x0b,
    0x24, 0x2b,
    0x25, 0x75,
    0x26, 0x1a,
    0x28, 0x00, // 2 = serial, 0 = parallel

    0x29, 0x1e,             // 1/2 threshold
    0x2a, 0x14,             // 2/3 threshold
    0x2b, 0x0f,             // 3/4 threshold
    0x2c, 0x09,             // 5/6 threshold
    0x2d, 0x05,             // 7/8 threshold
    0x31, 0x1f,             // test all FECs
    0x32, 0x19,             // viterbi and synchro search
    0x33, 0xfc,             // rs control
    0x34, 0x13,             // error control
    0x40, 0x00,

    0x41, 0x00,
    0x42, 0x00,
    0x43, 0x00,
    0x44, 0x00,
    0x45, 0x00,
    0x46, 0x00,
    0x47, 0x00,
    0x48, 0x00,
    0x49, 0x00,
    0x4a, 0x00,

    0x4b, 0x00,
    0x4c, 0x00,
    0x4d, 0x00,
    0x4e, 0x00,
    0x4f, 0x00,

    0xff, 0xff
};

#ifdef CONFIG_DVB_STV0299

static int which_fe = 0;

static int stem299_read(struct dvb_frontend *fe)
{
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);

    u8 reg2[]     = { 0x0 };
    struct i2c_msg read[]    =  {{ .addr = st_tuner->i2c_addr_tuner, .flags = I2C_M_RD, .buf = reg2,  len: 1 }};

    if (i2c_transfer(st_tuner->i2c_bus, read, 1)!=1) printk("%s: Couldn't read the STEM PIO controller\n",__FUNCTION__);

    return reg2[0];
}

static int stem299_write(struct dvb_frontend *fe,u8 value)
{
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);

    u8 reg2[]     = { 0x0 };
    struct i2c_msg write[]   =  {{ .addr = st_tuner->i2c_addr_tuner, .flags = 0,        .buf = reg2,  len: 1 }};

    reg2[0] = value;

    if (i2c_transfer(st_tuner->i2c_bus, write, 1)!=1) dprintk("%s: Couldn't write the STEM PIO controller\n",__FUNCTION__); else return 0;

    return 1;
}

static int sharp_stem299_pll_init(struct dvb_frontend *fe)
{
    static unsigned char regs[] ={ 0x22,0x10,0x86,0x8a};
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);
    //  struct st_tuner* st_tuner = (struct st_tunner*)((struct stfe*)fe->dvb->priv)->tuner_data;
    struct i2c_msg write[] = {{ .addr = 0x60, .flags = 0, .buf = regs, len: 4 }};

    if (i2c_transfer(st_tuner->i2c_bus,write,1)!=1) printk("%s: Couldn't initilise SHARP HZ1184 PLL\n",__FUNCTION__); else return 0;

    return 1;
}

static u32 bandwidth;
static int last_buf[4] = { 0x22,0x10,0x86,0x8a };

static int sharp_stem299_pll_set (struct dvb_frontend *fe, struct dvb_frontend_parameters *params)
{
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);
    //  struct st_tuner* st_tuner = (struct st_tunner*)((struct stfe*)fe->dvb->priv)->tuner_data;

    u8 buf [4];
    u32 div;
    u32 swallow;
    u32 nbsteps;
    u32 stepsize;
    u32 freq = params->frequency;
    struct i2c_msg msg = {.addr = 0x60,.flags = 0,.buf = buf,.len = sizeof(buf) };

    stepsize = 2 * (4000000 / ( 1 << (last_buf[2] & 0x7)));
    nbsteps  = (freq * 1000) / (stepsize);
    div      = nbsteps / 32;
    swallow  = nbsteps % 32;

#if 1
    buf[0] = (last_buf[0] & 0x80)      | ((div >> 3) & 0x7f);
    buf[1] =                             ((div  & 0x7) << 5) | (swallow & 0x1f);
    buf[2] = (last_buf[2] & (~(3<<5))) | (((div >> 10) & 0x3) << 5);
    buf[3] = (last_buf[3] & ~2)        | ((freq < 1450000 ? 0x2:0x0));

    last_buf[0] = buf[0];
    last_buf[1] = buf[1];
    last_buf[2] = buf[2];
    last_buf[3] = buf[3];
#endif

    if (i2c_transfer(st_tuner->i2c_bus,&msg,1)!=1) { printk(KERN_WARNING "%s: Failed setting I2C\n",__FUNCTION__); return -EIO; }

    return 0;
}

static int sharp_stem299_set_symbol_rate(struct dvb_frontend *fe, u32 srate, u32 ratio) 
{
    unsigned long Result = 1;
    long MasterClock, n1, n2;
    int i = 0;
    u8 nsbuffer[3];
    //  struct st_tuner* st_tuner = (struct st_tunner*)((struct stfe*)fe->dvb->priv)->tuner_data;
    //struct i2c_msg msg = {.addr = 0x60,.flags = 0,.buf = last_buf,.len = sizeof(last_buf) };

    MasterClock = /*64000000;*/88000000;
    n1 = srate;
    n2 = MasterClock;
    while(i <= 20)   /* n1 > 0  */
    {
        if(n1<n2)
        {
            Result *= 2;
            n1 *= 2;
        }
        else
        {
            Result = Result * 2 + 1;
            n1 = (n1-n2) * 2;
        }
        i++;
    }

    /*Result = ST_TUNER_Util_BinaryFloatDiv(SymbolRate, MasterClock, 20);*/
    nsbuffer[2] = (u8)(Result & 0x0F)<<4;
    nsbuffer[1] = (u8)(Result>>4) & 0xFF;
    nsbuffer[0] = (u8)(Result>>12) & 0xFF;

    stv0299_writereg(fe,0x1f,nsbuffer[0]);
    stv0299_writereg(fe,0x20,nsbuffer[1]);
    stv0299_writereg(fe,0x21,nsbuffer[2]);

    if (srate < 20000000) bandwidth = 2; 
    else if (srate >= 60000000) bandwidth = 10;
    else bandwidth = (srate - 10000000)/5000000 + 1;

    last_buf[2] = (last_buf[2] & ~( 0x18 )) | ((bandwidth & 0x1) << 4) | (((bandwidth >> 1) & 0x1) << 3);
    last_buf[3] = (last_buf[3] & ~( 0x0c )) | (((bandwidth >> 2) & 0x1) << 3) | (((bandwidth >> 3) & 0x1) << 2);

    return 0;
}

static int stem299_set_tone(struct dvb_frontend* fe, fe_sec_tone_mode_t tone)
{
    //  struct stv0299_state* state = (struct stv0299_state*) fe->demodulator_priv;
    //  u8 val;

#if 0
    if (stv0299_wait_diseqc_idle (state, 100) < 0)
        return -ETIMEDOUT;
#endif

    switch (tone) {
        case SEC_TONE_ON:
            stem299_write(fe,stem299_read(fe) | (0x8 << which_fe));
            break;
        case SEC_TONE_OFF:
            stem299_write(fe,stem299_read(fe) & ~(0x8 << which_fe));
            break;
        default:
            return -EINVAL;
    }

    return 0;
}

static int stem299_set_voltage (struct dvb_frontend* fe, fe_sec_voltage_t voltage)
{
    //        struct stv0299_state* state = (struct stv0299_state*) fe->demodulator_priv;

    dprintk("%s: %s\n", __FUNCTION__,
            voltage == SEC_VOLTAGE_13 ? "SEC_VOLTAGE_13" : 
            voltage == SEC_VOLTAGE_18 ? "SEC_VOLTAGE_18" : "??");

    if (voltage == SEC_VOLTAGE_OFF) {
        return stem299_write(fe,stem299_read(fe) & ~(0x4 << which_fe));
    }

    switch (voltage) {
        case SEC_VOLTAGE_13:
            return stem299_write(fe,(stem299_read(fe) & ~(0x2<<which_fe)) | (0x4<<which_fe) );

        case SEC_VOLTAGE_18:
            return stem299_write(fe,stem299_read(fe) | ((0x4 | 0x2) << which_fe));
        default:
            return -EINVAL;
    };
}


static struct stv0299_config sharp_stem299_config = {
    .demod_address = 0x68,
    .inittab = sharp_stem299_inittab,
    .mclk = 88000000UL,
    .invert = 0,
    //        .enhanced_tuning = 0,
    .skip_reinit = 0,
    .lock_output = STV0229_LOCKOUTPUT_1,
    .volt13_op0_op1 = STV0299_VOLT13_OP0,
    .min_delay_ms = 100,
    .set_symbol_rate = sharp_stem299_set_symbol_rate,
    .pll_set = sharp_stem299_pll_set,
    .pll_init = sharp_stem299_pll_init
};

/* STB6000 */
static char          stb6000_enable[] = { 0x5,0xb5 };
static unsigned char buf_nim299[]     = { 0x01, 0xc1, 0x3a, 0xe3, 0x04, 0x07, 0x9f, 0xdf,0xd0, 0x50, 0xeb, 0x4f  };

static int stb6000_nim299_pll_init(struct dvb_frontend *fe)
{
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);

    struct i2c_msg write[] = {
        { .addr = st_tuner->i2c_addr_demod, .flags = 0, .buf =stb6000_enable , len: 2 },
        { .addr = st_tuner->i2c_addr_tuner, .flags = 0, .buf = buf_nim299, len: sizeof(buf_nim299) }};

    dprintk("%s: Entered\n",__FUNCTION__);

    if (i2c_transfer(st_tuner->i2c_bus,write,2)!=2) printk("%s: Couldn't initilise STB6000 PLL\n",__FUNCTION__); else return 0;

    return 1;
}

#define STB6000_VCO  1
#define STB6000_N    2
#define STB6000_A    3
#define STB6000_K_R  4
#define STB6000_G    5
#define STB6000_F    6
#define STB6000_FCL  7
#define STB6000_LPEN 0xA
#define STB6000_XOG  0xB

/* STB6000_VCO__ */
#define STB6000_VCO__OSM0 0
#define STB6000_VCO__OSM1 1
#define STB6000_VCO__OSM2 2
#define STB6000_VCO__OSM3 3
#define STB6000_VCO__ODIV 4
#define STB6000_VCO__OCK0 5
#define STB6000_VCO__OCK1 6
#define STB6000_VCO__OSCH 7

#define STB6000_VCO__OCK (3<<5)
#define STB6000_VCO__OSM  0x0f

/* STB6000_A__ */
#define STB6000_A__N0 7
#define STB6000_A__A   0x1f

/* STB6000_K_R__ */
#define STB6000_K_R__R 0x3f

/* STB6000_FCL__ */
#define STB6000_FCL__FCL 0x7

/* STB6000_F__ */
#define STB6000_F__F 0x1f

/* STB6000_LPEN__ */
#define STB6000_LPEN__LPEN 0x10

static unsigned int STB6000_LOOKUP[11][3]= {
    /* low         high         osm */
    {  950000,         1000000,        0xA},
    { 1000000,        1076000,        0xC},
    { 1076000,        1200000,        0x0},
    { 1200000,        1300000,        0x1},
    { 1300000,        1370000,        0x2},
    { 1370000,        1470000,        0x4},
    { 1470000,        1530000,        0x5},
    { 1530000,        1650000,        0x6},
    { 1650000,        1800000,        0x8},
    { 1800000,        1950000,        0xA},
    { 1950000,        2150000,        0xC} };

#define modify_bits(reg,clear,set) reg = (reg & (~(clear))) | (set)
#define range(x,y,z) (y>=x && y<z)

static int stb6000_nim299_pll_set (struct dvb_frontend *fe, struct dvb_frontend_parameters *params)
{
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);
    //struct st_tuner* st_tuner = (struct st_tunner*)((struct stfe*)fe->dvb->priv)->tuner_data;
    //static u32 bandwidth;

    u32 div;
    u32 swallow;
    u32 nbsteps;
    u32 stepsize;
    //int err,ret;
    u32 freq = params->frequency,Freq;// << (params->frequency <= 1075000 ? 1 : 0);
    unsigned char osm;
    int G;

    struct i2c_msg msg[] = {
        { .addr = st_tuner->i2c_addr_demod,  .flags = 0,.buf = stb6000_enable ,.len = 2 },
        { .addr = st_tuner->i2c_addr_tuner,.flags = 0,.buf = buf_nim299 ,.len = sizeof(buf_nim299) } };

    G=-1;

    if (st_tuner->srate <= 10000000) G = 10;
    else if (st_tuner->srate <= 20000000) G = 9;

    if (G>0) buf_nim299[STB6000_G] = G & 0x0f;

    modify_bits(buf_nim299[STB6000_VCO],
            1 << STB6000_VCO__ODIV, 
            (freq <= 1075000 ? 1 : 0) << STB6000_VCO__ODIV);

    osm = 0;
    while(!range(STB6000_LOOKUP[osm][0],freq,STB6000_LOOKUP[osm][1]) && (osm<10)) osm++;

    modify_bits(buf_nim299[STB6000_VCO],
            STB6000_VCO__OSM,
            STB6000_LOOKUP[osm][2] & STB6000_VCO__OSM);

    Freq     = freq << (((buf_nim299[STB6000_VCO] >> STB6000_VCO__ODIV) & 0x1) + 1);
    stepsize = 2*(4000000 / ((u32)(buf_nim299[STB6000_K_R] & STB6000_K_R__R)));
    nbsteps  = (Freq * 100) / (stepsize /10);
    div      = nbsteps / 16;
    swallow  = nbsteps % 16;

    modify_bits(buf_nim299[STB6000_A],
            STB6000_A__A | (1 << STB6000_A__N0),
            ((div & 1) << STB6000_A__N0) | (swallow & STB6000_A__A));

    modify_bits(buf_nim299[STB6000_N],
            0xff,
            (div>>1) & 0xff);

    /* PLL loop diable */
    modify_bits(buf_nim299[STB6000_LPEN],
            STB6000_LPEN__LPEN,
            0);

    /* VCO search enable, search clock on */
    modify_bits(buf_nim299[STB6000_VCO],
            STB6000_VCO__OCK | (1 << STB6000_VCO__OSCH),
            (1 << STB6000_VCO__OCK0) | (1 << STB6000_VCO__OSCH));

    if (i2c_transfer(st_tuner->i2c_bus,msg,2)!=2) { printk(KERN_WARNING "%s: Failed setting I2C-0\n",__FUNCTION__); return -EIO; }

    /* PLL loop enable */
    modify_bits(buf_nim299[STB6000_LPEN],
            STB6000_LPEN__LPEN,
            STB6000_LPEN__LPEN);  

    if (i2c_transfer(st_tuner->i2c_bus,msg,2)!=2) { printk(KERN_WARNING "%s: Failed setting I2C-1\n",__FUNCTION__); return -EIO; }

    msleep(20);

    /* VCO search disable, search clock off */
    modify_bits(buf_nim299[STB6000_VCO],
            STB6000_VCO__OCK | (1 << STB6000_VCO__OSCH),
            (3 << STB6000_VCO__OCK0) | (0 << STB6000_VCO__OSCH));

    if (i2c_transfer(st_tuner->i2c_bus,msg,2)!=2) { printk(KERN_WARNING "%s: Failed setting I2C-2\n",__FUNCTION__); return -EIO; }

    return 0;
}

static int stb6000_nim299_set_symbol_rate(struct dvb_frontend *fe, u32 srate, u32 ratio) 
{
    //unsigned long Result;
    //long MasterClock, n1, n2;
    //int i = 0;
    //u8 nsbuffer[3];
    //struct st_tuner* st_tuner = (struct st_tunner*)((struct stfe*)fe->dvb->priv)->tuner_data;
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);

    struct i2c_msg msg[] = {
        { .addr = st_tuner->i2c_addr_demod,.flags = 0,.buf = stb6000_enable ,.len = 2 },
        { .addr = st_tuner->i2c_addr_tuner,.flags = 0,.buf = buf_nim299 ,.len = sizeof(buf_nim299) } };
    //  struct i2c_msg msg = {.addr = 0x60,.flags = 0,.buf = buf_nim299,.len = sizeof(buf_nim299) };
    u8 aclk = 0;
    u8 bclk = 0;

    if (srate < 1500000) {
        aclk = 0xb7;
        bclk = 0x47;
    } else if (srate < 3000000) {
        aclk = 0xb7;
        bclk = 0x4b;
    } else if (srate < 7000000) {
        aclk = 0xb7;
        bclk = 0x4f;
    } else if (srate < 14000000) {
        aclk = 0xb7;
        bclk = 0x53;
    } else if (srate < 30000000) {
        aclk = 0xb6;
        bclk = 0x53;
    } else if (srate < 45000000) {
        aclk = 0xb4;
        bclk = 0x51;
    }


    //aclk = 0x99;
    //bclk = 0x97;
    //stv0299_writereg(fe, 0x13, aclk);
    //stv0299_writereg(fe, 0x14, bclk);

    stv0299_writereg(fe, 0x1f, (ratio >> 16) & 0xff);
    stv0299_writereg(fe, 0x20, (ratio >> 8) & 0xff);
    stv0299_writereg(fe, 0x21, (ratio) & 0xf0);

    mdelay(5);

    if((srate/2) > 36000000)
        bandwidth = 31;
    else bandwidth = (srate/2)/1000000;

    st_tuner->srate = srate;

    buf_nim299[STB6000_FCL] =  (buf_nim299[STB6000_FCL] & (~STB6000_FCL__FCL)) | 0;
    if (i2c_transfer(st_tuner->i2c_bus,msg,2)!=2) { printk(KERN_WARNING "%s: Failed 0 setting I2C\n",__FUNCTION__);    stv0299_writereg(fe, 0x05, 0x35);	return -EIO; }

    mdelay(10);

    buf_nim299[STB6000_F] =  0x80 | (bandwidth &  STB6000_F__F);
    if (i2c_transfer(st_tuner->i2c_bus,msg,2)!=2) { printk(KERN_WARNING "%s: Failed 1 setting I2C\n",__FUNCTION__);    stv0299_writereg(fe, 0x05, 0x35);	return -EIO; }

    mdelay(10);

    buf_nim299[STB6000_FCL] =  (buf_nim299[STB6000_FCL] & (~STB6000_FCL__FCL)) | 7;
    if (i2c_transfer(st_tuner->i2c_bus,msg,2)!=2) { printk(KERN_WARNING "%s: Failed 2 setting I2C\n",__FUNCTION__);    stv0299_writereg(fe, 0x05, 0x35);	return -EIO; }

    return 0;
}

static struct stv0299_config stb6000_nim299_config_serial = {
    .demod_address = 0x68,
    .inittab = stb6000_nim299_inittab_serial,
    .mclk = 88000000UL,
    .invert = 0,
    //        .enhanced_tuning = 0,
    .skip_reinit = 0,
    .lock_output = STV0229_LOCKOUTPUT_0,
    .volt13_op0_op1 = STV0299_VOLT13_OP0,
    .min_delay_ms = 100,
    .set_symbol_rate = stb6000_nim299_set_symbol_rate,
    .pll_set = stb6000_nim299_pll_set,
    .pll_init = stb6000_nim299_pll_init
};

static struct stv0299_config stb6000_nim299_config_parallel = {
    .demod_address = 0x68,
    .inittab = stb6000_nim299_inittab_parallel,
    .mclk = 88000000UL,
    .invert = 0,
    //        .enhanced_tuning = 0,
    .skip_reinit = 0,
    .lock_output = STV0229_LOCKOUTPUT_0,
    .volt13_op0_op1 = STV0299_VOLT13_OP0,
    .min_delay_ms = 100,
    .set_symbol_rate = stb6000_nim299_set_symbol_rate,
    .pll_set = stb6000_nim299_pll_set,
    .pll_init = stb6000_nim299_pll_init
};
#endif

static int lnbh221_write(struct dvb_frontend* fe, u8 val)
{
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);
    //  struct st_tuner* st_tuner = (struct st_tunner*)((struct stfe*)fe->dvb->priv)->tuner_data;
    u8 buf = val ;
    struct i2c_msg msg[] = {
        { .addr = st_tuner->i2c_addr_lnb,.flags = 0,.buf = &buf ,.len = 1 } };

    if (i2c_transfer(st_tuner->i2c_bus,msg,1)!=1) { printk(KERN_WARNING "%s: Failed setting I2C\n",__FUNCTION__); return -EIO; }

    return 0;
}

static int lnbh221_read(struct dvb_frontend* fe)
{
    struct stfe *stfe = (struct stfe*)(fe->dvb->priv);
    struct st_tuner *st_tuner = (struct st_tuner*)(stfe->tuner_data);
    //struct st_tuner* st_tuner = (struct st_tunner*)((struct stfe*)fe->dvb->priv)->tuner_data;
    u8 buf;
    struct i2c_msg msg[] = {
        { .addr = st_tuner->i2c_addr_lnb,.flags = I2C_M_RD,.buf = &buf ,.len = 1 } };

    if (i2c_transfer(st_tuner->i2c_bus,msg,1)!=1) printk(KERN_WARNING "%s: Failed setting I2C\n",__FUNCTION__);

    return buf;
}

static int lnbh221_set_tone(struct dvb_frontend* fe, fe_sec_tone_mode_t tone)
{
    //  struct stv0299_state* state = (struct stv0299_state*) fe->demodulator_priv;
    //u8 val;

#if 0
    if (stv0299_wait_diseqc_idle (state, 100) < 0)
        return -ETIMEDOUT;
#endif

    switch (tone) {
        case SEC_TONE_ON:
            return lnbh221_write(fe,lnbh221_read(fe) | 0x20);
            //stem299_write(fe,stem299_read(fe) | (0x8 << which_fe));
            break;
        case SEC_TONE_OFF:
            return lnbh221_write(fe,lnbh221_read(fe) & ~0x20);
            //stem299_write(fe,stem299_read(fe) & ~(0x8 << which_fe));
            break;
        default:
            return -EINVAL;
    }

    return 0;
}

static int lnbh221_set_voltage (struct dvb_frontend* fe, fe_sec_voltage_t voltage)
{
    //        struct stv0299_state* state = (struct stv0299_state*) fe->demodulator_priv;

    dprintk("%s: %s\n", __FUNCTION__,
            voltage == SEC_VOLTAGE_13 ? "SEC_VOLTAGE_13" : 
            voltage == SEC_VOLTAGE_18 ? "SEC_VOLTAGE_18" : 
            voltage == SEC_VOLTAGE_OFF ? "SEC_VOLTAGE_OFF" : "??");

    if (voltage == SEC_VOLTAGE_OFF) {
        return lnbh221_write(fe,lnbh221_read(fe) & ~0x20/*~0x48*/);
    }

    switch (voltage) {
        case SEC_VOLTAGE_13:
            return lnbh221_write(fe,(lnbh221_read(fe) & ~0x20) /*| 0x40*/ | 0x4);

        case SEC_VOLTAGE_18:
            return lnbh221_write(fe,(lnbh221_read(fe) & ~0x20)/*| 0x48*/ | 0x8 | 0x4);

        default:
            return -EINVAL;
    };

}


/*
   .diseqc_send_master_cmd = stv0299_send_diseqc_msg,
   .diseqc_send_burst = stv0299_send_diseqc_burst,
   */

static unsigned long diseqc_registers;
static volatile int diseqc_int_flag;

#define DSQ_TX_EN             ( diseqc_registers + 0x00 )
#define DSQ_TX_MSG_CFG        ( diseqc_registers + 0x04 )/* 00 [2:8] */
#define DSQ_TX_PRESCALER      ( diseqc_registers + 0x08 )/* 2 */
#define DSQ_TX_SUBCARR_DIV    ( diseqc_registers + 0x0C )/* 1511 */
#define DSQ_TX_SILENCE_PER    ( diseqc_registers + 0x10 )/* ?? 1000 */
#define DSQ_TX_DATA_BUF       ( diseqc_registers + 0x14 )
#define DSQ_TX_SYMBOL_PER     ( diseqc_registers + 0x18 )/* 33 */
#define DSQ_TX_SYMBOL0_ONTIME ( diseqc_registers + 0x1c )/* 11 */
#define DSQ_TX_SYMBOL1_ONTIME ( diseqc_registers + 0x20 )/* 11 */
#define DSQ_TX_SW_RST         ( diseqc_registers + 0x24 )
#define DSQ_TX_START          ( diseqc_registers + 0x28 )

#define DSQ_TX_INT_EN         ( diseqc_registers + 0x2c )
#define DSQ_TX_INT_STA        ( diseqc_registers + 0x30 )
#define DSQ_TX_STA            ( diseqc_registers + 0x34 )
#define DSQ_TX_CLR_INT_STA    ( diseqc_registers + 0x38 )
#define DSQ_TX_AGC            ( diseqc_registers + 0x40 )

static int stm_wait_diseqc_idle ( int timeout )
{
    unsigned long start = jiffies;
    int status;

    dprintk ("%s\n", __FUNCTION__);

    while ((status = readl(DSQ_TX_STA)) & 0x8 ) {
        if (jiffies - start > timeout) {
            dprintk ("%s: timeout!!\n", __FUNCTION__);
            return -ETIMEDOUT;
        }
        msleep(10);
    };

    return 0;
}


/*
   SEC_MINI_A = 0x00 = Unmodulated
   SEC_MINI_B = 0xff = Modulated
   */

static int stm_send_diseqc_burst (struct dvb_frontend* fe, fe_sec_mini_cmd_t burst)
{
    dprintk ("%s\n", __FUNCTION__);

    //fe->ops->set_voltage(fe, SEC_VOLTAGE_OFF);

    if (stm_wait_diseqc_idle ( 100) < 0)
        return -ETIMEDOUT;


    diseqc_int_flag = 0;

    switch(burst)
    {
        case SEC_MINI_A:
            writel ( 0x01, DSQ_TX_MSG_CFG );
            writel ( 0x01 | (0x09 << 9), DSQ_TX_MSG_CFG );
            break;
        case SEC_MINI_B:
            writel ( 0x01, DSQ_TX_MSG_CFG );
            writel ( 0x01 | (0x09 << 9), DSQ_TX_MSG_CFG );
            break;
    }

    writel( 1   , DSQ_TX_EN );
    writel( 0x9 , DSQ_TX_CLR_INT_STA);
    writel( 1   , DSQ_TX_START);
    writel( 0x9 , DSQ_TX_INT_EN);

    while ( !diseqc_int_flag ) { dprintk("%s: waiting for interrupt\n",__FUNCTION__); msleep(10); }

    writel( 0   , DSQ_TX_EN );

    return 0;
}

static int stm_diseqc_send_msg ( struct dvb_frontend* fe,
        struct dvb_diseqc_master_cmd *m)
{
    int n,i,j;
    int buffer;

    dprintk("%s: entered %x\n",__FUNCTION__,m->msg_len);

    writel(   22, DSQ_TX_SILENCE_PER );

    writel( 0                , DSQ_TX_MSG_CFG);
    writel( (m->msg_len) << 2, DSQ_TX_MSG_CFG);

    diseqc_int_flag = 0;

    for (n=0;n<m->msg_len;n+=4)
    {
        buffer = 0;

        for (j=0;j<4;j++)
        {
            unsigned char c = m->msg[j+n];
            unsigned char d = 0;
            for (i=0;i<8;i++)
            {
                if (i>4)
                    d |= ( c & ( 1 << i)) >> (2*i - 7);
                else
                    d |= ( c & ( 1 << i)) << (7 - 2*i);

            }

            buffer |= d << (j*8);
        }

        writel( buffer,  DSQ_TX_DATA_BUF );
    } 

    writel( 1   , DSQ_TX_EN );
    writel( 0x9 , DSQ_TX_CLR_INT_STA);
    writel( 1   , DSQ_TX_START);
    writel( 0x9 , DSQ_TX_INT_EN);

    while ( !diseqc_int_flag ) { dprintk("%s: waiting for interrupt\n",__FUNCTION__); msleep(10); }

    writel( 0   , DSQ_TX_EN );

    return 0;
}

static irqreturn_t stm_diseqc_irq(int irq, void *dev_id, struct pt_regs *regs)
{
    unsigned int status;

    status = readl( DSQ_TX_INT_STA );

    dprintk("%s: %x\n",__FUNCTION__,status);

    /* TX Ready */
    if ( status & 0x8 )
    {
        writel ( 0x8, DSQ_TX_CLR_INT_STA );
        writel ( readl( DSQ_TX_INT_EN ) &~ 0x8, DSQ_TX_INT_EN );
        diseqc_int_flag = 1;
    }

    return IRQ_HANDLED;
}

/* 0x18068000 */
int stm_diseqc_init ( struct plat_frontend_config *config )
{
    diseqc_registers = ioremap( config->diseqc_address,0x100 );	

    if (request_irq(config->diseqc_irq, stm_diseqc_irq ,SA_INTERRUPT , "Diseqc", NULL)) goto err1;

    stpio_request_pin( config->diseqc_pio_bank, config->diseqc_pio_pin,"DiseqC", STPIO_ALT_OUT); 

    writel(    2, DSQ_TX_PRESCALER );
    writel( 1511, DSQ_TX_SUBCARR_DIV );

    writel(   33, DSQ_TX_SYMBOL_PER );
    writel(   22, DSQ_TX_SYMBOL0_ONTIME );
    writel(   11, DSQ_TX_SYMBOL1_ONTIME );

    //writel(   22, DSQ_TX_SILENCE_PER );

    writel( 0   , DSQ_TX_SW_RST );
    writel( 0   , DSQ_TX_EN );
    writel( 0   , DSQ_TX_INT_EN );

    return 0;

err1:
    iounmap( diseqc_registers );
    return -ENODEV;
}

void stm_diseqc_register_frontend ( struct dvb_frontend *fe )
{
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17)
    fe->ops.diseqc_send_master_cmd = stm_diseqc_send_msg;
    fe->ops.diseqc_send_burst      = stm_send_diseqc_burst;
#else	/* STLinux 2.2 or older */
    fe->ops->diseqc_send_master_cmd = stm_diseqc_send_msg;
    fe->ops->diseqc_send_burst      = stm_send_diseqc_burst;
#endif
}

#ifdef CONFIG_DVB_STV0360
static int tmm360_pll_init(struct dvb_frontend* fe)
{
    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    char buf[] = {0x2,0x34,0x93,0x05};
    struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = buf, .len = sizeof(buf) };

    dprintk ("%s\n", __FUNCTION__);

    return i2c_transfer (config->tuner_i2c, &msg, 1);
}

static int dtt7300x_pll_init(struct dvb_frontend* fe)
{
    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    char buf[] = {0xb,0xf4,0x9c,0x20};
    struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = buf, .len = sizeof(buf) };

    dprintk ("%s\n", __FUNCTION__);
    config->bypass(fe,0);

    return i2c_transfer (config->tuner_i2c, &msg, 1);
}
static int dtt754xx_pll_init(struct dvb_frontend* fe)
{
    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    char buf[] = {0x05,0x88,0x88,0x12,0xD3};
    struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = buf, .len = sizeof(buf) };
    dprintk ("%s\n", __FUNCTION__);
    /*  return i2c_transfer (config->tuner_i2c, &msg, 1);*/
    return 1;
}
static int dtt73200_pll_init(struct dvb_frontend* fe)
{
    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    char buf[] = {0x05,0x88,0x88,0x12,0xD3};
    struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = buf, .len = sizeof(buf) };
    dprintk ("%s\n", __FUNCTION__);
    /*  return i2c_transfer (config->tuner_i2c, &msg, 1);*/
    return 1;
}

#ifdef CONFIG_DVB_LGS8G52
//add by hug start
static int sharp6093_pll_init(struct dvb_frontend* fe)
{
	struct lgs8g52_config* config = *((struct lgs8g52_config**) fe->demodulator_priv);
	char buf[] = {0x10,0x75,0xca,0xd4,0x84};
	struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = buf, .len = sizeof(buf) };
	dprintk ("%s\n", __FUNCTION__);

	/* return i2c_transfer (config->tuner_i2c, &msg, 1);  */
	return 1;
}//end of add by hug
#endif
static int tdqd3_pll_init(struct dvb_frontend* fe)
{
    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    u8 buf[] = {0x0B,0xF5,0xA0,0xEC,0xC6};
    struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = buf, .len = sizeof(buf) };

    dprintk ("%s\n", __FUNCTION__);

    return i2c_transfer (config->tuner_i2c, &msg, 1);
}

static int tmm360_pll_set(struct dvb_frontend* fe, struct dvb_frontend_parameters* params)
{
    u32 div;
    unsigned char bs = 0;
    unsigned char cp = 0;
    u8 pllbuf[4];
    int ret;
    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = pllbuf, .len = sizeof(pllbuf) };

#define IF_FREQUENCYx6 217    /* 6 * 36.16666666667MHz */
    div = (((params->frequency + 83333) * 3) / 500000) + IF_FREQUENCYx6;

    if (params->frequency < 542000000) cp = 0xb4;
    else if (params->frequency < 771000000) cp = 0xbc;
    else cp = 0xf4;

    if (params->frequency == 0) bs = 0x03;
    else if (params->frequency < 443250000) bs = 0x02;
    else bs = 0x08;

    // pllbuf[0] = 0xc0;
    pllbuf[0] = div >> 8;
    pllbuf[1] = div & 0xff;
    pllbuf[2] = cp;
    pllbuf[3] = bs;

    ret = i2c_transfer (config->tuner_i2c, &msg, 1);	

    if (ret!=1) printk("%s: error writting to PLL\n",__FUNCTION__);

    return ret;
}

#define DTT7300x_MSB 0,0,7
#define DTT7300x_LSB 1,0,8

#define DTT7300x_CP  2,6,1
#define DTT7300x_T   2,3,3
#define DTT7300x_RS  2,1,2

#define DTT7300x_P   3,0,5

#define DTT7300x_ATC 3,7,1
#define DTT7300x_AL  3,4,3

static int dtt7300x_pll_set(struct dvb_frontend* fe, struct dvb_frontend_parameters* params)
{
    u32 freq,frequency;
    u32 div;
    u8  pllbuf[] = {0x0,0x0,0x80,0x00};
    //u8  pllbuf[4];
    int CP,T,BW_AUX,P,RS,SS,ATC;
    int ret;

    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = pllbuf, .len = sizeof(pllbuf) } ;

#define DTT_7300X_IF_FREQUENCY 36000   /* 6 * 36.16666666667MHz */

    RS = 2;

    switch(RS)
    {
        case 0:
            SS = 50000;
            break;
        case 1:
            SS = 31250;
            break;
        case 2:
            SS = 166667;
            break;
        case 3:
            SS = 62500;
            break;
    };

    freq = params->frequency / 1000;
    frequency = freq + DTT_7300X_IF_FREQUENCY;
    div  = ((frequency * 1000) / ( SS ) );

    //printk("%d %d %d\n",div,params->frequency,SS);

    if (frequency <= 305000) {
        BW_AUX = 0x02;
        CP     = 0;
        T      = 0x6;
    } else if (frequency  <= 405000) {
        BW_AUX = 0x02;
        CP     = 0;
        T      = 0x7;
    } else if (frequency <= 445000) {
        BW_AUX = 0x02;
        CP     = 1;
        T      = 0x6;
    } else if (frequency <= 465000) {
        BW_AUX = 0x02;
        CP     = 1;
        T      = 0x7;
    } else if (frequency <= 735000) {
        BW_AUX = 0x08;
        CP     = 0;
        T      = 0x7;
    } else if (frequency <= 835000) {
        BW_AUX = 0x08;
        CP     = 1;
        T      = 0x6;
    } else if (frequency <= 896000) {
        BW_AUX = 0x08;
        CP     = 1;
        T      = 0x7;
    }

    P = 0x8; // 8Mhz

    //printk("freq:%d %d %d %d %d\n",frequency,freq,div,RS,CP);

    WRITE_BIT(pllbuf,DTT7300x_MSB,div >> 8);
    WRITE_BIT(pllbuf,DTT7300x_LSB,div & 0xff);

    WRITE_BIT(pllbuf,DTT7300x_CP,CP);
    WRITE_BIT(pllbuf,DTT7300x_T ,T);
    WRITE_BIT(pllbuf,DTT7300x_RS,RS);

    WRITE_BIT(pllbuf,DTT7300x_P,P);

    //printk("%s: %x %x %x %x\n",__FUNCTION__,pllbuf[0],pllbuf[1],pllbuf[2],pllbuf[3]);
    config->bypass(fe,0);
    ret = i2c_transfer (config->tuner_i2c, &msg, 1);	
    if (ret!=1) printk("%s: error writting to PLL 1\n",__FUNCTION__);

    WRITE_BIT(pllbuf,DTT7300x_T ,3);
    pllbuf[3] = 0xa0;


    config->bypass(fe,0);
    //printk("%s: %x %x %x %x\n",__FUNCTION__,pllbuf[0],pllbuf[1],pllbuf[2],pllbuf[3]);
    ret = i2c_transfer (config->tuner_i2c, &msg, 1);	
    if (ret!=1) printk("%s: error writting to PLL 2\n",__FUNCTION__);


    msleep(100);

    pllbuf[3] = 0x20;


    config->bypass(fe,0);
    //printk("%s: %x %x %x %x\n",__FUNCTION__,pllbuf[0],pllbuf[1],pllbuf[2],pllbuf[3]);
    ret = i2c_transfer (config->tuner_i2c, &msg, 1);	
    if (ret!=1) printk("%s: error writting to PLL 3\n",__FUNCTION__);

    return ret;
}

static int dtt754xx_pll_set(struct dvb_frontend* fe, struct dvb_frontend_parameters* params)
{

#define DTT754XX_MSG_LEN	((int)5) // Message lenght
/*#define DB1 0*/
/*#define DB2 1*/
#define CB1 2
#define BB  3
#define CB2 4

    u32 freq,frequency;
    u32 div = 0;//N value = Fosc/Fref
    u8  pllbuf[DTT754XX_MSG_LEN] = {0}; ///<! Initialize the buffers
    u8  lockbuf[] = {0};	
    int SS  = 0;	
    int ret = 0;


    ///<! JBL Message
    u8 ucBandSwitch   = 0x00;///<! BB  : CP1, CP2, 0,    P5,   BS4,  BS3, BS2, BS1
    u8 ucControlByte1 = 0x00;///<! CB1 : 1,   0  , ATP2, ATP1, ATP0, RS2, RS1, RS0
    u8 ucControlByte2 = 0xE3;///<! CB2 : 1,   1  , ATC,  STBY, 0,    0,   1,   XTO

    /* --  demodulator associate to the */
    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    struct dvb_ofdm_parameters *ofdm = &params->u.ofdm;

    ///<! JBL struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = pllbuf, .len = sizeof(pllbuf) } ;
    struct i2c_msg message [2] = { { .addr = config->tuner_address, .flags = 0, .buf = pllbuf, .len = sizeof(pllbuf) },/* Write */
        { .addr = config->tuner_address, .flags = I2C_M_RD, .buf = lockbuf, .len = 1 } };/* Read */
#define DTT_754XX_IF_FREQUENCY 36000   /*36MHz */

    SS = 166667;
    dprintk("tuner_address = 0x%02X\n",config->tuner_address);
    dprintk("frequency = %d\n",params->frequency);
    /*
     *	Calculate the divider ration byte 1 and byte 2: N = fosc/fref
     */	
    freq = params->frequency / 1000;
    frequency = freq + DTT_754XX_IF_FREQUENCY;
    div  = ((frequency * 1000) / ( SS ) ); 

    /*	
        Table 5: internal RF-AGC take over point + 
        Table 2: Ref division ration select (fref) */
    ucControlByte1 = 0x88; ///<! 109dB/uV fref = 166.67khz

    /*	Table 3 : Charge pump current, band select */
    /*    dprintk("%s: freq:%d \n",__FUNCTION__,freq);*/
    if (frequency <= 121000) 
    {
        ucBandSwitch = 0x05;//0x15; ///<! VL 83 ..121Mhz BS3 = "1"
    } 
    else if (frequency  <= 141000) 
    {
        ucBandSwitch = 0x45;//0x55;///<! VL 83 ..121Mhz BS3 = "1"
    } 
    else if (frequency <= 166000) 
    {
        ucBandSwitch = 0x85;//0x95;///<! VL 83 ..121Mhz BS3 = "1"
    } 
    else if (frequency <= 182000) 
    {
        ucBandSwitch = 0xC5;//0xD5;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else if (frequency <= 286000) 
    {
        ucBandSwitch = 0x06;//0x16;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else if (frequency <= 386000) 
    {
        ucBandSwitch = 0x46;//0x56;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else if (frequency <= 446000) 
    {
        ucBandSwitch = 0x86;//0x96;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else if (frequency <= 466000) 
    {
        ucBandSwitch = 0xC6;//0xD6;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else if (frequency <= 506000) 
    {
        ucBandSwitch = 0x0C;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else if (frequency <= 761000) 
    {
        ucBandSwitch = 0x4C;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else if (frequency <= 846000) 
    {
        ucBandSwitch = 0x8C;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else if (frequency <= 905000) 
    {
        ucBandSwitch = 0xCC;///<! VL 83 ..121Mhz BS3 = "1"
    }
    else
    {
        printk("%s: error writting to PLL 1\n",__FUNCTION__);
        return (-1);
    }
    
    if(ofdm->bandwidth == BANDWIDTH_7_MHZ)
    {
        ucBandSwitch|= 0x10;
    }
    else
    {
        if(ofdm->bandwidth != BANDWIDTH_8_MHZ)
        {
            printk("%s: error bandwidth is not supported %d\n",__FUNCTION__,ofdm->bandwidth);
            return (-1);
        }
    }

    //printk("freq:%d %d %d %d %d\n",frequency,freq,div,RS,CP);
#define DTT754xx_MSB 0,0,7
#define DTT754xx_LSB 1,0,8

    WRITE_BIT(pllbuf,DTT754xx_MSB,div >> 8); //DB1
    WRITE_BIT(pllbuf,DTT754xx_LSB,div & 0xff);///DB2	

    pllbuf[CB1] = ucControlByte1;
    pllbuf[BB]  = ucBandSwitch;
    pllbuf[CB2] = ucControlByte2;

    /*    printk("%s: DB1[%x] DB2[%x] CB1[%x] BB[%x] BB[%x]\n",__FUNCTION__,pllbuf[DB1],pllbuf[DB2],pllbuf[CB1],pllbuf[BB],pllbuf[CB2]);	*/

    ///<! Set new frequency
    config->bypass(fe,0);
    ret = i2c_transfer (config->tuner_i2c, &message[0], 1);	
    if (ret!=1) 
    {
        printk("%s: error writting to PLL High AGC : %d\n",__FUNCTION__,ret);
    }

    ///<! Wait pll lock
    msleep(80); ///<! 100ms	

    config->bypass(fe,0);
    pllbuf[CB2] = 0xC3;
    /*    printk("%s: DB1[%x] DB2[%x] CB1[%x] BB[%x] BB[%x]\n",__FUNCTION__,pllbuf[DB1],pllbuf[DB2],pllbuf[CB1],pllbuf[BB],pllbuf[CB2]);	*/
    ret = i2c_transfer (config->tuner_i2c, &message[0], 1);	
    if (ret!=1) 
    {
        printk("%s: error writting to PLL Low AGC : %d\n",__FUNCTION__,ret);
    }
    ///<! Read the lock bit
    config->bypass(fe,0);
    ret = i2c_transfer (config->tuner_i2c, &message[1], 1);
    if (ret!=1) 
    {
        printk("%s: error reading status : %d\n",__FUNCTION__,ret);	
    }


    ///<! Check if the pll is locked
    /*    printk("%s:lock:0x%02x\n",__FUNCTION__,lockbuf[0]);	 */

#define CAL_LOCK_BIT  (6)

    if( lockbuf[0]  & (1 << CAL_LOCK_BIT) )
    {
        dprintk("%s:LOCKED 0x%02x \n",__FUNCTION__,lockbuf[0]);	
    }
    else
    {
        dprintk("%s:NOT LOCKED 0x%02x \n",__FUNCTION__,lockbuf[0]);	

    }


    return ret;

}
static int dtt73200_pll_set(struct dvb_frontend* fe, struct dvb_frontend_parameters* params)
{
#define DTT73200_MSG_LEN	((int)6) // Message lenght
/*#define DB1 0*/
/*#define DB2 1*/
#define CD1 2
#define CD2 3
#define CD3 4
#define CD4 5
#define DTT_73200_IF_FREQUENCY 36000   /*36MHz */
#define DTT73200_MSB 0,0,7
#define DTT73200_LSB 1,0,8

    u32 freq,frequency;
    u32 div = 0;//N value = Fosc/Fref
    u8  pllbuf[DTT73200_MSG_LEN] = {0}; ///<! Initialize the buffers
    u8  lockbuf[] = {0};	
    int SS  = 0;	
    int ret = 0;


    /* --  demodulator associate to the */
    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    struct dvb_ofdm_parameters *ofdm = &params->u.ofdm;

/*    JBL struct i2c_msg msg = { .addr = config->tuner_address, .flags = 0, .buf = pllbuf, .len = sizeof(pllbuf) } ;*/
    struct i2c_msg message [2] = { { .addr = config->tuner_address, .flags = 0, .buf = pllbuf, .len = sizeof(pllbuf) },/* Write */
        { .addr = config->tuner_address, .flags = I2C_M_RD, .buf = lockbuf, .len = 1 } };/* Read */

    SS = 166667;
/*    printk("tuner_address = 0x%02X\n",config->tuner_address);*/
/*    printk("frequency = %d\n",params->frequency);*/
    
/*    Calculate the divider ration byte 1 and byte 2: N = fosc/fref*/
     	
    freq = params->frequency / 1000;
    frequency = freq + DTT_73200_IF_FREQUENCY;
    div  = ((frequency * 1000) / ( SS ) ); 

    WRITE_BIT(pllbuf,DTT73200_MSB,div >> 8); //DB1
    WRITE_BIT(pllbuf,DTT73200_LSB,div & 0xff);//DB2	


    if((frequency>=209000)&&(frequency <= 236000))
    {
/*        VHF III 209-236MHz*/
        pllbuf[CD1] = 0xB3;
        pllbuf[CD2] = 0x39;

    }
    else if((frequency>236000)&&(frequency <= 263000))
    {
/*        VHF III 236-263MHz*/
        pllbuf[CD1] = 0xD3;
        pllbuf[CD2] = 0x39;
    }
    else if((frequency>=506000)&&(frequency <= 536000))
    {
/*        UHF 506-536MHz*/
        pllbuf[CD1] = 0xB3;
        pllbuf[CD2] = 0x30;
    }
    else if((frequency>536000)&&(frequency <= 811000))
    {
/*        UHF 506-536MHz*/
        pllbuf[CD1] = 0xD3;
        pllbuf[CD2] = 0x30;
    }
    else if((frequency>811000)&&(frequency <= 898000))
    {
/*        UHF 506-536MHz*/
        pllbuf[CD1] = 0xF3;
        pllbuf[CD2] = 0x30;
    }
    else
    {
        printk("%s: error frequency is not supported %d\n",__FUNCTION__,freq);
        return -1;
    }


    if(ofdm->bandwidth == BANDWIDTH_8_MHZ)
    {
        pllbuf[CD2] |= 0x80;
    }
    else if(ofdm->bandwidth == BANDWIDTH_7_MHZ)
    {
        pllbuf[CD2] |= 0x40;
    }
    else
    {
        printk("%s: error bandwidth is not supported %d\n",__FUNCTION__,ofdm->bandwidth);
        return -1;
    }


    pllbuf[CD3] = 0x51; //channel select/search

    pllbuf[CD4] = 0x90; //normal mode

/*    printk("%s: %x %x %x %x %x %x\n",__FUNCTION__,pllbuf[0],pllbuf[1],pllbuf[2],pllbuf[3],pllbuf[4],pllbuf[5]);	*/

/*    Set new frequency*/
    config->bypass(fe,0);
    ret = i2c_transfer (config->tuner_i2c, &message[0], 1);	
    if (ret!=1) 
    {
        printk("%s: error writting to PLL High AGC : %d\n",__FUNCTION__,ret);
    }

/*    Wait pll lock*/
    msleep(100); ///<! 100ms	

    pllbuf[CD3] = 0x71; //channel lock
    config->bypass(fe,0);
    ret = i2c_transfer (config->tuner_i2c, &message[0], 1);	
    if (ret!=1) 
    {
        printk("%s: error writting to PLL Low AGC : %d\n",__FUNCTION__,ret);
    }
/*    Read the lock bit*/
    config->bypass(fe,0);
    ret = i2c_transfer (config->tuner_i2c, &message[1], 1);
    if (ret!=1) 
    {
        printk("%s: error reading status : %d\n",__FUNCTION__,ret);	
    }


/*    Check if the pll is locked*/
/*    printk("%s:lock:0x%02x\n",__FUNCTION__,lockbuf[0]);	 */

#define CAL_LOCK_BIT  (6)

    if( lockbuf[0]  & (1 << CAL_LOCK_BIT) )
    {
        dprintk("%s:LOCKED 0x%02x \n",__FUNCTION__,lockbuf[0]);	
    }
    else
    {
        dprintk("%s:NOT LOCKED 0x%02x \n",__FUNCTION__,lockbuf[0]);	

    }

    return ret;
}

#ifdef CONFIG_DVB_LGS8G52
//add by hug start
static int sharp6093_pll_set(struct dvb_frontend* fe, struct dvb_frontend_parameters* params)
{
	u8 pllbuf[5] = {0};
	u8 lockbuf[] = {0};
	u8 cb3a, cb3b;
	u32 freq, frequency;
	u32 tunerdivi;
	int ret = 0;

#define CB1		2
#define CB2		3
#define CB3		4

	struct lgs8g52_config* config = *((struct lgs8g52_config ** ) fe->demodulator_priv);
	struct i2c_msg message[2] = { { .addr = config->tuner_address, .flags = 0, .buf = pllbuf, .len = sizeof(pllbuf) }, /* write */
				{ .addr = config->tuner_address, .flags = I2C_M_RD, .buf = lockbuf, .len = 1 } }; /*read */

	dprintk(KERN_INFO "current frequency is %d\n", params->frequency);
	printk(KERN_INFO "current frequency is %d\n", params->frequency);
	freq = params->frequency / 1000;
	frequency = freq;

	if (frequency < 50500)
	{
		printk ("%s: error frequency is not supported %d\n", __FUNCTION__, freq);
		return -1;
	}
	else if (frequency >= 50500 && frequency < 100000)
	{
		pllbuf[CB2] = 0x51;
	}
	else if (frequency >= 100000 && frequency < 122000)
	{
		pllbuf[CB2] = 0x71;
	}
	else if (frequency >= 122000 && frequency < 129000)
	{
		pllbuf[CB2] = 0x91;
	}
	else if (frequency >= 129000 && frequency < 136000)
	{
		pllbuf[CB2] = 0xB1;
	}
	else if (frequency >= 136000 && frequency < 142500)
	{
		pllbuf[CB2] = 0xD1;
	}
	else if (frequency >= 147000 && frequency < 240000)
	{
		pllbuf[CB2] = 0x3a;
	}
	else if (frequency >= 240000 && frequency < 310000)
	{
		pllbuf[CB2] = 0x52;
	}
	else if (frequency >= 310000 && frequency < 380000)
	{
		pllbuf[CB2] = 0x72;
	}
	else if (frequency >= 380000 && frequency < 430000)
	{
		pllbuf[CB2] = 0x92;
	}
	else if (frequency >= 430000 && frequency < 578000)
	{
		pllbuf[CB2] = 0x94;
	}
	else if (frequency >= 578000 && frequency < 650000)
	{
		pllbuf[CB2] = 0xB4;
	}
	else if (frequency >= 650000 && frequency < 746000)
	{
		pllbuf[CB2] = 0xD4;
	}
	else if (frequency >= 746000 && frequency <= 858000)
	{
		pllbuf[CB2] = 0xF4;
	}
	else
	{
		printk("%s: error frequency is not supported %d\n",__FUNCTION__,freq);
		return -1;
	}
	
	pllbuf[CB1] = 0xCA;

	cb3a = 0x8C;
	cb3b = 0x84;

	tunerdivi = ((freq / 1000) * 6) + 217;
	pllbuf[0] = (u8)((tunerdivi >> 8) & 0xff);
	pllbuf[1] = (u8)(tunerdivi & 0xff);
	pllbuf[CB3] = cb3a;

	config->bypass(fe, 0);
	ret = i2c_transfer (config->tuner_i2c, &message[0], 1);
	if (ret != 1)
	{
		printk("%s: error writting to PLL : %d\n",__FUNCTION__,ret);
	}

	msleep(100);

	config->bypass(fe, 0);
	ret = i2c_transfer (config->tuner_i2c, &message[1], 1);
	if (ret != 1)
	{
		 printk("%s: error reading status : %d  \n",__FUNCTION__,ret);
	}

#define CAL_LOCK_BIT  (6)

	if( lockbuf[0]  & (1 << CAL_LOCK_BIT) )
	{
		config->bypass(fe, 1);
		printk("%s:LOCKED 0x%02x\n",__FUNCTION__,lockbuf[0]);	
	}
	else
	{
		printk("%s:NOT LOCKED 0x%02x\n",__FUNCTION__,lockbuf[0]);	
        }

	return ret;
}//end of add by hug
#endif
#define TDQD3_ZERO  0,7,1
#define TDQD3_N_MSB 0,0,7
#define TDQD3_N_LSB 1,0,8

#define TDQD3_F1    2,6,2
#define TDQD3_AGST  2,3,3
#define TDQD3_R     2,0,3

#define TDQD3_CP    3,6,2
#define TDQD3_TS    3,4,2
#define TDQD3_BS4   3,3,1
#define TDQD3_BS3   3,2,1
#define TDQD3_BS21  3,0,2

#define TDQD3_F2    4,6,2
#define TDQD3_IFW   4,4,2
#define TDQD3_CP2   4,3,1
#define TDQD3_XLO   4,1,2
#define TDQD3_POW   4,0,1

static int tdqd3_pll_set(struct dvb_frontend* fe, struct dvb_frontend_parameters* params)
{
    u32 freq,frequency;
    u32 div;
    u8  pllbuf[] = {0x0B,0xF5,0xA0,0xEC,0xC6};
    //u8  pllbuf[4];
    int R,SS,CP=-1,BS21,BS4;
    int ret;

    struct stv036x_config* config = *((struct stv036x_config**) fe->demodulator_priv);
    struct i2c_msg msg =  { .addr = config->tuner_address, .flags = 0, .buf = pllbuf, .len = sizeof(pllbuf) };

    printk("%s: %x %x %x %x %x\n",__FUNCTION__,pllbuf[0],pllbuf[1],pllbuf[2],pllbuf[3],pllbuf[4]);

#define TDQD3_IF_FREQUENCY 36000

    R  = READ_BIT(pllbuf ,TDQD3_R);

    switch(R)
    {
        case 0:
            SS = 166667;
            break;
        case 1:
            SS = 142857;
            break;
        case 2:
            SS = 80000;
            break;
        case 3:
            SS = 62500;
            break;
        case 4:
            SS = 31250;
            break;
    };

    freq = params->frequency / 1000;
    frequency = freq + TDQD3_IF_FREQUENCY;
    div  = ((frequency * 100) / ( SS ) / 10 );

    WRITE_BIT(pllbuf,TDQD3_N_MSB,(div >> 8));
    WRITE_BIT(pllbuf,TDQD3_N_LSB,(div & 0xff));

    if (freq <= 230000) {
        CP   = 0x02;
        BS21 = 0x02;
        BS4  = 0x00;
    } else if (freq  <= 782000) {
        CP   = 0x02;
        BS21 = 0x02;
        BS4  = 0x00;
    } else if (freq <= 862000) {
        CP   = 0x02;
        BS21 = 0x02;
        BS4  = 0x00;
    }

    if (CP!=-1) {
        WRITE_BIT(pllbuf,TDQD3_CP  ,CP);
        WRITE_BIT(pllbuf,TDQD3_BS21,BS21);
        WRITE_BIT(pllbuf,TDQD3_BS4 ,BS4);
    }

    printk("freq:%d %d %d %d\n",frequency,freq,div,SS);

    printk("%s: %x %x %x %x %x\n",__FUNCTION__,pllbuf[0],pllbuf[1],pllbuf[2],pllbuf[3],pllbuf[4]);
    ret = i2c_transfer (config->tuner_i2c, &msg, 1);	
    if (ret!=1) printk("%s: error writting to PLL\n",__FUNCTION__);

    return ret;
}
#endif

static int i2c_bus_count;
static struct plat_frontend_config *i2c_config;
static struct stfe ** i2c_stfe;

struct i2c_detect_results
{
    int demod_type;
    struct i2c_adapter *demod_adapter;
    u8                  demod_addr;

    int tuner_type;
    struct i2c_adapter *tuner_adapter;
    u8                  tuner_addr;

    int lnb_type;
    struct i2c_adapter *lnb_adapter;
    u8                  lnb_addr;
};

static struct i2c_detect_results results[4];

static int st_i2c_autodetect (struct i2c_adapter *adapter, u8 i2c_addr, u8 dev_addr)
{
        u8 buf[2] = { 0,0 };
        struct i2c_msg msg[] = {
                {
                        .addr = i2c_addr,
                        .flags = 0,
                        .buf = &dev_addr,
                        .len = 1
                },
                {
                        .addr = i2c_addr,
                        .flags = I2C_M_RD,
                        .buf = &buf,
                        .len = 1
                }
        };
        int b;

        b = i2c_transfer(adapter,msg,1);
        b |= i2c_transfer(adapter,msg+1,1);

        if (b!=1) return -1;

        return buf[0];
}

#if 0
static int test(struct i2c_adapter *adapter)
{
    u8 buf[16] = { 0,0,0,0 ,0,0,0,0 };
    int a,i,n;
    struct i2c_msg msg =  { .addr = 0x61, .flags = 0, .buf = &buf, .len = 1 };

    u8 buf2[] = { 0x1, 0xd1 };

    struct i2c_msg msg2 =  { .addr = 0x1c, .flags = 0, .buf = &buf2, .len = 2 };

    a = st_i2c_autodetect(adapter,0x1c,0x0);

    if (a==-1) return -1;
    printk("found %x\n",a);

    for (i=0;i<8;i++) {
        msg.len = i;
        //stv036x_writereg (state, R_I2CRPT 0x1, 0xd1);  /* open i2c bus switch */
        a = i2c_transfer(adapter,&msg2,1);

        if (a!=1) printk("ARRGHH\n");

        a = i2c_transfer(adapter,&msg,1);
        printk("%d: %d\n",i,a);
    }


}
#endif

/* Autodetecting tuner's is not as easy as it might sound */

static int st_tuner_i2c_attach(struct i2c_adapter *adapter )
{
    int n;
    char found[256];
    memset(found,0,256);
    i2c_bus_count++;

    for (n=0;n<i2c_config->nr_channels;n++)
    {
        struct plat_frontend_channel *channel = &i2c_config->channels[n];

        if (channel->demod_i2c_bus == 0xff || channel->demod_i2c_bus == i2c_bus_count)
        {
            if (!results[n].demod_type)
            {
                if (!(channel->option & STM_DEMOD_ALL) || (channel->option & STM_DEMOD_299))
                {
                    u8 addr = 0x68;
                    if (channel->demod_i2c_address != 0xff) 
                    {
                        addr = channel->demod_i2c_address;
                    }

                    if ( !found[addr] && st_i2c_autodetect(adapter, addr /* 299 Address */, 0x0) != -1 )
                    {
                        results[n].demod_type    = STM_DEMOD_299;
                        results[n].demod_adapter = adapter;
                        results[n].demod_addr    = addr;

                        results[n].tuner_type    = STM_TUNER_STB6000;
                        results[n].tuner_adapter = adapter;
                        results[n].tuner_addr    = 0x60;

                        results[n].lnb_type      = channel->option & STM_LNB_ALL;
                        results[n].lnb_adapter   = adapter;
                        results[n].lnb_addr      = channel->lnb_i2c_address;

                        found[addr] = 1;
                    }
                }
            }
            if (!results[n].demod_type)
            {
                if (!(channel->option & STM_DEMOD_ALL) || (channel->option & STM_DEMOD_360))
                {
                    u8 addr, autodetect;
                    
                    switch (channel->demod_i2c_ad01)
                    {
                        case 1:
                            addr = 0x1d;
                            break;
                        case 2:
                            addr = 0x1e;
                            break;
                        case 3:
                            addr = 0x1f;
                            break;
                        case 0:
                        default:
                            addr = 0x1c;
                    }

                    if (channel->demod_i2c_address != 0xff) 
                    {
                        addr = channel->demod_i2c_address;
                    }

                    autodetect = st_i2c_autodetect(adapter, addr , 0x0);
                    if ( !found[addr] && (autodetect != -1) )
                    {
                        results[n].demod_type    = STM_DEMOD_360;
                        results[n].demod_adapter = adapter;
                        results[n].demod_addr    = addr;

                        results[n].tuner_type    = channel->option&STM_TUNER_MASK;
                        results[n].tuner_adapter = adapter;
                        results[n].tuner_addr    = channel->tuner_i2c_address;

                        found[addr] = 1;
                    }

                }
            }
	    //add by hug start
	    if (!results[n].demod_type)
	    {
		u8 addr;

	  	if (!(channel->option & STM_DEMOD_ALL) || (channel->option & STM_DEMOD_LGS8G52))
	    	{
		    addr = 0x19;
	   	    if (channel->demod_i2c_address != 0xff)
		    {
		         addr = channel->demod_i2c_address;
		    }

		    if (!found[addr] && st_i2c_autodetect(adapter, addr, 0x00) != -1)
		    {
		        results[n].demod_type = STM_DEMOD_LGS8G52;
		        results[n].demod_adapter = adapter;
		        results[n].demod_addr = addr;

		        results[n].tuner_type = STM_TUNER_SHARP;
		        results[n].tuner_adapter = adapter;
		        results[n].tuner_addr = channel->tuner_i2c_address;

		        found[addr] = 1;
		    }
		}
	    }//end of add by hug start
	    #ifdef CONFIG_DVB_S5H1432
            if (!results[n].demod_type)//add by hug start
            {
                if (!(channel->option & STM_DEMOD_ALL) || (channel->option & STM_DEMOD_S5H1432))
                {
                    u8 addr, autodetect;
                    
                    if (channel->demod_i2c_address != 0xff) 
                    {
                        addr = channel->demod_i2c_address;
                    }

                    autodetect = st_i2c_autodetect(adapter, addr , 0x0);
                    if ( !found[addr] && (autodetect != -1) )
                    {
                        results[n].demod_type    = STM_DEMOD_S5H1432;
                        results[n].demod_adapter = adapter;
                        results[n].demod_addr    = addr;

                        results[n].tuner_type    = channel->option & STM_TUNER_MASK;
                        results[n].tuner_adapter = adapter;
                        results[n].tuner_addr    = channel->tuner_i2c_address;

                        found[addr] = 1;
			mxl5007_adapter = adapter;
                    }

                }
            }//add by hug end
	    #endif
	    if (!results[n].demod_type)
	    {
	    	if (!(channel->option & STM_DEMOD_ALL) || (channel->option &STM_DEMOD_INTEL))
		{
		    u8 addr, autodetect;

		    if (channel->demod_i2c_address != 0xff)
		    {
		    	addr = channel->demod_i2c_address;
		    }

		    autodetect = st_i2c_autodetect(adapter, addr, 0x0);
		    if (!found[addr] && (autodetect != -1))
		    {
				printk("%s found intel demod\n", __FUNCTION__);

		    	results[n].demod_type = STM_DEMOD_INTEL;
				results[n].demod_adapter = adapter;
				results[n].demod_addr = addr;

				results[n].tuner_type = channel->option & STM_TUNER_MASK;
				results[n].tuner_adapter = adapter;
				results[n].tuner_addr = channel->tuner_i2c_address;

				found[addr] = 1;
				tuner_adapter = adapter;
		    }
		}
	    }
#if defined(CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION) || defined(CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION_MODULE)
            if (!results[n].demod_type)
            {
                if (channel->option & STM_DEMOD_SAA6752) 
                {
                    printk("%s: Test SAA6752\n", __FUNCTION__);
                    // Detect SAA6752 presence
                    u8 addr = 0x1c;
                    if (channel->demod_i2c_address != 0xff) addr = channel->demod_i2c_address;
                    if (st_i2c_autodetect(adapter, addr, 0x00) != -1) 
                    {
                        printk("%s : SAA6752 detected !\n", __FUNCTION__);
                        /*                        if (st_i2c_autodetect(adapter, 0x21, 0x00) != -1) */
                        {
                            printk("%s : SAA711x detected !\n", __FUNCTION__);
                            printk("%s : => Analog Acquisition device detected !\n",
                                    __FUNCTION__);
                            results[n].demod_type = STM_DEMOD_SAA6752;
                            results[n].demod_adapter = adapter;
                            results[n].demod_addr = addr;

                            results[n].tuner_type = STM_TUNER_SAA6752;
                            results[n].tuner_adapter = adapter;
                            results[n].tuner_addr = 0x60;
                        }
                    }
                }
            }
#endif
        }
    }

}

static int st_tuner_i2c_detatch(struct i2c_adapter *adapter)
{
#warning todo, if you ever want to be able to unload this module
    return 0;
}

struct i2c_driver st_tuner_i2c_driver = {
    .id = 129,
    .attach_adapter = st_tuner_i2c_attach,
    .detach_adapter = st_tuner_i2c_detatch,
    .command        = 0,
    .driver         = {
        .owner          = THIS_MODULE,
        .name           = "i2c_stm_frontend"
    }
};

int stm_tuner_register_frontend ( struct plat_frontend_config *config, 
        struct stfe ** stfe,
        void         * private_data,
        void         * start_feed,
        void         * stop_feed )

{
    int n;
    int m = 0;

    i2c_bus_count = -1;
    i2c_config    = config;
    i2c_stfe      = stfe;

    memset( results, 0, sizeof(results) );

    for (n=0;n<config->nr_channels;n++)
    {
        if ((config->channels[n].pio_reset_bank < 19) &&
            !(config->channels[n].pio_reset_bank == 0x00))
        {  
            struct stpio_pin *reset = stpio_request_pin( config->channels[n].pio_reset_bank, config->channels[n].pio_reset_pin,"Nim Reset", STPIO_OUT);

            if (reset)
            {
                stpio_set_pin(reset,0);
                udelay(10000);
                stpio_set_pin(reset,1);
            } 
            else 
            {
                printk(KERN_WARNING "STM Frontend, couldn't get pin for reset\n");
                printk("STM Frontend, couldn't get pin for reset\n");
            }
        }
    }

    i2c_add_driver (&st_tuner_i2c_driver);

    for (n=0;n<config->nr_channels;n++)
    {
        struct dvb_frontend* new_fe = NULL;
        new_fe = NULL;

//add by hug start
#ifdef CONFIG_DVB_S5H1432
        if (results[n].demod_type == STM_DEMOD_S5H1432 && results[n].tuner_type == STM_TUNER_MXL5007)
        {
 			struct s5h1432_config *a = (struct s5h1432_config*)kmalloc(sizeof(struct s5h1432_config), GFP_KERNEL);

			a->tuner_address       = results[n].tuner_addr;
			a->serial_not_parallel = config->channels[n].option & STM_SERIAL_NOT_PARALLEL;
			
			new_fe = s5h1432_attach(a,results[n].demod_adapter,results[n].demod_addr);
         }
#endif	//add by hug end

#define CONFIG_DVB_INTEL6353
#ifdef CONFIG_DVB_INTEL6353
	if (results[n].demod_type == STM_DEMOD_INTEL && results[n].tuner_type == STM_TUNER_MXL5007)
	{
		struct intel_config *a = (struct intel_config*)kmalloc(sizeof(struct intel_config), GFP_KERNEL);

		a->tuner_address = results[n].tuner_addr;
		a->tuner_i2c = results[n].tuner_adapter;
		a->serial_not_parallel = config->channels[n].option & STM_SERIAL_NOT_PARALLEL;
		new_fe = intel_attach(a, results[n].demod_adapter, results[n].demod_addr);
	}
#endif

#ifdef CONFIG_DVB_STV0299
        if (results[n].demod_type == STM_DEMOD_299 && results[n].tuner_type == STM_TUNER_STB6000)
        {
            static struct stv0299_config *a;

            if ((config->channels[n].option & STM_SERIAL_NOT_PARALLEL))
            {
                a = &stb6000_nim299_config_serial; 
            }
            else 
            {
                a = &stb6000_nim299_config_parallel;
            }
            new_fe = stv0299_attach(a ,results[n].demod_adapter);
        }
#endif

#ifdef CONFIG_DVB_STV0360
        if (results[n].demod_type == STM_DEMOD_360 && results[n].tuner_type == STM_TUNER_DTT73)
        { 
            struct stv036x_config *tmm = (struct stv036x_config*)kmalloc(sizeof(struct stv036x_config),GFP_KERNEL);
            printk("DEMOD stv36x with DTT7300 tuner\n");
            tmm->tuner_i2c           = results[n].tuner_adapter;
            tmm->tuner_address       = results[n].tuner_addr;
            tmm->pll_init            = &dtt7300x_pll_init;
            tmm->pll_set             = &dtt7300x_pll_set;

            tmm->serial_not_parallel = config->channels[n].option & STM_SERIAL_NOT_PARALLEL;

            new_fe = stv036x_attach(tmm,results[n].demod_adapter,results[n].demod_addr);  
        }
        if (results[n].demod_type == STM_DEMOD_360 && results[n].tuner_type == STM_TUNER_DTT75)
        { 
            struct stv036x_config *tmm = (struct stv036x_config*)kmalloc(sizeof(struct stv036x_config),GFP_KERNEL);
            printk("DEMOD stv36x with DTT754xx tuner\n");
            tmm->tuner_i2c           = results[n].tuner_adapter;
            tmm->tuner_address       = results[n].tuner_addr;
            tmm->pll_init            = &dtt754xx_pll_init;
            tmm->pll_set             = &dtt754xx_pll_set;

            tmm->serial_not_parallel = config->channels[n].option & STM_SERIAL_NOT_PARALLEL;

            new_fe = stv036x_attach(tmm,results[n].demod_adapter,results[n].demod_addr);  
        }
        if (results[n].demod_type == STM_DEMOD_360 && results[n].tuner_type == STM_TUNER_DTT73200)
        { 
            struct stv036x_config *tmm = (struct stv036x_config*)kmalloc(sizeof(struct stv036x_config),GFP_KERNEL);
            printk("DEMOD stv36x with DTT73200 tuner\n");
            tmm->tuner_i2c           = results[n].tuner_adapter;
            tmm->tuner_address       = results[n].tuner_addr;
            tmm->pll_init            = &dtt73200_pll_init;
            tmm->pll_set             = &dtt73200_pll_set;

            tmm->serial_not_parallel = config->channels[n].option & STM_SERIAL_NOT_PARALLEL;

            new_fe = stv036x_attach(tmm,results[n].demod_adapter,results[n].demod_addr);  
        }
#endif
	//add by hug start
#ifdef CONFIG_DVB_LGS8G52
        if (results[n].demod_type == STM_DEMOD_LGS8G52 && results[n].tuner_type == STM_TUNER_SHARP)
	{
		struct lgs8g52_config *tmm = (struct lgs8g52_config*)kmalloc(sizeof(struct lgs8g52_config),GFP_KERNEL);
		printk("DEMOD lgs8g52 with SHARP 6093 tuner\n");
		tmm->tuner_i2c           = results[n].tuner_adapter;
		tmm->tuner_address       = results[n].tuner_addr;
		tmm->pll_init            = &sharp6093_pll_init;
		tmm->pll_set             = &sharp6093_pll_set;

		tmm->serial_not_parallel = config->channels[n].option & STM_SERIAL_NOT_PARALLEL;

		new_fe = lgs8g52_attach(tmm,results[n].demod_adapter,results[n].demod_addr);
	}
#endif
//end add by hug

        /* WYPLAY ACQUISITION */
#if defined(CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION) || defined(CONFIG_DVB_WYPLAY_ANALOG_ACQUISITION_MODULE)
        if( results[n].demod_type == STM_DEMOD_SAA6752	&& results[n].tuner_type == STM_TUNER_SAA6752) 
        {
            printk("%s: ADDING frontend for SAA6752\n", __FUNCTION__);
            printk("%s : Attach SAA6752 !\n", __FUNCTION__);
            new_fe = wyvideocapture_dvb_fe_attach();
        }
#endif
        /* WYPLAY ACQUISITION */

        if (results[n].lnb_type == STM_LNB_LNBH221 && new_fe )
        {
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17)
            new_fe->ops.set_tone    = lnbh221_set_tone;
            new_fe->ops.set_voltage = lnbh221_set_voltage;
#else	/* STLinux 2.2 or older */
            new_fe->ops->set_tone    = lnbh221_set_tone;
            new_fe->ops->set_voltage = lnbh221_set_voltage;
#endif

            printk("DVB: registering frontend %d with ONCHIP DiseqC support\n",m);

            if (config->channels[n].option & STM_DISEQC_STONCHIP && new_fe )
            {
                stm_diseqc_register_frontend ( new_fe );
            }
        }

        if ( new_fe || ((config->channels[n].option & STM_DEMOD_ALL) == STM_DEMOD_NONE) ) 
        {
            stfe[m] = stfe_create( private_data, start_feed, stop_feed );
            stfe[m]->mapping = n;
            if (new_fe) 
            {
                dvb_register_frontend( &stfe[m]->adapter, new_fe );
            }
            m++;
        }

        if ( new_fe && results[n].tuner_type)
        {
            struct st_tuner *tuner = (struct st_tuner*)kmalloc(sizeof(struct st_tuner),GFP_KERNEL);

            tuner->type = 1;
            tuner->i2c_addr_demod = results[n].demod_addr;
            tuner->i2c_addr_tuner = results[n].tuner_addr;
            tuner->i2c_addr_lnb   = results[n].lnb_addr;
            tuner->i2c_bus        = results[n].tuner_adapter;

            stfe[m-1]->tuner_data = tuner;
        }
    }

    return 0;
}

int stm_tuner_deregister_frontend ( void )
{
    i2c_del_driver (&st_tuner_i2c_driver);

    return 0;
}
